************************************************************************
*****                                                              *****
***** @program name: x1_ActuationModeCy2O2u                        *****
***** @task:   compute actuation mode for oscillating cylinder     *****
***** @input:  ...                                                 *****
***** @output: flow (file name from y1_Modes)                      *****
*****                                                              *****
************************************************************************
*****                                                              *****
***** @author:  Bernd Noack                                        *****
***** @see:     res.20100408.xAMC_ToDo                             *****
***** @revised: 2010-11-07 Poitiers                                *****
***** @revised: 2010-07-04 Poitiers                                *****
***** @created: 2010-07-04 Poitiers                                *****
*****                                                              *****
************************************************************************
*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012
      program x1ActuationModeCy2O2u
      implicit none
* --- Variables for grid and flow
      include 'xv3X.include'           ! NX, ...
      include 'xv3X_Grid.include'      ! xyzG
      include 'xv3X_Flow.include'      ! fFlowG
* --- Auxiliary variables
      real*8  fX, fY, fU, fV
      integer iMode, iGrid
* --- Auxiliary variables for file name
      character*120 fn        ! file name
      integer       fnLength  ! length of file name (# of characters)

* === Begin main segment ============================================= *
      write(*,*) '****************************************'
      write(*,*) '*******   Add actuation mode    ********'
      write(*,*) '*******   see Noack+ 2004 ACC   ********'
      write(*,*) '****************************************'
      write(*,*) '-------------- BN 101109 ---------------'
      write(*,*)
* --- Read grid
      call readGrid
* --- Compute actuation mode
      iMode = -1
      do iGrid = 1, NX
        fX = xyzG(iGrid,1,1,1)
        fY = xyzG(iGrid,1,1,2)
        call compActuationMode(fX,fY,fU,fV)
        fFlowG(iGrid,1,1,1) = fU
        fFlowG(iGrid,1,1,2) = fV
      end do
* --- Write actuation mode
      call readFilenameMode1(iMode,fn,fnLength)
      call writeFlow        (fn,fnLength,fFlowG)
* === End of main segment ============================================ *
      stop
      end

************************************************************************
* === Actuation mode ================================================= *
      subroutine compActuationMode(x,y,u,v)
      implicit none
      real*8 x,  y,  u,  v
      real*8 r2, r,  cs, sn, ur, ut
      real*8 Re, delta, expf, corrf
* --- Compute radial coordinates
      r2    = x*x + y*y
      r     = sqrt(r2)
      cs    = x/r
      sn    = y/r
* --- Compute delta from Re
      Re    = 10.
      delta = 2.45/sqrt(Re)  ! from PF94
      r     = r*2.
      r2    = r*r
      expf  = exp(-(r-1)/delta)
      corrf = 1-expf
* --- Compute radial and azimuthal velocity component
* --- of the transverse actuation mode.
* ......................................................................
* === Note the transformation r' =r/R = r/0.5
* === Psi     = -(r-1/r) (1-exp(-(r-1)/d)) cos(theta)
* === u_r     =   1./r *D_theta Psi
* === u_theta = -       D_r Psi
* ......................................................................
      ur    = sn*(1-1/r2)*corrf
      ut    = cs*((1+1/r2)*corrf 
     &           +(r-1/r )*expf/delta)
C DEB WRITE(*,*) ur, ut, cs, sn, cs*cs+sn*sn 
* --- Compute Cartesian velocity components
      u =  cs*ur - sn*ut
      v =  sn*ur + cs*ut
* --- End of subroutine
      return
      end

