************************************************************************
*****                                                              *****
***** @program name: x0_Statistics                                 *****
***** @task:   compute mean flow and variances of snapshots        *****
***** @input:  ...                                                 *****
***** @output: ...                                                 *****
*****                                                              *****
************************************************************************
*****                                                              *****
***** @author:  Bernd Noack                                        *****
***** @see:     res.20100408.xAMC_ToDo                             *****
***** @revised: 2010-11-09 Poitiers                                *****
***** @revised: 2010-04-15 Poitiers                                *****
***** @created: 2010-04-14 Poitiers                                *****
*****                                                              *****
************************************************************************
*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012
      program x0Statistics
      implicit none
* --- Variables for grid and flow
      include 'xv3X.include'           ! NXP, ...
      include 'xv3X_Flow.include'      ! fFlowG
      include 'xv3X_Flow1.include'     ! fFlowG
      include 'xv3X_Flow2.include'     ! fFlowG
* ---
      integer NFlow0, iFlow
* --- Auxiliary variables for file name
      character*120 fn        ! file name
      integer       fnLength  ! length of file name (# of characters)
* --- Further auxiliary files
      real*8  fNorm
* === Begin main segment ============================================= *
      write(*,*) '****************************************'
      write(*,*) '******* Statistics of snapshots ********'
      write(*,*) '****************************************'
      write(*,*) '-------------- BN 101109 ---------------'
      write(*,*)
* --- Initialize
      call readGrid
      call zeroFlow(fFlow1G) ! -> mean flow
      call zeroFlow(fFlow2G) ! -> variances
      call countFlows0(NFlow0)
* --- Read the snapshots
      do iFlow = 1, NFlow0
        call readFilenameFlow0(iFlow,fn,fnLength)
        call readFlow(fn,fnLength,fFlowG)
        call maxnormFlow(fFlowG,fNorm)
        call addScaledFlow (1.0D0, fFlowG,fFlow1G) !-> mean
        call addProductFlow(fFlowG,fFlowG,fFlow2G) !-> product
      end do
* --- Compute the mean flow
      write(*,*) '*** Computing the mean flow ...'
      call scaleFlow  (1.D0/NFlow0,fFlow1G) ! -> mean
      call maxnormFlow (fFlow1G,fNorm)      ! -> mean
* --- Compute the fluctuation flow
      write(*,*) '*** Computing the fluctuation level ...'
      call scaleFlow  (1.D0/NFlow0,fFlow2G)    ! a := second moment^2 
      call zeroFlow(fFlowG)
      call addProductFlow(fFlow1G,fFlow1G,fFlowG)  ! b := mean^2
      call addScaledFlow (-1.D0,  fFlowG, fFlow2G) ! variance = a-b
      call maxnormFlow (fFlow2G,fNorm)
* --- Write the files
* ......... 123456789012345
      fn = 'z0_FlowMean.dat'
      fnLength = 15
      call writeFlow(fn,fnLength,fFlow1G)
* ......... 1234567890123456789
      fn = 'z0_FlowVariance.dat'
      fnLength = 19
      call writeFlow(fn,fnLength,fFlow2G)
* === End of main segment ============================================ *
      stop
      end
