************************************************************************
*****                                                              *****
*****        U N F I N I S H E D                                   *****
*****                                                              *****
*****                                                              *****
***** @program name: x0Statistics3c                                *****
*****                                                              *****
***** @task: general purpose program for statistics                *****
*****        of first and second moments from snapshots            *****
***** @input:  yX_GM.dat and files therein                         *****
***** @output: z0_Flow_m1.dat (first moments)                      *****
*****          z0_Flow_c2.dat (standard deviations)                *****
*****                                                              *****
************************************************************************
*****                                                              *****
***** @author:  Bernd Noack                                        *****
*****                                                              *****
***** @see:  1/Proc/xAMC/docu/xGM2c060617plan.tex                  *****
*****                                                              *****
***** @revised: 2011-01-25 Berlin                                  *****
***** @revised: 2010-11-09 Poitiers                                *****
***** @revised: 2010-10-25 Poitiers                                *****
***** @created: 2007-07-31 Berlin                                  *****
*****                                                              *****
************************************************************************
*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012
      program x0Statistics3c
      implicit none
* --- Variables for file system
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
* --- Variables for original data
      include 'xv3X.include'           ! for NxRawP, ...
      include 'xv3X_Flow.include'      ! for fFlowG
      include 'xv3X_Flow1.include'     ! for fFlow1G

* --- Auxiliary variables
      integer iFile          ! index of file
      integer iFlow          ! counter for snapshots
      integer iDomain        ! counter for domains
      integer ix, iy         ! counter for grid points
      character*120 fn       ! file name
      integer       fnLength ! file name length
      integer       iDummy
      real*8        fDummy

***** Begin of main segment ********************************************
* --- User info ------------------------------------------------------ *
      write(*,*) '**********************************'
      write(*,*) 'First and second moment statistics'
      write(*,*) 'of xAMC data on 3d Cartesian grid'
      write(*,*) '----- Bernd Noack, 2010-11-09 ----'
      write(*,*) '**********************************'
      write(*,*)

* --- Read job and related file -------------------------------------- *
      call readYGA
* ---
      call countFlows1(iDummy)
      NFlows = iDummy
* ---
      call readGrid
* --- Initialize statistics
      call zeroFlow(fFlow1G)
      fDummy = 1./NFlows
* --- loop over snapshots -------------------------------------------- *
      do iFlow = 1, NFlows
        call readFilenameFlow1(iFlow,fn,fnLength)
        call readFlow(               fn,fnLength,fFlowG)
        call addScaledFlow(fDummy,fFlowG,fFlow1G)
* --- end of loop over snapshots
      end do
* --------------- 12345678901
      fn       = 'z0_Mean.dat'
      fnLength =  11
      call writeFlow(fn,fnLength,fFlowG)
***** end of main segment **********************************************
      stop
      end