************************************************************************
*****                                                              *****
***** @program name: x0_Tecplot3c                                  *****
*****                                                              *****
***** @task: generate Tecplot files                                *****
*****                                                              *****
***** @input:  -- y1_GM and associated files                       *****
*****          -- file name (specified by user)                    *****
*****          -- 3D flow field (specified by file name)           *****
***** @output: tmp.tec (TecPlot file)                              *****
*****                                                              *****
************************************************************************
*****                                                              *****
***** @author:  Bernd Noack                                        *****
*****                                                              *****
***** @see:     1/Proc/xAMC/docu/                                  *****
*****                                                              *****
***** @revised: 2010-11-09 Poitiers                                *****
***** @created: 2007-08-03 Berlin                                  *****
*****                                                              *****
************************************************************************
*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012
      program x0Tecplot3c
      implicit none
* --- Variables for original data
      include 'xv3X.include'        ! for NX, ...
      include 'xv3X_Grid.include'   ! for grid
      include 'xv3X_Flow.include'   ! for fFlowG
* --- Auxiliary variables
      integer iOpt, iFile
      character*120 fn       ! file name
      integer       fnLength ! file name length
      real*8        iDummy
      real*8        fDummy

***** Begin of main segment ********************************************
* --- User info ------------------------------------------------------ *
      write(*,*) '*************************************'
      write(*,*) 'Create TecPlot file for visualization'
      write(*,*) '  of flow data on 3d Cartesian grid'
      write(*,*) '--- Bernd R. Noack, 2010-11-09 ---'
      write(*,*) '**********************************'
      write(*,*)
      write(*,*) '< 1 > ... snapshot'
      write(*,*) '< 2 > ... POD mode'
* --- User input
      read (*,*) iOpt
      write(*,*) 'File number:'
      read (*,*) iFile 
* --- Load grid and flow --------------------------------------------- *
* --- Read the job file (to get the file names, ...)
      call readYGA
      call readYGP
* --- Read the grid
      call readGrid
* --- Determine the file name
      if (iOpt.eq.1) then
        call readFilenameFlow1(iFile,fn,fnLength)
      else
        call readFilenameMode1(iFile,fn,fnLength)
      end if
* --- Read the flow field
      call readFlow(fn,fnLength,fFlowG)
* --- Create TecPlot file -------------------------------------------- *
      fnLength = 7
      fn       = 'tmp.tec'
      call writeTecPlot3c(fn,fnLength,fFlowG)
* --- Output --------------------------------------------------------- *
      write(*,*) 'Visualize TecPlot file with'
      write(*,*) '>>> visit tmp.tec' 
***** End of main segment **********************************************
      stop
      end

************************************************************************
************************************************************************
* === Write in the format of the original snapshot =================== *
      subroutine writeTecPlot3c(fn,fnLength,fFlowG)
      implicit none
* --- Index and flow field (raw data=entire field)
      include 'xv3X.include'           ! for NX, ...
      include 'xv3X_Grid.include'      ! for xG, ...
      include 'xv3X_Flow.include'      ! for fFlowG
* --- File name
      character*120 fn                 ! file name of snapshot
      integer       fnLength           ! file name length
* --- Auxiliary variables
      integer       ix, iy, iz, iVar   ! counters
* --- Begin of segment ----------------------------------------------- *
* --- User info
      write(*,'(a,i4,a,i4,a,i4,a,i2,3a)') 
     &           'Writing the TecPlot file ',
     &            NX,'x',NY,'x',NZ,'x',NVarFlow,
     &           ' in << ', fn(1:fnLength), ' >> ...'
* --- Writing the flow ----------------------------------------------- *
      open(10,file=fn(1:fnLength),status='UNKNOWN',form='formatted')
C XXX write(10,'(a)') 'VARIABLES = X Y Z U V P' ! for 2D flow + p
      write(10,'(a)') 'VARIABLES = X Y Z U V W'
      write(10,'(4(a,i4.4),a)') 
     &         'ZONE T ="Element ',1,
     &         '" I=',NX,' J=',NY,' K=',NZ,' F=POINT'
      do iz = 1, NZ
      do iy = 1, NY
      do ix = 1, NX
        write(10,*) xG(ix), yG(iy), zG(iz), 
     &             (fFlowG(ix,iy,iz,iVar),iVar=1,NVarFlow)  ! for 3D flow
C XXX&             (fFlowG(ix,iy,iz,iVar),iVar=1,NVarFlow), ! for 2D flow
C XXX&             (0.0,                  iVar=NVarFlow+1,3)
      end do
      end do
      end do
      close(10)
* --- End of segment ------------------------------------------------- *
      return
      end
