************************************************************************
*****                                                              *****
***** @program name: x0_addForce2u                                 *****
***** @task:   create a volume force                               *****
***** @input:  ...                                                 *****
***** @output: ...                                                 *****
*****                                                              *****
************************************************************************
*****                                                              *****
***** @author:  Bernd Noack                                        *****
***** @see:     res.20100408.xAMC_ToDo                             *****
***** @revised: 2010-11-07 Poitiers                                *****
***** @revised: 2010-05-16 Poitiers                                *****
***** @created: 2010-05-16 Poitiers                                *****
*****                                                              *****
************************************************************************
*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012
      program x0addForce2u
      implicit none
* --- Variables for grid and flow
      include 'xv3X.include'           ! NGrid, ..., NTriangle, ...
      include 'xv3X_Grid.include'      ! xyzG
      include 'xv3X_Flow.include'      ! fFlowG
* --- Auxiliary variables for file name
      integer       iForce             ! # of volume force
      character*120 fn                 ! file name
      integer       fnLength           ! length of file name (# of characters)
* --- Further auxiliary variables
      integer iGrid
      real*8  fPosXForce,   fPosYForce ! position
      real*8  fRadiusForce, fRadius    ! radius
      

* === Begin main segment ============================================= *
      write(*,*) '****************************************'
      write(*,*) '******* Add a volume force *************'
      write(*,*) '****************************************'
      write(*,*) '-------------- BN 100516 ---------------'
      write(*,*)
      write(*,*) 'Read # of volume force:'
      read (*,*)  iForce
      write(*,*) 'Read (x,y) position of center of disc:'
      read (*,*)  fPosXForce, fPosYForce
      write(*,*) 'Read radius of disc:'
      read (*,*)  fRadiusForce
      write(*,*) '------------------------------------'
      write(*,'(a12,a12,a12)') 'x_c', 'y_c', 'r'
      write(*,'(3f12.5)') fPosXForce, fPosYForce, fRadiusForce
      write(*,*) '------------------------------------'
      
* --- Initialize
      call readGrid
      call zeroFlow(fFlowG)
* --- Determine transverse unit force in disc 
      do iGrid = 1, NX
        fRadius = (xyzG(iGrid,1,1,1)-fPosXForce)**2
     &          + (xyzG(iGrid,1,1,2)-fPosYForce)**2
        fRadius = sqrt(max(0.0,fRadius))
        if (fRadius.lt.fRadiusForce) then
          fFlowG(iGrid,1,1,2) = 1.0
        end if
      end do
* --- Write force field
* ......... 123456789012
      fn = 'z0_Force.000'
      fnLength = 12
      write(fn,'(a,i3.3)') 'z0_Force.',iForce
      call writeFlow(fn,fnLength,fFlowG)
* === End of main segment ============================================ *
      stop
      end
