************************************************************************
*****                                                              *****
***** @program name: x0_plotFlow3c                                 *****
***** @task:   plot flow of whole domain                           *****
***** @input:  ...                                                 *****
***** @output: ...                                                 *****
*****                                                              *****
************************************************************************
*****                                                              *****
***** @author:  Bernd Noack                                        *****
***** @see:     Noack & Cordier 2010 CEAT report                   *****
***** @revised: 2011-02-03 Poitiers                                *****
***** @created: 2011-02-03 Poitiers                                *****
*****                                                              *****
************************************************************************
*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012
      program x0plotFlow3c
      implicit none
* --- Variables for grid and flow
      include 'xv3X.include'           ! NXP, ...
      include 'xv3X_Flow.include'      ! fFlowG, ...
* === Begin main segment ============================================= *
* --- User info
      write(*,*) '****************************************'
      write(*,*) '******* Plot 3-D flow           ********'
      write(*,*) '******* (mid-plane slices)      ********'
      write(*,*) '******* of the whole domain     ********'
      write(*,*) '****************************************'
      write(*,*) '-------------- BN 110203 ---------------'
      write(*,*)
      call readData(fFlowG)
      call writeGNUPlotData(fFlowG)
      call writeGNUPlotScript(fFlowG)
* === End of main segment ============================================ *
      stop
      end

************************************************************************
* === Read data ====================================================== *
      subroutine readData(fFlowG)
* --- Variables for grid and flow
      include 'xv3X.include'           ! NXP, ...
      include 'xv3X_Grid.include'      ! xG, ...
      include 'xv3X_Flow.include'      ! fFlowG, ...
* --- Auxiliary variables for file name and its existence
      character*120 fn        ! file name
      integer       fnLength  ! length of file name (# of characters)
* --- Further auxiliary variables
      integer iOption, iFlow
      common / IDFlow / iOption, iFlow
* --- Begin of subroutine -------------------------------------------- *
      write(*,*) '< 1 > ... shapshot << y1_Flows.dat >>'
      write(*,*) '< 2 > ... mode     << y1_Modes.dat >>'
* --- User input
      read (*,*) iOption
      if (iOption.eq.1) then
        write(*,*) 'snapshot number:'
        read(*,*) iFlow
        call readFilenameFlow1(iFlow,fn,fnLength)
      else if (iOption.eq.2) then
        write(*,*) 'mode number:'
        read(*,*) iFlow
        call readFilenameMode1(iFlow,fn,fnLength)
      end if
* --- Read grid and flow
      call readGrid
      call readFlow(fn,fnLength,fFlowG)
* --- End of subroutine ---------------------------------------------- *
      return
      end

************************************************************************
* === Write data for gnuplot figures ================================= *
      subroutine writeGNUPlotData(fFlowG)
* --- Variables for grid and flow
      include 'xv3X.include'           ! NXP, ...
      include 'xv3X_Grid.include'      ! xG, ...
      include 'xv3X_Flow.include'      ! fFlowG, ...
* --- Auxiliary variables for file name and its existence
      character*120 fn        ! file name
      integer       fnLength  ! length of file name (# of characters)
* --- Further auxiliary variables
      integer iVar
* --- Begin of subroutine -------------------------------------------- *
      write(*,*) 'Preparing << gnu_xxuu.dat >> ... / dim=', NVarGrid
      do iVar = 1, NVarFlow
* ----- xy plane
        if (NVarGrid.ge.2) then
* ....................123456789012
          fn       = 'gnu_xyu1.dat'
          fnLength = 12
          write(fn,'(a,i1.1,a)') 'gnu_xyu',iVar,'.dat'
          write(*,'(a,a,a)') 'Writing << ',fn(1:fnLength),' >> ...'
          open(10,file=fn(1:fnLength),form='FORMATTED')
          iz = max(1,NZ/2) ! for 2D flow
          do iy = 1, NY
          do ix = 1, NX
            write(10,*) xG(ix), yG(iy), fFlowG(ix,iy,iz,iVar)
          end do
          write(10,*)
          end do
          close(10)
        end if
* ----- xz plane
        if (NVarGrid.ge.3) then
* ....................123456789012
          fn       = 'gnu_xzu1.dat'
          fnLength = 12
          write(fn,'(a,i1.1,a)') 'gnu_xzu',iVar,'.dat'
          write(*,'(a,a,a)') 'Writing << ',fn(1:fnLength),' >> ...'
          open(10,file=fn(1:fnLength),form='FORMATTED')
          iy = NY/2
          do iz = 1, NZ
          do ix = 1, NX
            write(10,*) xG(ix), zG(iz), fFlowG(ix,iy,iz,iVar)
          end do
          write(10,*)
          end do
          close(10)
        end if
* ----- zy plane (abscissa = z, ordinate = y)
        if (NVarGrid.ge.3) then
* ....................123456789012
          fn       = 'gnu_yzu1.dat'
          fnLength = 12
          write(fn,'(a,i1.1,a)') 'gnu_zyu',iVar,'.dat'
          write(*,'(a,a,a)') 'Writing << ',fn(1:fnLength),' >> ...'
          open(10,file=fn(1:fnLength),form='FORMATTED')
          ix = NX/2
          do iy = 1, NY
          do iz = 1, NZ
            write(10,*) zG(iz), yG(iy), fFlowG(ix,iy,iz,iVar)
          end do
          write(10,*)
          end do
          close(10)
        end if
      end do
* --- End of subroutine ---------------------------------------------- *
      return
      end

************************************************************************
* === Write scripts for gnuplot figures ============================== *
      subroutine writeGNUPlotScript(fFlowG)
* --- Variables for grid and flow
      include 'xv3X.include'           ! NXP, ...
      include 'xv3X_Flow.include'      ! fFlowG, ...
      integer iOption, iFlow
      common / IDFlow / iOption, iFlow
* --- Auxiliary variables for file name and its existence
      character*120 fn        ! file name
      integer       fnLength  ! length of file name (# of characters)
* --- Further auxiliary variables
      integer iPlane, NPlane, iVar
* --- Begin of subroutine -------------------------------------------- *
      write(*,*) 'Preparing << gnu_xxuu.[gnuplot,sh] >> ...'
* --- initialize # of planes
      NPlane = 0
      if (NVarGrid.eq.2) NPlane=1
      if (NVarGrid.eq.3) NPlane=3
      write(*,*) '# of planes         = ',NPlane
      write(*,*) '# of flow variables = ',NVarFlow
      write(*,*) 'Writing preamble of gnuplot script ...'
* --- initialize LaTeX file
      open(12,file='gnu_xxuu.tex',
     &        form='formatted',status='UNKNOWN')
* === NOTE: char(92)='\' should run on all fortran compilers.
* === Different fortran compilers  assume 
* === '\' exclusive or '\\' for the backslash.
      write(12,'(2a)') char(92)//'documentclass[a4paper]',
     &               '{article}'
      write(12,'(a)') char(92)//'usepackage{graphicx}'
      write(12,'(a)') char(92)//'DeclareGraphicsExtensions{png}'
      write(12,'(a)') char(92)//'begin{document}'
      write(12,'(a)') char(92)//'pagestyle{empty}'
* --- Linux script for showing all figures
      open(11,file='gnu_xxuu.sh',
     &        form='formatted',status='UNKNOWN')
      write(11,'(a)') 'source x_defineLocalVariables.sh'
* --- initialize preamble of gnuplot
      open(10,file='gnu_xxuu.gnuplot',
     &        form='formatted',status='UNKNOWN')
      write(10,'(a)') 'set view map          # 2D perspective'
      write(10,'(a)') 'unset surface         # no surface'
      write(10,'(a)') 'set style data lines  # plot some lines'
      write(10,'(a)') 'set contour #  add background color to lines'
      write(10,'(a)') 'set cntrparam levels 10 # 10 lines/colors'
      write(10,'(a)') 'set pm3d    # user-definer color code'
      write(10,'(a,a)') 'set palette model RGB defined ',
     &         '(0 "blue", 0.5 "yellow", 1 "red")'
* --- Begin of loops over planes and variables
      write(*,*) 'Running the loop over planes and variables ...'
      do iPlane = 1, NPlane
      write(12,'(a)') char(92)//'enlargethispage{20mm}'
C XXX if (iPlane.eq.1) write(12,'(a)') char(92)//'fbox{$x-y$ plane}'
C XXX if (iPlane.eq.2) write(12,'(a)') char(92)//'fbox{$x-z$ plane}'
C XXX if (iPlane.eq.3) write(12,'(a)') char(92)//'fbox{$z-y$ plane}'
C XXX write(12,'(a)') char(92)//'newline'
      do iVar   = 1, NVarFlow
* ----- data file name
* ..................123456789012
        fn       = 'gnu_xyu1.dat'
        fnLength = 12
        write(fn,'(a,i1.1,a)') 'gnu_xyu',iVar,'.dat'
        if (iPlane.eq.2) fn(5:6) = 'xz'
        if (iPlane.eq.3) fn(5:6) = 'zy'
        write(*,'(a,a,a)') 'Script for << ',fn(1:fnLength),' >> ...'
* ----- gnuplot script "axis caption"
        if (iPlane.eq.1) then
          write(10,'(a)') 'set xlabel "x"'
          write(10,'(a)') 'set ylabel "y"'
        else if (iPlane.eq.2) then
          write(10,'(a)') 'set xlabel "x"'
          write(10,'(a)') 'set ylabel "z"'
        else if (iPlane.eq.3) then
          write(10,'(a)') 'set xlabel "z"'
          write(10,'(a)') 'set ylabel "y"'
        end if
* ----- gnuplot script "variable"
        if (iOption.eq.1) then
        if (iVar.eq.1) then
          write(10,'(a,i4,a)') 
     &             'set title "xAMC: u of snapshot #',iFlow,'"'
        else if (iVar.eq.2) then
          write(10,'(a,i4,a)') 
     &             'set title "xAMC: v of snapshot #',iFlow,'"'
        else if (iVar.eq.3) then
          write(10,'(a,i4,a)') 
     &             'set title "xAMC: w of snapshot #',iFlow,'"'
        else if (iVar.eq.4) then
          write(10,'(a,i4,a)') 
     &             'set title "xAMC: p of snapshot #',iFlow,'"'
        end if
        else
        if (iVar.eq.1) then
          write(10,'(a,i4,a)') 
     &             'set title "xAMC: u of mode #',iFlow,'"'
        else if (iVar.eq.2) then
          write(10,'(a,i4,a)') 
     &             'set title "xAMC: v of mode #',iFlow,'"'
        else if (iVar.eq.3) then
          write(10,'(a,i4,a)') 
     &             'set title "xAMC: w of mode #',iFlow,'"'
        else if (iVar.eq.4) then
          write(10,'(a,i4,a)') 
     &             'set title "xAMC: p of mode #',iFlow,'"'
        end if
        end if
* ----- gnuplot visualization of the scalar
        write(10,'(3a)') 'splot  ''',fn(1:fnLength),''''
        write(10,'(3a)') 'set out "',fn(1:fnLength-3)//'png','"'
        write(10,'(3a)') 'set terminal png'
        write(10,'(3a)') 'replot'
* ----- Update Linux script
        write(11,'(3a)') '$PNGViewer ',fn(1:fnLength-3)//'png'
* ----- Update LaTeX file
        write(12,'(a)') char(92)//'vspace{-20mm}'
C XXX   write(12,'(a)') char(92)//'begin{figure}[b]'
        write(12,'(a)') char(92)//'begin{flushright}'
        write(12,'(4a)') 
     &  char(92)//'includegraphics[height=70mm,width=120mm,angle=0]{',
     &         fn(1:fnLength-3),'png}' 
C XXX   if (iVar.eq.1) write(12,'(a)') '{$u$}'
C XXX   if (iVar.eq.2) write(12,'(a)') '{$v$}'
C XXX   if (iVar.eq.3) write(12,'(a)') '{$w$}'
C XXX   if (iVar.eq.4) write(12,'(a)') '{$p$}'
C XXX   write(12,'(a)') char(92)//'newline'
        write(12,'(a)') char(92)//'end{flushright}'
C XXX   write(12,'(a,i4,a)') 
C XXX &         char(92)//'caption{',iVar,'-th velocity component'
C XXX    if (iPlane.eq.1) write(12,'(a)') 'in the $x-y$ plane.}'
C XXX    if (iPlane.eq.2) write(12,'(a)') 'in the $x-z$ plane.}'
C XXX    if (iPlane.eq.3) write(12,'(a)') 'in the $z-y$ plane.}'
C XXX    write(12,'(a)') char(92)//'end{figure}'
* --- End of loops over planes and variables
      end do
      write(12,'(a)') char(92)//'clearpage'
      end do
* --- close gnuplot script
      close(10) ! gnuplot script
      close(11) ! Linux script for showing the png files
      write(12,'(a)') char(92)//'end{document}'
      close(12) ! LaTeX file including the png files
      call system('chmod u+x gnu_xxuu.sh')
      call system('gnuplot gnu_xxuu.gnuplot')
      call system('pdflatex gnu_xxuu.tex')
* --- End of subroutine ---------------------------------------------- *
      return
      end
