************************************************************************
*****                                                              *****
***** @program name: x0_plotFlows2u                                *****
***** @task:   plot flow of whole domain                           *****
***** @input:  ...                                                 *****
***** @output: ...                                                 *****
*****                                                              *****
************************************************************************
*****                                                              *****
***** @author:  Bernd Noack                                        *****
***** @see:     Noack & Cordier 2010 CEAT report                   *****
***** @revised: 2012-01-25 Paris (./yTools)                        *****
***** @revised: 2011-02-11 Poitiers                                *****
***** @created: 2011-02-04 Poitiers                                *****
*****                                                              *****
************************************************************************
*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012
      program x0plotFlows2u
      implicit none
* --- Galerkin system
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
* --- auxiliary variables
      integer iOption, iFlow, iFlowMin, iFlowMax
      integer       fnLength
      character*120 fn, Line
* === Begin main segment ============================================= *
* --- User info
      write(*,*) '****************************************'
      write(*,*) '******* Plot 2u flow sets       ********'
      write(*,*) '******* of the whole domain     ********'
      write(*,*) '****************************************'
      write(*,*) '-------------- BN 110212 ---------------'
      write(*,*)
      call readYGA
      write(*,*)
***** user input *******************************************************
      write(*,*) '< 1 > ... snapshots'
      write(*,*) '< 2 > ... modes'
      read (*,*) iOption
      if (iOption.eq.1) then
        iFlowMin = 1
        call countFlows1(iFlowMax)
        write(*,*) 'snapshots:',iFlowMin,iFlowMax
      else if (iOption.eq.2) then
        iFlowMin = -NAM
        iFlowMax = +NEM
        write(*,*) 'modes:',iFlowMin,iFlowMax
      end if
***** preparing png files of all figures *******************************
      do iFlow = iFlowMin, iFlowMax
        open(10,file='tmp.dat',form='formatted',status='UNKNOWN')
        write(10,*) iOption
        write(10,*) iFlow
        write(10,*)
        close(10)
        call system('./xTools/Y0_plotFlow2u < tmp.dat')
      end do
***** LaTeX documentation **********************************************
      iFlow = 0
      open (12,file='gnu_xxuu.tex', form='formatted',status='UNKNOWN')
      write(12,'(2a)') char(92)//'documentclass[a4paper]',
     &               '{article}'
      write(12,'(a)') char(92)//'usepackage{graphicx}'
      write(12,'(a)') char(92)//'begin{document}'
      write(12,'(a)') char(92)//'pagestyle{empty}'
* ---
      if (iOption.eq.1) call system('ls *Flow*_S*.ps > tmp.lis')
      if (iOption.eq.2) call system('ls *Mode*_S*.ps > tmp.lis')
      open(10,file='tmp.lis',     form='formatted',status='UNKNOWN')
 301  read(10,'(a)',end=302,err=302) fn
      iFlow = iFlow + 1
      fnLength = index(fn,'.ps') +2
* ....................12345678901234567
* ....expecting fn = 'gnu_xxuu_0000.png'
 
      write(12,'(a)') char(92)//'vspace{-20mm}'
      write(12,'(a)') char(92)//'begin{figure}'
      write(12,'(a)') char(92)//'begin{center}'
      write(12,'(4a)') 
     &char(92)//'includegraphics[height=70mm,width=120mm,angle=0]{',
     &           fn(1:fnLength),'}' 
        write(12,'(a)') char(92)//'end{center}'
        write(12,'(3a)') char(92)//'caption{Streamlines of ',
     &                             fn(1:fnLength-5),'}'
        write(12,'(a)') char(92)//'end{figure}'
* --- The following command prevents a LaTeX compilation error
* --- "too many unprocessed floats"
      if (mod(iFlow,2).eq.0) write(12,'(a)') char(92)//'clearpage'
      goto 301
 302  close(10)
      write(12,'(a)') char(92)//'end{document}'
      close(12)
      call system('latex gnu_xxuu.tex')
      call system('dvips gnu_xxuu.dvi -o gnu_xxuu.ps')
* === End of main segment ============================================ *
      stop
      end
