************************************************************************
*****                                                              *****
***** @program name: x1_CorrelationMatrix                          *****
***** @task:   compute correlation matrix of snapshots             *****
*****          in the whole domain                                 *****
***** @input:  ...                                                 *****
***** @output: ...                                                 *****
*****                                                              *****
************************************************************************
*****                                                              *****
***** @author:  Bernd Noack                                        *****
***** @see:     res.20100408.xAMC_ToDo                             *****
***** @revised: 2012-01-15 Syracuse (binaries in yTools/Y*)        *****
***** @revised: 2010-11-09 Poitiers                                *****
***** @created: 2010-04-18 Poitiers                                *****
*****                                                              *****
************************************************************************
*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012
      program x1CorrelationMatrix
      implicit none
* --- Variables for reading the flow
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
* --- Variables for matrices
      real*8  fMatrixSub(NFlowsP,NFlowsP)
      real*8  fMatrix   (NFlowsP,NFlowsP)
      logical bOK                         ! true if file ex. & non-empty
* --- Vurther auxiliary files
      integer iDummy, iDomain, NDomain, iFlow, jFlow
* === Begin main segment ============================================= *
      write(*,*) '****************************************'
      write(*,*) '******* Correlation matrix      ********'
      write(*,*) '****************************************'
      write(*,*) '-------------- BN 101109 ---------------'
      write(*,*)
* --- Initialize NFlows and matrix
      call countFlows1(iDummy)
      NFlows = iDummy
      do jFlow = 1, NFlowsP
      do iFlow = 1, NFlowsP
        fMatrix(iFlow,jFlow) = 0.0
      end do
      end do
* --- Read the chosen subdomain
      call countDomains(NDomain)
      do iDomain = 1, NDomain
* ----- Create keyboard file
        open(10,file='tmp.input',status='UNKNOWN',form='formatted')
        write(10,*) iDomain
        write(10,*)
        write(10,*)
        write(10,*)
        close(10)
* ----- Run computation on subdomain
        call system("./xTools/Y1_CorrelationMatrixSub < tmp.input")
* ----- Read and add contribution of subdomain
        call readCorrMatrSub(iDomain,fMatrixSub,bOK)
        do jFlow=1,NFlows
        do iFlow=1,NFlows
          fMatrix(iFlow,jFlow) = fMatrix   (iFlow,jFlow)
     &                         + fMatrixSub(iFlow,jFlow)
        end do
        end do
      end do
* --- Save result
      call writeCorrMatr(fMatrix)
* --- Save gnuplot file of i -> TKE ---------------------------------- *
      write(*,*) 'Saving << gnu_iK.dat >> ...'
      open(10,file='gnu_iK.dat',
     &        status='UNKNOWN', form='formatted')
      do iFlow=1,NFlows
* ----- Note that the 1/M normalization has to be removed
        write(10,*) fMatrix(iFlow,iFlow)*NFlows/2
      end do
      close(10)
* --- Save gnuplot file of m,n -> C^mn ------------------------------- *
      write(*,*) 'Saving << gnu_CorrelationMatrix.dat >> ...'
      open(10,file='gnu_CorrelationMatrix.dat',
     &        status='UNKNOWN', form='formatted')
      do jFlow=1,NFlows
      do iFlow=1,NFlows
        write(10,*) fMatrix(iFlow,jFlow)
      end do
      write(10,*)
      end do
      close(10)
* === End of main segment ============================================ *
      stop
      end
