************************************************************************
*****                                                              *****
***** @program name: x1_CorrelationMatrixSub                       *****
***** @task:   compute correlation matrix of snapshots             *****
*****          in a subdomain                                      *****
***** @input:  ...                                                 *****
***** @output: ...                                                 *****
*****                                                              *****
************************************************************************
*****                                                              *****
***** @author:  Bernd Noack                                        *****
***** @see:     res.20100408.xAMC_ToDo                             *****
***** @revised: 2010-11-09 Poitiers                                *****
***** @revised: 2010-07-10 Poitiers                                *****
***** @created: 2010-04-18 Poitiers                                *****
*****                                                              *****
************************************************************************
*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012
      program x1CorrelationMatrixSub
      implicit none
* --- Variables for matrix
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
      real*8  fAim(-NAMP:NEMP,NFlowsP)
      real*8  fMatrix(NFlowsP,NFlowsP)
* --- Variables for grid and flow
      include 'xv3X.include'           ! NXSubP, ...
      include 'xv3X_ModesSub.include'  ! fModesSubG
      include 'xv3X_FlowsSub.include'  ! fFlowsSubG
* --- Auxiliary variables for file name and its existence
      character*120 fn        ! file name
      integer       fnLength  ! length of file name (# of characters)
      logical       bOK       ! = .true. if file exists
* --- Further auxiliary files
      integer iDomain, NDomain
* === Begin main segment ============================================= *
      write(*,*) '****************************************'
      write(*,*) '******* Correlation matrix      ********'
      write(*,*) '******* in a subdomain          ********'
      write(*,*) '****************************************'
      write(*,*) '-------------- BN 101109 ---------------'
      write(*,*)
* --- Reading the chosen subdomain
      write(*,*) 'Read domain #'
      read (*,*) iDomain
      call countDomains(NDomain)
      call readCorrMatrSub(iDomain,fMatrix,bOK)
      if (bOK) then
        write(*,*) 'The correlation matrix has already been computed!'
        stop
      end if
* --- Reading the grid
      call readGridSub(iDomain)
* --- Read the snapshots
      call readFlowsSub(iDomain,fFlowsSubG)
* --- Read the base flow
      call readYGA ! necessary for NAM
      call readBaseFlowSub (iDomain,fAim,fModesSubG)
      call subtractBaseFlowSub     (fAim,fModesSubG,fFlowsSubG)
* --- Compute correlation matrix from fluctuation around base flow
      call compCorrMatrSub                         (fFlowsSubG,fMatrix)
      call writeCorrMatrSub(iDomain,                           fMatrix)
* === End of main segment ============================================ *
      stop
      end
