************************************************************************
*****                                                              *****
***** @program name: x1_DMD                                        *****
***** @task:   compute DMD from transformation matrix              *****
***** @input:  ...                                                 *****
***** @output: ...                                                 *****
*****                                                              *****
************************************************************************
*****                                                              *****
***** @author:  Bernd Noack                                        *****
***** @see:     res.20100408.xAMC_ToDo                             *****
***** @revised: 2010-11-11 Poitiers                                *****
***** @revised: 2010-06-19 Princeton                               *****
***** @created: 2010-06-13 Princeton                               *****
*****                                                              *****
************************************************************************
*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012
      program x1DMD
      implicit none
* --- 
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
* --- Variables for grid and flow
      include 'xv3X.include'           ! NXP, ...
      include 'xv3X_Flow.include'      ! fFlowG
      include 'xv3X_Flow1.include'     ! fFlow1G
      include 'xv3X_Flow2.include'     ! fFlow2G
* --- Variables read 
      real*8  fNormDMD(NFlowsP)
      real*8  fTim(NFlowsP,NFlowsP)    ! transformation matrix
* --- Auxiliary variables
      real*8  fFctInnerProduct
      integer iDummy, jDummy, kDummy, iMode, iFlow
* --- Auxiliary variables for file name
      character*120 fn                 ! file name
      integer       fnLength           ! length of file name (# of char.)
* === Begin main segment ============================================= *
      write(*,*) '****************************************'
      write(*,*) '*******           DMD           ********'
      write(*,*) '****************************************'
      write(*,*) '-------------- BN 101109 ---------------'
      write(*,*)       
* --- Read the transformation matrix --------------------------------- *
*.................123456789012345678901234567890
      fn       = 'z1_TransformationMatrixDMD.dat'
      fnLength =  30
      call readList2d(fn,fnLength,
     &                fTim,iDummy,jDummy,1,NFlowsP,
     &                     iDummy,kDummy,1,NFlowsP,1.0D0)
      NFlows = jDummy+1
* ... NOTE that NKL and NFlows is determined from the transf. matrix
* --- Initialize
      call readGrid
* --- 0-th mode ... -> fFlow1G
* ... Note that the correct averages includes m = 1 ... NFlows
      iMode = 0
      call zeroFlow(fFlow1G)
      do iFlow = 1, NFlows
        call readFilenameFlow1(iFlow,fn,fnLength)
        call readFlow(fn,fnLength,fFlowG)
        call addScaledFlow (1.D0/NFlows,fFlowG,fFlow1G)
      end do
      call readFilenameModeX(iMode,fn,fnLength)
      call writeFlow(fn,fnLength,fFlow1G)
* --- i-th modes ... -> fFlow2G
      do iMode = 1, NFlows-1
        call zeroFlow(fFlow2G)
        do iFlow = 1, NFlows-1
          call readFilenameFlow1(iFlow,fn,fnLength)
          call readFlow     (fn,fnLength,fFlowG)
          call addScaledFlow(-1.0D0, fFlow1G,fFlowG) ! subtract u_0
          call addScaledFlow (fTim(iMode,iFlow), fFlowG,fFlow2G)
        end do
        fNormDMD(iMode)=sqrt(max(0.,fFctInnerProduct(fFlow2G,fFlow2G)))
        call readFilenameModeX(iMode,fn,fnLength)
        call writeFlow(fn,fnLength,fFlow2G)
      end do
* --- Write norms of DMD modes
      write(*,*) 'Writing << z1_NormDMD.dat >> ...'
      open(10,file='z1_NormDMD.dat',status='UNKNOWN',form='formatted')
      do iMode=1,NFlows-1
        write(10,*) iMode, fNormDMD(iMode), fNormDMD(iMode)**2
      end do
      close(10)
* --- Write names of DMD modes
      write(*,*) 'Writing << y1_Modes.dat >> ...'
      open(10,file='y1_Modes.dat',status='UNKNOWN',form='formatted')
      do iMode=1,NFlows-1
        call readFilenameModeX(iMode,fn,fnLength)
        write(10,'(i5,2x,f5.2,2x,a)') iMode, 1.0, fn(1:fnLength)
      end do
      close(10)
* === End of main segment ============================================ *
      stop
      end
