************************************************************************
*****                                                              *****
***** @program name: x1_GalerkinApproximation                      *****
***** @task:   compute additional modes by linear combinations     *****
***** @input:  ...                                                 *****
***** @output: ...                                                 *****
*****                                                              *****
************************************************************************
*****                                                              *****
***** @author:  Bernd Noack                                        *****
***** @see:     res.20100408.xAMC_ToDo                             *****
***** @revised: 2012-01-27 Poitiers (writeYGA removed)             *****
***** @revised: 2010-11-09 Poitiers                                *****
***** @created: 2010-04-22 Poitiers                                *****
*****                                                              *****
************************************************************************
*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012
      program x1GalerkinApproximation
      implicit none
* ---
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
      include 'xv_GA2.include'    ! tasks for construction of modes
* --- Variables for grid and flow
      include 'xv3X.include'           ! NXP, ...
      include 'xv3X_Flow.include'      ! fFlowG
      include 'xv3X_Flow1.include'     ! fFlow1G
* --- Auxiliary variables
      real*8  fFctInnerProduct, fFactor
      integer iTask, iMode, jMode, kMode, iDummy
* --- Auxiliary variables for file name
      character*120 fn        ! file name
      integer       fnLength  ! length of file name (# of characters)

* === Begin main segment ============================================= *
      write(*,*) '****************************************'
      write(*,*) '*******  Adding non-POD modes   ********'
      write(*,*) '****************************************'
      write(*,*) '-------------- BN 101109 ---------------'
      write(*,*)
* --- Read files
      call readYGA
* --- 
      call readGrid
* --- 
      if (NKL.eq.NEM) then
        write(*,*) 'POD modes only'
        stop
      end if
* --- Carry out GA tasks
      do iTask = 1, NTaskGA
        if (aTaskGA(iTask).eq.0) then
          write(*,'(a,i5,a,i5,a,i5)') 
     &            'task #',iTask,'): identity for i=',
     &             iTaskGA(iTask),' ...',jTaskGA(iTask)
          write(*,*) 'The modes are already computed!'
        else if (aTaskGA(iTask).eq.1) then
          iMode = iTaskGA(iTask)
          write(*,'(a,i5,a,i5)') 
     &            'task #',iTask,'): initialize mode',iMode
          call zeroFlow(fFlowG)
          call readFilenameMode1(iMode,fn,fnLength)
          call writeFlow        (fn,fnLength,fFlowG)
        else if (aTaskGA(iTask).eq.2) then
          iMode = iTaskGA(iTask)
          jMode = jTaskGA(iTask)
          fFactor = fTaskGA(iTask)
          write(*,'(a,i5,a,i5,a,i5,a,f12.5,a,i5)') 
     &            'task #',iTask,'): mode',iMode,
     &                          ' <- mode',iMode,
     &                         ' + ',fFactor,
     &                         ' x mode',jMode
          call readFilenameMode1(iMode,fn,fnLength)
          call readFlow         (fn,fnLength,fFlowG)
          call readFilenameMode1(jMode,fn,fnLength)
          call readFlow         (fn,fnLength,fFlow1G)
          call addScaledFlow    (fFactor,fFlow1G,fFlowG) 
          call readFilenameMode1(iMode,fn,fnLength)
          call writeFlow        (fn,fnLength,fFlowG)
        else if (aTaskGA(iTask).eq.3) then
          iMode = iTaskGA(iTask)
          jMode = jTaskGA(iTask)
          kMode = kTaskGA(iTask)
          write(*,'(a,i5,a,i5,a,i5,a,i5)') 
     &            'task #',iTask,'): orthonormalize mode ',
     &                     iMode, ' w.r.t. modes',
     &                     jMode, ' ...',
     &                     kMode
* ....... Read original mode
          call readFilenameMode1(iMode,fn,fnLength)
          call readFlow         (fn,fnLength,fFlowG)
* ....... Subtract orthogonal projections on other modes
* ....... which are assumed to be an ONS
          if (max(jMode,kMode).ne.iMode.and.
     &        min(jMode,kMode).ne.iMode) then 
          do iDummy = jMode, kMode
            call readFilenameMode1(iDummy,fn,fnLength)
            call readFlow         (fn,fnLength,fFlow1G)
            fFactor = -fFctInnerProduct(fFlowG,fFlow1G)
            call addScaledFlow    (fFactor,fFlow1G,fFlowG) 
          end do
          end if
* ....... Normalize
          fFactor = fFctInnerProduct(fFlowG,fFlowG)
          fFactor = 1/sqrt(fFactor)
          call scaleFlow(fFactor,fFlowG)
* ....... Save mode
          call readFilenameMode1(iMode,fn,fnLength)
          call writeFlow        (fn,fnLength,fFlowG)
        end if
      end do
* === End of main segment ============================================ *
      stop
      end
