************************************************************************
*****                                                              *****
***** @program name: x1_POD                                        *****
***** @task:   compute POD from transformation matrix              *****
***** @input:  ...                                                 *****
***** @output: ...                                                 *****
*****                                                              *****
************************************************************************
*****                                                              *****
***** @author:  Bernd Noack                                        *****
***** @see:     res.20100408.xAMC_ToDo                             *****
***** @revised: 2010-11-09 Poitiers                                *****
***** @revised: 2010-07-10 Berlin (with actuation modes)           *****
***** @created: 2010-04-22 Poitiers                                *****
*****                                                              *****
************************************************************************
*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012
      program x1POD
      implicit none
* --- Variables for transformation matrix
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
      real*8  fAim(-NAMP:NEMP,NFlowsP) ! a_i^m (needed for actuation modes)
      real*8  fTim(NEMP,NFlowsP)       ! transformation matrix
* --- Variables for grid and flow
      include 'xv3X.include'           ! NXP, ...
      include 'xv3X_Flow.include'      ! fFlowG
      include 'xv3X_Flow1.include'     ! fFlow1G
      include 'xv3X_Flow2.include'     ! fFlow2G
      include 'xv3X_Flow3.include'     ! fFlow3G
* --- Auxiliary variables
      integer iDummy, iMode, iMode2, iFlow
* --- Auxiliary variables for file name
      character*120 fn        ! file name
      integer       fnLength  ! length of file name (# of characters)
* === Begin main segment ============================================= *
      write(*,*) '****************************************'
      write(*,*) '*******           POD           ********'
      write(*,*) '****************************************'
      write(*,*) '-------------- BN 101109 ---------------'
      write(*,*)
* --- Read amplitudes of actuation modes
      call readYGA ! for NAM
      call readAmplitudes1(fAim)
* --- Read the transformation matrix --------------------------------- *
*.................123456789012345678901234567890
      fn       = 'z1_TransformationMatrixPOD.dat'
      fnLength =  30
      call readList2d(fn,fnLength,
     &                fTim,iDummy,NKL,1,NEMP,
     &                     iDummy,NFlows,1,NFlowsP,1.0D0)
* === NOTE that NKL and NFlows is determined from the transf. matrix
* --- initialize
      call readGrid
* --- 0-th mode !............................................ -> fFlow1G
      iMode = 0
      call zeroFlow(fFlow1G)
      do iFlow = 1, NFlows
        call readFilenameFlow1(iFlow,fn,fnLength)
        call readFlow(fn,fnLength,fFlowG)
        call addScaledFlow (1.D0/NFlows,fFlowG,fFlow1G)
      end do
      call readFilenameMode1(iMode,fn,fnLength)
      call writeFlow(fn,fnLength,fFlow1G)
* --- i-th modes ... -> fFlow3G -------------------------------------- *
      do iMode = 1, NKL
* ----- Initialize mode u_i = 0 !............................... fFlow3G
        call zeroFlow(fFlow3G)
* ----- Add T_i^m (u^m-u_0) to mode
        do iFlow = 1, NFlows
          call readFilenameFlow1(iFlow,fn,fnLength)
          call readFlow(fn,fnLength,fFlowG) ! read u^m in ....... fFlowG
* ------- "Purify" snapshot from mean flow, i.e. subtract u_0 from u^m
          call addScaledFlow(-1.0D0, fFlow1G,fFlowG) ! ........ -fFlow1G
* ------- "Purify" snapshot from boundary imposed unsteadiness,
* ------- i.e. subtract a_i^m u_i from snapshot
          if (NAM.gt.0) then
          do iMode2 = -NAM,-1
            call readFilenameMode1(iMode2,fn,fnLength)
            call readFlow(fn,fnLength,fFlow2G) ! ....... - a_i^m fFlow2G
            call addScaledFlow(-fAim(iMode2,iFlow),fFlow2G,fFlowG)
          end do
          end if
* ------- Now, add homogenized snapshot with approxiated weight to mode,
* ------- i.e. T_i^m (u^m - u_0 - Sum_{i<0} a_i^m u_i)
          call addScaledFlow (fTim(iMode,iFlow), fFlowG,fFlow3G)
        end do
        call readFilenameMode1(iMode,fn,fnLength)
        call writeFlow(fn,fnLength,fFlow3G) ! ...... save u_i in fFlow3G
* --- End of Loop over snapshot
      end do
* === End of main segment ============================================ *
      stop
      end
