************************************************************************
*****                                                              *****
***** @program name: x1_StatisticsSub                              *****
***** @task:   compute mean flow and variances of snapshots        *****
*****          in a subdomain                                      *****
***** @input:  ...                                                 *****
***** @output: ...                                                 *****
*****                                                              *****
************************************************************************
*****                                                              *****
***** @author:  Bernd Noack                                        *****
***** @see:     res.20100408.xAMC_ToDo                             *****
***** @revised: 2010-11-09 Poitiers                                *****
***** @created: 2010-04-16 Poitiers                                *****
*****                                                              *****
************************************************************************
*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012
      program x1StatisticsSub
      implicit none
* --- variables for grid and flow
      include 'xv3X.include'           ! NXSubP, ...
      include 'xv3X_FlowSub.include'   ! fFlowSubG
      include 'xv3X_Flow1Sub.include'  ! fFlow1SubG
      include 'xv3X_Flow2Sub.include'  ! fFlow2SubG
* --- Auxiliary variables (counters)
      integer iDomain, NDomain
      integer NFlow1, iFlow
* --- Auxiliary variables for file name
      character*120 fn        ! file name
      integer       fnLength  ! length of file name (# of characters)
* --- Further auxiliary files
      real*8  fNorm

* === Begin main segment ============================================= *
      write(*,*) '****************************************'
      write(*,*) '******* Statistics of snapshots ********'
      write(*,*) '******* in a subdomain          ********'
      write(*,*) '****************************************'
      write(*,*) '-------------- BN 101109 ---------------'
      write(*,*)
      call countDomains(NDomain)
      write(*,*) 'Read domain #'
      read (*,*) iDomain
* --- Read the grid
      call readGridSub(iDomain)
      call zeroFlowSub(fFlow1SubG)
      call zeroFlowSub(fFlow2SubG)
      call countFlows1(NFlow1)
* --- Read the flows
      do iFlow = 1, NFlow1
        call readFilenameFlow1(iFlow,fn,fnLength)
        call readFlowSub(iDomain,fn,fnLength,fFlowSubG)
        call maxnormFlowSub(fFlowSubG,fNorm)
        call addScaledFlowSub (1.0D0, fFlowSubG,fFlow1SubG)
        call addProductFlowSub(fFlowSubG,fFlowSubG,fFlow2SubG)
      end do
* --- Note that the only ouput is the norm on the screen ------------- *
* --- Compute the mean flow
      write(*,*) '*** Computing the mean flow ...'
      call scaleFlowSub  (1.D0/NFlow1,fFlow1SubG)
      call maxnormFlowSub (fFlow1SubG,fNorm)
* --- Compute the fluctuation level
      write(*,*) '*** Computing the fluctuation level ...'
      call scaleFlowSub  (1.D0/NFlow1,fFlow2SubG)    ! a := second moment^2 
      call zeroFlowSub(fFlowSubG)
      call addProductFlowSub(fFlow1SubG,fFlow1SubG,fFlowSubG)  ! b := mean^2
      call addScaledFlowSub (-1.D0,  fFlowSubG, fFlow2SubG) ! variance = a-b
      call maxnormFlowSub (fFlow2SubG,fNorm)
* === End of main segment ============================================ *
      stop
      end
