************************************************************************
*****                                                              *****
***** @program name: x1_listSpectrumPOD                            *****
*****                (similar to x1_plotSpectrumPOD)               *****
*****                                                              *****
***** @task:  evaluation POD spectrum                              ***** 
*****                                                              *****
***** @input:  - z1_SpectrumPOD.dat  [spectrum]                    *****
*****          - z1_CorrelationMatrix.dat (for E)                  *****
*****                                                              *****
***** @output: - zz1_SpectrumPOD.dat                               *****
*****          - zz1_SpectrumPOD2.dat (pairs)                      *****
*****                                                              *****
***** @dependency: requires the execution of following programs:   *****
*****          - x1_eigenproblemPOD.f (and the previous programs)  *****
*****                                                              *****
************************************************************************
*****                                                              *****
***** @author: Bernd Noack                                         *****
*****                                                              *****
***** @see:    x1_eigenproblemPOD.f                                *****
*****                                                              *****
***** @revised: 2011-05-18 London                                  *****
***** @created: 2011-05-18 London                                  *****
*****                                                              *****
************************************************************************
*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012
      program x1listSpectrumPOD
      implicit none

* --- Galerkin system
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
* --- POD analysis
      real*8  fTKE
      real*8  fSpectrum (NEMP,4)
      real*8  fSpectrum2(NEMP,4)
      real*8  fSum
* --- Auxiliary values
      integer iDummy, jDummy
      real*8  fDummy

* === Begin main segment ============================================= *
      write(*,*) '**************************************************'
      write(*,*) '****     Analyze spectrum from POD data       ****'
      write(*,*) '**************************************************'
      write(*,*) '------------------- BRN 110518 -------------------'
      write(*,*)
* --- Initialize
      NEM = 0
      do iDummy = 1, NEMP
      do jDummy = 1, 4
        fSpectrum (iDummy,jDummy)=0.0
        fSpectrum2(iDummy,jDummy)=0.0
      end do
      end do
      fSum = 0.0
* --- Read correlation matrix ---------------------------------------- *
      fTKE = 0.
      write(*,'(a,a)') 'Reading from the ',
     &                  '<< z1_CorrelationMatrux.dat>> ...'
      open(10,file='z1_CorrelationMatrix.dat',
     &        status='old',form='formatted')
      NEM = 0
  101 read(10,*,end=102) iDummy, jDummy, fDummy
      if (iDummy.eq.jDummy) fTKE = fTKE + fDummy/2
      fSum = fSum + fDummy
      goto 101
  102 close(10)
      write(*,*) 'TKE = ',fTKE
* --- Read spectrum -------------------------------------------------- *
      write(*,'(a,a)') 'Reading the spectrum from ',
     &                  '<< z1_SpectrumPOD.dat>> ...'
      open(10,file='z1_SpectrumPOD.dat',status='old',form='formatted')
      NEM = 0
  201 read(10,*,end=202) iDummy, fDummy
      fDummy = fDummy / 2   !   E_i = lambda_i / 2
* --- update N
      NEM = max(NEM,iDummy)
* --- update modal spectrum
      fSpectrum(iDummy,1) = fDummy
      jDummy = 1 + (iDummy-1)/2
* --- update pair spectrum
      fSpectrum2(jDummy,1) = fSpectrum2(jDummy,1)+fDummy
      goto 201
  202 close(10)
      write(*,*) 'NEM = ',NEM

* --- Compute normalized spectra ------------------------------------- *
      write(*,*) 'Compute normalized spectra ...'
      do iDummy = 1, NEM
        fSpectrum (iDummy,2) = fSpectrum (iDummy,1)/fTKE
        fSpectrum2(iDummy,2) = fSpectrum2(iDummy,1)/fTKE
      end do
* --- Compute the normalized cumulative spectra
      fSpectrum (1,3) = fSpectrum (1,2)
      fSpectrum2(1,3) = fSpectrum2(1,2)
      do iDummy = 2, NEM
        fSpectrum (iDummy,3) = fSpectrum (iDummy-1,3)
     &                       + fSpectrum (iDummy,2)
        fSpectrum2(iDummy,3) = fSpectrum2(iDummy-1,3)
     &                       + fSpectrum2(iDummy,2)
      end do
* --- Compute the normalized residual
      do iDummy = 1, NEM
        fSpectrum (iDummy,4) = 1.-fSpectrum (iDummy,3)
        fSpectrum2(iDummy,4) = 1.-fSpectrum2(iDummy,3)
      end do
* ---
      close(10)
* --- Writing the complete set of spectra ---------------------------- *
      write(*,'(a,a)') 'Writing the POD spectra in ',
     &                  '<< gnu_SpectrumPOD.dat>> ...'
      open(10,file='gnu_SpectrumPOD.dat',
     &        status='unknown',form='formatted')
      do iDummy = 1, NEM
        write(10,'(i5,4(1x,f20.10))')
     &            iDummy,(fSpectrum(iDummy,jDummy),jDummy=1,4)
      end do
      close(10)
* --- Writing the complete set of spectra (lumped for POD pairs) ----- *
      write(*,'(a,a)') 'Writing the POD spectra in ',
     &                  '<< gnu_SpectrumPOD2.dat>> ...'
      open(10,file='gnu_SpectrumPOD2.dat',
     &        status='unknown',form='formatted')
      do iDummy = 1, NEM/2
        write(10,'(i5,4(1x,f20.10))')
     &            iDummy,(fSpectrum2(iDummy,jDummy),jDummy=1,4)
      end do
      close(10)
* --- Writing the complete set of spectra (lumped for POD pairs) ----- *
      write(*,'(a,a)') 'Writing the oscillation amplitudes in ',
     &                  '<< gnu_Amplitudes.dat>> ...'
      open(10,file='gnu_Amplitudes.dat',
     &        status='unknown',form='formatted')
      do iDummy = 1, NEM/2
        write(10,'(i5,4(1x,f20.10))')
     &            iDummy, sqrt(2*fSpectrum2(iDummy,1))
      end do
      close(10)
* === End of main segment ============================================ *
      stop
      end 
