************************************************************************
*****                                                              *****
***** @program name: x1_plotSpectrumDMD                            *****
*****                                                              *****
***** @task:   plot Fourier spectrum from DMD data                 ***** 
*****                                                              *****
***** @input:  - z1_SpectrumDMD.dat  [spectrum]                    *****
*****          - z1_NormDMD.dat      [norm of Ritz eigenvectors]   *****
*****                                                              *****
***** @output: - gnu_SpectrumDMD.dat,  [DMD eigenvalues]           *****
*****                                                              *****
***** @dependency: requires the execution of following programs:   *****
*****          - x1_DMD.f (and the previous programs)              *****
*****                                                              *****
************************************************************************
*****                                                              *****
***** @author: Bernd Noack                                         *****
*****                                                              *****
***** @see:    x1_CorrelationMatrix.f                              *****
*****                                                              *****
***** @revised: 2010-11-07 Poitiers                                *****
***** @created: 2006-06-20 Poitiers                                *****
*****                                                              *****
************************************************************************
*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012
      program x1plotSpectrumDMD
      implicit none

* --- Galerkin system
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ... 
* --- DMD analysis
      real*8  fNorm(NFlowsP,2)
      real*8  fSpectrum(NFlowsP,5)
* --- auxiliary values
      integer iDummy, jDummy
      real*8  fVal(5)

* === Begin main segment ============================================= *
      write(*,*) '**************************************************'
      write(*,*) '****    Plot Fourier spectrum from DMD data   ****'
      write(*,*) '**************************************************'
      write(*,*) '------------------- BRN 101107 -------------------'
      write(*,*)
* --- Initialize (errors shall show up!!!)
      do iDummy = 1, NFlowsP
      do jDummy = 1, 5
        fSpectrum(iDummy,jDummy)=1.E36
      end do
      end do
      NFlows = 0
* --- Read spectrum -------------------------------------------------- *
      write(*,'(a,a)') 'Reading the spectrum from ',
     &                  '<< z1_SpectrumDMD.dat>> ...'
      open(10,file='z1_SpectrumDMD.dat',status='old',form='formatted')
      NFlows = 0
  101 read(10,*,end=102) iDummy, (fVal(jDummy),jDummy=1,4)
      NFlows = max(NFlows,iDummy)
      do jDummy = 1, 4
        fSpectrum(iDummy,jDummy) = fVal(jDummy)
      end do
      goto 101
  102 close(10)
      write(*,*) 'NFlows = ',NFlows
* --- Read norms ----------------------------------------------------- *
      write(*,'(a,a)') 'Reading the norms from ',
     &                  '<< z1_NormDMD.dat>> ...'
      open(10,file='z1_NormDMD.dat',status='old',form='formatted')
      NFlows = 0
  201 read(10,*,end=202) iDummy, (fVal(jDummy),jDummy=1,2)
      NFlows = max(NFlows,iDummy)
      if (abs(fVal(2)).gt.10E10) fVal(2) = 0.
      fSpectrum(iDummy,5)=fVal(2)
      goto 201
  202 close(10)
      write(*,*) 'NFlows = ',NFlows
* --- Write spectrum ------------------------------------------------- *
      write(*,'(a,a)') 'Writing the spectrum in',
     &                  '<< gnu_SpectrumDMD3.dat>> ...'
      open(10,file='gnu_SpectrumDMD3.dat',
     &        status='unknown',form='formatted')
      do iDummy = 1, NFlows
        if (fSpectrum(iDummy,5).gt.0)
     &  write(10,'(i5,2(1x,f20.10))')
     &             iDummy,abs(fSpectrum(iDummy,4)),fSpectrum(iDummy,5)
      end do
      close(10)
* --- 
      write(*,'(a,a)') 'Writing the COMPLETE spectrum in',
     &                  '<< gnu_SpectrumDMD.dat>> ...'
      open(10,file='gnu_SpectrumDMD.dat',
     &        status='unknown',form='formatted')
      do iDummy = 1, NFlows
        write(10,'(i5,5(1x,f20.10))') 
     &             iDummy,(fSpectrum(iDummy,jDummy),jDummy=1,5)
      end do
      close(10)
* --- Write spectrum ------------------------------------------------- *
* === End of main segment ============================================ *
      stop
      end 
