************************************************************************
*****                                                              *****
***** @program name: x1_plotSpectrumPOD                            *****
*****                                                              *****
***** @task:   plot Fourier spectrum from POD data                 ***** 
*****                                                              *****
***** @input:  - z1_SpectrumPOD.dat  [spectrum]                    *****
*****                                                              *****
***** @output: - gnu_SpectrumPOD.dat,  [POD eigenvalues]           *****
*****                                                              *****
***** @dependency: requires the execution of following programs:   *****
*****          - x1_eigenproblemPOD.f (and the previous programs)  *****
*****                                                              *****
************************************************************************
*****                                                              *****
***** @author: Bernd Noack                                         *****
*****                                                              *****
***** @see:    x1_eigenproblemPOD.f                                *****
*****                                                              *****
***** @revised: 2010-11-07 Poitiers                                *****
***** @revised: 2010-06-20 Princeton                               *****
***** @created: 2006-06-20 Poitiers                                *****
*****                                                              *****
************************************************************************
*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012
      program x1plotSpectrumPOD
      implicit none

* --- Galerkin system
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
* --- POD analysis
      real*8  fSpectrum(NEMP,4)
      real*8  fSum
* --- Auxiliary values
      integer iDummy, jDummy
      real*8  fDummy

* === Begin main segment ============================================= *
      write(*,*) '**************************************************'
      write(*,*) '****        Plot spectrum from POD data       ****'
      write(*,*) '**************************************************'
      write(*,*) '------------------- BRN 101107 -------------------'
      write(*,*)
* --- Initialize
      NEM = 0
      do iDummy = 1, NEMP
      do jDummy = 1, 4
        fSpectrum(iDummy,jDummy)=0.0
      end do
      end do
      fSum = 0.0
* --- Read spectrum -------------------------------------------------- *
      write(*,'(a,a)') 'Reading the spectrum from ',
     &                  '<< z1_SpectrumPOD.dat>> ...'
      open(10,file='z1_SpectrumPOD.dat',status='old',form='formatted')
      NEM = 0
  101 read(10,*,end=102) iDummy, fDummy
      fSum = fSum + fDummy
      NEM = max(NEM,iDummy)
      fSpectrum(iDummy,1) = fDummy
      goto 101
  102 close(10)
      write(*,*) 'NEM = ',NEM

* --- Compute normalized spectra ------------------------------------- *
      write(*,*) 'Compute normalized spectra ...'
      do iDummy = 1, NEM
        fSpectrum(iDummy,2) = fSpectrum(iDummy,1)/fSum
      end do
* --- Compute the normalized cumulative spectrum
      fSpectrum(1,3) = fSpectrum(1,2)
      do iDummy = 2, NEM
        fSpectrum(iDummy,3) = fSpectrum(iDummy-1,3)+fSpectrum(iDummy,2)
      end do
* --- Compute the normalized residual
      do iDummy = 1, NEM
        fSpectrum(iDummy,4) = 1.-0-fSpectrum(iDummy,3)
      end do
* ---
      close(10)
* --- Writing the complete set of spectra ---------------------------- *
      write(*,'(a,a)') 'Writing the POD spectra in',
     &                  '<< gnu_SpectrumPOD.dat>> ...'
      open(10,file='gnu_SpectrumPOD.dat',
     &        status='unknown',form='formatted')
      do iDummy = 1, NEM
        write(10,'(i5,4(1x,f20.10))')
     &            iDummy,(fSpectrum(iDummy,jDummy),jDummy=1,4)
      end do
      close(10)
* --- Write spectrum ------------------------------------------------- *
* === End of main segment ============================================ *
      stop
      end 
