************************************************************************
*****                                                              *****
***** @program name: x2_GalerkinProjection                         *****
***** @task:   compute Galerkin model for the whole domain         *****
***** @input:  ...                                                 *****
***** @output: ...                                                 *****
*****                                                              *****
************************************************************************
*****                                                              *****
***** @author:  Bernd Noack                                        *****
***** @see:     res.20100408.xAMC_ToDo                             *****
***** @revised: 2012-01-15 Syracuse (binaries in yTools/Y*)        *****
***** @revised: 2010-11-09 Poitiers                                *****
***** @created: 2010-05-02 Poitiers                                *****
*****                                                              *****
************************************************************************
*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012
      program x2GalerkinProjection
      implicit none
* --- Variables for Galerkin system
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
      real*8  fMij   (NEMP,-NAMP:NEMP)
      real*8  fLijV  (NEMP,-NAMP:NEMP)
      real*8  fQijkC (NEMP,-NAMP:NEMP,-NAMP:NEMP)
      real*8  fGil   (NEMP,NVFP)
* --- Auxiliary variables
      real*8  fMatrix (NEMP,-NAMP:NEMP)
      real*8  fMatrix2(NEMP, NVFP)
      real*8  fTensor (NEMP,-NAMP:NEMP,-NAMP:NEMP)
      logical bOK                         ! true if file ex. & non-empty
* --- Further auxiliary files
      integer iDummy, iDomain, NDomain
* === Begin main segment ============================================= *
      write(*,*) '****************************************'
      write(*,*) '******* Galerkin projection     ********'
      write(*,*) '******* of the whole domain     ********'
      write(*,*) '****************************************'
      write(*,*) '-------------- BN 101109 ---------------'
      write(*,*)
* --- Read Galerkin model dimensions
      call readYGA
      call readYGP
* --- Initialize the Galerkin system
      write(*,*) 'Initializing the Galerkin system ...'
      call zeroMatrix (fMij)
      call zeroMatrix (fLijV)
      call zeroTensor (fQijkC)
      call zeroMatrix2(fGil)
* --- Read the chosen subdomain
      call countDomains(NDomain)
* --- Loop over domains
      do iDomain = 1, NDomain
* ----- Create keyboard file
        open(10,file='tmp.input',status='UNKNOWN',form='formatted')
        write(10,*) iDomain
        write(10,*)
        write(10,*)
        write(10,*)
        close(10)
* ----- Run computation on subdomain
        call system("./xTools/Y2_GalerkinProjectionSub < tmp.input")
* ----- Read and add contribution of subdomain
        call readMassMatrixSub (iDomain,fMatrix,bOK)
        call add2Matrix                (fMatrix,fMij)
* -----
        call readViscousTermSub(iDomain,fMatrix,bOK)
        call add2Matrix                (fMatrix,fLijV)
* -----
        call readConvectiveTermSub(iDomain,fTensor,bOK)
        call add2Tensor                   (fTensor,fQijkC)
* -----
        call readForceTermSub(iDomain,fMatrix2,bOK)
        call add2Matrix2             (fMatrix2,fGil)
* --- End of loop over the domains
      end do
* --- Save result
      call cleanMatrix        (fMij)
      call writeMassMatrix    (fMij)
* ---
      call cleanMatrix        (fLijV)
      call writeViscousTerm   (fLijV)
* ---
      call writeConvectiveTerm(fQijkC)
* ---
      call writeForceTerm     (fGil)
* --- Read the result
      call readMassMatrix     (fMij,  bOK)
      call readViscousTerm    (fLijV, bOK)
      call readConvectiveTerm (fQijkC,bOK)
      call readForceTerm      (fGil,  bOK)
* === End of main segment ============================================ *
      stop
      end
