************************************************************************
*****                                                              *****
***** @program name: x2_GalerkinProjectionSub                      *****
***** @task:   compute Galerkin system of modes                    *****
*****          --- contributions from a subdomain                  *****
***** @input:  ...                                                 *****
***** @output: ...                                                 *****
*****                                                              *****
************************************************************************
*****                                                              *****
***** @author:  Bernd Noack                                        *****
***** @see:     x2_GalerkinProjection.f                            *****
***** @revised: 2010-11-09 Poitiers                                *****
***** @revised: 2010-05-16 Poitiers                                *****
***** @created: 2010-05-02 Poitiers                                *****
*****                                                              *****
************************************************************************
*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012
      program x2_GalerkinProjectionSub
      implicit none
* --- Galerkin system
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
* --- Variables for grid and flow
      include 'xv3X.include'           ! NXSubP, ...
      include 'xv3X_FlowSub.include'   ! fFlowSubG
      include 'xv3X_ModesSub.include'  ! fModesSubG
      include 'xv3X_ForcesSub.include' ! fForcesSubG
* --- Variables for the Galerkin system
      real*8  fMatrix(NEMP,-NAMP:NEMP) ! viscous term
      real*8  fMatrix2(NEMP,     NVFP) ! volume force term
      real*8  fTensor(NEMP,-NAMP:NEMP,-NAMP:NEMP) ! convective term
* --- Auxiliary variables for file name and its existence
      character*120 fn        ! file name
      integer       fnLength  ! length of file name (# of characters)
      logical       bOK       ! = .true. if file exists
      logical       bAll      ! = .true. if all files exist
* --- Further auxiliary files
      integer iDomain, NDomain
* === Begin main segment ============================================= *
      write(*,*) '****************************************'
      write(*,*) '******* Galerkin projection     ********'
      write(*,*) '******* in a subdomain          ********'
      write(*,*) '****************************************'
      write(*,*) '-------------- BN 100502 ---------------'
      write(*,*)
* --- Reading the chosen subdomain
      write(*,*) 'Read domain #'
      read (*,*) iDomain
      call countDomains(NDomain)
* --- Read mode expansion dimensions
      call readYGA
* --- Read Galerkin projection (e.g. # volume forces)
      call readYGP
* --- Determine if the GP of the subdomain has already been done ----- *
      bAll = .true.
* ... Mandatory terms
      call readMassMatrixSub (iDomain,fMatrix,bOK)
      if (.not.bOK) bAll =.false.
      call readViscousTermSub(iDomain,fMatrix,bOK)
      if (.not.bOK) bAll =.false.
      call readConvectiveTermSub(iDomain,fTensor,bOK)
      if (.not.bOK) bAll =.false.
* ... Optional terms
      call readForceTermSub(iDomain,fMatrix2,bOK) 
      if (.not.bOK) bAll =.false. ! only if N_V>0 and file empy/non-ex.
      if (bAll) then
        write(*,*) "All GS quantities of this subdomain computed!"
        stop
      end if
* --- Read the grid
      call readGridSub(iDomain)
* --- Read the modes
      call readModesSub(iDomain,fModesSubG)
* --- Read the forces
      call readForcesSub(iDomain,fForcesSubG)
* --- Compute the mass matrix
      call readMassMatrixSub(iDomain,fMatrix,bOK)
      if (.not.bOK) then
        call compMassMatrixSub(fModesSubG,fMatrix)
        call writeMassMatrixSub(iDomain,  fMatrix)
      end if
* --- Compute the viscous term
      call readViscousTermSub(iDomain,fMatrix,bOK)
      if (.not.bOK) then
        call compViscousTermSub(fModesSubG,fMatrix)
        call writeViscousTermSub(iDomain,  fMatrix)
      end if
* --- Compute the convective term
      call readConvectiveTermSub(iDomain,fTensor,bOK)
      if (.not.bOK) then
        call compConvectiveTermSub(fModesSubG,fTensor)
        call writeConvectiveTermSub(iDomain,  fTensor)
      end if
* --- Compute the force term
      call readForceTermSub(iDomain,fMatrix2,bOK)
      if (.not.bOK) then
        call compForceTermSub(fModesSubG,fForcesSubG,fMatrix2)
        call writeForceTermSub(iDomain,              fMatrix2)
      end if
* === End of main segment ============================================ *
      stop
      end

