************************************************************************
*****                                                              *****
***** @program name: x2_PressureTermSub                            *****
*****                                                              *****
***** @task:   Compute pressure term (u_i,-nabla p^m)              *****
*****          on subdomains                                       *****
*****                                                              *****
***** @input/output: see x2_PressureTerm.f                         *****
*****                                                              *****
***** @dependency:   see x2_PressureTerm.f                         *****
*****                                                              *****
************************************************************************
*****                                                              *****
***** @author: Bernd Noack                                         *****
*****                                                              *****
***** @see:    see x2_PressureTerm.f                               *****
*****                                                              *****
***** @revised: 2012-03-11 K"oln -> Paris                          *****
***** @created: 2012-03-10 Berlin/K"oln/Bonn                       *****
*****                                                              *****
************************************************************************
*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012
      program x2PressureTermSub
      implicit none
* --- Variables (dimensions) for file system
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
* --- Variables for xAMC data
      include 'xv3X.include'           ! for NXSub, ...
      include 'xv3X_GridSub.include'   ! for xSubG, ...
      include 'xv3X_FlowSub.include'   ! for fFlowSubG ! one snapshot
      include 'xv3X_ModesSub.include'  ! for fModesSubG (modes)
* --- Variables for energy flow analysis
      real*8 fAim (-NAMP:NEMP,NFlowsP)    ! a from POD
      real*8 fPValSub(0:NEMP,0:NFlowsP)   ! P
* --- Variables for NSE operators
      real*8 fFctPressureTermSub
* --- File name
      character*120 fn
      integer       fnLength
* --- Auxiliary variables
      integer iDomain, NDomain         ! for xAMC domains
      integer iMode                    ! counter for expansion modes
      integer iFlow                    ! counter for snapshots
      integer iDummy
      real*8  fDummy
* === Begin main segment ============================================= *
      write(*,*) '***********************************'
      write(*,*) '    Compute GP of pressure term    '
      write(*,*) '----- Bernd Noack, 2012-03-11 -----'
      write(*,*) '***********************************'
      write(*,*)
      write(*,*)
* --- Check if pressure fields are included
      if (NVarFlowSub.le.NVarGridSub) then
        write(*,'(a,i3,a,i3)') 'dim(Grid)=',NVarGridSub,
     &                     '; dim(Flow)=',NVarFlowSub
        write(*,*) 'No pressure fields available.'
        stop
      end if
* --- Read parameters ------------------------------------------------ *
* --- Read parameters
      call readYGA                     ! read NEM
      call readAmplitudes1(fAim)        ! read a_i^m
      write(*,*) 'Read domain #'
      read (*,*) iDomain
* --- Compute NFlows
      call countFlows1(iDummy) 
      NFlows = iDummy
      write(*,*) NFlows
      read (*,*)
* --- Zero array
      call initPressureTerm(fPValSub) 
* --- Read grid ------------------------------------------------------ *
      call readGridSub(iDomain)       ! needed for initWeight, etc.
* --- Read modes ----------------------------------------------------- *
      do iMode   = -NAM, NEM
        call readFilenameMode1(iMode,fn,fnLength)
        call readFlowSub(iDomain,fn,fnLength,
     &                   fModesSubG(1,1,1,1,iMode))
C        call testMode(iMode,fModesSubG(1,1,1,1,iMode))
      end do
* --- Read flows ----------------------------------------------------- *
      do iFlow   = 1, NFlows
        call readFilenameFlow1(iFlow,fn,fnLength)
        call readFlowSub(iDomain,fn,fnLength,fFlowSubG)
* ----- DO NOT Subtract mean flow (compute also constant terms)
C XXX   call addFlowOrig(fFlowSubG,
C XXX&                   fModesSubG(1,1,1,1,0),    -1.0D0)
* ----- Update energy flow analysis -------------------------------- *
        write(*,*) 'Updating the pressure terms for iDomain=',
     &              iDomain,'...'
* ..... assume pressure field in "last" component
        do iMode = 1, NEM
          fPValSub(iMode,iFlow) = 
     &    fFctPressureTermSub(fModesSubG(1,1,1,1,iMode), 
     &                        fFlowSubG(1,1,1,NVarFlowSub))
        end do
* --- End of loop over snapshots
      end do

* --- Output of the pressure term computation ------------------------ *
      call writePressureTerm(iDomain,fPValSub)
* === End of main segment ============================================ *
      stop
      end

************************************************************************
* === Add fFactor x flow field ======================================= *
      subroutine addFlowOrig(fFlowSubG,fFlow2SubG,fFactor)
      implicit none
      include 'xv3X.include'
      include 'xv3X_FlowSub.include'
      include 'xv3X_Flow2Sub.include'
      real*8  fFactor
* --- Auxiliary variables
      integer ix, iy, iz, iVar
* --- Begin of segment ----------------------------------------------- *
      do iVar=1,NVarFlowSub
      do iz  =1,NZSub
      do iy  =1,NYSub
      do ix  =1,NXSub
        fFlowSubG(ix,iy,iz,iVar) = fFlowSubG (ix,iy,iz,iVar)
     &                    + fFlow2SubG(ix,iy,iz,iVar) * fFactor
      end do
      end do
      end do
      end do
* --- End of segment ------------------------------------------------- *
      return
      end

