************************************************************************
*****                                                              *****
***** @segment name: xs_ioPressureTerm.f                           *****
*****                                                              *****
***** @task: io routines for pressure term (GP)                    *****
*****                                                              *****
***** @see: header of x2_PressureTerm.f                            *****
*****                                                              *****
************************************************************************
*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012
************************************************************************
* === initialize modal pressure term ================================= *
      subroutine initPressureTerm(fPVal)
      implicit none
* --- variables (dimensions) for file system
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
* --- variables for pressure term
      real*8 fPVal(0:NEMP,0:NFlowsP)
* --- auxiliary variables
      integer iMode, iFlow
* --- begin of segment ----------------------------------------------- *
      do iMode = 0, NEMP
      do iFlow = 0, NFlowsP
        fPVal(iMode,iFlow) = 0.0
      end do
      end do
* --- end of segment ------------------------------------------------- *
      return
      end

************************************************************************
* === output of pressure term (u_i, -nabla p^m) ====================== *
      subroutine writePressureTerm(iDomain,fPVal)
      implicit none
* --- variables (dimensions) for file system
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
* --- 
      integer iDomain
* --- variables for pressure term
      real*8 fPVal(0:NEMP,0:NFlowsP)

* --- file name
      character*120 fn
      integer       fnLength
* --- auxiliary variables
      integer iMode, iFlow
* --- begin of segment ----------------------------------------------- *
* --- opening the file with instantaneous values
      if (iDomain.ge.0) then
* ............................123456789012345.7890
        write(fn,'(a,i4.4)') 'z1_PressureTerm.',iDomain
        fnLength = 20
      else
* .......................123456789012345.789
        write(fn,'(a)') 'z1_PressureTerm.dat'
        fnLength = 19
      end if
      write(*,*) 'Writing in << ',fn(1:fnLength),' >> ...',
     &            NEM,' x ',NFlows
      open(10,file=fn(1:fnLength),status='UNKNOWN',form='formatted')
      do iFlow = 1, NFlows
      do iMode = 1, NEM
* >>>>> Note the index order if flipped during I/O: m, i -> K_i^m
        write(10,'(2i6,1e20.10)') iFlow, iMode, fPVal(iMode,iFlow) 
      end do
      end do
      close(10)
* --- end of segment ------------------------------------------------- *
      return
      end

************************************************************************
* === reading the pressure term ====================================== *
      subroutine readPressureTerm(iDomain,fPVal)
      implicit none
* --- variables (dimensions) for file system
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
* --- 
      integer iDomain
* --- variables for pressure term
      real*8 fPVal(0:NEMP,0:NFlowsP)

* --- file name
      character*120 fn
      integer       fnLength
* --- auxiliary variables
      integer iMode, iFlow
      integer iDummy, jDummy
* --- begin of segment ----------------------------------------------- *
* --- opening the file with instantaneous values
      if (iDomain.ge.0) then
* ............................123456789012345.7890
        write(fn,'(a,i4.4)') 'z1_PressureTerm.',iDomain
        fnLength = 20
      else
* .......................123456789012345.789
        write(fn,'(a)') 'z1_PressureTerm.dat'
        fnLength = 19
      end if
      write(*,*) 'Reading << ',fn(1:fnLength),' >> ...',
     &            NEM,' x ',NFlows
      open(10,file=fn(1:fnLength),status='UNKNOWN',form='formatted')
      do iFlow = 1, NFlows
      do iMode = 1, NEM
* >>>>>>> note the index order is flipped during I/O: m, i -> X_i^m
        read(10,*) jDummy, iDummy, fPVal(iDummy,jDummy) 
      end do
      end do
      close(10)
* --- end of segment ------------------------------------------------- *
      return
      end

