************************************************************************
*****                                                              *****
***** @program name: x3_FixedPoint                                 *****
***** @task:   compute fixed point of dynamical system             *****
***** @input:  y3_DS***.dat ........ parameters of DS              *****
*****          z3_FixedPoint.dat ... initial condition             *****
***** @output: z3_FixedPoint.dat ... solution                      *****
*****                                                              *****
************************************************************************
*****                                                              *****
***** @author:  Bernd Noack                                        *****
***** @see:     res090905                                          *****
***** @revised: 2010-11-07 Poitiers                                *****
***** @revised: 2010-04-03 Berlin->Poitiers (renaming)             *****
***** @created: 2009-09-16 Bad Ems                                 *****
*****                                                              *****
************************************************************************
* === subroutine compFixedPoint(fAi)
*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012
      program x3FixedPoint
      implicit none
      include 'xv_DS.include'     ! NAP,... | dimensions of DS
      include 'xv_DS_State.include'   ! fAi     | state of DS
* === Begin main segment ============================================= *
      write(*,*) '************************************'
      write(*,*) '**** simple fixed point solver *****'
      write(*,*) '************************************'
      write(*,*) '------------ BN 101107 ------------'
      write(*,*)
* --- Initialize
      call readFixedPoint(fAi)
      call initFlow
* --- Main part
      call compFixedPoint(fAi)
* --- Terminate
      call saveFixedPoint(fAi)
* === end of main segment ============================================ *
      stop
      end

************************************************************************
* === Compute fixed point of DS ====================================== *
      subroutine compFixedPoint(fAi)
      implicit none
      include 'xv_DS.include'       ! NAP,... | dimensions of DS
      include 'xv_DS_State.include'     ! fAi     | state of DS
      include 'xv_DS_Flow.include'      ! fFi     | flow of DS
      include 'xv_DS_Jacobian.include'  ! fDFij   | Jacobian of DS
      real*8  fInverseDFij(NAP,NAP)               ! Inverse of Jacobian 

      integer iDummy, jDummy, kDummy, NIteration
      real*8  fDummy
* --- Begin of segment ----------------------------------------------- *
* --- Compute flow
      fDummy = 0 ! dummy parameter for time
      call compFlow(fDummy,fAi,fFi)
* --- Newton iterations
      NIteration = 0
  101 NIteration = NIteration + 1
* --- Compute Jacobian
      call compJacobian(fAi,fDFij)
* --- Compute inverse of Jacobian
      call MatInv(fDFij,fInverseDFij,NA,NAP)
* --- Compute new estimate of a_s with Newton iteration
      do iDummy = 1, NA
         fDummy = 0
         do jDummy = 1, NA
            fDummy = fDummy - fInverseDFij(iDummy,jDummy)*fFi(jDummy)
         end do
         fAi(iDummy) =  fAi(iDummy) + 0.5*fDummy ! underrelaxation for stable operation
      end do
* ... Compute residual of new a_s
      fDummy = 0
      call compFlow(fDummy,fAi,fFi)
      fDummy = 0
      do iDummy = 1, NA
         fDummy    = fDummy + fFi(iDummy)**2
      end do
      fDummy = sqrt(fDummy)
      write(*,'(i5,a,f10.5,a,4f10.5)') NIteration,
     &        'th FP iteration, ||f||=',fDummy,
     &        '; a_i=',       (fAi(iDummy),iDummy=1,min(4,NA))
      if (NIteration.gt.1000) stop 'CompFixedPoint: no convergence!'
      if (fDummy.gt.0.000001) goto 101
* --- End of segment ------------------------------------------------- *
      return
      end
