************************************************************************
*****                                                              *****
***** @program name: x3_StabilityAnalysis                          *****
***** @task:   compute normal modes of dynamical system            *****
***** @input:  y3_ds***.dat ............... parameters of DS       *****
*****          z3_FixedPoint.dat .......... fixed point            *****
***** @output: z3_StabilitySpectrum.dat ... spectrum               *****
*****          z3_StabilityMode.0001 ...... first eigenmode        *****
*****                                                              *****
************************************************************************
*****                                                              *****
***** @author:  Bernd Noack                                        *****
***** @see:     res090905                                          *****
***** @revised: 2011-02-26 Poitiers (output format)                *****
***** @revised: 2010-11-07 Poitiers                                *****
***** @revised: 2010-02-29 Berlin (new nomenclatures)              *****
***** @revised: 2009-09-16 Bad Ems                                 *****
***** @created: 2009-09-16 Bad Ems                                 *****
*****                                                              *****
************************************************************************
* === subroutine compStabilityAnalysis(fAi)
*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012
      program x3StabilityAnalysis
      implicit none
      include 'xv_DS.include'     ! NAP,... | dimensions of DS
      include 'xv_DS_State.include'   ! fAi     | state of DS
* === Begin main segment ============================================= *
      write(*,*) '********************************************'
      write(*,*) '**** stability analysis of fixed point *****'
      write(*,*) '********************************************'
      write(*,*) '---------------- BN 101107 -----------------'
      write(*,*)
* --- Initialize
      call readFixedPoint(fAi)
      call initFlow
* --- Main part
      call compStabilityAnalysis(fAi)
* === End of main segment ============================================ *
      stop
      end

************************************************************************
* === Perform stability analysis of DS =============================== *
      subroutine compStabilityAnalysis(fAi)
      implicit none
      include 'xv_DS.include'       ! NAP,... | dimensions of DS
      include 'xv_DS_State.include'     ! fAi     | state of DS
      include 'xv_DS_Jacobian.include'  ! fDFij   | Jacobian of DS
      real*8  fEigValRe(NAP), fEigValIm(NAP)
      integer iDummy, jDummy, kDummy
      real*8  fDummy
* --- for eigenvector computation
      complex*16 cEigVal, cEigVec(NAP)
      complex*16 cMatrA(NAP,NAP), cMatrB(NAP,NAP)
* ---
      character sFileName*20
* --- Begin of segment ----------------------------------------------- *
* --- Compute Jacobian
      call compJacobian(fAi,fDFij)
* === Stability analysis (spectrum) ================================== *
      call EigVal(fDFij,fEigValRe,fEigValIm,NA,NAP)
      call SortEVal(fEigValRe,fEigValIm,NA)
* --- Save result
*--------1---------2---------3---------4---------5---------6---------7--
      open(11,file  ='z3_StabilitySpectrum.dat', form  ='formatted',
     &        access='sequential', status='unknown')
      write(*,*) 'Listing << z3_SpabilitySpectrum.dat >> ...'
      do iDummy = 1, NA
         write(*, '(i5,2f12.6)') 
     &             iDummy,fEigValRe(iDummy),fEigValIm(iDummy)
         write(11,'(i5,2f12.6)') 
     &             iDummy,fEigValRe(iDummy),fEigValIm(iDummy)
      end do
      close(11)
* === Stability analysis (eigenvectors) ============================== *
* --- Compute Jacobian (it was destroyed in EigVal)
      call compJacobian(fAi,fDFij)
* --- Loop over eigenvalues
      do iDummy = 1, NA
* ----- Complex eigenvalue
        cEigVal = cmplx(fEigValRe(iDummy),fEigValIm(iDummy))
        write(*,'(i5,a,2f12.5)')
     &          iDummy,'th eigenvector for lambda=', cEigVal
* ----- Complex stability matrix
        do jDummy = 1, NA
        do kDummy = 1, NA
          cMatrA(jDummy,kDummy) = fDFij(jDummy,kDummy)
          cMatrB(jDummy,kDummy) = 0.0
        end do
        end do
* -----
        call EigVec(cEigVec,cMatrA,cMatrB,cEigVal,NA,NAP)
* ----- Save result
*---------------------------1---------2
*------------------12345678901234567890
C === sFileName = 'z3_Eigenvector.0000'
        write(sFileName,'(a,i4.4)') 'z3_Eigenvector.',iDummy
        open(11,file  =sFileName, form  ='formatted',
     &          access='sequential', status='unknown')
        write(*,'(a,a,a)') 'Writing << ',sFileName,' >> ...'
        do jDummy = 1, NA
          write(11,'(i5,3f12.5)') jDummy, real(cEigVec(jDummy)), 
     &                                    imag(cEigVec(jDummy)), 
     &                                    abs (cEigVec(jDummy))
        end do
        close(11)
* --- End of loop over eigenvalues
      end do 
* --- End of segment ------------------------------------------------- *
      return
      end
