************************************************************************
*****                                                              *****
***** @program name: x3_StabilityAnalysisTKE                       *****
***** @task:   compute normal modes of A^T A (energy growth)       *****
***** @input:  y3_ds***.dat ............... parameters of DS       *****
*****          z3_FixedPoint.dat .......... fixed point            *****
***** @output: z3_SpectrumTKE.dat ... spectrum                     *****
*****          z3_ModeTKE.0001 ...... first eigenmode              *****
*****                                                              *****
************************************************************************
*****                                                              *****
***** @author:  Bernd Noack                                        *****
***** @see:     Noack & Cordier 2012 report                        *****
***** @created: 2012-04-07 Canberra                                *****
*****                                                              *****
************************************************************************
* === subroutine compStabilityAnalysis(fAi)
*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012
      program x3StabilityAnalysisTKE
      implicit none
      include 'xv_DS.include'     ! NAP,... | dimensions of DS
      include 'xv_DS_State.include'   ! fAi     | state of DS
* === Begin main segment ============================================= *
      write(*,*) '*********************************************'
      write(*,*) '**** Energy growth rates at fixed point *****'
      write(*,*) '*********************************************'
      write(*,*) '----------------- BN 120407 -----------------'
      write(*,*)
* --- Initialize
      call readFixedPoint(fAi)
      call initFlow
* --- Main part
      call compStabilityAnalysisTKE(fAi)
* === End of main segment ============================================ *
      stop
      end

************************************************************************
* === Perform stability analysis of DS =============================== *
      subroutine compStabilityAnalysisTKE(fAi)
      implicit none
      include 'xv_DS.include'           ! NAP,... | dimensions of DS
      include 'xv_DS_State.include'     ! fAi     | state of DS
      include 'xv_DS_Jacobian.include'  ! fDFij   | Jacobian of DS
* --- POD
      real*8  fMatrixG   (NAP,NAP)   ! Gramian
      real*8  fMatrixEVec(NAP,NAP)   ! eigenvectors of Gramian
      real*8  fSpectrumG (NAP)       ! eigenvectors of Gramian
      integer iDummy, jDummy, kDummy
      real*8  fDummy
* ---
      character sFileName*20
* --- Begin of segment ----------------------------------------------- *
* --- Compute Jacobian ----------------------------------------------- *
      call compJacobian(fAi,fDFij)
* --- Compute Gramian ------------------------------------------------ *
      do iDummy = 1,NA
      do jDummy = 1,NA
         fDummy = 0.
        do kDummy = 1,NA
          fDummy = fDummy + fDFij(iDummy,kDummy)*fDFij(kDummy,jDummy) 
        end do
        fDummy = fDFij(iDummy,jDummy)+fDFij(jDummy,iDummy) 
        fMatrixG(iDummy,jDummy) = fDummy
      end do
      end do
* --- Compute spectrum of Gramian ------------------------------------ *
      call SpecDecSym(fMatrixG,fMatrixEVec,fSpectrumG,NA,NAP)
* --- Save spectrum -------------------------------------------------- *
      write(*,'(a,a,a)') 'Writing << z3_SpectrumTKE.dat >> ...'
      open(11,file  ='z3_SpectrumTKE.dat', form  ='formatted',
     &        access='sequential',         status='unknown')
      write(*,*) 'Stability spectrum (TKE):'
      do iDummy = 1, NA
         write(*, '(i5,2f12.6)') 
     &             iDummy, fSpectrumG(iDummy)
         write(11,'(i5,2f12.6)') 
     &             iDummy, fSpectrumG(iDummy)
      end do
      close(11)
* --- Save eigenvectors ---------------------------------------------- *
        write(*,'(a,a,a)') 'Writing << z3_EigenvectorsTKE.dat >> ...'
      open(11,file='z3_EigenvectorsTKE.dat', form  ='formatted',
     &          access='sequential',         status='unknown')
      do iDummy = 1, NA
      do jDummy = 1, NA
        write(11,'(2i5,f12.6)') iDummy, jDummy, 
     &                          fMatrixEVec(iDummy,jDummy) 
      end do
      end do
      close(11)
* --- End of segment ------------------------------------------------- *
      return
      end
