************************************************************************
*****                                                              *****
***** @program name: x3_compStatistics                             *****
***** @task:   compute moments                                     *****
***** @input:  z3_Times.dat ........ sampling time                 *****
****           z3_Amplitudes.dat ... saved states                  *****
***** @output: gnu_DS_Amplitudes.dat                               *****
*****          gnu_DS_Statistics.dat                               *****
*****                                                              *****
************************************************************************
*****                                                              *****
***** @author:  Bernd Noack                                        *****
***** @see:     ---                                                *****
***** @revised: 2011-07-14 Poitiers (amplitudes)                   *****
***** @revised: 2010-11-07 Poitiers                                *****
***** @revised: 2010-03-29 Berlin (new nomenclature)               *****
***** @revised: 2010-03-02 Poitiers                                *****
***** @created: 2010-03-02 Poitiers                                *****
*****                                                              *****
************************************************************************
* === readSolution(fTm,fAim,NSnap)

*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012
      program x3Statistics
      implicit none
      include 'xv_DS.include'     ! NAP,... | dimensions of DS
      include 'xv_DS_Solution.include'! NSnap,... | dimensions of array
* === Begin main segment ============================================= *
      write(*,*) '********************************'
      write(*,*) '***** statististics of a_i *****'
      write(*,*) '********************************'
      write(*,*) '---------- BN 101107 -----------'
      write(*,*)
* --- Initialize
      call readSolution(fTm,fAim,NSnap)
      call compMoments   (fAim,NSnap)
      call compAmplitudes(fAim,NSnap)
* === End of main segment ============================================ *
      stop
      end

************************************************************************
* === Reading the solution =========================================== *
      subroutine readSolution(fTm,fAim,NSnap)
      implicit none
      include 'xv_DS.include'     ! NAP,... | dimensions of DS
      include 'xv_DS_Solution.include'! NSnap,... | dimensions of array
      integer*8 iDummy, iSnap
      real*8    fDummy, fMax, fMin
* --- Begin of subroutine -------------------------------------------- *
* --- Reading the time
      write(*,*) 'Reading << z3_Times.dat >> ...'
      NSnap = 1
 111  open(10,file='z3_Times.dat',form='formatted',status='old')
      read(10,*,end=199,err=199) iSnap, fDummy
      fTm(iSnap) = fDummy
      NSnap = max(iSnap,NSnap)
      goto 111
 199  close(10)
      write(*,'(2(a,f15.8),a,i8)') 
     &        't_min=',fTm(1),'; t_max=',fTm(NSnap),'; M=',NSnap
* --- Reading the amplitudes
      write(*,*) 'Reading << z3_Amplitudes.dat >> ...'
      NA    = 1
      NSnap = 1
      fMin  =  1.E37
      fMax  = -1.E37
 211  open(10,file='z3_Amplitudes.dat',form='formatted',status='old')
      read(10,*,end=299,err=299) iSnap, iDummy, fDummy
      fAim(iDummy,iSnap) = fDummy !
      fMax  = max(fMax,  fDummy) 
      fMin  = min(fMin,  fDummy) 
      NA    = max(iDummy,NA)
      NSnap = max(iSnap, NSnap)
      goto 211
 299  close(10)
      write(*,'(2(a,f15.8),2(a,i8))') 
     &        'min(a_i)=',fMin,'; max(a_i)=',fMax,
     &        '; N=',NA, '; M=',NSnap
* --- End of subroutine ---------------------------------------------- *
      return
      end

************************************************************************
* === Compute statistical moments  =================================== *
      subroutine compMoments(fAim,NSnap)
      implicit none
      include 'xv_DS.include'     ! NAP,...   | dimensions of DS
      include 'xv_DS_Solution.include'! NSnap,... | dimensions of array
      real*8  fAiM1, fAiM2, fAiM3, fAiM4
      integer*8 iDummy, iSnap
      real*8    fDummy, fMax, fMin
* --- begin of subroutine -------------------------------------------- *
      write(*,*) 'Compute moments ...'
      write(*,'(a5,4a12)') 'i', 'm_i','V_i','S_i','F_i'
      open(10, file='gnu_DS_Statistics.dat', form='formatted',
     &         status='UNKNOWN', access='sequential')
* --- Initialize
      do iDummy = 1, NA
* ----- First moment
        fDummy = 0
        do iSnap  = 1, NSnap
          fDummy = fDummy + fAim(iDummy,iSnap)
        end do
        fAiM1 = fDummy / NSnap
* ----- Subtract mean value
        do iSnap  = 1, NSnap
          fAim(iDummy,iSnap) = fAim(iDummy,iSnap) - fAiM1
        end do
* ----- Compute second to fourth moment
        fAiM2 = 0
        fAiM3 = 0
        fAiM4 = 0
        do iSnap  = 1, NSnap
          fDummy = fAim(iDummy,iSnap)
          fAiM2  = fAiM2 + fDummy**2
          fAiM3  = fAiM3 + fDummy**3
          fAiM4  = fAiM4 + fDummy**4
        end do
        fDummy = 1.0
        fAiM2 = (fAiM2 / NSnap)**(1./2.) ! no Bessel correction
        fAiM3 = (fAiM3 / NSnap)/fAiM2**3
        fAiM4 = (fAiM4 / NSnap)/fAiM2**4
* ----- Write moment
        write(*,'(i5,4f15.8)') iDummy,fAiM1,fAiM2**2,fAiM3,fAiM4
        write(10,'(i5,4f15.8)') iDummy,fAiM1,fAiM2**2,fAiM3,fAiM4
      end do
      close(10)
* --- End of subroutine ---------------------------------------------- *
      return
      end

************************************************************************
* === Compute amplitudes ============================================= *
      subroutine compAmplitudes(fAim,NSnap)
      implicit none
      include 'xv_DS.include'     ! NAP,...   | dimensions of DS
      include 'xv_DS_Solution.include'! NSnap,... | dimensions of array
      real*8  fAiM1, fAiM2, fDummy, fDummy2
      real*8  fVariance(NAP)
      real*8  fLambda  (NAP)
      integer*8 iDummy, jDummy, kDummy, iSnap
      logical bExist
* --- begin of subroutine -------------------------------------------- *
      write(*,*) 'Compute amplitudes ...'
* --- Initialize
      do iDummy = 1, NA
* ----- First moment
        fDummy = 0
        do iSnap  = 1, NSnap
          fDummy = fDummy + fAim(iDummy,iSnap)
        end do
        fAiM1 = fDummy / NSnap
* ----- Subtract mean value
        do iSnap  = 1, NSnap
          fAim(iDummy,iSnap) = fAim(iDummy,iSnap) - fAiM1
        end do
* ----- Compute second to fourth moment
        fAiM2 = 0
        do iSnap  = 1, NSnap
          fDummy = fAim(iDummy,iSnap)
          fAiM2  = fAiM2 + fDummy**2
        end do
        fVariance(iDummy) = max(0.0,fAiM2 / NSnap -fAiM1**2)
      end do
* --- load POD spectrum
      do iDummy=1,NAP
        fLambda(iDummy) = 0
      end do
      inquire(file='z1_SpectrumPOD.dat',EXIST=bExist)
      if (bExist) then
        open(10,file='z1_SpectrumPOD.dat',form='formatted',status='old')
  101   read(10,*,end=102,err=102) iDummy, fDummy
        fLambda(iDummy) = fDummy
        goto 101
  102   close(10)
      else
        write(*,*) '<< z1_SpectrumPOD.dat >> does not exist.'
      end if
* --- compute amplitudes
      write(*,*) 'Compute amplitudes ...'
      write(*,'(a5,4a12)') 'i', 'A_i', 'A_i(POD)'
      open(10, file='gnu_DS_Amplitudes.dat', form='formatted',
     &         status='UNKNOWN', access='sequential')
      do iDummy = 1, NA
       fDummy  = sqrt(2.*fVariance(iDummy))
       fDummy2 = sqrt(2.*fLambda(iDummy))
        write(* ,'(i5,4f15.8)') iDummy, fDummy, fDummy2,
     &                   (fDummy2-fDummy)/fDummy
        write(10,'(i5,4f15.8)') iDummy, fDummy, fDummy2,
     &                   (fDummy2-fDummy)/fDummy
      end do
      close(10)
* --- compute amplitudes
      write(*,*) 'Compute amplitudes of "l"-th harmonics...'
      write(*,'(a5,4a12)') 'l', 'A_l', 'A_l(POD)'
      open(10, file='gnu_DS_Amplitudes2.dat', form='formatted',
     &         status='UNKNOWN', access='sequential')
      do iDummy = 1, NA/2
       jDummy = 2*iDummy-1
       kDummy = 2*iDummy
       fDummy  = sqrt(fVariance(jDummy)+fVariance(kDummy))
       fDummy2 = sqrt(fLambda  (jDummy)+fLambda  (kDummy))
        write(* ,'(i5,4f15.8)') iDummy, fDummy, fDummy2,
     &                   (fDummy2-fDummy)/fDummy
        write(10,'(i5,4f15.8)') iDummy, fDummy, fDummy2,
     &                   (fDummy2-fDummy)/fDummy
      end do
      close(10)
* --- Compute TKE
      fDummy  = 0.0
      fDummy2 = 0.0
      do iDummy = 1, NA
        fDummy  = fDummy  + fLambda(iDummy)/2
        fDummy2 = fDummy2 + fVariance(iDummy)/2
      end do
      open(10, file='gnu_TKE.dat', form='formatted',
     &         status='UNKNOWN', access='sequential')
      write(10,'(3f12.5)') fDummy2, fDummy, fDummy2/fDummy-1.
      close(10)
* --- End of subroutine ---------------------------------------------- *
      return
      end
