************************************************************************
*****                                                              *****
***** @program name: x3_plotEigenvector                            *****
***** @task:   plots eigenvectors from x3_StabilityAnalysis        *****
***** @input:  z3_Eigenvector                                      *****
***** @output: gnu_DS_ev.000i ..... for eigenvector                *****
*****                                                              *****
************************************************************************
*****                                                              *****
***** @author:  Bernd Noack                                        *****
***** @see:     ---                                                *****
***** @revised: 2010-11-07 Poitiers                                *****
***** @revised: 2010-02-29 Berlin (new nomenclature)               *****
***** @created: 2010-02-24 Poitiers                                *****
*****                                                              *****
************************************************************************
*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012
      program x3plotEigenvector
      implicit none
      include 'xv_DS.include'     ! NAP,... | dimensions of DS
      complex*16 cEigVal(NAP)
      complex*16 cEigVec(NAP,NAP)     ! eigenvectors in columns (!)
* === Begin main segment ============================================= *
      write(*,*) '*****************************'
      write(*,*) '***** plot eigenvectors *****'
      write(*,*) '*****************************'
      write(*,*) '-------- BN 101107 ---------'
      write(*,*)
* --- Initialize
      call readSpectrum   (cEigVal) ! determines "N"
      call readEigenvector(cEigVec)
      call plotEigenvector(cEigVal,cEigVec)
* === End of main segment ============================================ *
      stop
      end

************************************************************************
* === Reading the stability spectrum ================================= *
      subroutine readSpectrum(cEigVal)
      implicit none
      include 'xv_DS.include'     ! NAP,... | dimensions of DS
      complex*16 cEigVal(NAP)         ! eigenvalues
* --- Dummy variables
      integer*8 iDummy,  jDummy
      real*8    f1Dummy, f2Dummy 
      character*20, sFileName
* --- Begin of subroutine -------------------------------------------- *
* --- Initialize
      do iDummy = 1, NAP
        cEigVal(iDummy)=0.0
      end do
* --- Read spectrum
      NA = 0
      write(*,'(a,a,a)') 'Reading << z3_StabilitySpectrum.dat >> ...'
      open(10,file='z3_StabilitySpectrum.dat',
     &        form='formatted',status='old')
 211  read(10,*,end=299,err=299) iDummy, f1Dummy, f2Dummy
      cEigVal(iDummy) = cmplx(f1Dummy,f2Dummy)
      NA = max(NA,iDummy)
      goto 211
 299  close(10)
* --- Show spectrum
      write(*,*) 'stability spectrum:'
      do iDummy=1,NA
        write(*,'(i5,2f12.5)') iDummy, cEigVal(iDummy)
      end do
* --- End of subroutine ---------------------------------------------- *
      return
      end

************************************************************************
* === Reading the eigenvector ======================================== *
      subroutine readEigenvector(cEigVec)
      implicit none
      include 'xv_DS.include'     ! NAP,... | dimensions of DS
      complex*16 cEigVec(NAP,NAP)     ! eigenvector in rows
* --- Dummy variables
      integer*8 iDummy,  jDummy
      real*8    f1Dummy, f2Dummy 
      character*20, sFileName
* --- Begin of subroutine -------------------------------------------- *
* --- Initialize
      do iDummy = 1, NAP
      do jDummy = 1, NAP
        cEigVec(iDummy,jDummy)=0.0
      end do
      end do
* --- Read eigenvector
      do iDummy = 1, NA
        write(sFileName,'(a,i4.4)') 'z3_Eigenvector.',iDummy
        write(*,'(a,a,a)') 'Reading << ',sFileName,' >> ...'
        open(10,file=sFileName,form='formatted',status='old')
 211    read(10,*,end=299,err=299) jDummy, f1Dummy, f2Dummy
        cEigVec(iDummy,jDummy) = cmplx(f1Dummy,f2Dummy)
        goto 211
 299    close(10)
      end do
* --- End of subroutine ---------------------------------------------- *
      return
      end

************************************************************************
* === Plotting the eigenvector ======================================= *
      subroutine plotEigenvector(cEigVal,cEigVec)
      implicit none
      include 'xv_DS.include'     ! NAP,...   | dimensions of DS
      complex*16 cEigVal(NAP)         ! eigenvalue in rows
      complex*16 cEigVec(NAP,NAP)     ! eigenvector in rows
      integer*8 iDummy, jDummy
      real*4    f1Dummy, f2Dummy, f3Dummy
      character*17 fName
* --- Begin of subroutine -------------------------------------------- *
      write(*,*) 'Plotting the eigenvectors ...'
      do iDummy = 1, NA
* ----- Creating gnuplot data file
* ..... 12345678901234567
* ..... gnu_DS_ev0000.dat
        write(fName,'(a,i4.4,a)') 'gnu_DS_ev',iDummy,'.dat'
        write(*,'(a,a,a)') 'Creating <<',fName,'>> ...'
        open(10,file=fName,status='UNKNOWN',form='FORMATTED')
        do jDummy=1,NA
          f1Dummy = sngl(real(cEigVec(iDummy,jDummy)))
          f2Dummy = sngl(imag(cEigVec(iDummy,jDummy)))
          f3Dummy = sngl(abs (cEigVec(iDummy,jDummy)))
          write(10,*) jDummy, f1Dummy,f2Dummy,f3Dummy
        end do
        close(10)
* ----- Creating the gnuplot script
        open(10,file='tmp.gnuplot',status='UNKNOWN',form='FORMATTED')
        write(10,'(3a)') 'plot "',fName,'" using 1:2 w l title "Re",\'
        write(10,'(3a)') '"',fName,'" using 1:3 w l title "Im",\'
        write(10,'(3a)') '"',fName,'" using 1:4 w l title "abs"'
        write(10,'(a,i4,a)') 'set title "',iDummy,'th eigenvector"'
        write(10,'(a)') 'set xlabel "i"'
        write(10,'(a)') 'set ylabel "a_i"'
        write(10,'(a)') 'set terminal png'
        write(fName,'(a,i4.4,a)') 'gnu_DS_ev',iDummy,'.png'
        write(10,'(3a)') 'set output "',fName(1:17),'"'
        write(10,'(a)') 'replot'
        close(10)
* ----- Executing the gnuplot script
        open(10,file='tmp.sh',status='UNKNOWN',form='FORMATTED')
        write(10,'(a)') 'gnuplot tmp.gnuplot'
        close(10)
        call system("chmod u+x tmp.sh")
        call system("./tmp.sh")
      end do
* --- Creating the LaTeX file comprising all EPS files
      write(*,*) 'Creating gnu_DS_ev.[tex,pdf] ...'
      open(unit=10,file  ='gnu_DS_ev.tex',    status='unknown',
     &             access='sequential', form  ='formatted')
* === NOTE: char(92)='\' should run on all fortran compilers.
* === Different fortran compilers  assume 
* === '\' exclusive or '\\' for the backslash.
      write(10,'(a)') char(92)//'documentclass[twocolumn]{article}'
      write(10,'(a)') char(92)//'usepackage{graphicx}'
      write(10,'(a)') char(92)//'DeclareGraphicsExtensions{png}'
      write(10,'(a)') char(92)//'begin{document}'
      do iDummy=1,NA
      write(10,'(a)') char(92)//'begin{figure}'
      write(10,'(2a,i4.4,a)') 
     &         char(92)//'includegraphics[width=75mm,angle=-00]',
     &         '{gnu_DS_ev',iDummy,'.png}'
      write(10,'(a,i4,a)') 
     &         char(92)//'caption{',iDummy,'th eigenvector.'
      f1Dummy = sngl(real(cEigVal(iDummy)))
      f2Dummy = sngl(imag(cEigVal(iDummy)))
* --- Positive imaginary part
      if (f2Dummy.gt.0) then
        write(10,'(a,f12.5,a,f12.5,a)') 
     &   '$\lambda_i=$,',f1Dummy,'+',f2Dummy,'$\imath$.}'
* --- Negative imaginary part
      else if (f2Dummy.lt.0) then
        write(10,'(a,f12.5,a,f12.5,a)') 
     &   '$\lambda_i=$,',f1Dummy,'-',-f2Dummy,'$\imath$.}'
* --- Vanishing imaginary part
      else
        write(10,'(a,f12.5,a)') 
     &   '$\lambda_i=$,',f1Dummy,'.}'
      end if
      write(10,'(a)') char(92)//'end{figure}'
      end do
      write(10,'(a)') char(92)//'end{document}'
      close(10)
* --- Executing the LaTeX file
      call system("pdflatex gnu_DS_ev.tex")
* ---
  901 format(1h a)
* --- End of subroutine ---------------------------------------------- *
      return
      end
