************************************************************************
*****                                                              *****
***** @program name: x3_solveDS                                    *****
***** @task:   dynamical systems solver                            *****
***** @input:  y3_Time.dat .... parameters for time integration    *****
****           y3_State.dat ... initial condition                  *****
*****          y3_DS***.dat ... parameters of DS                   *****
***** @output: gnu_DS_ta.dat .... DS solution                      *****
*****                                                              *****
************************************************************************
*****                                                              *****
***** @author:  Bernd Noack                                        *****
***** @see:     res090905                                          *****
***** @revised: 2011-07-14 Poitiers (write final state)            *****
***** @revised: 2011-01-17 Poitiers                                *****
***** @revised: 2010-11-09 Poitiers                                *****
***** @revised: 2010-03-29 Berlin (new nomenclature)               *****
***** @revised: 2009-09-16 Bad Ems                                 *****
***** @created: 2009-09-16 Bad Ems                                 *****
*****                                                              *****
************************************************************************
* === subroutine nextState(timeNow,fAi)
*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012
      program x3solveDS
      implicit none
      include 'xv_DS.include'     ! NAP,... | dimensions of DS
      include 'xv_DS_Time.include'    ! timeBegin, time End, ...
      include 'xv_DS_State.include'   ! fAi     | state of DS
      include 'xv_DS_Flow.include'    ! fFi      | propagator of DS
      real*8   timeNow, fNorm
* === Begin main segment ============================================= *
      write(*,*) '*****************************************'
      write(*,*) '**** simple dynamical system solver *****'
      write(*,*) '*****************************************'
      write(*,*) '--------------- BN 101107 --------------'
      write(*,*)
* --- Initialize
      call initTime(timeNow)
      call initState(        fAi)
      call initFlow
      fNorm = 0
C XXX call antisymmetrizeQijk
* --- Output of parameters and initial state
      call showTime(timeNow)
      call showState(timeNow,fAi)
      call openFiles
      call saveState(timeNow,fAi)
* --- Time integration
      do while (timeNow.lt.timeEnd-0.5*timeStep
     &          .and.fNorm.lt.1.E6)
        call nextState(timeNow,fAi)
        call showState(timeNow,fAi)
        call saveState(timeNow,fAi)
        call normState(fAi,fNorm)
      end do
* --- Terminate
      call closeFiles
      call writeState(fAi) ! write final a_i in << y3_State >>
* === End of main segment ============================================ *
      stop
      end

************************************************************************
* === Enforce energy preservation of the quadratic term  ============= *
      subroutine antisymmetrizeQijk
      implicit none
      include 'xv_DS.include'        ! NAP,... | dimensions of DS
      include 'xv_DS_Propagator.include' ! fQijk
      real*8    fDummy
      integer*8 iDummy, jDummy, kDummy
* --- Begin of subroutine -------------------------------------------- *
* --- symmetrization of the main diagonal
      do iDummy = 1, NA
        fQijk(iDummy,iDummy,iDummy)=0
      end do
* --- symmetrization of the side diagonal
      do iDummy = 1, NA
      do jDummy = 1, NA
        if (iDummy.ne.jDummy) then
          fDummy = 0
          fDummy = fDummy + fQijk(iDummy,iDummy,jDummy)
          fDummy = fDummy + fQijk(iDummy,jDummy,iDummy)
          fDummy = fDummy + fQijk(jDummy,iDummy,iDummy)
          fDummy = fDummy / 3
          fQijk(iDummy,iDummy,jDummy)=fQijk(iDummy,iDummy,jDummy)-fDummy
          fQijk(iDummy,jDummy,iDummy)=fQijk(iDummy,jDummy,iDummy)-fDummy
          fQijk(jDummy,iDummy,iDummy)=fQijk(jDummy,iDummy,iDummy)-fDummy
        end if
      end do
      end do
* --- symmetrization of the off-diagonal terms i<>j<>k
      do iDummy = 1, NA
      do jDummy = 1, iDummy-1
      do kDummy = 1, jDummy-1
        fDummy = 0
        fDummy = fDummy + fQijk(iDummy,jDummy,kDummy)
        fDummy = fDummy + fQijk(iDummy,kDummy,jDummy)
        fDummy = fDummy + fQijk(jDummy,iDummy,kDummy)
        fDummy = fDummy + fQijk(jDummy,kDummy,iDummy)
        fDummy = fDummy + fQijk(kDummy,iDummy,jDummy)
        fDummy = fDummy + fQijk(kDummy,jDummy,iDummy)
        fDummy = fDummy / 6
        fQijk(iDummy,jDummy,kDummy)=fQijk(iDummy,jDummy,kDummy)-fDummy
        fQijk(iDummy,kDummy,jDummy)=fQijk(iDummy,kDummy,jDummy)-fDummy
        fQijk(jDummy,iDummy,kDummy)=fQijk(jDummy,iDummy,kDummy)-fDummy
        fQijk(jDummy,kDummy,iDummy)=fQijk(jDummy,kDummy,iDummy)-fDummy
        fQijk(kDummy,iDummy,jDummy)=fQijk(kDummy,iDummy,jDummy)-fDummy
        fQijk(kDummy,jDummy,iDummy)=fQijk(kDummy,jDummy,iDummy)-fDummy
      end do
      end do
      end do     
* --- End of subroutine ---------------------------------------------- *
      return
      end

************************************************************************
* === Compute next step ============================================== *
      subroutine nextState(timeNow,fAi)
      implicit none
      include 'xv_DS.include'     ! NAP,... | dimensions of DS
      include 'xv_DS_Time.include'    ! timeShow, ...
      include 'xv_DS_State.include'   ! fAi     | state of DS
      include 'xv_DS_State2.include'  ! fAi2
      include 'xv_DS_Flow.include'    ! fFi     | propagator of DS
      include 'xv_DS_Flow2.include'   ! fFi2
      real*8    timeNow
      integer*8 iDummy
* --- Begin of subroutine -------------------------------------------- *
* --- Predictor --------------------------------------------------------
      call compFlow(timeNow,fAi,fFi)
      do iDummy = 1, NA
* ..... ODE if z_i >=0
        if (iMode(iDummy).ge.0) then
          fAi2(iDummy) = fAi(iDummy)
     &                 + timeStep*fFi(iDummy)
* ..... algebraic equation if z_i < 0
        else
          fAi2(iDummy) = fFi(iDummy)
        end if
      end do
      timeNow = timeNow + timeStep
* --- Corrector --------------------------------------------------------
      call compFlow(timeNow,fAi2,fFi2)
      do iDummy = 1, NA
* ..... ODE if z_i >=0
        if (iMode(iDummy).ge.0) then
          fAi(iDummy) = fAi(iDummy)
     &                 + 0.5*timeStep*(fFi(iDummy)+fFi2(iDummy))
* ..... algebraic equation if z_i < 0
        else
          fAi(iDummy) = fFi(iDummy)
        end if

      end do
* --- End of subroutine ---------------------------------------------- *
      return
      end

************************************************************************
* === Compute the norm of the state ================================== *
      subroutine normState(fAi,fNorm)
      implicit none
      include 'xv_DS.include'     ! NAP,... | dimensions of DS
      include 'xv_DS_State.include'   ! fAi     | state of DS
      integer*8 iDummy
      real*8    fNorm
* --- Begin of subroutine -------------------------------------------- *
      fNorm = 0
      do iDummy = 1, NA
        fNorm = fNorm + fAi(iDummy)**2
      end do
      fNorm = sqrt(max(0.0,fNORM))
* --- End of subroutine ---------------------------------------------- *
      return
      end
