************************************************************************
*****                                                              *****
***** @program name: x3_makeDS                                     *****
***** @task:   specify dynamical system                            *****
***** @input:  ...                                                 *****
***** @output: ...                                                 *****
*****                                                              *****
************************************************************************
*****                                                              *****
***** @author:  Bernd Noack                                        *****
***** @see:     res.20100408.xAMC_ToDo                             *****
***** @revised: 2012-03-18 Poitiers                                *****
***** @revised: 2010-11-07 Poitiers                                *****
***** @revised: 2010-07-14 Poitiers                                *****
***** @revised: 2010-05-16 Poitiers                                *****
***** @created: 2010-05-07 Royan                                   *****
*****                                                              *****
************************************************************************
*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012
      program x3makeDS
      implicit none
* --- Variables for Galerkin system
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
* === Begin main segment ============================================= *
      write(*,*) '****************************************'
      write(*,*) '******* Make Dynamical System   ********'
      write(*,*) '****************************************'
      write(*,*) '-------------- BN 101107 ---------------'
      write(*,*)
* --- Read the Galerkin model
      call readYGA
      call readYGP
* --- Write the descriptor system
      call writeYDS
      call writeTime
      call writeModes
      call writeState
      call writeDS
* === End of main segment ============================================ *
      stop
      end

************************************************************************
* === Write job file ***************************************************
      subroutine writeYDS
      implicit none
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
      integer NModes
* --- Begin segment -------------------------------------------------- *
      NModes = NEM + 2 *NAM + NVF
      write(*,*) 'Writing << y3_DS.dat >>...', NModes
* --- Open file ------------------------------------------------------ *
      open(10,file='y3_DS.dat',form='formatted',status='UNKNOWN')
* --- Write in file -------------------------------------------------- *
      write(10,'(a)') '[*** standard file from xGS ***]'
      write(10,'(i5, 5x,a,i5,a,i5,a,i5,a)') 
     &                NModes,'[N; N_E=',NEM,
     &                       ' ; N_A=',NAM,
     &                       ' ; N_V=,',NVF,']'
      write(10,'(a14, 8x,a)') 'y3_DS_Qijk.dat', 
     &                        '[DS for natural dynamics]'
      write(10,'(a12,10x,a)') 'y3_State.dat',
     &                        '[initial condition]'
      write(10,'(a11,11x,a)') 'y3_Time.dat',
     &                        '[time]'
* --- Close file ----------------------------------------------------- *
      close(10)
* --- End of segment ------------------------------------------------- *
      return
      end

************************************************************************
* === Write time integration parameters for dynamical system ***********
      subroutine writeTime
      implicit none
      integer*8 iDummy
      real*8    fDummy, tMin, tMax
      logical   bDummy
* --- Begin segment -------------------------------------------------- *
      write(*,*) 'Writing << y3_Times.dat >> ...'
* --- Adopting the t_min and t_max from z1_Times, if available
      inquire(file='z1_Times.dat',exist=bDummy)
      if (bDummy) then
        tMin =  1.E37
        tMax = -tMin
        open(10,file='z1_Times.dat',form='formatted',status='old')
 101    read(10,*,end=102,err=102) iDummy, fDummy
        tMin = min(tMin,fDummy)
        tMax = max(tMax,fDummy)
        goto 101
 102    close(10)
      else
        tMin =   0.0
        tMax = 250.0
      end if
C XXX tMax = max(5000.0,tMax) ! EXCEPTION for cylinder wake models

* --- Open file ------------------------------------------------------ *
      open(10,file='y3_Time.dat',form='formatted',status='UNKNOWN')
* --- Write in file -------------------------------------------------- *
      write(10,'(f12.5,5x,a)')   tMin,  '[t_0 ]'
      write(10,'(f12.5,5x,a)')   tMax,  '[t_1 ]'
      write(10,'(f12.5,5x,a)')   0.001, '[dt, 1000 steps per unit]'
      write(10,'(f12.5,5x,a)')   1.0,   '[dt_Show, 10 steps per unit]'
      write(10,'(f12.5,5x,a)')   0.1,   '[dt_Save, 10 steps per unit]'
* --- Close file ----------------------------------------------------- *
      close(10)
* --- End of segment ------------------------------------------------- *
      return
      end

************************************************************************
* === Classify modes of descriptor system ******************************
      subroutine writeModes
      implicit none
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
      integer iMode, NModes
* --- Begin segment -------------------------------------------------- *
      NModes = NEM + 2*NAM + NVF
      write(*,*) 'Writing << y3_Modes.dat >> ...'
      write(*,'(4i5)') NEM, NAM, NVF, NModes
* --- Open file ------------------------------------------------------ *
      open(10,file='y3_Modes.dat',form='formatted',status='UNKNOWN')
* --- Write in file -------------------------------------------------- *
      do iMode = 1, NModes
* ..... ODE for expansion modes
        if (iMode.le.NEM) then
          write(10,'(2i5,2x,a)') iMode, iMode, '! expansion mode'
* ..... differential equations for actuation mode (da_i/dt = a_j)
        else if (iMode.gt.NEM.and.iMode.le.NEM+NAM) then
          write(10,'(2i5,2x,a,i5)') iMode, NEM+1,
     &             '! actuation mode a_i, i=',-(iMode-NEM)
* ..... algebraic equations for derivatives of actuation modes
        else if (iMode.gt.NEM+NAM.and.iMode.le.NEM+2*NAM) then
          write(10,'(2i5,2x,a,i5)') iMode,-1,
     &             '! actuation mode da_i/dt, i=',-(iMode-NEM-NAM)
* ..... algebraic equations for actuation modes and volume forces
        else 
          write(10,'(2i5,2x,a,i5)') iMode, -1,
     &             '! volume force b_l, l=',(iMode-NEM-2*NAM)
        end if
      end do
* --- Close file ----------------------------------------------------- *
      close(10)
* --- End of segment ------------------------------------------------- *
      return
      end

************************************************************************
* === Write IC for dynamical system ************************************
      subroutine writeState
      implicit none
      integer*8 iDummy, jDummy
      real*8    fDummy
      logical   bDummy
* --- Begin segment -------------------------------------------------- *
      write(*,*) 'Writing << y3_State.dat >> ...'
* --- Adopting the t_min and t_max from z1_Times, if available
      inquire(file='z1_Amplitudes.dat',exist=bDummy)
* --- Open file ------------------------------------------------------ *
      open(11,file='y3_State.dat',form='formatted',status='UNKNOWN')
      if (bDummy) then
        write(*,*) 'Copying first state of << z1_Amplitudes.dat >> ...'
        open(10,file='z1_Amplitudes.dat',form='formatted',status='old')
 101    read(10,*,end=102,err=102) iDummy, jDummy, fDummy
        if (iDummy.eq.1.and.jDummy.gt.0) then
        write(11,'(i5,e18.10,5x,a)') jDummy, fDummy,
     &           '! a_i(t_1) from z1_Amplitudes.dat'
        end if
        goto 101
 102    close(10)
      else
        write(*,*) 'Initializing with a_1 = 0.01 ...'
        write(11,'(i5,f12.5,5x,a)') 1, 0.01,' ! a_1=0.01'
      end if
* --- Close file ----------------------------------------------------- *
      close(11)
* --- End of segment ------------------------------------------------- *
      return
      end

************************************************************************
* === Write the dynamical system ===================================== *
      subroutine writeDS
      implicit none
* --- Variables for Galerkin system
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
* ---
      real*8  fMij   (NEMP,-NAMP:NEMP)            ! m_ij: mass matric
      real*8  fLijV  (NEMP,-NAMP:NEMP)            ! l_ij^nu: visc. term
      real*8  fLijP  (NEMP,-NAMP:NEMP)            ! l_ij^p: pressure term
      real*8  fQijkC (NEMP,-NAMP:NEMP,-NAMP:NEMP) ! q_ijk^c: conv. term
      real*8  fGil   (NEMP,NVFP)                  ! g_il: force term
* --- Descriptor system
      integer NMode, NModeP
      parameter (NModeP=NEMP+2*NAMP+NVFP)
      real*8  fQijk  (NModeP,0:NModeP,0:NModeP) ! dyn. system
* --- Auxiliary variables
      logical bOK                         ! true if file ex. & non-empty
* --- Further auxiliary files
      integer iMode, jMode, kMode, jjMode, kkMode, iForce
      real*8  fDummy
* --- Begin of segment ----------------------------------------------- *
* --- Initiaize the Galerkin system
      write(*,*) '____________________________________'
      write(*,*) 'Initializing the Galerkin system ...'
      call zeroMatrix (fMij)
      call zeroMatrix (fLijV)
      call zeroTensor (fQijkC)
      call zeroMatrix (fLijP)
      call zeroMatrix2(fGil)
* --- Read the Galerkin system
      write(*,*) '_______________________________'
      write(*,*) 'Reading the Galerkin system ...'
      call readMassMatrix     (fMij,  bOK)
      call readViscousTerm    (fLijV, bOK)
      call readConvectiveTerm (fQijkC,bOK)
      call readPressureTerm   (fLijP, bOK)
      call readForceTerm      (fGil,  bOK)
* --- Initialize the descriptor system
      NMode = NEM + 2*NAM + NVF
      write(*,*) '_____________________________________'
      write(*,*) 'Initializing the dynamical system ...', NMode
      do iMode = 1, NModeP
      do jMode = 0, NModeP
      do kMode = 0, NModeP
        fQijk(iMode,jMode,kMode) = 0.0
      end do
      end do
      end do
* --- Create the dynamical system
      write(*,*) '_________________________________'
      write(*,*) 'Creating the dynamical system ...', NAM, NEM, NMode
* --- Incorporate acceleration term from actuation modes -------------- *
      if (NAM.gt.0) then
        write(*,*) '+ actuation mode: N_A=',NAM
* ----- Add acceleration effect on expansion mode
        do iMode  =    1, NEM
        do jMode  = -NAM, -1
* ....... Index of da_i/dt of act.mode
          jjMode = NEM + NAM + abs(jMode)
* ------- Add acceleraton term
          fQijk(iMode,jjMode,0) = - fMij(iMode,jMode)
        end do
        end do
* ----- Add trivial equations
        do iMode  = NEM+1, NEM+NAM
* ....... Index of da_i/dt of act.mode
          jMode = iMode + NAM 
* ------- Add << da_i/dt = da_i/dt >>
          fQijk(iMode,jMode,0) = 1.0
        end do
      end if
* --- Incorporate force term ----------------------------------------- *
      if (NVF.gt.0) then
        write(*,*) '+ volume force: N_V=',NVF
        do iMode  = 1, NEM
        do iForce = 1, NVF
          jjMode = NEM + 2*NAM + iForce
* ------- Add force term
          fQijk(iMode,jjMode,0) = fQijk(iMode,jjMode,0) 
     &                          + fGil (iMode,iForce)
        end do
        end do
      end if
* --- Incorporate viscous term --------------------------------------- *
      write(*,*) '+ viscous term: j=',-NAM,',...,',NEM
      do iMode = 1, NEM
      do jMode = -NAM, NEM
        jjMode = jMode
* ----- Remove negative indices by putting them on the stack
        if (jMode.lt.0) jjMode = NEM + abs(jMode)
* ----- Add viscous term
        fQijk(iMode,jjMode,0) = fQijk(iMode,jjMode,0)
     &                        + fLijV (iMode,jMode) / fRe
      end do
      end do
* --- Incorporate pressure term --------------------------------------- *
      write(*,*) '+ pressure term: j=',-NAM,',...,',NEM
      do iMode = 1, NEM
      do jMode = -NAM, NEM
        jjMode = jMode
* ----- Remove negative indices by putting them on the stack
        if (jMode.lt.0) jjMode = NEM + abs(jMode)
* ----- Add pressure term
        fQijk(iMode,jjMode,0) = fQijk(iMode,jjMode,0)
     &                        + fLijP (iMode,jMode)
      end do
      end do
* --- Incorporate convective term ------------------------------------ *
      write(*,*) '+ convective term: j,k=',-NAM,',...,',NEM
      do iMode = 1, NEM
      do jMode = -NAM, NEM
      do kMode = -NAM, NEM
        jjMode = jMode
        kkMode = kMode
* ----- Remove negative indices by putting them on the stack
        if (jMode.lt.0) jjMode = NEM + abs(jMode)
        if (kMode.lt.0) kkMode = NEM + abs(kMode)
        fQijk(iMode,jjMode,kkMode) = fQijk (iMode,jjMode,kkMode)
     &                             + fQijkC(iMode,jMode, kMode)
      end do
      end do
      end do
* --- Symmetrize quadratic term
      write(*,*) '_________________________________'
      write(*,*) 'Symmetrize quadratic term ...'
      do iMode = 1, NMode
      do jMode = 0, NMode
      do kMode = 0, jMode-1
        fDummy = fQijk(iMode,jMode,kMode) + fQijk(iMode,kMode,jMode)
        fQijk(iMode,jMode,kMode) = fDummy
        fQijk(iMode,kMode,jMode) = 0.0
      end do
      end do
      end do
* --- Add auxiliary equations as reminder for prescribed modes
      if (NEM.lt.NMode) then
        do iMode = NEM+1,NMode
          fQijk(iMode,0,0) = 1.0E-37
        end do
      end if
* --- Write out the quadratic term for natural dynamics -------------- *
*.................12345678901234
      open(10,file='y3_DS_Qijk.dat',status='UNKNOWN',form='formatted')
      do iMode = 1, NMode
      do jMode = 0, NMode
      do kMode = 0, NMode
        fDummy = fQijk(iMode,jMode,kMode)
        if (fDummy.ne.0) then
* ------- EQUATIONS FOR ACTUATION AMPLITUDES
* ....... equation for actuation mode
          if (iMode.gt.NEM.and.iMode.le.NEM+NAM) then
            write(10,'(3i5,e20.10,2x,a,i5)') 
     &                iMode, jMode, kMode, fDummy,
     &               '! ODE for actuation mode i=',NEM-iMode
* ....... equation for derivative of actuation mode
          else if (iMode.gt.NEM+NAM.and.iMode.le.NEM+2*NAM) then
            write(10,'(3i5,e20.10,2x,a,i5)') 
     &                iMode, jMode, kMode, fDummy,
     &               '! ODE for derivative of actuation mode i=',
     &                NEM+NAM-iMode
* ....... equation for volume force
          else if (iMode.gt.NEM+2*NAM) then
            write(10,'(3i5,e20.10,2x,a,i5)') 
     &                iMode, jMode, kMode, fDummy,
     &               '! ODE for force amplitude l=',
     &                iMode-NEM-2*NAM
* ------- EQUATIONS FOR EXPANSION MODES
* ....... only expansion modes 
          else if (jMode.le.NEM.and.kMode.le.NEM) then
            write(10,'(3i5,e20.10,2x,a,i5)') 
     &                iMode, jMode, kMode, fDummy
* ....... volume force effect
          else if (jMode.gt.NEM+2*NAM) then
            write(10,'(3i5,e20.10,2x,a,i5)') 
     &                iMode, jMode, kMode, fDummy, 
     &               '! effect of volume force l=',jMode-NEM-2*NAM
* ....... volume force effect
          else if (kMode.gt.NEM+2*NAM) then
            write(10,'(3i5,e20.10,2x,a,i5)') 
     &                iMode, jMode, kMode, fDummy, 
     &               '! effect of volume force l=',kMode-NEM-2*NAM
* ....... actuation mode effect
          else if (jMode.gt.NEM.and.jMode.le.NEM+NAM) then
            write(10,'(3i5,e20.10,2x,a,i5)') 
     &                iMode, jMode, kMode, fDummy, 
     &               '! effect of actuation mode i=',
     &                NEM-jMode
* ....... actuation mode effect
          else if (kMode.gt.NEM.and.kMode.le.NEM+NAM) then
            write(10,'(3i5,e20.10,2x,a,i5)') 
     &                iMode, jMode, kMode, fDummy, 
     &               '! effect of actuation mode i=',
     &                NEM-kMode
* ....... actuation mode effect
          else if (jMode.gt.NEM+NAM.and.jMode.le.NEM+2*NAM) then
            write(10,'(3i5,e20.10,2x,a,i5)') 
     &                iMode, jMode, kMode, fDummy, 
     &               '! effect of derivative of actuation mode i=',
     &                NEM+NAM-jMode
* ....... actuation mode effect
          else if (kMode.gt.NEM+NAM.and.kMode.le.NEM+2*NAM) then
            write(10,'(3i5,e20.10,2x,a,i5)') 
     &                iMode, jMode, kMode, fDummy, 
     &               '! effect of acc. of actuation mode i=',
     &                NEM+NAM-kMode
* ....... else
            write(10,'(a)') 'ERROR IN CASES'
          end if
        end if
      end do
      end do
      end do
      close(10)
* --- End of segment ------------------------------------------------- *
      return
      end
