************************************************************************
*****                                                              *****
***** @program name: a1_initGM                                     *****
*****                                                              *****
***** @task: generate xv.include & y-files for POD GM              *****
*****                                                              *****
***** @input:  NFlows & NEM from user                              *****
***** @output: xv.include                                          *****
*****          y[1,2]_GM.dat, y[1,2]_Flows.dat, y[1,2]_Modes.dat   *****
*****          z1_Times.dat (if it does not exist)                 *****
*****          yAMC.sh                                             *****
*****                                                              *****
************************************************************************
*****                                                              *****
***** @author:  Bernd Noack                                        *****
*****                                                              *****
***** @see:     Noack & Cordier 2011 CEAT report                   *****
*****                                                              *****
***** @revised: 2011-02-27 Poitiers                                *****
***** @created: 2011-02-27 Poitiers                                *****
*****                                                              *****
************************************************************************
*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012
      program a1initGM
      implicit none

* --- Variables for flow data
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...

***** Begin of main segment ********************************************
* --- User info ------------------------------------------------------ *
      write(*,*) '****************************************'
      write(*,*) '        Generation of xAMC files'
      write(*,*) '    for standard POD Galerkin model'
      write(*,*) '------ Bernd R. Noack, 2011-02-27 ------'
      write(*,*) '****************************************'
      write(*,*)
      write(*,*) '# of snapshots:'
      read (*,*) NFlows
      write(*,*) '# of modes:'
      read (*,*) NEM
      write(*,*) 'Reynolds number:'
      read (*,*) fRe
* --- Initialize GM parameters --------------------------------------- *
      NKL = NEM
      NVF  = 0          ! # of volume forces
      NAM  = 0          ! # of actuation modes

* --- Write job file << y?_GM.dat >> and mass matrix << z2_M....dat -- *
      call writeY1GM    ! new file for current xAMC
      call writeY2GM    ! new file for current xAMC

* --- Write snapshots ------------------------------------------------ *
C XXX call writeZ1Times ! for m -> t^m
      call system('cp z0_Times.dat z1_Times.dat')
      call writeY1Flows ! for POD
      call writeY1Modes ! for expansion
      call writeXV      ! xv.include
      call writeYAMC    ! yAMC.sh

* --- Show results --------------------------------------------------- *
      write(*,*) '**************************************************'
      write(*,*) 'Listing of << y1_GM.dat >> ...'
      call system('head -5 y1_GM.dat')
      write(*,*) '...'
      call system('tail -5 y1_GM.dat')
      call pressRETURN

      write(*,*) '=================================================='
      write(*,*) 'Listing of << y2_GM.dat >> ...'
      call system('head -5 y2_GM.dat')
      write(*,*) '...'
      call system('tail -5 y2_GM.dat')
      call pressRETURN

C XXX write(*,*) '=================================================='
C XXX write(*,*) 'Listing of << z1_Times.dat >> ...'
C XXX call system('head -5 z1_Times.dat')
C XXX write(*,*) '...'
C XXX call system('tail -5 z1_Times.dat')
C XXX call pressRETURN

      write(*,*) '=================================================='
      write(*,*) 'Listing of << y1_Flows.dat >> ...'
      call system('head -5 y1_Flows.dat')
      write(*,*) '...'
      call system('tail -5 y1_Flows.dat')
      call pressRETURN

      write(*,*) '=================================================='
      write(*,*) 'Listing of << y1_Modes.dat >> ...'
      call system('head -1 y1_Modes.dat')
      write(*,*) '...'
      call system('tail -1 y1_Modes.dat')
      call pressRETURN

      write(*,*) '=================================================='
      write(*,*) 'Listing of << xv.include >> ...'
      call system('cat xv.include')
      call pressRETURN

      write(*,*) '=================================================='
      write(*,*) 'Listing of << yAMC.sh >> ...'
      call system('cat yAMC.sh')
***** End of main segment **********************************************
      stop
      end

************************************************************************
* === Write job file ***************************************************
      subroutine writeY1GM
      implicit none
* --- Galerkin system
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...

* --- Open file ------------------------------------------------------ *
      open(10,file='y1_GM.dat',form='formatted',status='UNKNOWN')

* --- Write in file -------------------------------------------------- *
      write(10,'(i5,a)') NAM,' [N_A:   number of actuation modes]'
      write(10,'(i5,a)') NEM,' [N:     total number of modes]'
      write(10,'(i5,a)') NKL,' [N_POD: number of POD/DMD/... modes]'
C === write(10,'(a)') ' '
C === write(10,'(2i5,a)') 0, NEM, ' ! identity'
      write(10,'(a)') 'END'
* --- Close file ----------------------------------------------------- *
      close(10)
* --- End of segment ------------------------------------------------- *
      return
      end

************************************************************************
* === Write job file ***************************************************
      subroutine writeY2GM
      implicit none
* --- Galerkin system
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...

* --- Open file ------------------------------------------------------ *
      open(10,file='y2_GM.dat',form='formatted',status='UNKNOWN')

* --- Write in file -------------------------------------------------- *
      write(10,'(i5,a)') nint(fRe),' [Re: Reynolds number]'
      write(10,'(i5,a)') NVF,      ' [N_V: number of volume forces]'
C === write(10,'(a)') ' '
C === write(10,'(a)') ' 1.0    [compute mass matrix]'
C === write(10,'(a)') ' 1.0    [project viscous term]'
C === write(10,'(a)') ' 1.0    [project convective term]'
C === write(10,'(a)') ' 1.0    [project pressure term]'
      write(10,'(a)') 'END'
* --- Close file ----------------------------------------------------- *
      close(10)
* --- End of segment ------------------------------------------------- *
      return
      end

************************************************************************
* === Write times for plotting a_i *************************************
      subroutine writeZ1Times
      implicit none
* --- Galerkin system
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
* --- Variables for file 
      integer iFile
      integer fnLength       ! length of file name
      character*120 fn       ! file name
      logical bExist

* --- Begin of segment ----------------------------------------------- *
      write(*,*) '**************************************************'
        write(*,*) 'Writing << z1_Times.dat >> ...'
      inquire(file='z1_Times.dat',EXIST=bExist)
      if (.not.bExist) then
        open(10,file='z1_Times.dat',form='formatted',status='UNKNOWN')
        do iFile = 1, NFlows
          write(10,'(i5,2x,f10.4)') iFile, 1.0*iFile
        end do
        close(10)
        write(*,*) '<< z1_Times.dat >> created.'
      else
        write(*,*) '<< z1_Times.dat >> exists already.'
      end if
* --- End of segment ------------------------------------------------- *
      return
      end

************************************************************************
* === Write snapshots for POD ******************************************
      subroutine writeY1Flows
      implicit none
* --- Galerkin system
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
* --- Variables for file 
      integer iFile
      integer fnLength       ! length of file name
      character*120 fn       ! file name

* --- Begin of segment ----------------------------------------------- *
      write(*,*) '**************************************************'
      write(*,*) 'Writing << y1_Flows.dat >> ...'
      open(10,file='y1_Flows.dat',form='formatted',status='UNKNOWN')
      do iFile = 1, NFlows
* ----- Specify (canonical) file name
        call readFilenameFlowX(iFile,fn,fnLength)
* ----- Update << y1_Modes.dat >>
        write(10,'(i5,2x,f5.2,2x,a)') iFile, 1.0, fn(1:fnLength)
      end do
      close(10)
* --- End of segment ------------------------------------------------- *
      return
      end

************************************************************************
* === Write modes for expansion ****************************************
      subroutine writeY1Modes
      implicit none
* --- Galerkin system
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
* --- Variables for file 
      integer iFile
      integer fnLength       ! length of file name
      character*120 fn       ! file name

* --- Begin of segment ----------------------------------------------- *
      write(*,*) '**************************************************'
      write(*,*) 'Writing << y1_Modes.dat >> ...'
      open(10,file='y1_Modes.dat',form='formatted',status='UNKNOWN')
      do iFile = NAM, NEM
* ----- Specify (canonical) file name
        call readFilenameModeX(iFile,fn,fnLength)
* ----- Update << y1_Modes.dat >>
        write(10,'(i5,2x,f5.2,2x,a)') iFile, 1.0, fn(1:fnLength)
      end do
      close(10)
* --- End of segment ------------------------------------------------- *
      return
      end

************************************************************************
* === Write variable segment *******************************************
      subroutine writeXV
      implicit none
* --- Galerkin system
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
* --- Variables for file 
      integer iFile
      integer fnLength       ! length of file name
      character*120 fn       ! file name

* --- Begin of segment ----------------------------------------------- *
      write(*,*) '**************************************************'
      write(*,*) 'Writing << xv.include >> ...'
      open(10,file='xv.include',form='formatted',status='UNKNOWN')
      write(10,'(a)') '***** created by a1_initGM ****'
      write(10,'(6x,a)') 'integer NFlows, NFlowsP'
      write(10,'(6x,a,i5,a)') 'parameter (NFlowsP=',NFlows,')'
      write(10,'(6x,a,i5,a)') 'common / cbFlows / NFlows'
      write(10,'(6x,a)') 'integer NAM,NAMP,NEM,NEMP,NKL,NKLP'
      write(10,'(6x,3(a,i5),a)') 
     &         'parameter (NAMP=',NAM,',NEMP=',NEM,',NKLP=NEMP)'
      write(10,'(6x,a)') 'common / cbGA / NEM, NKL, NAM'
      write(10,'(6x,a)') 'real*8  fRe'
      write(10,'(6x,a)') 'integer NVF,NVFP'
      write(10,'(6x,a,i5,a)') 'parameter (NVFP=',NVF,')'
      write(10,'(6x,a)') 'common / cbGP / fRe, NVF'
      close(10)
* --- End of segment ------------------------------------------------- *
      return
      end

************************************************************************
* === Write job file ***************************************************
      subroutine writeYAMC
      implicit none

* --- Begin of segment ----------------------------------------------- *
      write(*,*) '**************************************************'
      write(*,*) 'Writing << yAMC.sh >> ...'
      open(10,file='yAMC.sh',form='formatted',status='UNKNOWN')
      write(10,'(a)') '#***** created with a1_initGM.f *****'
      write(10,'(a)') 'source x_defineLocalVariables.sh'
      write(10,'(a)') '# ./INIT2u.sh # grid not defined yet'
      write(10,'(a)') 'mkdir z1_Data || echo "z1_Data exists!"'
      write(10,'(a)') '#----- begin of modelling -----'
      write(10,'(a)') './x1_CorrelationMatrix.sh'
      write(10,'(a)') './x1_eigenproblemPOD.sh'
      write(10,'(a)') './x1_POD.sh'
      write(10,'(a)') './x2_GalerkinProjection.sh'
      write(10,'(a)') './x3_makeDS.sh'
      write(10,'(a)') './x3_solveDS.sh'
      write(10,'(a)') './x3_plotAmplitudes.sh'
      write(10,'(a)') '# Plotting the grid:  ./x0_plotGridXX.sh'
      write(10,'(a)') '# Plotting the flows: ./x0_plotFlowsXX.sh'
      write(10,'(a)') '# Consider:           ./x1_plotSpectrumPOD.sh'
      write(10,'(a)') '# Plotting a_i(t^m):  ./x1_plotAmplitudes.sh'
      write(10,'(a)') '# Plotting the DS:    ./x3_plotDS.sh'
      write(10,'(a)') '# Consider: ./x3_plotPhasePortrait.sh'
      close(10) 
      call system('chmod u+x yAMC.sh')
* --- End of segment ------------------------------------------------- *
      return
      end
