************************************************************************
*****                                                              *****
***** @program name: aToolsInitGrid                                *****
*****                                                              *****
***** @task: generate xv.include & y-files for POD GM              *****
*****                                                              *****
***** @input:  NX, NY, ... NXSub, ... from user                    *****
***** @output: xv3X.include                                        *****
*****          y1_Domains.dat                                      *****
*****                                                              *****
************************************************************************
*****                                                              *****
***** @author:  Bernd Noack                                        *****
*****                                                              *****
***** @see:     Noack & Cordier 2012 CEAT report                   *****
*****                                                              *****
***** @revised: 2012-01-25 Poitiers                                *****
***** @created: 2011-02-27 Poitiers                                *****
*****                                                              *****
************************************************************************
*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012
      program aToolsInitGrid
      implicit none
      integer iOption
* --- Variables for flow data
      include 'aToolsInitGrid.include' 

***** Begin of main segment ********************************************
* --- User info ------------------------------------------------------ *
      write(*,*) '****************************************'
      write(*,*) '        Generation of xAMC files'
      write(*,*) '      for grid-dependent segments'
      write(*,*) '------ Bernd R. Noack, 2011-02-27 ------'
      write(*,*) '****************************************'
      write(*,*)
* --- InitGridialize variables of original grid
      NVarGrid = 1
      NVarFlow = 1
      NX       = 1
      NY       = 1
      NZ       = 1
      NMesh    = 1 
      NMesh2   = 1 
* --- user info
      write(*,*) 'Type of grid:'
      write(*,*) '< 20 > ... 2-D unstructured'   
      write(*,*) '< 21 > ... 2-D Cartesian'   
      write(*,*) '< 30 > ... 3-D unstructured'   
      write(*,*) '< 31 > ... 3-D Cartesian'
      read (*,*) iOption
      if (iOption.eq.20) then
* ----- initialize << z0.dat >> ----------------------------------------
        open(10,file='z0.dat',status='UNKNOWN',form='formatted')
        write(10,'(a)') '2u'
        close(10)
* ----- user input -----------------------------------------------------
        NVarGrid = 2
        write(*,*) 'Number of nodes:'
        read (*,*) NX
        write(*,*) 'Number of triangles:'
        read (*,*) NMesh
        NMesh2 = 3  ! interpolation with triangle (3 edges)
        call system('./aTools2u.sh')
      else if (iOption.eq.30) then
* ----- initialize << z0.dat >> ----------------------------------------
        open(10,file='z0.dat',status='UNKNOWN',form='formatted')
        write(10,'(a)') '3u'
        close(10)
* ----- user input -----------------------------------------------------
        NVarGrid = 3
        write(*,*) 'Number of nodes:'
        read (*,*) NX
        write(*,*) 'Number of tetraeders:'
        read(*,*) NMesh
        NMesh2 = 4  ! interpolation with tetraeder (4 edges)
        call system('./aTools3u.sh')
      else if (iOption.eq.21) then
* ----- initialize << z0.dat >> ----------------------------------------
        open(10,file='z0.dat',status='UNKNOWN',form='formatted')
        write(10,'(a)') '2c'
        close(10)
* ----- user input -----------------------------------------------------
        NVarGrid = 2
        write(*,*) 'NX, NY:'
        read(*,*) NX, NY
* ----- Note the 2c grid is processed by 3c routines
        call system('./aTools2c.sh')
      else if (iOption.eq.31) then
* ----- initialize << z0.dat >> ----------------------------------------
        open(10,file='z0.dat',status='UNKNOWN',form='formatted')
        write(10,'(a)') '3c'
        close(10)
* ----- user input -----------------------------------------------------
        NVarGrid = 3
        write(*,*) 'NX, NY, NZ:'
        read(*,*) NX, NY,NZ
        call system('./aTools3c.sh')
      end if
      NVarFlow    = NVarGrid
* --- subdomain = original domain (most simple default)
      NVarGridSub = NVarGrid  
      NVarFlowSub = NVarFlow   
      NXSub       = NX  
      NYSub       = NY  
      NZSub       = NZ
      NMeshSub    = NMesh
      NMesh2Sub   = NMesh2
      if (iOption.eq.21) then ! 3D computation of 2D data !
        NVarGridSub = 3 
        NVarFlowSub = max(3,NVarFlowSub)
        NZSub       = 2
      end if
* --- Writing the files ---------------------------------------------- *
      call writeY0Grid
      call writeY1Domains
      call writeXV3X
* --- Show results --------------------------------------------------- *
      write(*,*) '**************************************************'
      write(*,*) 'Listing of << y0_Grid.dat >> ...'
      call system('cat y0_Grid.dat')
      call pressRETURN

      write(*,*) '**************************************************'
      write(*,*) 'Listing of << y1_Domains.dat >> ...'
      call system('cat y1_Domains.dat')
      call pressRETURN

      write(*,*) '=================================================='
      write(*,*) 'Listing of << xv3XC.include >> ...'
      call system('cat xv3X.include')
      call pressRETURN

***** End of main segment **********************************************
      stop
      end

************************************************************************
* === Write link to grid ***********************************************
      subroutine writeY0Grid
      implicit none
* --- Variables for file 
      integer fnLength       ! length of file name
      character*120 fn       ! file name
* --- Begin of segment ----------------------------------------------- *
      write(*,*) '**************************************************'
      write(*,*) 'Writing << y0_Grid.dat >> ...'

      open(10,file='y0_Grid.dat',status='UNKNOWN',form='formatted')
      write(10,'(a)') 'z0_Data/Grid.dat' 
      close(10)
* --- End of segment ------------------------------------------------- *
      return
      end

************************************************************************
* === Write list of domains ********************************************
      subroutine writeY1Domains
      implicit none
* --- Variables for file 
      integer fnLength       ! length of file name
      character*120 fn       ! file name
* --- Begin of segment ----------------------------------------------- *
      write(*,*) '**************************************************'
      write(*,*) 'Writing << y1_Domains.dat >> ...'

      open(10,file='y1_Domains.dat',status='UNKNOWN',form='formatted')
      write(10,'(i5,2x,a)') 1, 'z0_Data/Grid.dat' 
      close(10)
* --- End of segment ------------------------------------------------- *
      return
      end

************************************************************************
* === Write variable segment *******************************************
      subroutine writeXV3X
      implicit none
* --- Galerkin system
      include 'aToolsInitGrid.include' ! NX, ...

* --- Begin of segment ----------------------------------------------- *
      write(*,*) '**************************************************'
      write(*,*) 'Writing << xv3X.include >> ...'
      open(10,file='xv3X.include',form='formatted',status='UNKNOWN')
      write(10,'(a)') '***** created by aToolsInitGrid ****'
*                       1234567890123456789012345678901234567890
      write(10,'(2a)') '* === dimensions of original grid ======',
     &                 '============================== *'
      write(10,'(6x,a)')      'integer NVarGrid'
      write(10,'(6x,a)')      'integer NVarFlow'
      write(10,'(6x,a)')      'integer NX, NXP'
      write(10,'(6x,a)')      'integer NY, NYP'
      write(10,'(6x,a)')      'integer NZ, NZP'
      write(10,'(6x,a,i5,a)') 'parameter (NVarGrid=',NVarGrid,')'
      write(10,'(6x,a,i5,a)') 'parameter (NVarFlow=',NVarFlow,')'
      write(10,'(6x,a,i5,a)') 'parameter (NXP=',NX,')'
      write(10,'(6x,a,i5,a)') 'parameter (NYP=',NY,')'
      write(10,'(6x,a,i5,a)') 'parameter (NZP=',NZ,')'
      write(10,'(6x,a)') 'common / cbIndex3X / NX, NY, NZ'
      write(10,'(6x,a)')      'integer NMesh, NMeshP'
      write(10,'(6x,a)')      'integer NMesh2, NMesh2P'
      write(10,'(6x,a,i5,a)') 'parameter (NMeshP=',NMesh,')'
      write(10,'(6x,a,i5,a)') 'parameter (NMesh2P=',NMesh2,')'
      write(10,'(6x,a)')      'integer indexMesh(NMesh2P,NMeshP)'
      write(10,'(6x,a)')
     &         'common / cbMesh3X / NMesh, NMesh2, indexMesh'
      write(10,'(2a)')  '* === dimensions of subdomain grid =====',
     &                  '============================== *'
      write(10,'(6x,a)')      'integer NVarGridSub'
      write(10,'(6x,a)')      'integer NVarFlowSub'
      write(10,'(6x,a)')      'integer NXSub, NXSubP'
      write(10,'(6x,a)')      'integer NYSub, NYSubP'
      write(10,'(6x,a)')      'integer NZSub, NZSubP'
      write(10,'(6x,a,i5,a)') 'parameter (NVarGridSub=',NVarGridSub,')'
      write(10,'(6x,a,i5,a)') 'parameter (NVarFlowSub=',NVarFlowSub,')'
      write(10,'(6x,a,i5,a)') 'parameter (NXSubP=',NXSub,')'
      write(10,'(6x,a,i5,a)') 'parameter (NYSubP=',NYSub,')'
      write(10,'(6x,a,i5,a)') 'parameter (NZSubP=',NZSub,')'
      write(10,'(6x,a)') 'common / cbIndex3XSub / NXSub, NYSub, NZSub'
      write(10,'(6x,a)')      'integer NMeshSub, NMeshSubP'

      write(10,'(6x,a)')      'integer NMesh2Sub, NMesh2SubP'
      write(10,'(6x,a,i5,a)') 'parameter (NMeshSubP=',NMeshSub,')'
      write(10,'(6x,a,i5,a)') 
     &                      'parameter (NMesh2SubP=',NMesh2Sub,')'
      write(10,'(6x,a)')    'integer indexMeshSub(NMesh2SubP,NMeshSubP)'
      write(10,'(6x,a)')
     &       'common / cbMesh3XSub / NMeshSub, NMesh2Sub, indexMeshSub'
      close(10)
* --- End of segment ------------------------------------------------- *
      return
      end
