************************************************************************
* --- 2D Graphics (Subroutines for PostScript Output) ---------------- *
* --- created by Bernd R. Noack, 28 Dec 1992 ------------------------- *
* --- revised, 2004-12-28 (real*4 -> real*8) ------------------------- *
* --- revised, 2006-05-26 (InitNamedFig, Init->init, Plot->plot) ----- *
************************************************************************
* --- call specBB (xPSBBMin,xPSBBMax,yPSBBMin,yPSBBMax)
* --- call initBB (UnitFile)
* --- call initFig(UnitPS)
* --- call initMFig(UnitPS, FileNo)
* --- call initDefs(UnitPS)
* --- call TermFig(UnitPS)
* --- call specDR(XPSMin,XPSMax,YPSMin,YPSMax)
* --- call Rescale(Fraction)
* --- call plotFrame(UnitPS)
* --- call plotSFig(UnitPS,NRow,NCol,AspectRatio,
* ---&              RowCaption,ColCaption,
* ---&              XCaption,XMin,XMax,XLMin,XLMax,XLDiff,XCPos,
* ---&              YCaption,YMin,YMax,YLMin,YLMax,YLDiff,YCPos)
* --- call initSFig(IRow,ICol,NRow,NCol,XMin,XMax,YMin,YMax)
* --- call ClipDR(UnitPS)
* --- call NoClip(UnitPS)
* --- call specXAxis(XMin,XMax)
* --- call specYAxis(YMin,YMax)
* --- call plotXAxis(UnitPS,XCaption,XMin,XMax,
* ---&                               XLMin,XLMax,XLDiff,XCPos)
* --- call plotYAxis(UnitPS,YCaption,YMin,YMax,
* ---&                               YLMin,YLMax,YLDiff,YCPos)
* --- call plotCapt(UnitPS,Strg,RefStrg,XRel,YRel,Angle,Pos)
* --- call plotStrg(UnitPS,Strg,XPS,YPS,Angle,Pos)
* --- call plotCurve(UnitPS,XVal,YVal,NVal)
* --- call plotSymbol(UnitPS,XVal,YVal,NVal,ISymbol)
* --- call plotCircle(UnitPS,X,Y,R,Fill)
* --- call plotRectangle(UnitPS,XC,YC,DX,DY,Fill)
* --- call plotPolygon(UnitPS,XVal,YVal,NVal,Fill)
* --- call plotArrow(UnitPS,XTail,YTail,XHead,YHead)
* --- call plotCL(UnitPS,XVal,NX,NXP,YVal,NY,NYP,FVal,FCL,NCL)
* --- call plotCLSquare(UnitPS,X,Y,F,FCL)
* --- call plotCLTriangle(UnitPS,X,Y,F,FCL)


************************************************************************
* --- Specify the bounding box ------...------------------------------ *
      subroutine specBB (xPSBBMin,xPSBBMax,yPSBBMin,yPSBBMax)
      implicit none
      real*8  xPSBBMin,xPSBBMax,yPSBBMin,yPSBBMax
      include 'gv2d_PostScript.include'
* --- 
      XPSBB0 = xPSBBMin
      XPSBB1 = xPSBBMax
      YPSBB0 = yPSBBMin
      YPSBB1 = yPSBBMax
* --- end of specBB
      return
      end

************************************************************************
* --- initialize the bounding box ------------------------------------ *
      subroutine initBB(UnitFile)
      implicit none
      integer      UnitFile, TXPS, TYPS, DXPS, DYPS, SizeFont
      character*80 Line
      include 'gv2d_PostScript.include'
* --- default = maxium size
      if (UnitFile.le.0) then
         XPSBB0 =   0
         XPSBB1 = 550
         YPSBB0 =   0
         YPSBB1 = 700
* --- otherwise, the bounding box is estimated
      else
* ------ Open the file with a macro-header
         open(unit   = UnitFile,
     &        file   = 'f.dat',
     &        status = 'unknown',
     &        access = 'sequential',
     &        form   = 'formatted')
         read(UnitFile,'(a)') Line ! comment
         read(UnitFile,*)  TXPS, TYPS, DXPS, DYPS
* ------ determine data region
         XPSBB0 = 0.1*TXPS
         YPSBB0 = 0.1*TYPS
         XPSBB1 = XPSBB0 + 0.1*DXPS 
         YPSBB1 = YPSBB0 + 0.1*DYPS
* ------ add margins for caption
         SizeFont = 0.075*max(0.1*DXPS,0.1*DYPS)
         XPSBB0 = XPSBB0 - 4.0*SizeFont 
         YPSBB0 = YPSBB0 - 1.7*SizeFont 
         XPSBB1 = XPSBB1 + 0.3*SizeFont 
         YPSBB1 = YPSBB1 + 0.5*SizeFont 
* ------ close the file
         close(UnitFile)
      end if
* --- end of initBB
      return
      end

************************************************************************
* --- initialize the graphics ---------------------------------------- *
      subroutine initFig(UnitPS)
      implicit none
      include 'gv2d_PostScript.include'
* --- Open the file with a macro-header
      open(unit   = UnitPS,
     &     file   = 'f.ps',
     &     status = 'unknown',
     &     access = 'sequential',
     &     form   = 'formatted')
      call initDefs(UnitPS)
* --- end of initFig
      return
      end

************************************************************************
* --- initialize the graphics (with PostScript file) ----------------- *
      subroutine initNamedFig(UnitPS,fn,fnLength)
      implicit none
      character*80 fn        ! file name
      integer      fnLength  ! file name length
      include 'gv2d_PostScript.include'
* --- Open the file with a macro-header
      open(unit   = UnitPS,
     &     file   = fn(1:fnLength),
     &     status = 'unknown',
     &     access = 'sequential',
     &     form   = 'formatted')
      call initDefs(UnitPS)
* --- end of initFig
      return
      end

************************************************************************
* --- initialize the graphics ---------------------------------------- *
      subroutine initMFig(UnitPS, FileNo)
      implicit none
      include 'gv2d_PostScript.include'
      integer FileNo, Counter, NumChar
      character*7 FileName
* --- Filename
      FileName = 'f000.ps'
      do Counter = 1,3
         NumChar = mod(FileNo,10**(Counter)) / 10**(Counter-1)
     &             + ichar('0')
         FileName(5-Counter:5-Counter) = char(NumChar)
      end do
* --- Open the file with a macro-header
      open(unit   = UnitPS,
     &     file   = FileName,
     &     status = 'unknown',
     &     access = 'sequential',
     &     form   = 'formatted')
      call initDefs(UnitPS)
* --- end of initMFig
      return
      end

* === initialize the PostScript definitions  ========================= *
      subroutine initDefs(UnitPS)
      implicit none
      include 'gv2d_PostScript.include'
* --- Define the default parameters (may be overwritten by axis ...)
      call specDR(2000.0D0,5000.0D0,800.0D0,2900.0D0)
* --- Definitions in the header
C     write(UnitPS,'(a)') '%!PostScript-Adobe'  ! for HFI
      write(UnitPS,'(a)') '%!PS-Adobe'           ! for EPFL
      write(UnitPS,'(a)') '%%Title: Graphics of Bernd R. Noack'
      if (max(XPSBB0,XPSBB1,YPSBB0,YPSBB1).gt.1.0) then ! initialized
        write(UnitPS,'(a,4i5)') '%%BoundingBox: ',
     &              nint(XPSBB0), nint(YPSBB0),
     &              nint(XPSBB1), nint(YPSBB1)
      else
        write(UnitPS,'(a,4i5)') '%%BoundingBox: 0 0 550 700' ! defaul
      end if
      write(UnitPS,'(1h 3a)')! 'initgraphics erasepage ',
     &             ' 0 0 translate 0 rotate 0.10 0.10 scale '
      write(UnitPS,'(1h 3a)') ' 0 0 translate ',  ! typically 84 52
     &             '2 setlinewidth 0 setlinecap ',
     &             '1 setlinejoin /#copies 1 def '
      write(UnitPS,'(1h  a)') '%*** Abbreviations for PS commands ***'
      write(UnitPS,'(1h 2a)') '/r  {       rotate} def     ',
     &                        '/t  {    translate} def     '
      write(UnitPS,'(1h 2a)') '/lw { setlinewidth} def     ',
     &                        '/n  {      newpath} def     '
      write(UnitPS,'(1h 2a)') '/m  {       moveto} def     ',
     &                        '/rm {      rmoveto} def     '
      write(UnitPS,'(1h 2a)') '/l  {       lineto} def     ',
     &                        '/rl {      rlineto} def     '
      write(UnitPS,'(1h 2a)') '/c  {    closepath} def     ',
     &                        '/s  {       stroke} def     '
      write(UnitPS,'(1h 2a)') '/f  {         fill} def     ',
     &                        '/g  {      setgray} def     '
      write(UnitPS,'(1h 2a)') '/gs {        gsave} def     ',
     &                        '/gr {     grestore} def     '
      write(UnitPS,'(1h 2a)') '/sf {      setfont} def     ',
     &                        '/mf {     makefont} def     '
      write(UnitPS,'(1h 2a)') '/ff {     findfont} def     ',
     &                        '/sc {    scalefont} def     '
      write(UnitPS,'(1h 2a)') '/sw {  stringwidth} def     ',
     &                        '/gr {     grestore} def     '
      write(UnitPS,'(1h 2a)') '/gs {        gsave} def     ',
     &                        '/sh {         show} def     '
      write(UnitPS,'(1h 2a)') '/p  {     showpage} def     '
      write(UnitPS,'(1h a)') '%*** PS-Macros ***'
      write(UnitPS,'(1h a)') '/NString 10 string def'
      write(UnitPS,'(1h a)') 
     &             '/PosStrgC { sw exch 2 div neg exch rm } def '
      write(UnitPS,'(1h a)') 
     &             '/PosStrgR { sw exch neg exch rm }       def '
      write(UnitPS,'(1h a)') 
     &             '/PosNoC   { NString cvs PosStrgC }      def '
      write(UnitPS,'(1h a)') 
     &             '/PosNoR   { NString cvs PosStrgR }      def '
      write(UnitPS,'(1h a)') 
     &             '/PrintNo  { NString cvs show }          def '
* --- end of initDefs
      return
      end

************************************************************************
* --- Terminate the graphics ----------------------------------------- *
      subroutine TermFig(UnitPS)
      implicit none
      integer UnitPS
* ---
      write(UnitPS,'(1h  a)') ' p'
      close(UnitPS)
* --- end of TermFig
      return
      end

************************************************************************
* --- Set default PostScript parameters ------------------------------ *
      subroutine specDR(XPSMin,XPSMax,YPSMin,YPSMax)
      implicit none
      include 'gv2d_PostScript.include'
      real*8 XPSMin, XPSMax, YPSMin, YPSMax, XPSDiff, YPSDiff, Fraction
* --- Define the default parameters
      XPS0       = XPSMin
      XPS1       = XPSMax
      YPS0       = YPSMin
      YPS1       = YPSMax
      XPSDiff    = XPSMax - XPSMin
      YPSDiff    = YPSMax - YPSMin
      SizeLabel  = 1.00* 0.075*max(XPSDiff,YPSDiff) 
      SizeTic    = 1.00* 0.025*max(XPSDiff,YPSDiff)
      SizeSymbol = 1.00* 0.025*max(XPSDiff,YPSDiff)
      SizeArrowL = 1.00* 0.025*max(XPSDiff,YPSDiff)
      SizeArrowW = 1.00* 0.80 *SizeArrowL
      XPSLabel   = YPS0 - 1.1*SizeLabel ! right bound of x-label
      YPSLabel   = XPS0 - 1.3*SizeLabel ! upper bound of y-label  [1.5]
      XPSCaption = XPSLabel - 1.2*SizeLabel
      YPSCaption = YPSLabel - 3.0*SizeLabel
      WidthFrame = 0.0100*max(XPSDiff,YPSDiff)
      WidthTic   = 0.0070*max(XPSDiff,YPSDiff)
      WidthCurve = 0.0070*max(XPSDiff,YPSDiff)
      WidthSymbol= 0.0035*max(XPSDiff,YPSDiff)
      WidthArrow = 0.0035*max(XPSDiff,YPSDiff)
* ---
      call Rescale(1.0D0) ! <=== CRITICAL PARAMETER
* --- end of specDR
      return
      end

************************************************************************
* --- Rescale size & width & dependent parameters -------------------- *
      subroutine Rescale(Fraction)
      implicit none
      include 'gv2d_PostScript.include'
      real*8  Fraction
* ---
      SizeLabel  =  Fraction*SizeLabel
      SizeTic    =  Fraction*SizeTic
      SizeSymbol =  Fraction*SizeSymbol
      SizeArrowL =  Fraction*SizeArrowL
      SizeArrowW =  Fraction*SizeArrowW
      XPSLabel   =  YPS0     - 1.1*SizeLabel
      YPSLabel   =  XPS0     - 1.3*SizeLabel
      XPSCaption =  XPSLabel - 1.2*SizeLabel
      YPSCaption =  YPSLabel - 3.0*SizeLabel
      WidthFrame =  Fraction*WidthFrame
      WidthTic   =  Fraction*WidthTic
      WidthCurve =  Fraction*WidthCurve
      WidthSymbol=  Fraction*WidthSymbol
      WidthArrow =  Fraction*WidthArrow
* --- end of Rescale
      return
      end

************************************************************************
* --- plot frame around data region ---------------------------------- *
      subroutine plotFrame(UnitPS)
      implicit none
      include 'gv2d_PostScript.include'
* ---
      write(UnitPS,'(1h  a)') '%*** FRAME ***'
      write(UnitPS,'(1h i3,a,4(2i6,a),a)')
     &              nint(WidthFrame),' lw  n ',
     &              nint(XPS0),nint(YPS0),' m ',
     &              nint(XPS1),nint(YPS0),' l ',
     &              nint(XPS1),nint(YPS1),' l ',
     &              nint(XPS0),nint(YPS1),' l  c  s'
* --- end of plotFrame
      return
      end


************************************************************************
* --- initialize subfigures with axes -------------------------------- *
      subroutine plotSFig(UnitPS,NRow,NCol,AspectRatio,
     &                    RowCaption,ColCaption,
     &                    XCaption,XMin,XMax,XLMin,XLMax,XLDiff,XCPos,
     &                    YCaption,YMin,YMax,YLMin,YLMax,YLDiff,YCPos)
      implicit none
      include 'gv2d_PostScript.include'    ! segment with PostScript   variables
      include 'gv2d_Cartesian.include'    ! segment with world coord. variables
      real*8  XL0, XL1, YL0, YL1, XCP, YCP
      real*8  XPSMin, XPSMax, YPSMin, YPSMax, AspectRatio
      integer IRow, ICol, NRow, NCol, IRef, NRef
      character*80 RowCaption(NRow), ColCaption(NCol), RefCapt
* --- initialize maximal data region
      DXPSDR = 4000          ! horizonatal lenght of whole data region
      DYPSDR = 5600          ! vertical    length of whole data region
      XPSDR0 = 1600          ! left margin of data region 
      XPSDR1 = XPSDR0 + DXPSDR
      YPSDR0 = 1200          ! bottom margin of data region
      YPSDR1 = YPSDR0 + DYPSDR
* --- Compute data region of subplot in PostScript coordinates
      DXPSSF = (XPSDR1-XPSDR0) / NCol    ! maximum horizontal length of subfig.
      DYPSSF = (YPSDR1-YPSDR0) / NRow    ! maximum vertical   length of subfig.
      if (DYPSSF.gt.AspectRatio*DXPSSF) then
         DYPSSF = AspectRatio*DXPSSF       ! Reduce vertical length of subfig.
      else
         DXPSSF = DYPSSF/AspectRatio       ! Reduce horizontal length of subfig.
      end if
* --- Determine reference caption
      NRef=0
      do IRow = 1, NRow
         if (index(RowCaption(IRow),'}').gt.NRef) then
            NRef = index(RowCaption(IRow),'}')
            IRef = IRow
         end if
      end do
      RefCapt = RowCaption(IRef)
      RefCapt(NRef:NRef+3) = 'XXX}'
* --- Loop over all subfigures
      do IRow = 1, NRow
      do ICol = 1, NCol
* ------ Size of data region in PS coordinates
         XPSMin = 0.5*(XPSDR0+XPSDR1)
     &          + (ICol-1.0-0.5*NCol)*DXPSSF ! Column captions from left to right
         XPSMax = XPSMin + DXPSSF
         YPSMin = 0.5*(YPSDR0+YPSDR1) 
     &          + (-IRow+0.5*NRow)*DYPSSF     ! Row captions from top to bottom
         YPSMax = YPSMin + DYPSSF
* ------ Postition of labels and captions
         XL0 = XLMin
         XL1 = XLMax
         XCP = XCPos
         YL0 = YLMin
         YL1 = YLMax
         YCP = YCPos
         if (IRow.ne.NRow.and.
     &       abs(XMin-XLMin).lt.0.1*(XMax-XMin)) 
     &       YL0 = YLMin + YLDiff
         if (ICol.gt.1.and.
     &       abs(YMin-YLMin).lt.0.1*(YMax-YMin))   
     &       XL0 = XLMin + XLDiff
* ------ plot axes -----------------------------------------------------
         call specDR(XPSMin,XPSMax,YPSMin,YPSMax)
* ------ Draw row- and column-captions
         SizeLabel = (3./2.)*SizeLabel     ! Increase font size for captions
         if (ICol.eq.1) then
            write(UnitPS,'(1h a)') '%*** row caption ***'
            call plotCapt(UnitPS,RowCaption(IRow),RefCapt,
     &                           0.00D0,0.999D0,0,1)
         end if
         if (IRow.eq.1) then
            write(UnitPS,'(1h a)') '%*** column caption ***'
            call plotCapt(UnitPS,ColCaption(ICol),ColCaption(ICol),
     &                           0.5D0,1.0D0,0,0)
         end if
         SizeLabel = (2./3.)*SizeLabel     ! Restore old font size 
* ------ plot X- & Y-axis
         call plotFrame(UnitPS)
         if (IRow.eq.NRow) then ! complete X-axis
          call plotXAxis(UnitPS,XCaption,XMin,XMax,XL0,XL1,XLDiff,XCP)
         else                  ! only tics at X-axis, no labels and caption
          call plotXAxis(UnitPS,XCaption,XMin,XMax,
     &                                   XL0,XL1,XLDiff,-1.0D0)
         end if
         if (ICol.eq.1) then   ! complete Y-axis
          call plotYAxis(UnitPS,YCaption,YMin,YMax,YL0,YL1,YLDiff,YCP)
         else                  ! only tics at Y-axis, no labels and caption
          call plotYAxis(UnitPS,YCaption,YMin,YMax,
     &                                   YL0,YL1,YLDiff,-1.0D0)
         end if
* --- end of loop over subfigures
      end do
      end do
* --- end of plotSFig
      return
      end

************************************************************************
* --- initialize subfigures ------------------------------------------ *
      subroutine initSFig(IRow,ICol,NRow,NCol,XMin,XMax,YMin,YMax)
      implicit none
      include 'gv2d_PostScript.include'    ! segment with PostScript   variables
      include 'gv2d_Cartesian.include'    ! segment with world coord. variables
      integer IRow, ICol, NRow, NCol
      real*8  XPSMin, XPSMax, YPSMin, YPSMax, AspectRatio
* ---
      XPSMin = 0.5*(XPSDR0+XPSDR1)
     &       + (ICol-1.0-0.5*NCol)*DXPSSF 
      XPSMax = XPSMin + DXPSSF
      YPSMin = 0.5*(YPSDR0+YPSDR1) 
     &       + (-IRow+0.5*NRow)*DYPSSF
      YPSMax = YPSMin + DYPSSF
      call specDR(XPSMin,XPSMax,YPSMin,YPSMax)
      call specXAxis(XMin,XMax)
      call specYAxis(YMin,YMax)
* --- end of initSFig
      return
      end

************************************************************************
* --- Clip data region ----------------------------------------------- *
      subroutine ClipDR(UnitPS)
      implicit none
      include 'gv2d_PostScript.include'
* ---
      if (UnitPS.gt.0) return
      write(UnitPS,'(1h a)') '%*** CLIP ***'
      write(UnitPS,'(1h a,4(2i6,a),a)')
     &              ' n ',
     &              nint(XPS0),nint(YPS0),' m ',
     &              nint(XPS1),nint(YPS0),' l ',
     &              nint(XPS1),nint(YPS1),' l ',
     &              nint(XPS0),nint(YPS1),' l  c  clip '
* --- end of ClipDR
      return
      end

************************************************************************
* --- Remove clip (set clipping path to DinA4 size) ------------------ *
      subroutine NoClip(UnitPS)
      implicit none
      include 'gv2d_PostScript.include'
* ---
      if (UnitPS.gt.0) return
      write(UnitPS,'(1h  a)') '%*** NO CLIP ***'
      write(UnitPS,'(1h  a,4(2i6,a),a)')
     &              ' n ',
     &              0,                    0,' m ',
     &              nint(5*20.0*72/2.54),0,' l ',
     &              nint(5*20.0*72/2.54),
     &              nint(5*29.0*72/2.54),   ' l ',
     &              0,nint(5*29.0*72/2.54), ' l c clip '
* --- end of NoClip
      return
      end

************************************************************************
* --- Specify X-axis ------------------------------------------------- *
      subroutine specXAxis(XMin,XMax)
      implicit none
      include 'gv2d_PostScript.include'
      include 'gv2d_Cartesian.include'
* ---
      APSX = (XPS1-XPS0)/(XMax-XMin)
      BPSX =  XPS0 - APSX*XMin
* --- end of specXAxis
      return
      end

************************************************************************
* --- specify Y-axis ------------------------------------------------- *
      subroutine specYAxis(YMin,YMax)
      implicit none
      include 'gv2d_PostScript.include'
      include 'gv2d_Cartesian.include'
* ---
      APSY = (YPS1-YPS0)/(YMax-YMin)
      BPSY =  YPS0 - APSY*YMin
* --- end of specYAxis
      return
      end

************************************************************************
* --- plot labels & caption of X-axis -------------------------------- *
      subroutine plotXAxis(UnitPS,XCaption,XMin,XMax,
     &                                     XLMin,XLMax,XLDiff,XCPos)
      implicit none
      include 'gv2d_PostScript.include'
      include 'gv2d_Cartesian.include'
      integer N, I0, J0, JA0, JA1, JB0, JB1
      real*8  X, Y, XPS, YPS
      character*80 Caption
* --- Compute positions of X-labels
      X   = XLMin
      NXL = 0
      do while (X.lt.XLMax+1.e-4)
         XL(NXL+1) = X
         NXL       = NXL + 1
         X         = X + XLDiff
      end do
* --- Compute transformation from user-defined to PostScript coordinates
      APSX = (XPS1-XPS0)/(XMax-XMin)
      BPSX =  XPS0 - APSX*XMin
* --- Compute labels reserved for X-caption
      IXC = 0
      if (NXL.gt.0) IXC = NXL + 1 - nint(XCPos)
      if (abs(XCPos-nint(XCPos)).gt.0.2.or.XCPos.le.0.0) IXC = 0
* --- Write macros for X-axis in PS-file
      write(UnitPS,'(1h  a)') '%*** X-axis ***'
* --- Write tics in PS-file
      write(UnitPS,'(1h  a)') '%--- tics at X-axes ---'
      write(UnitPS,'(1h i2,a)') nint(WidthTic), ' lw '
      do IXL = 1, NXL
         I0  = nint(APSX*XL(IXL)+BPSX)
         JA0 = nint(YPS0)
         JA1 = JA0 + SizeTic
         JB0 = nint(YPS1)
         JB1 = JB0 - SizeTic
         write(UnitPS,'(1h 2(a,2(2i6,a)))')
     &                ' n',I0,JA0,' m ',I0,JA1,' l s   ',
     &                ' n',I0,JB0,' m ',I0,JB1,' l s   '
         X = X + XLDiff
      end do
      if (XCPos.lt.0.0) return
      write(UnitPS,'(1h  a)') '%--- labels at X-axis ---'
      write(UnitPS,'(1h a,i4,a)') 
     &             '/Times-Bold ff ', nint(SizeLabel),' sc  sf '
      do IXL = 1, NXL
         if (IXL.ne.IXC) then
            I0  = nint(XPS0 + (XPS1-XPS0)*(XL(IXL)-XMin)/(XMax-XMin))
            J0  = nint(XPSLabel)
            write(UnitPS,'(1h 2i6,a,2(f7.2,a))') I0, J0, ' m ',
     &                   XL(IXL),' PosNoC ',
     &                   XL(IXL),' PrintNo '
         end if
      end do
* --- Write caption in PS-file
      write(UnitPS,'(1h  a)') '%--- caption at X-axis ---'
      if (XCPos.le.0.0) then
         XPS = 0.5*(XPS0+XPS1)
         YPS = XPSCaption
      else
         XPS = APSX*(XL(NXL)-(XCPos-1)*XLDiff) + BPSX
         YPS = XPSLabel -0.3*SizeLabel ! but caption a bit lower
      end if
      Caption = '                                          '
      Caption = 
     & '{'//XCaption(index(XCaption,'{')+1:index(XCaption,'}')-1)//'}'
      call plotStrg(UnitPS,Caption,XPS,YPS,0,0)
* --- end of plotXAxis
      return
      end

************************************************************************
* --- plot labels & caption of Y-axis -------------------------------- *
      subroutine plotYAxis(UnitPS,YCaption,YMin,YMax,
     &                                     YLMin,YLMax,YLDiff,YCPos)
      implicit none
      include 'gv2d_PostScript.include'
      include 'gv2d_Cartesian.include'
      integer N, I0, J0, JA0, JA1, JB0, JB1
      real*8  X, Y, XPS, YPS
      character*80 Caption
* --- Compute positions of Y-labels
      Y   = YLMin
      NYL = 0
      do while (Y.lt.YLMax+1.e-4)
         YL(NYL+1) = Y
         NYL       = NYL + 1
         Y         = Y + YLDiff
      end do
* --- Compute transformation from user-defined to PostScript coordinates
      APSY = (YPS1-YPS0)/(YMax-YMin)
      BPSY = YPS0 - APSY*YMin
* --- Compute labels reserved for Y-captions
      IYC = 0
      if (NYL.gt.0) IYC = NYL + 1 - nint(YCPos)
      if (abs(YCPos-nint(YCPos)).gt.0.2 .or.YCPos.le.0.0) IYC = 0
* --- plot tics in PS-file
      write(UnitPS,'(1h  a)') '%*** Y-axis ***'
      write(UnitPS,'(1h  a)') '%--- tics at Y-axis ---'
      write(UnitPS,'(1h i2,a)') nint(WidthTic), ' lw '
      do IYL = 1, NYL
         I0  = nint(APSY*YL(IYL) + BPSY)
         JA0 = nint(XPS0)
         JA1 = JA0 + SizeTic
         JB0 = nint(XPS1)
         JB1 = JB0 - SizeTic
         write(UnitPS,'(1h 2(a,2(2i6,a)))')
     &                ' n',JA0,I0,' m ',JA1,I0,' l s   ',
     &                ' n',JB0,I0,' m ',JB1,I0,' l s   '
         Y = Y + YLDiff
      end do
      if (YCPos.lt.0.0) return
      write(UnitPS,'(1h a)') '%--- labels at Y-axis ---'
      write(UnitPS,'(1h a,i4,a)') 
     &             '/Times-Bold ff ',nint(SizeLabel),' sc  sf '
      do IYL = 1, NYL
         if (IYL.ne.IYC) then
            J0  = nint(APSY*YL(IYL) + BPSY - 0.35*SizeLabel)
            I0  = nint(XPS0                - 0.35*SizeLabel)
            write(UnitPS,'(1h 2i6,a,2(f7.2,a))') I0, J0, ' m ',
     &                   YL(IYL),' PosNoR ',
     &                   YL(IYL),' PrintNo '
         end if
      end do
* --- Write caption in PS-file
      write(UnitPS,'(1h  a)') '%--- caption at Y-axis ---'
      Caption = '                    '
      Caption = 
     & '{'//YCaption(index(YCaption,'{')+1:index(YCaption,'}')-1)//'}'
      if (YCPos.le.0.0) then
         YPS = 0.5*(YPS1+YPS0)
         XPS = YPSCaption + 1.5*SizeLabel
         N   = 90
      else
         YPS = APSY*(YL(NYL)-(YCPos-1)*YLDiff) + BPSY
     &               - 0.35*SizeLabel
         XPS  = XPS0 - 0.35*SizeLabel
         N   = 0
      end if
      call plotStrg(UnitPS,Caption,XPS,YPS,N,1-min(1,N))
* --- end of plotYAxis
      return
      end

************************************************************************
* --- plot caption (w.r.t. the data region---------------------------- *
      subroutine plotCapt(UnitPS,Strg,RefStrg,XRel,YRel,Angle,Pos)
      implicit none
      include 'gv2d_PostScript.include'
      real*8  XRel, YRel, XPS, YPS
      integer Angle, Pos
      character*80 Strg, RefStrg
* --- Compute the point which corresponds to the relative coordinates
      XPS = XPS0 + XRel*(XPS1-XPS0)
      YPS = YPS0 + YRel*(YPS1-YPS0)
* --- Modify the position if (XPS,PYS) is on the frame
      if (XRel.eq.0.0 .or. XRel.eq.1.0) then    ! Left of right side
         XPS = XPS - (1-2*XRel) * SizeLabel
         YPS = YPS -   YRel     * SizeLabel
      end if
      if (YRel.eq.0.0 .or. YRel.eq.1.0) then    ! Top or bottom side
         YPS = YPS - (1-2*YRel) * SizeLabel
      end if
* --- If Strg=RefStrg plot the string at (XPS,YPS) according to (Angle,Pos)
* --- otherwise place the string w.r.t. StrgRef first. 
      if (Strg.eq.RefStrg) then
         call plotStrg(UnitPS,Strg,XPS,YPS,Angle,Pos)
      else
         call plotStrg(UnitPS,RefStrg,XPS,YPS,Angle,Pos+10)
         call plotStrg(UnitPS,   Strg,XPS,YPS,Angle,Pos+20)
      end if

* --- end of plotCapt
      return
      end

************************************************************************
* --- plot string ---------------------------------------------------- *
      subroutine plotStrg(UnitPS,Strg,XPS,YPS,Angle,Pos)
      implicit none
      include 'gv2d_PostScript.include'
      real*8    XPS, YPS
      integer   IToken, NToken, Pos, Angle, I
      integer   IChar0, IChar1, DXPS, DYPS, IFont0, IFontD, IPos0
      character Strg*80, Spec*5, Ch*1
      integer   SizeFont(10), ShiftFont(10)
      character NameFont(10)*12, Token(10)*40
* --- Decompose string in printable tokens
      IFontD = 1             ! Set default font (Bold)
      IFont0 = 1             ! Set actual default font (Bold)
      IPos0  = 0             ! Set actual default position (no ...script)
      IChar0 = 2             ! # of first character in token
      IChar1 = 2             ! # of last  character in token
      NToken = 0             ! # of token
    1 do while (Strg(IChar1:IChar1).ne.'['.and.  ! Look for end of token
     &          Strg(IChar1:IChar1).ne.'}')
         IChar1 = IChar1 + 1
      end do
      if (Strg(IChar1:IChar1).eq.'[') then
         Spec = Strg(IChar1+1:min(Len(Strg),IChar1+7))
         I    = 0
         I    = max(I,index(Spec,'BSUB'))
         I    = max(I,index(Spec,'ESUB'))
         I    = max(I,index(Spec,'BSUP'))
         I    = max(I,index(Spec,'ESUP'))
         I    = max(I,index(Spec,'FONT'))
         I    = max(I,index(Spec,'ROMAN'))
         I    = max(I,index(Spec,'BOLD'))
         I    = max(I,index(Spec,'SLANT'))
         I    = max(I,index(Spec,'GREEK'))
         if (I.eq.0) then                  ! If "[" is a printable character
            IChar1 = IChar1 + 1
            goto 1
         end if
      end if
* --- Define contents and style of new token
      if (IChar1.gt.IChar0) then
         NToken        = NToken + 1
         NameFont(NToken) = '            '
         if (IFont0.eq.-1) NameFont(NToken) = 'Symbol      '
         if (IFont0.eq. 0) NameFont(NToken) = 'Times-Roman '
         if (IFont0.eq. 1) NameFont(NToken) = 'Times-Bold  '
         if (IFont0.eq. 2) NameFont(NToken) = 'Times-Italic'
         SizeFont(NToken)  = nint(SizeLabel*(1.0-0.3*min(1,abs(IPos0))))
         ShiftFont(NToken) = nint(IPos0*0.3*SizeLabel)
         if (NToken.gt.1)        ! Neutralize the previous shift
     &      ShiftFont(NToken) = ShiftFont(NToken)-ShiftFont(NToken-1)
         Token(NToken) = '('//Strg(IChar0:IChar1-1)//')'
      end if
* --- Define style of new token
      if (Strg(IChar1:IChar1).eq.'[') then
         IChar0 = IChar1 + 1
         IChar1 = IChar0
         do while (Strg(IChar1:IChar1).ne.']')  ! Look for end of spec.
            IChar1 = IChar1 + 1
         end do
         Spec = '            '
         Spec = Strg(IChar0:IChar1-1)
         if (index(Spec,'BSUB').ne.0)  IPos0  = IPos0 -1
         if (index(Spec,'BSUP').ne.0)  IPos0  = IPos0 +2
         if (index(Spec,'ESUB').ne.0)  IPos0  = IPos0 +1
         if (index(Spec,'ESUP').ne.0)  IPos0  = IPos0 -2
         if (index(Spec,'ROMAN').ne.0) IFont0 =  0
         if (index(Spec,'FONT') .ne.0) IFont0 =  1
         if (index(Spec,'BOLD') .ne.0) IFont0 =  1
         if (index(Spec,'SLANT').ne.0) IFont0 =  2
         if (index(Spec,'GREEK').ne.0) IFont0 = -1
         IChar0 = IChar1 + 1
         IChar1 = IChar0
         goto 1
      end if
* --- Show resulting decomposition
C === write(*,*)
C === do IToken = 1, NToken
C ===    write(*,'(1h i6,a12,2i6,5x,a13)') IToken, 
C ===&            NameFont(IToken), SizeFont(IToken), ShiftFont(IToken),
C ===&            Token(IToken)
C === end do
* --- Compute the position of the string
      if (nint(0.1*Pos).le.1) then
         write(UnitPS,'(1h 2i6,a,i6,a)') 
     &                nint(XPS), nint(YPS), ' m ', Angle, ' r '
      else
         write(UnitPS,'(1h i6,a)')  Angle, ' r '
      end if
      if (Pos-10*nint(0.1*Pos).ge.0.and.nint(0.1*Pos).le.1) then
         IPos0 = Pos - 10*nint(0.1*Pos)
         if (IPos0.eq.0) Ch = 'C'
         if (IPos0.eq.1) Ch = 'R'
         DXPS  = min(NToken-1,1) * nint(0.05*(1+IPos0)*SizeLabel)
         do IToken = 1, NToken
           write(UnitPS,'(1h a,a12,a,i6,a,a40,a,a1)') 
     &                '/',NameFont(IToken),' ff ',
     &                    SizeFont(IToken),' sc  sf ',
     &                    Token(IToken), ' PosStrg',Ch
           write(UnitPS,'(1h 2i6,a)') -DXPS,0, ' rm '
         end do
      end if
* --- Display tokens
      if (mod(nint(0.1*Pos),2).gt.0) return
      do IToken = 1, NToken
         DXPS  = min(IToken-1,1) * nint(0.1*SizeLabel)
         DYPS  = ShiftFont(IToken)
         write(UnitPS,'(1h a,a12,a,i6,a,2i6,a)') 
     &                '/',NameFont(IToken),' ff ',
     &                    SizeFont(IToken),' sc  sf ',DXPS,DYPS,' rm '
         write(UnitPS,'(1h a40,a)') Token(IToken), ' sh '
      end do
      if (Angle.ne.0) write(UnitPS,'(1h i6,a)') -Angle, ' r '
* --- end of plotStrg
      return
      end

************************************************************************
* --- plot a solid curve --------------------------------------------- *
      subroutine plotCurve(UnitPS,XVal,YVal,NVal)
      implicit none
      include 'gv2d_PostScript.include'
      include 'gv2d_Cartesian.include'
      integer I, NPS, N, NVal
      real*8  XVal(NVal), YVal(NVal), Eps
      parameter (Eps=1.0)
      integer XPSVal(100000), YPSVal(100000)
      character*1 PSCom(100000)
* ---

      write(UnitPS,'(1h a)') '%*** SOLID CURVE ***'
      write(UnitPS,'(1h i3,a)') nint(WidthCurve),' lw  n '
      N    = 1
      NPS  = 1
    1 XPSVal(NPS) = nint(APSX*XVal(N) + BPSX)
      YPSVal(NPS) = nint(APSY*YVal(N) + BPSY)
      PSCom (NPS) = 'l'
      if (NPS.eq.1) PSCom(NPS) = 'm'

      if (XPSVal(NPS).ge.XPS0-Eps.and.XPSVal(NPS).le.XPS1+Eps .and.
     &    YPSVal(NPS).ge.YPS0-Eps.and.YPSVal(NPS).le.YPS1+Eps) then
         if (N.lt.NVal) then
            N   = N   + 1
            NPS = NPS + 1
            goto 1
         else
            goto 2
         end if
      else
         PSCom (NPS) = 'm'
      end if
    2 if (NPS.gt.1) then
         write(UnitPS,100) (XPSVal(I),YPSVal(I),PSCom(I),I=1,NPS)
         write(UnitPS,101) ' s '
      end if  
      if (N.lt.NVal) then
         NPS = 1
         N   = N + 1
         goto 1
      end if
* --- formats
  100 format(4(2i6,1x,a1,2x))
  101 format(a3)
* --- end of plotCurve
      return
      end

************************************************************************
* --- plot symbols --------------------------------------------------- *
      subroutine plotSymbol(UnitPS,XVal,YVal,NVal,ISymbol)
      implicit none
      include 'gv2d_PostScript.include'
      include 'gv2d_Cartesian.include'
      integer I, J, NPS, NVal, ISymbol
      real*8  XVal(NVal), YVal(NVal), XPS(1000), YPS(1000), DPS0, DPS1
      real*8  Eps
      character*1 Ch
* --- Compute PS coordinates of symbols
      NPS = 0
      Eps = 0.1*SizeSymbol
      do I = 1, NVal
         XPS(NPS+1) = APSX * XVal(I) + BPSX
         YPS(NPS+1) = APSY * YVal(I) + BPSY
         if (XPS(NPS+1).gt.XPS0+Eps.and.XPS(NPS+1).lt.XPS1-Eps.and.
     &       YPS(NPS+1).gt.YPS0+Eps.and.YPS(NPS+1).lt.YPS1-Eps) 
     &       NPS = NPS + 1
      end do
* --- Define symbol in PS-file
      Ch = ' '
      if (ISymbol.gt.0.and.ISymbol.lt.+6) Ch = 'f'
* --- ... circle
      if (abs(ISymbol).eq.1) then
         write(UnitPS,'(1h  a)') '%*** CIRCLES ***'
         write(UnitPS,'(1h i3,a)') nint(WidthSymbol),' lw '
         write(UnitPS,'(1h  a,i3,a,a1,a)') 
     &       '/Symbol { n  ',nint(0.5*SizeSymbol),
     &       ' 0 360 arc ',Ch,' s } def '
      end if
* --- ... square
      if (abs(ISymbol).eq.2) then
         write(UnitPS,'(1h  a)') '%*** SQUARE ***'
         write(UnitPS,'(1h i3,a)') nint(WidthSymbol),' lw '
         DPS0 = 0.5*SizeSymbol
         write(UnitPS,'(1h  a,4(2f6.1,a),a1,a)') 
     &       '/Symbol { m ',
     &         - DPS0, - DPS0, ' rm ',
     &         2*DPS0,   0.0 , ' rl '
         write(UnitPS,'(1h 12x, 2(2f6.1,a),a1,a)') 
     &           0.0 , 2*DPS0, ' rl ',
     &        -2*DPS0,   0.0 , ' rl c ',Ch,' s } def '
      end if
* --- ... triangle
      if (abs(ISymbol).eq.3) then
         write(UnitPS,'(1h  a)') '%*** TRIANGLE ***'
         write(UnitPS,'(1h i3,a)') nint(WidthSymbol),' lw '
         DPS0 = 0.5 * 1.22*SizeSymbol
         DPS1 = DPS0 * sqrt(3.0)/2.0
         write(UnitPS,'(1h  a,3(2f6.1,a),a1,a)') 
     &       '/Symbol { m ',
     &         - DPS0 , - DPS1, ' rm ',
     &         2*DPS0 ,   0.0 , ' rl ',
     &         - DPS0 , 2*DPS1, ' rl c ',Ch,' s } def '
      end if
* --- ... star
      if (abs(ISymbol).eq.4) then
         write(UnitPS,'(1h  a)') '%*** STAR ***'
         write(UnitPS,'(1h i3,a)') nint(WidthSymbol),' lw '
         DPS0 = 0.5*1.22*SizeSymbol
         DPS1 = DPS0/sqrt(2.0)
         write(UnitPS,'(1h a,3(2f6.1,a))') 
     &       '/Symbol { m ',
     &       - DPS0,   0.0, ' rm ',
     &       2*DPS0,   0.0, ' rl ',
     &       - DPS0,   0.0, ' rm '
         write(UnitPS,'(1h 12x,3(2f6.1,a))')
     &         0.0, - DPS0, ' rm ',
     &         0.0, 2*DPS0, ' rl ',
     &         0.0, - DPS0, ' rl '
         write(UnitPS,'(1h 12x,3(2f6.1,a))')
     &           DPS1,   DPS1, ' rm ',
     &        -2*DPS1,-2*DPS1, ' rl ',
     &           DPS1,   DPS1, ' rm '
         write(UnitPS,'(1h 12x,3(2f6.1,a))')
     &         DPS1,   - DPS1, ' rm ',
     &        -2*DPS1, 2*DPS1, ' rl ',
     &         DPS1,   - DPS1, ' rm s } def '
      end if
* --- Write symbols in PS-file
      write(UnitPS,100) (nint(XPS(I)), nint(YPS(I)), I=1,NPS)
* --- formats
  100 format(4(2i6,1x,'Symbol',1x))
* --- end of plotSymbol
      return
      end

************************************************************************
* --- plot circle ---------------------------------------------------- *
      subroutine plotCircle(UnitPS,X,Y,R,Fill)
      implicit none
      include 'gv2d_PostScript.include'
      include 'gv2d_Cartesian.include'
      real*8  X, XPS, Y, YPS, R, RPS, Fill, FillPS
      character*1 Ch
* --- Compute PS coordinate of object
      XPS = APSX * X + BPSX
      YPS = APSY * Y + BPSY
      RPS = APSX * R         ! Assuming APSX = APSY
      FillPS = max(0.0,min(1.0,abs(1.0-Fill))) 
* --- Check if a part of the circle is in the data region ...
      if (XPS+RPS.gt.XPS0.and.XPS-RPS.lt.XPS1.and.
     &    YPS+RPS.gt.YPS0.and.YPS-RPS.lt.YPS1) then
         write(UnitPS,'(1h  a)') '%*** CIRCLE ***'
* ------ plot only boundary of the circle
         if (Fill.eq.0.0) then
            write(UnitPS,'(1h i3,a,3i6,a)') 
     &         nint(WidthSymbol),' lw n ',nint(XPS),nint(YPS),nint(RPS),
     &         ' 0 360 arc c s '
* ------ Fill the area in the circle
         else if (Fill.gt.0.0) then
            write(UnitPS,'(1h i3,a,3i6,a,f5.2,a)')
     &         nint(WidthSymbol),' lw n ',nint(XPS),nint(YPS),nint(RPS),
     &         ' 0 360 arc c ',FillPS,' g f 0.00 g '
* ------ Fill the area in the circle and plot the contour
         else
            write(UnitPS,'(1h i3,a,3i6,a,f5.2,a)') 
     &         nint(WidthSymbol),' lw n ',nint(XPS),nint(YPS),nint(RPS),
     &         ' 0 360 arc c gs ',FillPS,' g f gr s 0.00 g '
         end if
      end if
* --- end of plotCircle
      return
      end

************************************************************************
* --- plot rectangle ------------------------------------------------- *
      subroutine plotRectangle(UnitPS,XC,YC,DX,DY,Fill)
      implicit none
      include 'gv2d_PostScript.include'
      include 'gv2d_Cartesian.include'
      integer I
      real*8  XC, YC, DX, DY, R, Fill, XPS(4), YPS(4), RPS, FillPS, Eps
      parameter (Eps = 1.0)
      character*1 Ch
* --- Compute PS coordinate of object
      XPS(1) = APSX * (XC - 0.5*DX) + BPSX
      XPS(2) = APSX * (XC + 0.5*DX) + BPSX
      XPS(3) = XPS(2)
      XPS(4) = XPS(1)

      YPS(1) = APSY * (YC - 0.5*DY) + BPSY
      YPS(2) = YPS(1)
      YPS(3) = APSY * (YC + 0.5*DY) + BPSY
      YPS(4) = YPS(3)
      FillPS = max(0.0,min(1.0,1.0-abs(Fill))) 
* --- Check if a part of the rectangle is in the data region ...
      if ((XPS(2).gt.XPS0-Eps.or.XPS(1).lt.XPS1+Eps).and.
     &    (YPS(3).gt.YPS0-Eps.or.YPS(1).lt.YPS1+Eps)) then
         write(UnitPS,'(1h  a)') '%*** RECTANGLE ***'
* ------ plot only boundary of the rectangle
         if (Fill.eq.0.0) then
            write(UnitPS,'(1h i3,a,4(2i6,a),a)') 
     &         nint(WidthSymbol),' lw n ',
     &         (nint(XPS(I)),nint(YPS(I)),' m ',I=1,1),
     &         (nint(XPS(I)),nint(YPS(I)),' l ',I=2,4),' c s '
* ------ Fill the area in the rectangle
         else if (Fill.gt.0.0) then
            write(UnitPS,'(1h i3,a,4(2i6,a),a)') 
     &         nint(WidthSymbol),' lw n ',
     &         (nint(XPS(I)),nint(YPS(I)),' m ',I=1,1),
     &         (nint(XPS(I)),nint(YPS(I)),' l ',I=2,4),' c '
            write(UnitPS,'(1h f5.2,a)') FillPS,' g f 0.00 g '
* ------ Fill the area in the rectangle and plot the contour
         else
            write(UnitPS,'(1h i3,a,4(2i6,a),a)') 
     &         nint(WidthSymbol),' lw n ',
     &         (nint(XPS(I)),nint(YPS(I)),' m ',I=1,1),
     &         (nint(XPS(I)),nint(YPS(I)),' l ',I=2,4),' c gs '
            write(UnitPS,'(1h f5.2,a)') FillPS,' g f gr s 0.00 g '
         end if
      end if
* --- end of plotRectangle
      return
      end

************************************************************************
* --- plot polygon (added 041228) ------------------------------------ *
      subroutine plotPolygon(UnitPS,XVal,YVal,NVal,Fill)
      implicit none
      include 'gv2d_PostScript.include'
      include 'gv2d_Cartesian.include'
      integer iVal, NVal
      real*8  X, XVal(NVal), XPS
      real*8  Y, YVal(NVal), YPS, Fill,  FillPS, EPS
      parameter (EPS=1.0)
      character*1 Ch
* --- initialize
      FillPS = max(0.0,min(1.0,1.0-abs(Fill))) 
* --- Check if a part of the rectangle is in the data region ...
C XXX      if ((XPS(2).gt.XPS0-Eps.or.XPS(1).lt.XPS1+Eps).and.
C XXX&    (YPS(3).gt.YPS0-Eps.or.YPS(1).lt.YPS1+Eps)) then
         write(UnitPS,'(1h  a)') '%*** POLYGON ***'
         write(UnitPS,'(1h i3,a,4(2i6,a))') 
     &         nint(WidthSymbol),' lw n ',
     &         nint(APSX*XVal(1)+BPSX), nint(APSY*YVal(1)+BPSY),
     &         ' m'
         write(UnitPS,'(1h  4(2i6,a))')
     &        (nint(APSX*XVal(iVal)+BPSX), 
     &         nint(APSY*YVal(iVal)+BPSY),' l',iVal=2,NVal)
* ------ plot only boundary of the rectangle
         if (Fill.eq.0.0) then
            write(UnitPS,'(1h a)') ' c s '
* ------ Fill the area in the rectangle
         else if (Fill.gt.0.0) then
            write(UnitPS,'(1h a,f5.2,a)') 
     &                   ' c ',FillPS,' g f 0.00 g '
* ------ Fill the area in the rectangle and plot the contour
         else
            write(UnitPS,'(1h a,f5.2,a)') 
     &                   ' c gs ',FillPS,' g f gr s 0.00 g '
         end if
C XXX end if
* --- end of plotRectangle
      return
      end

************************************************************************
* --- plot arrow ----------------------------------------------------- *
      subroutine plotArrow(UnitPS,XTail,YTail,XHead,YHead)
      implicit none
      include 'gv2d_PostScript.include'
      real*8  XTail, YTail, XHead, YHead
      real*8  XPSTail, YPSTail, XPSHead, YPSHead, XPSDiff, YPSDiff
      real*8  DPS, ScaleHead, AnglePS, Arg
      real*8  XPS(6), YPS(6)
      integer I
* --- Compute PostScript coordinates of the tail and head
      XPSTail = APSX*XTail + BPSX
      YPSTail = APSY*YTail + BPSY
      XPSHead = APSX*XHead + BPSX
      YPSHead = APSY*YHead + BPSY
* --- Compute the PostScript length and orientation of the arrow
      XPSDiff = XPSHead - XPSTail
      YPSDiff = YPSHead - YPSTail
      DPS     = sqrt(XPSDiff**2 + YPSDiff**2)
      AnglePS = 180.0 * Arg(XPSDiff,YPSDiff) / 3.1415926
* --- Compute contour of arrow
      ScaleHead = min(1.0, DPS / (1.75*SizeArrowL))
      XPS(1) =  0.0
      YPS(1) =  0.0
      XPS(2) =  DPS - 0.6667*ScaleHead*SizeArrowL
      YPS(2) =  0.0
      XPS(3) =  DPS - ScaleHead*SizeArrowL
      YPS(3) =  0.5*ScaleHead*SizeArrowW
      XPS(4) =  DPS
      YPS(4) =  0.0
      XPS(5) =  DPS - ScaleHead*SizeArrowL
      YPS(5) = -0.5*ScaleHead*SizeArrowW
      XPS(6) =  DPS - 0.6667*ScaleHead*SizeArrowL
      YPS(6) =  0.0
* --- plot arrow
      write(UnitPS,'(1h a)') '%*** ARROW ***'
      write(UnitPS,'(1h i3,a,2i6,a,i4,a)') 
     &      nint(ScaleHead*WidthArrow),' lw ',
     &      nint(XPSTail),nint(YPSTail),' t ',
     &      nint(AnglePS),' r '
      write(UnitPS,'(1h a,2(2i6,a))')
     &      ' n ',nint(XPS(1)),nint(YPS(1)),' m ',
     &            nint(XPS(2)),nint(YPS(2)),' l s ' 
      write(UnitPS,'(1h a,4(2i6,a))')
     &      ' n ',nint(XPS(2)),nint(YPS(2)),' m ',
     &            nint(XPS(3)),nint(YPS(3)),' l ', 
     &            nint(XPS(4)),nint(YPS(4)),' l ',
     &            nint(XPS(5)),nint(YPS(5)),' l c f ' 
      write(UnitPS,'(1h i4,a,2i6,a)') 
     &      -nint(AnglePS),' r ',-nint(XPSTail),-nint(YPSTail),' t '
* --- end of plotArrow
      return
      end

* === function for argument of (x,y) ================================= *
      real*8  function Arg(X,Y)
      implicit none
      real*8  X, Y, Phi, Pi
      parameter (Pi=3.1415296)
      if (X.eq.0) then
         if (Y.gt.0.0) then
            Phi = 0.5 * Pi
         else if (Y.lt.0.0) then
            Phi =-0.5 * Pi
         else
            Phi = 0.0
         end if
      else
         Phi = atan(abs(Y/X))
         if (X.ge.0.0) Phi = Phi
         if (X.lt.0.0) Phi = Pi - Phi
         if (Y.lt.0.0) Phi = - Phi
      end if
      if (Phi.lt.0.0) Phi = Phi + 2.0*Pi
      Arg = Phi
* --- end of Arg
      return
      end

* === plot contour-lines ============================================= *
      subroutine plotCL(UnitPS,XVal,NX,NXP,YVal,NY,NYP,FVal,FCL,NCL)
      implicit none
      integer UnitPS, NX, NXP, NY, NYP, NCL, I, J, N
      real*8  XVal(NX), YVal(NY), FVal(NXP,NYP), FCL(NCL)
      real*8  X(4),     Y(4),     F(4)
* --- plot contour-lines
      do N = 1, NCL
      do I = 1, NX-1
      do J = 1, NY-1
         X(1) = XVal(I)
         Y(1) = YVal(J)
         F(1) = FVal(I  ,J  )
         X(2) = XVal(I+1)
         Y(2) = YVal(J)
         F(2) = FVal(I+1,J  )
         X(3) = XVal(I+1)
         Y(3) = YVal(J+1)
         F(3) = FVal(I+1,J+1)
         X(4) = XVal(I)
         Y(4) = YVal(J+1)
         F(4) = FVal(I,  J+1)
         call plotCLSquare(UnitPS,X,Y,F,FCL(N))
      end do
      end do
      end do
* --- end of plotCL
      return
      end

* === plot contour-lines in a square ================================= *
      subroutine plotCLSquare(UnitPS,X,Y,F,FCL)
      implicit none
      integer UnitPS, I, J, NCL
      real*8  X(4),Y(4),F(4),FCL          ! coordinates and F-values at 4 pts
      real*8  XTria(3),YTria(3),FTria(3)  ! same for triangle
* --- Check, if there exists a contour-line in the square
      if (FCL.gt.max(F(1),F(2),F(3),F(4))) return
      if (FCL.lt.min(F(1),F(2),F(3),F(4))) return
* --- Linear interpolation at the center of the square
      XTria(1) = 0.25*(X(1)+X(2)+X(3)+X(4))
      YTria(1) = 0.25*(Y(1)+Y(2)+Y(3)+Y(4))
      FTria(1) = 0.25*(F(1)+F(2)+F(3)+F(4))
* --- Split up the square in four triangles around the center
* --- and plot the contour-segments in each triangle
      do I = 1, 4            ! Loop over vertices
         J = 1 + mod(I,4)    ! No. of subsequent vertex
         XTria(2) = X(I)
         XTria(3) = X(J)
         YTria(2) = Y(I)
         YTria(3) = Y(J)
         FTria(2) = F(I)
         FTria(3) = F(J)
C DEB    write(*,'(1h 3(3f6.2,2x))') 
C DEB&        (XTria(J),J=1,3), (YTria(J),J=1,3), (FTria(J),J=1,3)
         call plotCLTriangle(UnitPS,XTria,YTria,FTria,FCL)
      end do
* --- end of plotCLSquare
      return
      end

* === plot contour-lines in a triangle ============================== *
      subroutine plotCLTriangle(UnitPS,X,Y,F,FCL)
      implicit none
      include 'gv2d_PostScript.include'
      include 'gv2d_Cartesian.include'
      integer I, J, NCL
      real*8  X(3),Y(3),F(3),FCL              ! same X1..F3 but as arrays
      real*8  XCL(2),YCL(2)                   ! end-points of contour-lines
      real*8  DF,WA, WB, Eps                  ! aux. quantities
      parameter (Eps=1.e-6)
* --- Check, if there exists a contour-line in the triangle
C DEB write(*,*) FCL,min(F(1),F(2),F(3)),max(F(1),F(2),F(3))
      if (FCL.gt.max(F(1),F(2),F(3))) return
      if (FCL.lt.min(F(1),F(2),F(3))) return
* --- Find the contour-line
      NCL = 0
      do I = 1, 3            ! Loop over vertices
         J = 1 + mod(I,3)    ! No. of subsequent vertex
* ------ If the contour-segment coincides with an edge ...
         if (max(abs(FCL-F(I)),abs(FCL-F(J))).le.Eps) then
            XCL(1) = X(I)
            YCL(1) = Y(I)
            XCL(2) = X(J)
            YCL(2) = Y(J)
            NCL = 2
            goto 1
         end if
* ------ If the contour-segment passes the interior of the triangle ...
         if (FCL-F(I).le.Eps*abs(FCL).and.FCL-F(J).ge.Eps*abs(FCL).or.
     &       FCL-F(I).ge.Eps*abs(FCL).and.FCL-F(J).le.Eps*abs(FCL)) then
C XXX    else if ((FCL-F(I))*(FCL-F(J)).le.0.0) then
            NCL = NCL + 1
            WA = abs((F(J)-FCL) / (F(J)-F(I)))
            WA = max(0.0,min(1.0,WA))
            WB = 1.0 - WA
            XCL(NCL) = WA*X(I) + WB*X(J)
            YCL(NCL) = WA*Y(I) + WB*Y(J)
         end if
         if (NCL.ge.2) goto 1
      end do
* --- plot contour-segment
    1 continue
      if (NCL.ne.2) return  ! no line (due to truncation errors)
      write(UnitPS,'(1h i4,a,2(2i6,a))') nint(WidthCurve),
     &    ' lw n ',nint(APSX*XCL(1)+BPSX),nint(APSY*YCL(1)+BPSY),
     &     ' m ',  nint(APSX*XCL(2)+BPSX),nint(APSY*YCL(2)+BPSY),
     &     ' l s '
* --- end of plotCLTriangle
      return
      end
