C$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
C$$$$ Segment with subroutines for complex linear algebra $$$$$$$$$$$$$$
C$$$$ created by Bernd R. Noack, 24Mar92 $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
C$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
C === call CGauss(A,N,NP)
C === call CGaussJ(A,N,NP,B,M,MP)
C === complex = CDET(A,N,NP)
C === call CMatInv(A, AINV, N, NP)
C === call CLUDcmp(A,N,NP,Indx,D)
C === call CLUBksb(A,N,NP,Indx,B)
C === call CQUDec(A,M,MP,N,D,Err)
C === call CMatOut(A,M,MP,N,NP)
************************************************************************
* --- reduction of a general matrix to upper-triangular form --------- *
      subroutine CGauss(A,N,NP)
      implicit none
      integer    N, NP
      integer    I, J, K, L, LL, ICol, IRow, IRow0
      real*8     Big, Eps
      complex*16 A(NP,NP)
      complex*16 PivInv, Dum
      parameter  (Eps = 1.0E-5)
* --- main loop over columns
      IRow0 = 0
      do ICol = 1,N
         IRow0 = IRow0 + 1
* ------ outer loop of search for a pivot element (with largest abs. value)
*        in the same column
         Big = 0.0
         do J = 1,N
            if (abs(A(J,ICol)).GT.Big) then
               Big = abs(A(J,ICol))
               IRow = J
            end if
         end do
* ------ exchaning the rows so that pivot element is at the right place
         if (IRow.GT.IRow0) then
            do L = 1,N
               Dum        = A(IRow,L)
               A(IRow, L) = A(IRow0,L)
               A(IRow0,L) = Dum
            end do
         end if
* ------ dividing pivot row by pivot element
         if (abs(A(IRow0,ICol)).GT.Eps) then
            PivInv = 1.0 / A(IRow0,ICol)
            A(IRow0,ICol) = 1.0
            do L = ICol+1,N
               A(IRow0,L) = A(IRow0,L) * PivInv
            end do
* --------- producing zeros in the column below IRow0
            do L = 1,N
               if (L.GT.IRow0) then
                  Dum = A(L,ICol)
                  A(L,ICol) = 0.0
                  do J = ICol+1, N
                     A(L,J) = A(L,J) - A(IRow0,J) * Dum
                  end do
               end if
            end do
         ELSE
            IRow0 = IRow0 - 1
            do L = IRow0, N
               A(L,ICol) = 0.0
            end do
         end if
* ------ transformation which yields a unit vector in the IColth column
      end do
* --- end of CGauss
      return
      end

* --- Gauss-Jordan elimination --------------------------------------- *
* --- subroutine from Numerical Receipes, chapt. 2.1, p.28  ---------- *
      subroutine CGaussJ(A,N,NP,B,M,MP)
      implicit none
      integer    N, NP, M, MP, NMax
      parameter (NMax=2020)
      integer    IndxC(NMax), IndxR(NMax), IPIV(NMax)
      integer    I, J, K, L, LL, IRow, ICol
      real*8     Big
      complex*16 A(NP,NP), B(NP,MP)
      complex*16 PivInv, Dum
* ---
      do 11 J = 1,N
         IPIV(J) = 0
   11 continue
* --- main loop over columns
      do 22 I = 1,N
         Big = 0.0
* ------ outer loop of search for a pivot element (with largest abs. value)
*        index of pivot element (IRow, ICol)
         do 13 J = 1,N
            if (IPIV(J).NE.1) then
               do 12 K = 1,N
                  if (IPIV(K).EQ.0) then
                     if (abs(A(J,K)).GE.Big) then
                        Big = abs(A(J,K))
                        IRow = J
                        ICol = K
                     end if
                  ELSE if (IPIV(K).GT.1) then
                     STOP 'Greetings from CGaussJ: singular matrix!'
                  end if
   12          continue
            end if
   13    continue
* ------ exchaning the rows so that pivot element is on diagonal
         if (IRow.NE.ICol) then
            do 14 L = 1,N
               Dum       = A(IRow,L)
               A(IRow,L) = A(ICol,L)
               A(ICol,L) = Dum
   14       continue
            do 15 L = 1,M
               Dum       = B(IRow,L)
               B(IRow,L) = B(ICol,L)
               B(ICol,L) = Dum
   15       continue
         end if
*        bookkeeping Ith row -> IRowth row
*                    Ith column is pivoted
         IndxR(I) = IRow
         IndxC(I) = ICol
* ------ dividing pivot Row by pivot element
         if (A(ICol,ICol).EQ.0.0)
     &             STOP 'Greetings from CGaussJ: singular matrix!'
         PivInv = 1.0 / A(ICol,ICol)
         A(ICol,ICol) = 1.0
         do 16 L = 1,N
            A(ICol,L) = A(ICol,L) * PivInv
   16    continue
         do 17 L = 1,M
            B(ICol,L) = B(ICol,L) * PivInv
   17    continue
* ------ transformation which yields a unit vector in the IColth column
         do 21 LL = 1,N
            if (LL.NE.ICol) then
               Dum = A(LL,ICol)
               A(LL,ICol) = 0.0
               do 18 L = 1,N
                  A(LL,L) = A(LL,L) - A(ICol,L) * Dum
   18          continue
               do 19 L = 1,M
                  B(LL,L) = B(LL,L) - B(ICol,L) * Dum
   19          continue
            end if
   21    continue
   22 continue
* --- unscrambling with respect to column interchanges
      do 24 L = N,1,-1
         if (IndxR(L).NE.IndxC(L)) then
            do 23 K = 1,N
               Dum = A(K,IndxR(L))
               A(K,IndxR(L)) = A(K,IndxC(L))
               A(K,IndxC(L)) = Dum
   23       continue
         end if
   24 continue
* --- end of CGaussJ
      return
      end

* --- determinant of complex matrix ---------------------------------- *
      complex*16 function CDET(A,N,NP)
      implicit none
      integer    N, NP
      integer    I, J, K, L, LL, ICol, IRow, IRow0
      real*8     Big, Eps
      complex*16 A(NP,NP), Factor
      complex*16 PivInv, Dum
      parameter  (Eps = 1.0E-5)
* --- main loop over columns
      Factor = 1.0  ! no scaling of a row of the matrix, yet.
      IRow0 = 0
      do ICol = 1,N
         IRow0 = IRow0 + 1
* ------ outer loop of search for a pivot element (with largest abs. value)
*        in the same column
         Big = 0.0
         do J = 1,N
            if (abs(A(J,ICol)).GT.Big) then
               Big = abs(A(J,ICol))
               IRow = J
            end if
         end do
* ------ exchaning the rows so that pivot element is at the right place
         if (IRow.GT.IRow0) then
            do L = 1,N
               Dum        = A(IRow,L)
               A(IRow, L) = A(IRow0,L)
               A(IRow0,L) = Dum
            end do
         end if
* ------ dividing pivot row by pivot element
         if (abs(A(IRow0,ICol)).GT.Eps) then
            PivInv = 1.0 / A(IRow0,ICol)
            Factor = Factor * A(IRow0,ICol)
            A(IRow0,ICol) = 1.0
            do L = ICol+1,N
               A(IRow0,L) = A(IRow0,L) * PivInv
            end do
* --------- producing zeros in the column below IRow0
            do L = 1,N
               if (L.GT.IRow0) then
                  Dum = A(L,ICol)
                  A(L,ICol) = 0.0
                  do J = ICol+1, N
                     A(L,J) = A(L,J) - A(IRow0,J) * Dum
                  end do
               end if
            end do
         ELSE
            IRow0 = IRow0 - 1
            do L = IRow0, N
               A(L,ICol) = 0.0
            end do
         end if
      end do
* --- determining the determinant 
* --- (previous scalings X product of diagonal elements)
      do I = 1, N
         Factor = Factor * A(I,I)
      end do
      CDET = Factor
* --- end of CDET
      return
      end

* --- matrix inversion ----------------------------------------------- *
      subroutine CMatInv(A, AInv, N, NP)
      implicit none
      integer    NMax
      parameter (NMax=2020)
      integer    N, NP, I, J, Indx(NMax)
      complex*16 A(NP, NP), AInv(NP, NP)
      real*8  D
* --- identity matrix
      do 12 I = 1,N
         do 11 J = 1,N
            AInv(I,J) = 0.0
   11    continue
         AInv(I,I) = 1.0
   12 continue
      CALL CLUDcmp(A,N,NP,Indx,D)
      do 13 J = 1,N
         CALL CLUBksb(A,N,NP,Indx,AInv(1,J))
   13 continue
* --- end of CMatInv
      return
      end

* --- LU - decomposition -----------------------------------------------
      subroutine CLUDcmp(A,N,NP,Indx,D)
      implicit none
      integer    N, NP, NMax, I, J, K, IMax
      real*8     AAMax, TINY, Dum, D
      parameter (NMax=2020,TINY=1.0E-20)
      real*8     VV(NMax)
      complex*16 A(NP,NP), CDum
      complex*16 Sum
      integer Indx(N)
      D = 1.        ! No rows interchanged yet.
      do 12 I = 1,N ! loop over the Rows to get the implicit scaling information
         AAMax = 0.
         do 11 J = 1,N
            if (abs(A(I,J)).GT.AAMax) AAMax=abs(A(I,J))
   11    continue
         if (AAMax.EQ.0.) 
     &      STOP 'Greetings from CLUDcmp: singular matrix!'
         VV(I) = 1./AAMax
   12 continue
      do 19 J = 1,N
         do 14 I = 1,J-1
            Sum = A(I,J)
            do 13 K = 1,I-1
               Sum = Sum -A(I,K)*A(K,J)
   13       continue
            A(I,J) = Sum
   14    continue
         AAMax = 0.0
         do 16 I = J,N
            Sum = A(I,J)
            do 15 K = 1,J-1
               Sum = Sum-A(I,K)*A(K,J)
   15       continue
            A(I,J) = Sum
            Dum = VV(I)*abs(Sum)
            if (Dum.GE.AAMax) then
               IMax = I
               AAMax = Dum
            end if
   16    continue
         if (J.NE.IMax) then
            do 17 K = 1,N
               CDum      = A(IMax,K)
               A(IMax,K) = A(J,K)
               A(J,K)    = CDum
   17       continue
            D = -D
            VV(IMax) = VV(J)
         end if
         Indx(J) = IMax
         if (A(J,J).EQ.0) A(J,J) = TINY
         if (J.NE.N) then
            CDum = 1.0/A(J,J)
            do 18 I = J+1,N
               A(I,J) = A(I,J)*CDum
   18       continue
         end if
   19 continue
* --- end of CLUDcmp
      return
      end

* --- Back substitution ------------------------------------------------
      subroutine CLUBksb(A,N,NP,Indx,B)
      implicit none
      integer    N, NP, Indx(N), I, J, II, LL
      complex*16 A(NP,NP), B(N), Sum
      II = 0
      do 12 I = 1,N
         LL = Indx(I)
         Sum = B(LL)
         B(LL) = B(I)
         if (II.NE.0) then
            do 11 J = II,I-1
               Sum = Sum - A(I,J)*B(J)
   11       continue
         ELSE if (Sum.NE.0.0) then
            II = I
         end if
         B(I) = Sum
   12 continue
      do 14 I = N,1,-1
         Sum = B(I)
         if (I.LT.N) then
            do 13 J = I+1,N
               Sum = Sum -A(I,J)*B(J)
   13       continue
         end if
         B(I) = Sum / A(I,I)
   14 continue
* --- end of CLUBksb
      return
      end

* --- matrix output -------------------------------------------------- *
      subroutine CMatOut(A,M,N,MP,NP)
      implicit none
      integer    I, K, K0, K1, M, N, MP, NP
      complex*16 A(1:MP,1:NP)
* --- showing groups of 5 columns for all lines
      K0 = 1
      K1 = MIN(N,4)
   10 continue
      write(*,1010) (K,K=K0,K1)
      do I = 1, M
            write(*,1020) I, (A(I,K),K=K0,K1)
      end do
      if (K1.LT.N) then
         K0 = K0 + 4
         K1 = MIN(N,K1 + 4)
         write(*,1030)
         goto 10
      end if
* --- some formats
 1010 format('I/J', i9,4i18)
 1020 format(i3  , 4(2f8.4,2X))
 1030 format('matrix - continued:')
* --- end of subroutine CMatOut
      return
      end
