C$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
C$$$$ Segment with subroutines for real*8 eigenvalue problem $$$$$$$$$$$
C$$$$ created by Bernd R. Noack, 24Mar92 $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
C$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
C === call EigVal (A, EVRe, EVIm, N, NP)
C === call Balanc(A,N,NP)
C === call ElmHes(A,N,NP)
C === call HQR(A,N,NP,WR,WI)
C === call EigVec(X, A, B, EVal, N, NP)        ! 010301: New complex*16!
C === call SortEV(EVRe,EVIm,N)
C === call Jacobi(A,M,MP,D,V,NRot)
C === call SortEVal(EValRe,EValIm,N)
C === call SortEE(EigVal,EigVec,N,NP)
C === call SpecDecSym(MatrA,MatrEVec,EVal,N,NP)
C === call GramSchmidt(MatrCorr,MatrTran,N,NP)
************************************************************************
* === eigenvalues of a double precision matrix ======================= *
      subroutine EigVal (A, EVRe, EVIm, N, NP)
      implicit none
      integer N, NP
      real*8  A(NP,NP), EVRe(NP), EVIm(NP)
* --- Balance the matrix
      call Balanc(A,N,NP)
* --- Compute the real Hessenberg matrix
      call ElmHes(A,N,NP)
* --- computing the eigen-values of Fx =kx
      call HQR(A,N,NP,EVRe,EVIm)
* --- end of EigVal
      return
      end

* --- Balancing a matrix (for greater accuracy of eigen-values) ------ *
* --- Numerical Recipes, chapter 11.5, p. 366 ------------------------ *
      subroutine Balanc(A,N,NP)
      implicit none
      real*8 RADIX, SQRDX
      parameter (RADIX = 2.0, SQRDX = RADIX**2)
      integer NP, LAST, I, J, N
      real*8 A(NP,NP), C, F, G, R, S
* ---
    1 continue
         LAST = 1
         do 14 I = 1,N
            C = 0.0
            R = 0.0
            do 11 J = 1,N
               if (J.ne.I) then
                  C = C + abs(A(J,I))
                  R = R + abs(A(I,J))
               end if
   11       continue
            if (C.ne.0.0 .and. R.ne.0.0) then
               G = R/RADIX
               F = 1.0
               S = C + R
    2          if (C.lt.G) then
                  F = F * RADIX
                  C = C * SQRDX
                  goto 2
               end if
               G = R * RADIX
    3          if (C.gt.G) then
                  F = F/RADIX
                  C = C/SQRDX
                  goto 3
               end if
               if ((C+R)/F .lt. 0.95*S) then
                  LAST = 0
                  G = 1.0/F
                  do 12 J = 1,N
                     A(I,J) = A(I,J)*G
   12             continue
                  do 13 J = 1,N
                     A(J,I) = A(J,I)*F
   13             continue
               end if
            end if
   14    continue
         if (LAST.EQ.0) goto 1
* --- end of Balanc
      return
      end

* --- reduction to Hessenberg form (eigen values unchanged!) --------- *
* --- see Numerical Recipes, chapt. 11.5, p.368 ---------------------- *
      subroutine ElmHes(A,N,NP)
      implicit none
      integer I, J, N, M, NP
      real*8  A(NP,NP), X, Y
* --- N <= 2  => matrix is a Hessenberg one
      if (N.gt.2) then
* ------ loop for rows
         do 17 M = 2,N-1
            X = 0.0
            I = M
* --------- finding the pivot
            do 11 J = M,N
               if (abs(A(J,M-1)) .gt. abs(X)) then
                  X = A(J,M-1)
                  I = J
               end if
   11       continue
* --------- interchange rows and columns
            if (I.ne.M) then
               do 12 J = M-1,N
                  Y      = A(I,J)
                  A(I,J) = A(M,J)
                  A(M,J) = Y
   12          continue
               do 13 J = 1,N
                  Y      = A(J,I)
                  A(J,I) = A(J,M)
                  A(J,M) = Y
   13          continue
            end if
* --------- carry out the elimination
            if (X.ne.0.0) then
               do 16 I = M+1, N
                  Y = A(I,M-1)
                  if (Y.ne.0.0) then
                     Y = Y/X
                     A(I,M-1) = Y
                     do 14 J = M,N
                        A(I,J) = A(I,J) - Y*A(M,J)
   14                continue
                     do 15 J = 1,N
                        A(J,M) = A(J,M) + Y*A(J,I)
   15                continue
                  end if
   16          continue
            end if
   17    continue
      end if
* --- end of ElmHes
      return
      end

* --- computing the eigenvalues of an upper Hessenberg matrix -------- *
* --- see Numerical Recipes, chapt. 11.6, pp374 ---------------------- *
      subroutine HQR(A,N,NP,WR,WI)
      implicit none
      integer I, ITS, J, K, L, M, MM, N, NN, NP
      real*8 A(NP,NP), WR(NP), WI(NP), ANorm
      real*8 T, X, Y, Z, U, V, W
      real*8 R, S, P, Q
* --- compute matrix norm for possible use in locating single small
* --- subdiagonal elements
      ANorm = abs(A(1,1))
      do 12 I = 2,N
         do 11 J = I-1,N
            ANorm = ANorm + abs(A(I,J))
   11    continue
   12 continue
      NN = N
      T = 0.0
* --- ...gets changed only by exceptional shift
* --- Begin search for next eigenvalue
    1 if (NN.GE.1) then
         ITS = 0
C ===    write(*,*) 'Starting the iterations'
    2    do 13 L = NN,2,-1
            S = abs(A(L-1,L-1)) + abs(A(L,L))
            if (S.EQ.0.0) S = ANorm
            if (abs(A(L,L-1))+S.EQ.S) goto 3
   13    continue
         L = 1
    3    X = A(NN,NN)
* ------ One root found...
         if (L.EQ.NN) then
C ===       write(*,*) 'One root found ... '
C ===       write(*,*) X+T, 0.0
            WR(NN) = X + T
            WI(NN) = 0.0
            NN = NN-1
         else
C ===       write(*,*) 'More than one root.'
            Y = A(NN-1,NN-1)
            W = A(NN,NN-1) * A(NN-1,NN)
* --------- Two roots found...
            if (L.EQ.NN-1) then
C ===          write(*,*) 'Two roots found ... '
               P = 0.5*(Y-X)
               Q = P**2 + W
               Z = sqrt(abs(Q))
               X = X + T
* ------------ ...a real pair
               if (Q.GE.0.0) then
C ===             write(*,*) 'A real pair:'
                  Z = P + sign(Z,P)
C ===             write(*,*) X+Z, 0.0
C ===             write(*,*) X+Z, 0.0
                  WR(NN)   = X + Z
                  WR(NN-1) = WR(NN)
                  if (Z.ne.0.0) WR(NN) = X-W/Z
                  WI(NN)   = 0.0
                  WI(NN-1) = 0.0
* ------------ ...a complex conjugate pair
               else
C ===             write(*,*) 'A conjugate pair:'
C ===             write(*,*) X+P, Z
C ===             write(*,*) X+P, -Z
                  WR(NN)   = X + P
                  WR(NN-1) = WR(NN)
                  WI(NN)   =  Z
                  WI(NN-1)   = -Z
               end if
               NN = NN - 2
* --------- No roots found. continue iteration
            else
C ===          write(*,*) 'No roots found.'
               if (ITS.EQ.100) STOP 'too many iterations'
* ------------ Form exceptional shift
               if (ITS.EQ.10 .OR. ITS.EQ.20) then
                  T = T + X
                  do 14 I = 1,NN
                     A(I,I) = A(I,I) - X
   14             continue
                  S = abs(A(NN,NN-1))+abs(A(NN-1,NN-2))
                  X = 0.75*S
                  Y = X
                  W = -0.4375*S**2
               end if
               ITS = ITS + 1
* ------------ Form shift and then look for 2 consecutive small subdiag. el.
               do 15 M = NN-2,L,-1
                  Z = A(M,M)
                  R = X - Z
                  S = Y - Z
                  P = (R*S-W)/A(M+1,M)+A(M,M+1)
                  Q = A(M+1,M+1)-Z-R-S
                  R = A(M+2,M+1)
* --------------- scale to prevent overflow or underflow
                  S = abs(P)+abs(Q)+abs(R)
                  P = P/S
                  Q = Q/S
                  R = R/S
                  if (M.EQ.L) goto 4
                  U = abs(A(M,M-1)) * (abs(Q)+abs(R))
                  V = abs(P)*(abs(A(M-1,M-1))+abs(Z)+abs(A(M+1,M+1)))
                  if (U+V.EQ.V) goto 4
   15          continue
    4          do 16 I = M+2,NN
                  A(I,I-2) = 0.0
                  if (I.ne.M+2) A(I,I-3) = 0.0
   16          continue
* ------------ double QR step on rows L to NN and columns M to NN
               do 19 K = M,NN-1
                  if (K.ne.M) then
* ------------------ Begin setup of Householder vector
                     P = A(K,K-1)
                     Q = A(K+1,K-1)
                     R = 0.0
                     if (K.ne.NN-1) R = A(K+2,K-1)
                     X = abs(P) + abs(Q) + abs(R)
                     if (X.ne.0.0) then
                        P = P/X
                        Q = Q/X
                        R = R/X
                     end if
                  end if
                  S = sign(sqrt(P**2+Q**2+R**2),P)
                  if (S.ne.0.0) then
                     if (K.EQ.M) then
                        if (L.ne.M) A(K,K-1) = -A(K,K-1)
                     else
                        A(K,K-1) = -S*X
                     end if
                     P = P + S
                     X = P/S
                     Y = Q/S
                     Z = R/S
                     Q = Q/P
                     R = R/P
* ------------------ Row modification.
                     do 17 J = K,NN
                        P = A(K,J) + Q*A(K+1,J)
                        if (K.ne.NN-1) then
                           P = P + R*A(K+2,J)
                           A(K+2,J) = A(K+2,J) - P*Z
                        end if
                        A(K+1,J) = A(K+1,J) - P*Y
                        A(K,J)   = A(K,J)   - P*X
   17                continue
* ------------------ Column modification.
                     do 18 I = L,min(NN,K+3)
                        P = X*A(I,K) + Y*A(I,K+1)
                        if (K.ne.NN-1) then
                           P = P + Z*A(I,K+2)
                           A(I,K+2) = A(I,K+2) - P*R
                        end if
                        A(I,K+1) = A(I,K+1) - P*Q
                        A(I,K)   = A(I,K)   - P
   18                continue
                  end if
   19          continue
* ------------ ...for next iteration on current eigenvalue
               goto 2
            end if
         end if
* --- ... for next eigenvalue
      goto 1
      end if
* --- end of HQR
      return
      end

* === eigenvector of an eigenvalue =================================== *
      subroutine EigVec(X, A, B, EVal, N, NP)
      implicit none
      integer I, J, K, N, NP, NXP
      parameter (NXP=2020)
      real*8     Norm, Norm0
      complex*16 A(NP,NP), X(NP), EVal, Sum
      complex*16 B(NP,NP), X0(NXP)
      real*8     Eps, Diff
      integer    Iter, ItMax
      parameter (Eps = 1.0E-4, ItMax = 100)
* --- warning
      if (NP.gt.NXP) stop "EigVec: NP > NXP=2020"
* --- nearly random initial value for eigenvector
      Iter = 0
      Norm = sqrt(1.0*N)
      do I = 1, N
         if (IMAG(EVal).NE.0.0) then
            X(I) = cmplx(cos(I*0.2),sin(I*0.1)) / Norm
         else
            X(I) = cmplx(1.0*(-1)**I,0.0) / Norm
         end if
      end do
      Norm = 1.0     
* --- computing the matrix of the linear system to be solved
* --- S x_{i+1} = (A-lambda) x_{i+1} = x_i, 
* --- where x_i = i-th approx. for the eigenvector    
      do I = 1, N
         A(I,I) = A(I,I) - EVal + 10*Eps  ! +Eps to avoid a sing. matrix
      end do
* --- computing the inverse matrix S^{-1}
      call CMatInv(A,B,N,NP)
* --- starting point of iteration
    1 continue
      Iter = Iter + 1
* --- saving the old approximation for the eigenvector
      do I = 1, N
         X0(I) = X(I)
      end do
* --- computing the new eigenvector  x_{i+1} = (A-l)^{-1} x_i
      do I = 1, N
         Sum = 0.0
         do J = 1, N
            Sum = Sum + B(I,J)*X0(J)
         end do
         X(I) = Sum
      end do
* --- normalization of the eigenvector (for next run)
      Norm = 0.0
      do I = 1, N
         Norm = Norm + ABS(X(I))**2
      end do
      Norm = sqrt(Norm)
      do I = 1, N
         X(I) = X(I) / Norm
      end do
      Norm = 1.0
* --- quality of eigenvector
      Diff = 0.0
      do I = 1, N
         Diff = Diff + abs(X(I)-X0(I))**2
      end do
      Diff = sqrt(Diff)
* --- a new iteration necessary?
      WRITE(*,'(1X,I3,A,F12.5)')
     &     Iter, 'th iteration;  norm(x_{i}-x_{i+1}) = ',Diff
      if (Iter.LT.ItMax .AND. Diff.GT.Eps) goto 1
* --- end of EigVec
      return
      end

************************************************************************
* --- sorting the eigenvalues (w.r.t. to Re(lambda)) ----------------  *
      subroutine SortEV(EVRe,EVIm,N)
      implicit none
      integer N, I, J
      real*4  EVRe(N), EVIm(N), X, Y
      logical change
* --- 
      change = .TRUE.
      do while (change)
         change = .FALSE.
         do I = 1, N-1
            if (EVRe(I).lt.EVRe(I+1)) then
               X         = EVRe(I)
               Y         = EVIm(I)
               EVRe(I)   = EVRe(I+1)
               EVIm(I)   = EVIm(I+1)
               EVRe(I+1) = X
               EVIm(I+1) = Y
               change = .TRUE.
            end if
         end do
      end do
* --- end of SortEV
      return
      end

************************************************************************
* --- spectral decomposition of a real symmetric matrix -------------- *
* --- based on Jacobi rotations (see "Numerical Recipes", sec. 11.1, p. 346
* --- A : real symmetric matrix, of upper diagonal elements will be destroyed
* --- D : returns the eigenvalues
* --- V : returns the corresponding eigenvectors
* --- NRot : returns the amount of Jacobi rotations
* ---
      subroutine Jacobi(A,M,MP,D,V,NRot)
      implicit none 
      integer I, J, M, MP, NXP, NRot
      parameter (NXP=2020)
      real*8  A(MP,MP), D(MP), V(MP,MP), B(NXP), Z(NXP)
      integer IP, IQ
      real*8  SM, Tresh, G, H, C, S, T, Tau, Theta
* ---
      if (MP.gt.NXP) stop "Jacobi: NP > NXP=2020"
* --- Initialize the identity matrix V <- I.
      do IP = 1, M
         do IQ = 1, M
            V(IP,IQ) = 0.0
         end do
         V(IP,IP) = 1.0
      end do
* --- Initialize B, D, Z.
      do IP = 1, M
         B(IP) = A(IP,IP)              ! Store diagonal elements
         D(IP) = B(IP)
         Z(IP) = 0.0                   ! Z sums up shifts in the diagonal
      end do
* --- Loop over Jacobi rotations
      NRot = 0
      do I = 1, 50
* ------ Compute sum of off-diagonal elements
         SM = 0.0
         do IP = 1, M-1
         do IQ = IP+1, M
            SM = SM + abs(A(IP,IQ))
         end do
         end do
C XXX    WRITE(*,'(1h a,i3,a,f10.3)') 'S(',I,') = ',SM
         if (SM.eq.0.0) return
* ------ Initializate for the first 4 sweeps ... and thereafter.
         if (I.lt.4) then
            Tresh = 0.2*SM/(M*M)
         else
            Tresh = 0.0
         end if
* ------ Begin the sweep.
         do IP = 1, M-1
         do IQ = IP+1, M
* --------- Is the Jacobi rotation really necessary?
            G = 100.0 * abs(A(IP,IQ))
            if (I.gt.4.and.(abs(D(IP))+G .eq. abs(D(IP)))
     &                .and.(abs(D(IQ))+G .eq. abs(D(IQ)))) then
               A(IP,IQ) = 0.0          ! A-elem. = zero to machine precision
            else if (abs(A(IP,IQ)).gt.Tresh) then
               H = D(IQ) - D(IP)
* ------------ Compute Jacobi matrix.
               if (abs(H)+G.eq.abs(H)) then
                  T = A(IP,IQ)/H                 ! = tan(Phi)
               else 
                  Theta = 0.5*H/A(IP,IQ)         ! = cot(2 Phi)
                  T     = 1.0/(abs(Theta)+sqrt(1.0+Theta**2))
                  if (Theta.lt.0.0) T = -T
               end if
               C = 1.0 /sqrt(1+T*T)
               S = T*C
* ------------ Effect the Jacobi rotation.
               Tau   = S/(1.0+C)
               H     = T*A(IP,IQ)
               Z(IP) = Z(IP) - H
               Z(IQ) = Z(IQ) + H
               D(IP) = D(IP) - H
               D(IQ) = D(IQ) + H
               A(IP,IQ) = 0.0
               do J = 1, IP-1
                  G = A(J,IP)
                  H = A(J,IQ)
                  A(J,IP) = G - S*(H+G*Tau)
                  A(J,IQ) = H + S*(G-H*Tau)
               end do
               do J = IP+1, IQ-1
                  G = A(IP,J)
                  H = A(J,IQ)
                  A(IP,J) = G - S*(H+G*Tau)
                  A(J,IQ) = H + S*(G-H*Tau)
               end do
               do J = IQ+1, M
                  G = A(IP,J)
                  H = A(IQ,J)
                  A(IP,J) = G - S*(H+G*Tau)
                  A(IQ,J) = H + S*(G-H*Tau)
               end do
* ------------ Update eigenvectors
               do J = 1, M
                  G = V(J,IP)
                  H = V(J,IQ)
                  V(J,IP) = G - S*(H+G*Tau)
                  V(J,IQ) = H + S*(G-H*Tau)
               end do
* ------------ This is the end of this rotation.
               NRot = NRot + 1
            end if
* ------ End of the matrix operations in this sweep.
         end do                        ! Loop over columns
         end do                        ! Loop over rows
* ------ Update B, D, Z
         do IP = 1, M
            B(IP) = B(IP) + Z(IP)
            D(IP) = B(IP)
            Z(IP) = 0.0
         end do
* --- New sweep!
      end do 
* --- end of Jacobi
      return
      end

************************************************************************
* --- sorting the eigenvalues (w.r.t. to Re(lambda)) ----------------- *
      subroutine SortEVal(EValRe,EValIm,N)
      implicit none
      integer N, I, J
      real*8  EValRe(N), EValIm(N), X, Y
      logical change
* --- 
      change = .TRUE.
      do while (change)
         change = .FALSE.
         do I = 1, N-1
            if (EValRe(I).lt.EValRe(I+1) .or.
     &          EValRe(I).eq.EValRe(I+1).and.
     &          EValIm(I).lt.EValIm(I+1)) then
               X           = EValRe(I)
               Y           = EValIm(I)
               EValRe(I)   = EValRe(I+1)
               EValIm(I)   = EValIm(I+1)
               EValRe(I+1) = X
               EValIm(I+1) = Y
               change = .TRUE.
            end if
         end do
      end do
* --- end of SortEVal
      return
      end

************************************************************************
* --- sorting the real eigenvalues (& eigenvectors) ------------------ *
      subroutine SortEE(EigVal,EigVec,N,NP)   ! read: Real EigenValue Sort
      implicit none
      integer I, J, N,  NP
      real*8  EigVal(N), EigVec(NP,N), EV
      logical Change
* ---
    1 Change = .false.
      do I = 2, N
         if (EigVal(I-1).lt.EigVal(I)) then
            EV          = EigVal(I-1)
            EigVal(I-1) = EigVal(I)
            EigVal(I)   = EV
            do J = 1, N
               EV            = EigVec(J,I-1)
               EigVec(J,I-1) = EigVec(J,I)
               EigVec(J,I)   = EV
            end do
            Change = .true.
         end if
      end do
      if (Change) goto 1
* --- end of SortEE
      return
      end

************************************************************************
* === Compute spectral matrix decomposition of symm. matrix ========== *
      subroutine SpecDecSym(MatrA,MatrEVec,EVal,N,NP)
* --- MatrA: NxN matrix
* --- MatrV: Matrix with eigenvectors als columns
* ---        ordered by decreasing eigenvalues
* --- EVal:  (real) eigenvalues (decreasing with "i")
      implicit none
      integer I,J,K, N,NP,NRot,NXP
      parameter (NXP=2020)
      real*8  MatrA(NP, NP),  MatrEVec(NP,NP), EVal(NP)
      real*8  MatrB(NXP,NXP), Sum, Sum1, Sum2
* ---
      if (NP.gt.NXP) stop "SpecDecSym: NP > NXP=2020"
      if (N .gt.NXP) stop 'SpecDecSym: dimension error'
* --- Initialization ------------------------------------------------- *
      do J = 1, N
      do I = 1, N
         MatrB(I,J) = MatrA(I,J)
      end do
      end do
* --- User info ------------------------------------------------------ *
      write(*,*) 'Spectral decomposition of a symmetric matrix'
* --- Computation ---------------------------------------------------- *
      call Jacobi(MatrA,N,NP,EVal,MatrEVec,NRot)
      call SortEE(EVal,MatrEVec,N,NP)
      write(*,'(a,i5)') '# Jacobi rotations = ',NRot      
      write(*,'(a)')    'lambda_n  & eigenvectors (in columns)'
      write(*,'(a5,6i10)')   'n=   ', (I,I=1,min(6,N))
      write(*,'(a5,6f10.5)') 'l_n= ', (EVal(I),I=1,min(6,N))
      do I = 1, N
         write(*,'(i5,6f10.5)') I, (MatrEVec(I,J),J=1,min(6,N))
      end do
* --- Test eigenvector ----------------------------------------------- *
* --- Loop over eigenvectors
      do K = 1, N
         Sum1 = 0.0
* ------ Loop over components
         do I = 1, N
            Sum2 = -EVal(K)*MatrEVec(I,K)
            do J = 1, N
               Sum2 = Sum2 + MatrB(I,J)*MatrEVec(J,K)
            end do
            Sum1 = max(Sum1,abs(Sum2))
* ------ End of loop over components
         end do
         write(*,'(i5,a,f12.5)') K, ')  ||A v-lambda v||_max=',Sum1
* --- End of loop over eigenvectors
      end do
* --- end of subroutine
      return
      end


************************************************************************
* === Gram-Schmidt Orthogonormalization from correlation matrix ====== *
      subroutine GramSchmidt(MatrCorr,MatrTran,N,NP)
* --- MatrCorr: NxN correlation matrix (u_i,u_j) (bilinear form)
* --- MatrTran: NxN transformation matrix
* --- such that u_i^* = T_ij * u_j , T_ij=0 at i<j ! upper triag.
* --- and (u_i^*, u_j^*) = delta_ij. 
* --- (Gram-Schmidt orthonormalization)
      implicit none
      integer I, J, K, L, N, NP 
      real*8  MatrCorr(NP,NP), MatrTran(NP,NP)
      real*8  R11, R21, R22, alpha,  Eps, Sum
      parameter (Eps=1.0D-5)

***** WARNING: MatrCorr must be positive definite.
***** In case of unexpected results, check this positive definiteness!

* --- Initialization ------------------------------------------------- *
      do J = 1, N
         do I = 1, N
            MatrTran(I,J) = 0.0
         end do
         MatrTran(J,J) = 1.D0
      end do
      call matout(MatrTran,N,NP,N,NP)
* --- Orthogonormalization ------------------------------------------- *
      do K = 1,N
* ------ Orthogonalization step [effective at K>=2] ------------------ *
         do L = 1,K-1
* ......... "K" is the counter for the Gram Schmidt iteration
* ......... "L" is the counter for the projection
* ......... The offdiagonal element is at (J,I)
            R11 =  MatrCorr(L,L)   ! from orthogonalized vector
            R22 =  MatrCorr(K,K)   ! from vector to be orthogonalized
            R21 =  MatrCorr(K,L)
* ......... cos(u_i,u_j)>Eps => post-orthogonalize!
            if (R21*R21.gt.max(0.D0,Eps*Eps*R11*R22)) then
               alpha = -R21/R11
            else 
               alpha = 0.0
            end if
* ......... Update correlation matrix (u_i^*,u_j^*)
            MatrCorr(K,K) = R22 + 2*alpha*R21 + alpha**2 * R11
            do I = 1, N
               if (I.ne.K) then
                  MatrCorr(I,K) = MatrCorr(I,K) +alpha * MatrCorr(I,L)
                  MatrCorr(K,I) = MatrCorr(I,K)
               end if
            end do
* ......... Update transformation matrix
            do I = 1,N
               MatrTran(K,I) = MatrTran(K,I) + alpha*MatrTran(L,I)
            end do
         end do
* ------ normalization step u_j^* ->  u_j^* / alpha ------------------ *
* ...... Determine norm of "k"-th vector
         alpha =  sqrt(max(0.0,MatrCorr(K,K)))
* ...... Update correlation matrix for normalized vector
         if (alpha.gt.Eps) then
            do I = 1,N
               MatrCorr(I,K)= MatrCorr(I,K)/alpha 
               MatrCorr(K,I)= MatrCorr(K,I)/alpha
            end do 
* ...... Update transformation matrix for normalized vector
*            / a   0 \
*        T=  \ 0   1 /
            do I = 1, N
               MatrTran(K,I) = MatrTran(K,I)/alpha
            end do
         write(*,*) K,alpha
         end if
* --- End of loop
      end do
* === user output ==================================================== *
      write(*,*) 'Transformed matrix T Q T^t:'
      call matout(MatrCorr,N,NP,N,NP)
      write(*,*) 'Transformation matrix T:'
      call matout(MatrTran,N,NP,N,NP)
* --- end of subroutine ---------------------------------------------- *
      return
      end
         

         
      







