C$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
C$$$$ Segment with subroutines for real*8 linear algebra $$$$$$$$$$$$$$$
C$$$$ created by Bernd R. Noack, 24Mar92 $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
C$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
C === call Gauss(A,N,NP)
C === call GaussJ(A,N,NP,B,M,MP)
C === real*8 = DET(A,N,NP)
C === call MatInv(A, AINV, N, NP)
C === call LUDcmp(A,N,NP,Indx,D)
C === call LUBksb(A,N,NP,Indx,B)
C === call QUDec(A,M,MP,N,D,Err)
C === call GSO(A,N,NP,M,MP,IPar)
C === call MatOut(A,M,MP,N,NP)
************************************************************************
* --- reduction of a general matrix to upper-triangular form --------- *
      subroutine Gauss(A,N,NP)
      implicit none
      integer N, NP
      integer I, J, K, L, LL, ICol, IRow, IRow0
      real*8 Big, Eps
      real*8 A(NP,NP)
      real*8 PIVINV, Dum
      parameter (Eps = 1.0e-5)
* --- main loop over columns
      IRow0 = 0
      do ICol = 1,N
         IRow0 = IRow0 + 1
* ------ outer loop of search for a pivot element (with largest abs. value)
*        in the same column
         Big = 0.0
         do J = 1,N
            if (abs(A(J,ICol)).gt.Big) then
               Big = abs(A(J,ICol))
               IRow = J
            end if
         end do
* ------ exchaning the rows so that pivot element is at the right place
         if (IRow.gt.IRow0) then
            do L = 1,N
               Dum        = A(IRow,L)
               A(IRow, L) = A(IRow0,L)
               A(IRow0,L) = Dum
            end do
         end if
* ------ dividing pivot row by pivot element
         if (abs(A(IRow0,ICol)).gt.Eps) then
            PIVINV = 1.0 / A(IRow0,ICol)
            A(IRow0,ICol) = 1.0
            do L = ICol+1,N
               A(IRow0,L) = A(IRow0,L) * PIVINV
            end do
* --------- producing zeros in the column below IRow0
            do L = 1,N
               if (L.gt.IRow0) then
                  Dum = A(L,ICol)
                  A(L,ICol) = 0.0
                  do J = ICol+1, N
                     A(L,J) = A(L,J) - A(IRow0,J) * Dum
                  end do
               end if
            end do
         else
            IRow0 = IRow0 - 1
            do L = IRow0, N
               A(L,ICol) = 0.0
            end do
         end if
* ------ transformation which yields a unit vector in the IColth column
      end do
* --- end of Gauss
      return
      end

* --- Gauss-Jordan elimination --------------------------------------- *
* --- subroutine from Numerical Receipes, chapt. 2.1, p.28  ---------- *
      subroutine GaussJ(A,N,NP,B,M,MP)
      implicit none
      integer N, NP, M, MP, NMax
      parameter (NMax=2020)
      integer IndxC(NMax), IndxR(NMax), IPIV(NMax)
      integer I, J, K, L, LL, IRow, ICol
      real*8 Big
      real*8 A(NP,NP), B(NP,MP)
      real*8 PIVINV, Dum
* ---
      do 11 J = 1,N
         IPIV(J) = 0
   11 continue
* --- main loop over columns
      do 22 I = 1,N
         Big = 0.0
* ------ outer loop of search for a pivot element (with largest abs. value)
*        index of pivot element (IRow, ICol)
         do 13 J = 1,N
            if (IPIV(J).NE.1) then
               do 12 K = 1,N
                  if (IPIV(K).EQ.0) then
                     if (abs(A(J,K)).GE.Big) then
                        Big = abs(A(J,K))
                        IRow = J
                        ICol = K
                     end if
                  else if (IPIV(K).gt.1) then
                     STOP 'Greetings from CGaussJ: singular matrix!'
                  end if
   12          continue
            end if
   13    continue
* ------ exchaning the rows so that pivot element is on diagonal
         if (IRow.NE.ICol) then
            do 14 L = 1,N
               Dum       = A(IRow,L)
               A(IRow,L) = A(ICol,L)
               A(ICol,L) = Dum
   14       continue
            do 15 L = 1,M
               Dum       = B(IRow,L)
               B(IRow,L) = B(ICol,L)
               B(ICol,L) = Dum
   15       continue
         end if
*        bookkeeping Ith row -> IRowth row
*                    Ith column is pivoted
         IndxR(I) = IRow
         IndxC(I) = ICol
* ------ dividing pivot Row by pivot element
         if (A(ICol,ICol).EQ.0.0)
     &             STOP 'Greetings from CGaussJ: singular matrix!'
         PIVINV = 1.0 / A(ICol,ICol)
         A(ICol,ICol) = 1.0
         do 16 L = 1,N
            A(ICol,L) = A(ICol,L) * PIVINV
   16    continue
         do 17 L = 1,M
            B(ICol,L) = B(ICol,L) * PIVINV
   17    continue
* ------ transformation which yields a unit vector in the IColth column
         do 21 LL = 1,N
            if (LL.NE.ICol) then
               Dum = A(LL,ICol)
               A(LL,ICol) = 0.0
               do 18 L = 1,N
                  A(LL,L) = A(LL,L) - A(ICol,L) * Dum
   18          continue
               do 19 L = 1,M
                  B(LL,L) = B(LL,L) - B(ICol,L) * Dum
   19          continue
            end if
   21    continue
   22 continue
* --- unscrambling with respect to column interchanges
      do 24 L = N,1,-1
         if (IndxR(L).NE.IndxC(L)) then
            do 23 K = 1,N
               Dum = A(K,IndxR(L))
               A(K,IndxR(L)) = A(K,IndxC(L))
               A(K,IndxC(L)) = Dum
   23       continue
         end if
   24 continue
* --- end of GaussJ
      return
      end

* --- determinant of real*8 matrix ---------------------------------- *
      real*8 FUNCTION DET(A,N,NP)
      implicit none
      integer N, NP
      integer I, J, K, L, LL, ICol, IRow, IRow0
      real*8  Big, Eps
      real*8 A(NP,NP), FACTOR
      real*8 PIVINV, Dum
      parameter (Eps = 1.0d-5)
* --- main loop over columns
      FACTOR = 1.0  ! no scaling of a row of the matrix, yet.
      IRow0 = 0
      do ICol = 1,N
         IRow0 = IRow0 + 1
* ------ outer loop of search for a pivot element (with largest abs. value)
*        in the same column
         IRow = ICol
         Big  = A(ICol,ICol)
         do J = ICol,N   ! <=== BRN020129 corrected!!! before:1,N
            if (abs(A(J,ICol)).gt.Big) then
               Big = abs(A(J,ICol))
               IRow = J
            end if
         end do
* ------ exchaning the rows so that pivot element is at the right place
         if (IRow.gt.IRow0) then
            do L = 1,N
               Dum        = A(IRow,L)
               A(IRow, L) = A(IRow0,L)
               A(IRow0,L) = Dum
            end do
         end if
* ------ dividing pivot row by pivot element
         if (abs(A(IRow0,ICol)).gt.Eps) then
            PIVINV = 1.0 / A(IRow0,ICol)
            FACTOR = FACTOR * A(IRow0,ICol)
            A(IRow0,ICol) = 1.0
            do L = ICol+1,N
               A(IRow0,L) = A(IRow0,L) * PIVINV
            end do
* --------- producing zeros in the column below IRow0
            do L = 1,N
               if (L.gt.IRow0) then
                  Dum = A(L,ICol)
                  A(L,ICol) = 0.0
                  do J = ICol+1, N
                     A(L,J) = A(L,J) - A(IRow0,J) * Dum
                  end do
               end if
            end do
         else
            IRow0 = IRow0 - 1
            do L = IRow0, N
               A(L,ICol) = 0.0
            end do
         end if
      end do
* --- determining the determinant 
* --- (previous scalings X product of diagonal elements)
      do I = 1, N
         FACTOR = FACTOR * A(I,I)
      end do
      DET = FACTOR
* --- end of DET
      return
      end

* --- matrix inversion ----------------------------------------------- *
      subroutine MatInv(A, AINV, N, NP)
      implicit none
      integer N, NP, I, J, NMax
      parameter (NMax=2020)
      integer Indx(NMax)
      real*8 A(NP, NP), AINV(NP, NP)
      real*8 D
* --- identity matrix
      do 12 I = 1,N
         do 11 J = 1,N
            AINV(I,J) = 0.0
   11    continue
         AINV(I,I) = 1.0
   12 continue
      call LUDcmp(A,N,NP,Indx,D)
      do 13 J = 1,N
         call LUBksb(A,N,NP,Indx,AINV(1,J))
   13 continue
* --- end of MatInv
      return
      end

* --- LU - decomposition -----------------------------------------------
      subroutine LUDcmp(A,N,NP,Indx,D)
      implicit none
      integer N, NP, NMax, I, J, K, IMax
      real*8  AAMax, TINY, Dum, D
      parameter (NMax=2020,TINY=1.0e-20)
      real*8 VV(NMax)
      real*8 A(NP,NP), CDum
      real*8 Sum
      integer Indx(N)
      D = 1.        ! No rows interchanged yet.
      do 12 I = 1,N ! loop over the Rows to get the implicit scaling information
         AAMax = 0.
         do 11 J = 1,N
            if (abs(A(I,J)).gt.AAMax) AAMax=abs(A(I,J))
   11    continue
         if (AAMax.EQ.0.) 
     &      STOP 'Greetings from CLUDcmp: singular matrix!'
         VV(I) = 1./AAMax
   12 continue
      do 19 J = 1,N
         do 14 I = 1,J-1
            Sum = A(I,J)
            do 13 K = 1,I-1
               Sum = Sum -A(I,K)*A(K,J)
   13       continue
            A(I,J) = Sum
   14    continue
         AAMax = 0.0
         do 16 I = J,N
            Sum = A(I,J)
            do 15 K = 1,J-1
               Sum = Sum-A(I,K)*A(K,J)
   15       continue
            A(I,J) = Sum
            Dum = VV(I)*abs(Sum)
            if (Dum.GE.AAMax) then
               IMax = I
               AAMax = Dum
            end if
   16    continue
         if (J.NE.IMax) then
            do 17 K = 1,N
               CDum      = A(IMax,K)
               A(IMax,K) = A(J,K)
               A(J,K)    = CDum
   17       continue
            D = -D
            VV(IMax) = VV(J)
         end if
         Indx(J) = IMax
         if (A(J,J).EQ.0) A(J,J) = TINY
         if (J.NE.N) then
            CDum = 1.0/A(J,J)
            do 18 I = J+1,N
               A(I,J) = A(I,J)*CDum
   18       continue
         end if
   19 continue
* --- end of LUDcmp
      return
      end

* --- Back substitution ------------------------------------------------
      subroutine LUBksb(A,N,NP,Indx,B)
      implicit none
      integer N, NP, Indx(N), I, J, II, LL
      real*8 A(NP,NP), B(N), Sum
      II = 0
      do 12 I = 1,N
         LL = Indx(I)
         Sum = B(LL)
         B(LL) = B(I)
         if (II.NE.0) then
            do 11 J = II,I-1
               Sum = Sum - A(I,J)*B(J)
   11       continue
         else if (Sum.NE.0.0) then
            II = I
         end if
         B(I) = Sum
   12 continue
      do 14 I = N,1,-1
         Sum = B(I)
         if (I.lt.N) then
            do 13 J = I+1,N
               Sum = Sum -A(I,J)*B(J)
   13       continue
         end if
         B(I) = Sum / A(I,I)
   14 continue
* --- end of LUBksb
      return
      end

************************************************************************
* --- QU decomposition of a matrix using the Householder algorithm --- *
* --- The Householder-"u"s of Q are stored in a_{ij}, i<=j.
* --- The diagonal elements of "U" are returned in "D",
* --- the upper elements in "A".
      subroutine QUDec(A,M,MP,N,D,Err)
      implicit none 
      integer I, J, K, M, MP, MMax, N, Err
      parameter (MMax = 2020)
      real*8  A(MP,N), U(MMax), Z, D(N), Sum
      real*8  S, Sigma, Beta, Eps
      parameter (Eps = 3.0e-5)
* --- Loop over the columns
      Err = 0
      do J = 1, N
* ------ Compute the norm of "x_1".
         Sigma = 0.0
         do I = J, N
            Sigma = Sigma + A(I,J)**2
         end do
         if (Sigma.le.Eps) then
            Err =  0
            goto 1
         else
            Sigma = sqrt(Sigma)
         end if
* ------ Compute the diagonal element.
         S    = abs(A(J,J))
         if (S.gt.Eps) then            ! a_jj = Z abs(a_jj); Z =exp(i Alpha)
            Z = A(J,J) / S
         else
            Z = 1.0
         end if
         D(J)   = -Z*Sigma             ! k = - Sigma Z
         A(J,J) = Z*(S+Sigma)          ! u_j comp. of Householder-"u"
         Beta = 1.0 / (Sigma * (Sigma + S))
* ------ Effect the Householder transformation
         do K = J+1, N
* --------- Compute y^H_j = beta_j u^H_j A^{(j-1)}
            Sum = 0.0
            do I = J, N
               Sum = Sum + A(I,J)*A(I,K)     ! u^H_j A^{(j-1)} part
            end do
            Sum = Beta * Sum
* --------- Compute P_j A^{(j-1)} = A^{(j-1)} - u_j y^H_j
            do I = J, N
               A(I,K) = A(I,K) - Sum * A(I,J)
            end do
         end do
      end do
* --- end of QUDec
    1 continue
      return
      end

***********************************************************************
* --- Gram-Schmidt orthonormalization -------------------------------- *
      subroutine GSO(A,N,NP,M,MP,IPar)
* --- N : dimension of vectors to be orthonormalized
* --- M : amount    of vectors to be orthonormalized
* --- IPar: >0 orthonormalization; <=0 orthogonalization
      implicit none
      integer I, J, K, N, NP, M, MP, MX, IPar
      parameter (MX=2020)
      real*8  A(NP,MP), Sum, ScaPro(MX), Norm2(MX), Eps
      parameter (Eps = 0.001)
* --- 
C DEB write(*,'(1h 4(a,i3))') 
C DEB&         'GSO with N=',N,'; M=',M,'; NP=',NP,'; MP=',MP
* --- Loop over column vectors v_i
      do I = 1, M
* ------ Compute < v_k,v_i > 
         do K = 1, I-1
            Sum = 0.0
            do J = 1, N
               Sum = Sum + A(J,K)*A(J,I)
            end do
            ScaPro(K) = Sum
         end do
* ------ Compute v_i = v_i - Sum_k <v_k,v_i> v_k / <v_k,v_k>
* ------ parallel to the space spanned by the orthogonalized vectors
         do K = 1, I-1
         do J = 1, N
            A(J,I) = A(J,I) - ScaPro(K)*A(J,K)/Norm2(K)
         end do
         end do
* ------ Compute ||v_i|| and normalize v_i
         Sum = 0.0
         do J = 1, N
            Sum = Sum + A(J,I)**2
         end do
         if (IPar.gt.0) then
           Sum = sqrt(Sum)
           do J = 1, N
               A(J,I) = A(J,I) / Sum
            end do
            Norm2(I) = 1.0
         else
            Norm2(I) = Sum
         end if
* ------ Check orthonormality
         do K = 1, I
            Sum = 0.0
            if (K.eq.I) Sum = -Norm2(I)
            do J = 1, N
               Sum = Sum + A(J,K)*A(J,I)
            end do
            if (abs(Sum).gt.Eps) then
               write(*,'(1h a,i3,a,f8.3,a)') 
     &                  'GRS: numerical error in v_{',I,'} = ',Sum,'!'
               stop 'GRS: numerical error too large!'
            end if
         end do
* --- End of loop over column vectors v_i
      end do
* --- End of GSO
      return
      end

* --- matrix output -------------------------------------------------- *
      subroutine MatOut(A,M,MP,N,NP)
      implicit none
      integer I, K, K0, K1, M, N, MP, NP
      real*8  A(1:MP,1:NP)
* --- showing groups of 8 columns for all lines
      K0 = 1
      K1 = min(N,8)
   10 continue
      write(*,1010) (K,K=K0,K1)
      do 100 I = 1, M
            write(*,1020) I, (A(I,K),K=K0,K1)
  100 continue
      if (K1.lt.N) then
         K0 = K0 + 8
         K1 = min(N,K1 + 8)
         write(*,1030)
         goto 10
      end if
* --- some formats
 1010 format('i/j', i5,7i9)
 1020 format(i3  , 8(f8.4,1x))
 1030 format('matrix - continued:')
* --- end of subroutine MatOut
      return
      end
