************************************************************************
*****                                                              *****
***** @segment name: xs2u_gxCORESub.f                              *****
*****                                                              *****
***** @task: core routines for Galerkin modelling                  *****
*****        for subdomain                                         *****
*****                                                              *****
***** @see: res.20100408.xAMC_ToDo.f                               *****
*****       Based on linear interpolation of flow field            ***** 
*****                                                              *****
************************************************************************
*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012
************************************************************************

*----------------------------------------------------------------------*
*     real*8 function fFctInnerProduct     (fFlowG,fFlow1G)
*----------------------------------------------------------------------*
*     real*8 function fFctInnerProductSub  (fFlowSubG,fFlow1SubG)
*     real*8 function fFctViscousTermSub   (fFlowSubG,fFlow1SubG)
*     real*8 function fFctConvectiveTermSub(fFlowSubG,fFlow1SubG,fFlow2SubG)
*     real*8 function fFctForceTermSub     (fFlowSubG,fForceSubG)
*     real*8 function fFctPressureTermSub  (fFlowSubG,fFieldSubG)
*----------------------------------------------------------------------*
*     real*8 function fFctAreaTriangle (xVal,yVal)
*     real*8 function fFctGaussTriangle(xVal,yVal,fVal)
*     subroutine      compGradTriangle(xVal,yVal,fVal,fx,fy)
*----------------------------------------------------------------------*

************************************************************************
* === inner product ================================================== *
      real*8 function fFctInnerProduct(fFlowG,fFlow1G)
      implicit none
* --- flow fields
      include 'xv3X.include'           ! NX, ..., NMesh, ...
      include 'xv3X_Grid.include'      ! xyzG
      include 'xv3X_Flow.include'      ! fFlowG
      include 'xv3X_Flow1.include'     ! fFlow1G
* --- auxiliary variables
      real*8  xVal(3), yVal(3), fVal(3), Sum, fFctGaussTriangle
      integer iX, jX, kX, iMesh
* --- initialize
      Sum = 0
* --- loop over triangles
      do iMesh = 1, NMesh
* ------ determine triangle
         iX = indexMesh(1,iMesh)
         jX = indexMesh(2,iMesh)
         kX = indexMesh(3,iMesh)
         xVal(1) = xyzG(iX,1,1,1)
         xVal(2) = xyzG(jX,1,1,1)
         xVal(3) = xyzG(kX,1,1,1)
         yVal(1) = xyzG(iX,1,1,2)
         yVal(2) = xyzG(jX,1,1,2)
         yVal(3) = xyzG(kX,1,1,2)
* ------ determine integrand
         fVal(1) = fFlowG(iX,1,1,1)*fFlow1G(iX,1,1,1)
     &           + fFlowG(iX,1,1,2)*fFlow1G(iX,1,1,2)
         fVal(2) = fFlowG(jX,1,1,1)*fFlow1G(jX,1,1,1)
     &           + fFlowG(jX,1,1,2)*fFlow1G(jX,1,1,2)
         fVal(3) = fFlowG(kX,1,1,1)*fFlow1G(kX,1,1,1)
     &           + fFlowG(kX,1,1,2)*fFlow1G(kX,1,1,2)
* -----
         Sum = Sum + fFctGaussTriangle(xVal,yVal,fVal)
* --- end of loop over triangles
      end do
      fFctInnerProduct = Sum
* --- end of function
      end

************************************************************************
* === inner product ================================================== *
      real*8 function fFctInnerProductSub(fFlowSubG,fFlow1SubG)
      implicit none
* --- flow fields
      include 'xv3X.include'           ! NXSub, ..., NMeshSub, ...
      include 'xv3X_GridSub.include'   ! xyzSubG
      include 'xv3X_FlowSub.include'   ! fFlowSubG
      include 'xv3X_Flow1Sub.include'  ! fFlow1SubG
* --- auxiliary variables
      real*8  xVal(3), yVal(3), fVal(3), Sum, fFctGaussTriangle
      integer iXSub, jXSub, kXSub, iMesh
* --- initialize
      Sum = 0
* --- loop over triangles
      do iMesh = 1, NMeshSub
* ------ determine triangle
         iXSub = indexMeshSub(1,iMesh)
         jXSub = indexMeshSub(2,iMesh)
         kXSub = indexMeshSub(3,iMesh)
         xVal(1) = xyzSubG(iXSub,1,1,1)
         xVal(2) = xyzSubG(jXSub,1,1,1)
         xVal(3) = xyzSubG(kXSub,1,1,1)
         yVal(1) = xyzSubG(iXSub,1,1,2)
         yVal(2) = xyzSubG(jXSub,1,1,2)
         yVal(3) = xyzSubG(kXSub,1,1,2)
* ------ determine integrand
         fVal(1) = fFlowSubG(iXSub,1,1,1)*fFlow1SubG(iXSub,1,1,1)
     &           + fFlowSubG(iXSub,1,1,2)*fFlow1SubG(iXSub,1,1,2)
         fVal(2) = fFlowSubG(jXSub,1,1,1)*fFlow1SubG(jXSub,1,1,1)
     &           + fFlowSubG(jXSub,1,1,2)*fFlow1SubG(jXSub,1,1,2)
         fVal(3) = fFlowSubG(kXSub,1,1,1)*fFlow1SubG(kXSub,1,1,1)
     &           + fFlowSubG(kXSub,1,1,2)*fFlow1SubG(kXSub,1,1,2)
* -----
         Sum = Sum + fFctGaussTriangle(xVal,yVal,fVal)
* --- end of loop over triangles
      end do
      fFctInnerProductSub = Sum
* --- end of function
      end

************************************************************************
* === dissipation term =============================================== *
      real*8 function fFctViscousTermSub(fFlowSubG,fFlow1SubG)
      implicit none
* --- flow fields
      include 'xv3X.include'           ! NXSub, ..., NMeshSub, ...
      include 'xv3X_GridSub.include'    ! xyzSubG
      include 'xv3X_FlowSub.include'    ! fFlowSubG
      include 'xv3X_Flow1Sub.include'   ! fFlow1SubG
* --- auxiliary variables
      integer iXSub, jXSub, kXSub, iMesh
      real*8  xVal(3), yVal(3), fVal(3)
      real*8  u, ux, uy, vx, vy, upx, upy, vpx, vpy
      real*8  Area, fFctAreaTriangle, Sum
* --- initialize
      Sum = 0
* --- loop over triangles
      do iMesh = 1, NMeshSub
* ------ determine triangle
         iXSub = indexMeshSub(1,iMesh)
         jXSub = indexMeshSub(2,iMesh)
         kXSub = indexMeshSub(3,iMesh)
         xVal(1) = xyzSubG(iXSub,1,1,1)
         xVal(2) = xyzSubG(jXSub,1,1,1)
         xVal(3) = xyzSubG(kXSub,1,1,1)
         yVal(1) = xyzSubG(iXSub,1,1,2)
         yVal(2) = xyzSubG(jXSub,1,1,2)
         yVal(3) = xyzSubG(kXSub,1,1,2)
         Area    = fFctAreaTriangle(xVal,yVal)
* ------ determine gradients of "u", "v"
         fVal(1) = fFlowSubG(iXSub,1,1,1)
         fVal(2) = fFlowSubG(jXSub,1,1,1)
         fVal(3) = fFlowSubG(kXSub,1,1,1)
         call compGradTriangle(xVal,yVal,fVal,ux,uy)
         fVal(1) = fFlowSubG(iXSub,1,1,2)
         fVal(2) = fFlowSubG(jXSub,1,1,2)
         fVal(3) = fFlowSubG(kXSub,1,1,2)
         call compGradTriangle(xVal,yVal,fVal,vx,vy)
* ------ determine gradients of "up"
         fVal(1) = fFlow1SubG(iXSub,1,1,1)
         fVal(2) = fFlow1SubG(jXSub,1,1,1)
         fVal(3) = fFlow1SubG(kXSub,1,1,1)
         call compGradTriangle(xVal,yVal,fVal,upx,upy)
* ------ determine gradients of "vp"
         fVal(1) = fFlow1SubG(iXSub,1,1,2)
         fVal(2) = fFlow1SubG(jXSub,1,1,2)
         fVal(3) = fFlow1SubG(kXSub,1,1,2)
         call compGradTriangle(xVal,yVal,fVal,vpx,vpy)
* ------ Add viscous term (after Green's formula).
* ------ Add boundary integral later.
         Sum = Sum + Area*(ux*upx + uy*upy + vx*vpx + vy*vpy)
* --- end of loop over triangles
      end do
      fFctViscousTermSub = -Sum ! Note the "-" sign!!!
* --- end of function
      end

************************************************************************
* === convection term ================================================ *
      real*8 function fFctConvectiveTermSub(fFlowSubG,
     &                                      fFlow1SubG,fFlow2SubG)
      implicit none
* --- flow fields
      include 'xv3X.include'           ! NXSub, ..., NMeshSub, ...
      include 'xv3X_GridSub.include'   ! xyzSubG
      include 'xv3X_FlowSub.include'   ! fFlowSubG
      include 'xv3X_Flow1Sub.include'  ! fFlow1SubG
      include 'xv3X_Flow2Sub.include'  ! fFlow2SubG
* --- auxiliary variables
      integer iXSub, jXSub, kXSub, iMesh
      real*8  xVal(3), yVal(3), fVal(3)
      real*8  u, v, uu, uv, vv
      real*8  uux, uuy, vvx, vvy, uvx, uvy, vux, vuy
      real*8  Area, fFctAreaTriangle, Sum
* --- initialize
      Sum = 0
* --- loop over triangles
      do iMesh = 1, NMeshSub
* ------ determine triangle
         iXSub = indexMeshSub(1,iMesh)
         jXSub = indexMeshSub(2,iMesh)
         kXSub = indexMeshSub(3,iMesh)
         xVal(1) = xyzSubG(iXSub,1,1,1)
         xVal(2) = xyzSubG(jXSub,1,1,1)
         xVal(3) = xyzSubG(kXSub,1,1,1)
         yVal(1) = xyzSubG(iXSub,1,1,2)
         yVal(2) = xyzSubG(jXSub,1,1,2)
         yVal(3) = xyzSubG(kXSub,1,1,2)
         Area    = fFctAreaTriangle(xVal,yVal)
* ------ determine "u"
         u = (fFlowSubG(iXSub,1,1,1)+fFlowSubG(jXSub,1,1,1)
     &                          +fFlowSubG(kXSub,1,1,1))/3.D0
* ------ determine "v"
         v = (fFlowSubG(iXSub,1,1,2)+fFlowSubG(jXSub,1,1,2)
     &                          +fFlowSubG(kXSub,1,1,2))/3.D0
* ------ determine gradients of "uu"
         fVal(1) = fFlow1SubG(iXSub,1,1,1)*fFlow2SubG(iXSub,1,1,1)
         fVal(2) = fFlow1SubG(jXSub,1,1,1)*fFlow2SubG(jXSub,1,1,1)
         fVal(3) = fFlow1SubG(kXSub,1,1,1)*fFlow2SubG(kXSub,1,1,1)
         call compGradTriangle(xVal,yVal,fVal,uux,uuy)
* ------ determine gradients of "vv"
         fVal(1) = fFlow1SubG(iXSub,1,1,2)*fFlow2SubG(iXSub,1,1,2)
         fVal(2) = fFlow1SubG(jXSub,1,1,2)*fFlow2SubG(jXSub,1,1,2)
         fVal(3) = fFlow1SubG(kXSub,1,1,2)*fFlow2SubG(kXSub,1,1,2)
         call compGradTriangle(xVal,yVal,fVal,vvx,vvy)
* ------ determine gradients of "uv"
         fVal(1) = fFlow1SubG(iXSub,1,1,1)*fFlow2SubG(iXSub,1,1,2)
         fVal(2) = fFlow1SubG(jXSub,1,1,1)*fFlow2SubG(jXSub,1,1,2)
         fVal(3) = fFlow1SubG(kXSub,1,1,1)*fFlow2SubG(kXSub,1,1,2)
         call compGradTriangle(xVal,yVal,fVal,uvx,uvy)
* ------ determine gradients of "vu"
         fVal(1) = fFlow1SubG(iXSub,1,1,2)*fFlow2SubG(iXSub,1,1,1)
         fVal(2) = fFlow1SubG(jXSub,1,1,2)*fFlow2SubG(jXSub,1,1,1)
         fVal(3) = fFlow1SubG(kXSub,1,1,2)*fFlow2SubG(kXSub,1,1,1)
         call compGradTriangle(xVal,yVal,fVal,vux,vuy)
* ------ add convective term
         Sum = Sum + Area*(u*uux + u*vuy + v*vvy + v*uvx)
* --- end of loop over triangles
      end do
      fFctConvectiveTermSub = -Sum
* --- end of function
      end

************************************************************************
* === force [need really needed because of fFctInnerProductSub) ====== *
      real*8 function fFctForceTermSub(fFlowSubG,fForceSubG)
      implicit none
* --- flow fields
      include 'xv3X.include'           ! NXSub, ..., NMeshSub, ...
      include 'xv3X_GridSub.include'   ! xyzSubG
      include 'xv3X_FlowSub.include'   ! fFlowSubG
      include 'xv3X_ForceSub.include'  ! fForceSubG
* --- auxiliary variables
      real*8  xVal(3), yVal(3), uVal(3), vVal(3), fVal(3)
      real*8  Sum, fFctGaussTriangle
      real*8  xc, yc
      integer iXSub, jXSub, kXSub, iMesh, iVal
* --- initialize
      Sum = 0
* --- loop over triangles
      do iMesh = 1, NMeshSub
* ------ determine triangle
         iXSub = indexMeshSub(1,iMesh)
         jXSub = indexMeshSub(2,iMesh)
         kXSub = indexMeshSub(3,iMesh)
         xVal(1) = xyzSubG(iXSub,1,1,1)
         xVal(2) = xyzSubG(jXSub,1,1,1)
         xVal(3) = xyzSubG(kXSub,1,1,1)

         yVal(1) = xyzSubG(iXSub,1,1,2)
         yVal(2) = xyzSubG(jXSub,1,1,2)
         yVal(3) = xyzSubG(kXSub,1,1,2)

* ------ determine integrand on triangle
         fVal(1) = fFlowSubG(iXSub,1,1,1)*fForceSubG(iXSub,1,1,1)
     &           + fFlowSubG(iXSub,1,1,2)*fForceSubG(iXSub,1,1,2)
         fVal(2) = fFlowSubG(jXSub,1,1,1)*fForceSubG(jXSub,1,1,1)
     &           + fFlowSubG(jXSub,1,1,2)*fForceSubG(jXSub,1,1,2)
         fVal(3) = fFlowSubG(kXSub,1,1,1)*fForceSubG(kXSub,1,1,1)
     &           + fFlowSubG(kXSub,1,1,2)*fForceSubG(kXSub,1,1,2)
* ------ add volume force term term
         Sum = Sum + fFctGaussTriangle(xVal,yVal,fVal)
* --- end of loop over triangles
      end do

      fFctForceTermSub = Sum
* --- end of function
      end

************************************************************************
* === pressure term ================================================== *
      real*8 function fFctPressureTermSub(fFlowSubG,fFieldSubG)
      implicit none
* --- flow fields
      include 'xv3X.include'           ! NXSub, ..., NMeshSub, ...
      include 'xv3X_GridSub.include'   ! xyzSubG
      include 'xv3X_FlowSub.include'   ! fFlowSubG
      include 'xv3X_FieldSub.include'  ! fFieldSubG
* --- auxiliary variables
      integer iXSub, jXSub, kXSub, iMesh
      real*8  xVal(3), yVal(3), fVal(3)
      real*8  u, v, p, px, py
      real*8  Area, fFctAreaTriangle, Sum
* ---
      Sum = 0
* --- loop over triangles
      do iMesh = 1, NMeshSub
* ------ determine triangle
         iXSub = indexMeshSub(1,iMesh)
         jXSub = indexMeshSub(2,iMesh)
         kXSub = indexMeshSub(3,iMesh)
         xVal(1) = xyzSubG(iXSub,1,1,1)
         xVal(2) = xyzSubG(jXSub,1,1,1)
         xVal(3) = xyzSubG(kXSub,1,1,1)
         yVal(1) = xyzSubG(iXSub,1,1,2)
         yVal(2) = xyzSubG(jXSub,1,1,2)
         yVal(3) = xyzSubG(kXSub,1,1,2)
* ------ determine "u"
         u = (fFlowSubG(iXSub,1,1,1)+fFlowSubG(jXSub,1,1,1)
     &                          +fFlowSubG(kXSub,1,1,1))/3.D0
* ------ determine "v"
         v = (fFlowSubG(iXSub,1,1,2)+fFlowSubG(jXSub,1,1,2)
     &                          +fFlowSubG(kXSub,1,1,2))/3.D0
* ------ determine gradients of "p"
         fVal(1) = fFieldSubG(iXSub,1,1)
         fVal(2) = fFieldSubG(jXSub,1,1)
         fVal(3) = fFieldSubG(kXSub,1,1)
         call compGradTriangle(xVal,yVal,fVal,px,py)
* ------ add pressure term
         Area    = fFctAreaTriangle(xVal,yVal)
* ------ 
         Sum = Sum + Area*( - u*px - v*py)
* --- end of loop over triangles
      end do
      fFctPressureTermSub = Sum
* --- end of function
      end

************************************************************************
* === area of triangle (from cross-product) ========================== *
      real*8 function fFctAreaTriangle(xVal,yVal)
      implicit none
      real*8  xVal(3), yVal(3)
      real*8  dx1, dx2, dy1, dy2
* ---
      dx1 = xVal(2)-xVal(1)
      dx2 = xVal(3)-xVal(1)
      dy1 = yVal(2)-yVal(1)
      dy2 = yVal(3)-yVal(1)
      fFctAreaTriangle = abs(dx1*dy2-dx2*dy1)/2.D0
* --- end of function
      end

************************************************************************
* === Gauss integration (Here: 3pt formula) ========================== *
      real*8 function fFctGaussTriangle(xVal,yVal,fVal)
      implicit none
      real*8  xVal(3), yVal(3), fVal(3),  fFctAreaTriangle
* ---
      fFctGaussTriangle=fFctAreaTriangle(xVal,yVal)
     &                 *(fVal(1)+fVal(2)+fVal(3))/3.D0
* --- end of function
      end

************************************************************************
* === Gradient of scalar from data on triangle ======================= *
      subroutine compGradTriangle(xVal,yVal,fVal,fx,fy)
      implicit none
      real*8  xVal(3), yVal(3), fVal(3), fx, fy
      real*8  dx1, dx2, df1, dy1, dy2, df2, nx, ny, nz
* --- the gradient is derived from x,y projection of the cross product.
*     P1 -dx1--> P2
*       \
*        dx2
*          \
*          _|
*            P3 
* ---
      dx1 = xVal(2)-xVal(1)
      dx2 = xVal(3)-xVal(1)
      dy1 = yVal(2)-yVal(1)
      dy2 = yVal(3)-yVal(1)
      df1 = fVal(2)-fVal(1)
      df2 = fVal(3)-fVal(1)
      nx  = dy1*df2-df1*dy2
      ny  = df1*dx2-dx1*df2
      nz  = dx1*dy2-dy1*dx2
      nx = nx/nz
      ny = ny/nz
      nz = nz/nz
      fx = -nx
      fy = -ny
* --- end of function
      return
      end
