************************************************************************
*****                                                              *****
***** @segment name: xs3X_ioFlow.f                                 *****
***** @task: read/write flow for 2D unstructured grid              *****
***** @see:  header of res.20100408.xAMC_ToDo.tex                  *****
*****                                                              *****
************************************************************************
* === subroutine readFlow(fn,fnLength,fFlowG)
* === subroutine writeFlow(fn,fnLength,fFlowG)
*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012

************************************************************************ 
* === read the flow of a 2D unstructured grid ======================== *
      subroutine readFlow(fn,fnLength,fFlowG)
      implicit none
* --- input parameters
      character*120 fn        ! file name
      integer       fnLength  ! length of file name (# of characters)
* --- variables for grid and flow
      include 'xv3X.include'            ! NX, ...
      include 'xv3X_Flow.include'       ! fFlowG
* --- auxiliary dummy variables
      integer iX, iY, iZ      ! counter for grid nodes
      integer iDummy          ! dummy variable
* --- begin of segment ----------------------------------------------- *
* --- initialize pseudo 3D format
* >>> THIS IS ASSUMED TO HAVE BEEN DONE BY << readGrid >>
C XXX NY = 1
C XXX NZ = 1
* --- read the flow
      write(*,*) 'Reading the flow from <<', fn(1:fnLength),' >> ...'
* --- open the file
      open(10, file =fn(1:fnLength), status='old', form='formatted')
* --- read grid nodes
      read(10,*) NX
      write(*,*) 'Reading ',NX,' nodes ...'
      do iZ = 1, NZ
      do iY = 1, NY
      do iX = 1, NX
        read(10,*) (fFlowG(iX,iY,iZ,iDummy), iDummy=1,NVarFlow)
      end do
      end do
      end do
* --- close the file
      close(10)
* --- end of segment ------------------------------------------------- *
      return
      end

************************************************************************ 
* === write the flow on a 2D unstructured grid ======================= *
      subroutine writeFlow(fn,fnLength,fFlowG)
      implicit none
* --- input parameters for the file name
      character*120 fn        ! file name
      integer       fnLength  ! length of file name (# of characters)
* --- variables for grid and flow
      include 'xv3X.include'            ! NX, ...
      include 'xv3X_Flow.include'       ! fFlowG
* --- auxiliary dummy variables
      integer iX, iY, iZ      ! counter for grid nodes
      integer iDummy          ! counter for variables
* --- begin of segment ----------------------------------------------- *
* --- read the flow
      write(*,*) 'Writing the flow in <<', fn(1:fnLength),' >> ...'
* --- open the file
      open(10, file =fn(1:fnLength), status='UNKNOWN', form='formatted')
* --- read grid nodes
      write(10,*) NX,NY,NZ
      write(*,*) 'Writing ',NX,' nodes ...'
      do iZ = 1, NZ
      do iY = 1, NY
      do iX = 1, NX
        write(10,*) (fFlowG(iX,iY,iZ,iDummy), iDummy=1,NVarFlow)
      end do
      end do
      end do
* --- close the file
      close(10)
* --- end of segment ------------------------------------------------- *
      return
      end
