************************************************************************
*****                                                              *****
***** @segment name: xs2u_ioGridSub.f                              *****
***** @task: read grid for 2D unstructured grid (subdomain)        *****
***** @see:  header of res.20100408.xAMC_ToDo.tex                  *****
*****                                                              *****
************************************************************************
*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012
************************************************************************

************************************************************************ 
* === read the 2D unstructured grid ================================== *
      subroutine readGridSub(iDomain)
      implicit none
* --- input parameter
      integer       iDomain             ! domain number
* --- variables for grid
      include 'xv3X.include'   ! NXSub, NYSub, ...
      include 'xv3X_GridSub.include'    ! xyzSubG
* --- auxiliary variables for file name
      character*120 fn        ! file name
      integer       fnLength  ! length of file name (# of characters)
* --- auxiliary variables for readLine
      character*120 Line
      logical       bOK
* --- auxiliary dummy variables
      integer iXSub, iYSub, iZSub       ! counter for grid nodes
      integer iDummy                    ! counter for variables
      integer iMeshSub, iIndexSub   ! counters for triangular mesh
* --- begin of segment ----------------------------------------------- *
* --- initialize pseudo 3D format
      NYSub = 1
      NZSub = 1
* --- read file name of the grid
      call readFilenameDomain(iDomain,fn,fnLength)
      write(*,'(3a)') 'Locating the subgrid from <<',
     &                 fn(1:fnLength),'>>...'
      open(10,file=fn(1:fnLength),status='old',form='formatted')
      call readLine(10,Line,bOK)
      close(10)
* --- read the grid
      write(*,*) 'Reading the subgrid <<', fn(1:fnLength),' >> ...'
* --- open the file
      open(10, file =fn(1:fnLength), status='old', form='formatted')
* --- read grid nodes
      read(10,*) NXSub
      write(*,*) 'Reading ',NXSub,' nodes ...'
      do iZSub = 1, NZSub
      do iYSub = 1, NYSub
      do iXSub = 1, NXSub
        read(10,*) (xyzSubG(iXSub,iYSub,iZSub,iDummy),
     &                                        iDummy=1,NVarGridSub)
      end do
      end do
      end do
* --- reading the triangular mesh
      read(10,*) NMeshSub
      write(*,*) 'Reading ',NMeshSub,' triangles...'
      do iMeshSub = 1, NMeshSub
        read(10,*) (indexMeshSub(iIndexSub,iMeshSub),
     &             iIndexSub=1,3)
      end do
* --- close the file
      close(10)
* --- end of segment ------------------------------------------------- *
      return
      end
