************************************************************************
*****                                                              *****
***** @segment name: xs2u_plotGrid.f                               *****
***** @task:         plot flow for 2D unstructured grid            *****
***** @dependency:   requires gs2d_PostScript.f                    *****
***** @see:          res.20100408.xAMC_ToDo.tex                    *****
*****                                                              *****
************************************************************************
* === subroutine plotFlow(ug)
* ===  |_____ compPsi2u  (fFlowG,fFieldG)
* ===  |_____ plotField2u(fn,fnLength,fFieldG)
*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012
************************************************************************

************************************************************************
* === plot scalar field ============================================== *
      subroutine plotFlow(fn,fnLength,fFlowG)
      implicit none
* --- file name
      character*120 fn
      integer       fnLength
* --- variables for grid and field
      include 'xv3X.include'           ! NX, ...
      include 'xv3X_Grid.include'      ! xG, yG
      include 'xv3X_Flow.include'      ! fFlowG
      include 'xv3X_Field.include'     ! fFieldG
* --- auxiliary variables
      integer iCh, iChMin
* === begin of segment =============================================== *
      write(*,*) 'Plotting the flow ...'
* --- remove directory from file name (assuming a single '/') 
      iChMin = 1
      do iCh = fnLength,1,-1
        if (fn(iCh:iCh).eq.'/') iChMin = iCh+1
      end do
* --- add PS extension
      write(*,*) '****************************************'
      fn       = fn(iChMin:fnLength)//"_u.ps"
      fnLength = fnLength + 5 - iChMin + 1
      call plotField2u(fn,fnLength,fFlowG(1,1,1,1))
* ---
      write(*,*) '****************************************'
      fn(fnLength-3:fnLength-3) = 'v'
      call plotField2u(fn,fnLength,fFlowG(1,1,1,2))
* ---
      write(*,*) '****************************************'
      call compPsi2u(fFlowG,fFieldG)
      fn(fnLength-3:fnLength-3) = 'S'
      call plotField2u(fn,fnLength,fFieldG)
* ---
      write(*,*) '3 plots generated:'
      call system('ls *.ps')

* === end of segment ================================================= *
      return
      end

************************************************************************
* === plot scalar field ============================================== *
      subroutine plotField2u(fn,fnLength,fFieldG)
      implicit none
* --- file name
      character*120 fn
      integer       fnLength

* --- variables for grid and field
      include 'xv3X.include'           ! NX, ..., NMesh, ...
      include 'xv3X_Grid.include'      ! xG, yG
      include 'xv3X_Field.include'     ! fFieldG

* --- variables for grid visualization
      integer iGrid, jGrid, iMesh   ! dummy counters
      real*8  xGMin, xGMax, yGMin, yGMax, fgMin, fgMax ! data bounds
      real*8  xC, yC, R ! circle

* --- variables for graphics
      include 'gv2d_PostScript.include'
      real*8  Scale
      real*8  xPC0, xPC1, yPC0, yPC1, xPS, yPS ! for BB & captions
      character*80 Strg                        ! for caption
      real*8  xVal(4), yVal(4), fVal(4), fCL   ! for contour plot
      integer iCL, NCL
      real*8  Width0

* === begin of segment =============================================== *
      write(*,*) 'Plotting the field ...'
* --- initialize the graphics ---------------------------------------- *
* --- Compute bounds on selected triangles
      xGMin =  1.E37
      xGMax = -1.E37
      yGMin =  1.E37
      yGMax = -1.E37
      fgMin =  1.E37
      fgMax = -1.E37
      
      do iMesh = 1, NMesh
      do iGrid = 1, 3
         jGrid = indexMesh(iGrid,iMesh)
         xGMin = min(xGMin,xyzG(jGrid,1,1,1))
         xGMax = max(xGMax,xyzG(jGrid,1,1,1))
         yGMin = min(yGMin,xyzG(jGrid,1,1,2))
         yGMax = max(yGMax,xyzG(jGrid,1,1,2))
         fgMin = min(fgMin,fFieldG(jGrid,1,1))
         fgMax = max(fgMax,fFieldG(jGrid,1,1))

      end do
      end do
      write(*,*) 'Extremal x,y & f values ...'
      write(*,'(6f12.5)') xGMin,xGMax,yGMin,yGMax,fgMin,fgMax

* --- determine bounding box
      Scale = 500./max(xGMax-xGMin,yGMax-yGMin)
      xPS0 = 20.
      xPS1 = 20+Scale*(xGMax-xGMin)
      yPS0 = 20.
      yPS1 = 20+Scale*(yGMax-yGMin)
      call specBB(xPS0,xPS1,yPS0,yPS1) ! for header

* --- initialize graphics (Here xPS0, ... are re-initialized)
      call initNamedFig(1,fn,fnLength)

* --- initialize data region
      Scale = 5000./max(xGMax-xGMin,yGMax-yGMin)
      xPS0 = 200.
      xPS1 = 200.+Scale*(xGMax-xGMin)
      yPS0 = 200.
      yPS1 = 200+Scale*(yGMax-yGMin)
      call specDR(xPS0,xPS1,yPS0,yPS1)

* --- plot bounding box
      WidthFrame = 5
      WidthCurve = 3
      call specXAxis(xGMin,xGMax)
      call specYAxis(yGMin,yGMax)
      call plotFrame(1)

* --- figure caption ------------------------------------------------- *
      fCL=abs(fgMin)
*             123456789012
      Strg = '{Min= 00.000}'
      if (fgMin.lt.0) Strg(6:6)='-'
      Strg(7:7)=char(ichar('0')+mod(int(fCL/10),10))
      Strg(8:8)=char(ichar('0')+mod(int(fCL),10))
      fCL=fCL-int(fCL)
      Strg(10:10)=char(ichar('0')+mod(int(fCL*10),10))
      Strg(11:11)=char(ichar('0')+mod(int(fCL*100),10))
      Strg(12:12)=char(ichar('0')+mod(int(fCL*1000),10))
      SizeLabel=80.0
      xps= 0.2*SizeLabel+200.0
      yps=-1.2*SizeLabel+200.0+Scale*(yGMax-yGMin)
      call PlotStrg(1,Strg,xps,yps,0,-1)
      Width0 = WidthCurve      
      fCL=abs(fgMax)
*             123456789012
      Strg = '{Max= 00.000}'
      if (fgMax.lt.0) Strg(6:6)='-'
      Strg(7:7)=char(ichar('0')+mod(int(fCL/10),10))
      Strg(8:8)=char(ichar('0')+mod(int(fCL),10))
      fCL=fCL-int(fCL)
      Strg(10:10)=char(ichar('0')+mod(int(fCL*10),10))
      Strg(11:11)=char(ichar('0')+mod(int(fCL*100),10))
      Strg(12:12)=char(ichar('0')+mod(int(fCL*1000),10))
      SizeLabel=80.0
      xPS= 0.2*SizeLabel+200.0
      yPS=-2.2*SizeLabel+200.0+Scale*(yGMax-yGMin)
      call PlotStrg(1,Strg,xPS,yPS,0,-1)
      Width0 = WidthCurve

* --- contour plot --------------------------------------------------- *
      write(*,'(1h 6(a,f7.3),a)') 'Contour plot in [',
     &     xGMin,';',xGMax,'] x [',
     &     yGMin,';',yGMax,'] x [',
     &     fgMin,';',fgMax,'] ...' 
* --- plot triangles
      do iMesh = 1, NMesh
         do iGrid = 1, 3
            jGrid = indexMesh(1+mod(iGrid-1,3),iMesh) ! 1->2->3->1
            xVal(iGrid) =  xyzG(jGrid,1,1,1)
            yVal(iGrid) =  xyzG(jGrid,1,1,2)
            fVal(iGrid) =  fFieldG(jGrid,1,1)
         end do
         NCL = 10
         do iCL = 0, NCL
            fCL = ((NCL-iCL)*fgMin + iCL*fgMax)/NCL
            if (fCL.gt.0.) then
               WidthCurve =     Width0
            else
               WidthCurve = 0.2*Width0
            end if 
            call PlotCLTriangle(1,xVal,yVal,fVal,fCL)
         end do
C        call PlotCurve(1,xVal,yVal,3)
      end do
* --- cylinder ------------------------------------------------------- *
      call PlotCircle(1,0.D0,0.D0,0.5D0,1.D0)
* --- Terminate graphics --------------------------------------------- *
      call TermFig(1)
* === end of subroutine ============================================== *
      return
      end


************************************************************************
* === Compute stream function ======================================== *
      subroutine compPsi2u(fFlowG,fFieldG)
      implicit none
      include 'xv3X.include'            ! NX, ..., NMesh, ...
      include 'xv3X_Grid.include'       ! xG, yG
      include 'xv3X_Flow.include'       ! fFlowG
      include 'xv3X_Field.include'      ! fFieldG
      integer iGrid, iMesh, iLoop
      integer iPnt, jPnt, kPnt, NPnt, NPntP
      parameter (NPntP=6)
      real*8  xVal(NPntP), yVal(NPntP)
      real*8  uVal(NPntP), vVal(NPntP), PsiVal(NPntP), Psi2Val(NPntP)
      real*8  dx, dy, ds, n_x, n_y, uc, vc, dPsiVal(3), dPsiMax
      real*8  fx, fy
* --- initialization ------------------------------------------------- *
* --- determine number of points (needed for discretization)
      NPnt = 3
* --- initialization
      uc = 0
      vc = 0
      do iGrid = 1, NX
         uc = uc + fFlowG(iGrid,1,1,1) / NX
         vc = vc + fFlowG(iGrid,1,1,2) / NX
      end do
      write(*,'(a,2f12.5)') '(u,v)_av = ', uc, vc
* --- initialize stream function
      do iGrid = 1, NX
         fFieldG(iGrid,1,1) = 0                               ! e.g. K-L mode
         if (uc.gt.0.5) fFieldG(iGrid,1,1) = fFieldG(iGrid,1,1)
     &                  + xyzG(iGrid,1,1,2) ! e.g. u_0
         if (vc.gt.0.5) fFieldG(iGrid,1,1) = fFieldG(iGrid,1,1)
     &                  - xyzG(iGrid,1,1,1) ! e.g. Act. mode
* ------ enforce BC
         if (xyzG(iGrid,1,1,1)**2
     &       +xyzG(iGrid,1,1,2)**2.lt.0.2501) fFieldG(iGrid,1,1) = 0
C XXX    if (abs(xyzG(iGrid,1,1,2)).gt.+4.9999) fFieldG(iGrid,1,1) = xyzG(iGrid,1,1,2)
      end do
* --- iteration loop ove mesh ---------------------------------------- *
      do iLoop = 1, 1000 ! <<<< REPLACE BY 2000 for more accurate results
         dPsiMax = 0
* ------ loop over triangles
         do iMesh = 1, NMesh
* --------- determine triangle
            do iPnt = 1, NPnt
               iGrid = indexMesh(iPnt,iMesh)
               xVal  (iPnt) = xyzG(iGrid,1,1,1)
               yVal  (iPnt) = xyzG(iGrid,1,1,2)
               uVal  (iPnt) = fFlowG (iGrid,1,1,1)
               vVal  (iPnt) = fFlowG (iGrid,1,1,2)
               PsiVal(iPnt) = fFieldG(iGrid,1,1)
            end do
* --------- determine flux through all sides
            do iPnt=1,3
               jPnt = 1 + mod(iPnt+0,3)
               kPnt = 1 + mod(iPnt+3,3) ! mid point in 6 point elements
               dx = xVal(jPnt)-xVal(iPnt)
               dy = yVal(jPnt)-yVal(iPnt)
               ds = sqrt(dx*dx+dy*dy)
               n_x =  dy/ds
               n_y = -dx/ds
               if (NPnt.eq.6) then
                  uc = (uVal(iPnt)+2*uVal(kPnt)+uVal(jPnt))/4
                  vc = (vVal(iPnt)+2*vVal(kPnt)+vVal(jPnt))/4
               else
                  uc = (uVal(iPnt)+uVal(jPnt))/2
                  vc = (vVal(iPnt)+vVal(jPnt))/2
               end if
               dPsiVal(iPnt) = ds*(n_x*uc+n_y*vc)
            end do
* --------- iteration algorithm
            do iPnt=1,3
               jPnt = 1 + mod(iPnt+0,3)
               kPnt = 1 + mod(iPnt+1,3)
               Psi2Val(iPnt)=(1.0* PsiVal(iPnt)
     &                       +1.0*(PsiVal(jPnt)-dPsiVal(iPnt))
     &                       +1.0*(PsiVal(kPnt)+dPsiVal(kPnt)))/3.
            end do
* --------- enforce symmetry on y=0
C XXX       if (max(yVal(1),yVal(2),yVal(3)).gt.0.and.
C XXX&          min(yVal(1),yVal(2),yVal(3)).lt.0) then
C XXX           call CompGrad(xVal,yVal,Psi2Val,fx,fy)
* ------------- remove x-gradient so that Psi=0 and dPsi/dx=0 on y=0
C XXX           Psi2Val(1) =   fy*yVal(1)
C XXX           Psi2Val(2) =   fy*yVal(2)
C XXX           Psi2Val(3) =   fy*yVal(3)
C XXX       end if
            do iPnt=4,NPnt
               jPnt=1+mod(iPnt-1,3)
               kPnt=1+mod(iPnt-0,3)
               Psi2Val(iPnt) = (Psi2Val(jPnt)+Psi2Val(kPnt))/2.
            end do
* --------- monitor change (only on edges!)
            do iPnt=1,3
               dPsiMax = max(dPsiMax,abs(PsiVal(iPnt)-Psi2Val(iPnt)))
            end do
* --------- update Psi
            do iPnt=1,NPnt
               iGrid     = indexMesh(iPnt,iMesh)
               fFieldG(iGrid,1,1) = Psi2Val(iPnt)
            end do
* ------ end of loop over triangles
         end do
         if (mod(iLoop,100).eq.0) then
           write(*,*) iLoop,' Psi-iteration, change=',dPsiMax
         end if
* ------ enforce BC
         do iGrid = 1, NX
            if (xyzG(iGrid,1,1,1)**2
     &         +xyzG(iGrid,1,1,2)**2.lt.0.2501) 
     &          fFieldG(iGrid,1,1) = 0
         end do
* ------ jump to end if convergence reached
         if (dPsiMax.lt.1.e-4) goto 999
* --- end of loop of mesh
      end do
* --- end of subroutine
 999  return
      end
