************************************************************************
*****                                                              *****
***** @segment name: xs2u_plotGrid.f                               *****
***** @task:         plot grid for 2D unstructured grid            *****
***** @dependency:   requires gs2d_PostScript.f                    *****
***** @see:          res.20100408.xAMC_ToDo.tex                    *****
*****                                                              *****
************************************************************************
* === subroutine plotGrid
*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012
************************************************************************

************************************************************************
* === plot 2D unstructured grid ====================================== *
      subroutine plotGrid
      implicit none
* --- variables for grid
      include 'xv3X.include'           ! NX, ...
      include 'xv3X_Grid.include'      ! xG, yG
* === NOTE THAT xv2u_*.include will be copied on xv3X_*.include before
* --- variables for grid
      integer iGrid, jGrid, iMesh
      real*8  xGMin, xGMax, yGMin, yGMax
      real*8  xC, yC, R
* --- variables for graphics
      include 'gv2d_PostScript.include'
      real*8   Scale
      real*8  xPC0, xPC1, yPC0, yPC1
      real*8  xVal(4), yVal(4)
* --- file name
      character*120 FileName
      integer       FileNameLength

      write(*,*) 'Plotting the grid...'
      write(*,*) 'NX=', NX,';  NMesh=',NMesh

* ---                   1234567890
      FileName       = 'z0_Grid.ps'
      FileNameLength = 10
* --- Compute bounds on selected triangles
      xGMin =  1.E37
      xGMax = -1.E37
      yGMin =  1.E37
      yGMax = -1.E37
      do iMesh = 1, NMesh
      do iGrid = 1, 3
         jGrid = indexMesh(iGrid,iMesh)
         xGMin = min(xGMin,xyzG(jGrid,1,1,1))
         xGMax = max(xGMax,xyzG(jGrid,1,1,1))
         yGMin = min(yGMin,xyzG(jGrid,1,1,2))
         yGMax = max(yGMax,xyzG(jGrid,1,1,2))
      end do
      end do
      write(*,*) 'Extremal x,y values ...'
      write(*,'(4f12.5)') xGMin,xGMax,yGMin,yGMax


* --- determine bounding box
      Scale = 500./max(xGMax-xGMin,yGMax-yGMin)
      xPS0 = 20.
      xPS1 = 20+Scale*(xGMax-xGMin)
      yPS0 = 20.
      yPS1 = 20+Scale*(yGMax-yGMin)
      call specBB(xPS0,xPS1,yPS0,yPS1) ! for header

* --- initialize graphics (Here xPS0, ... are re-initialized)
C XXX      call initMFig(1,1)
      call initNamedFig(1,FileName,FileNameLength)

* --- initialize data region
      Scale = 5000./max(xGMax-xGMin,yGMax-yGMin)
      xPS0 = 200.
      xPS1 = 200.+Scale*(xGMax-xGMin)
      yPS0 = 200.
      yPS1 = 200+Scale*(yGMax-yGMin)
      call specDR(xPS0,xPS1,yPS0,yPS1)

* --- plot bounding box
      WidthFrame = 5
      WidthCurve = 3
      call specXAxis(xGMin,xGMax)
      call specYAxis(yGMin,yGMax)
      call plotFrame(1)

* --- plot triangles
      do iMesh = 1, NMesh
         do iGrid = 1, 4
            jGrid = indexMesh(1+mod(iGrid-1,3),iMesh) ! 1->2->3->1
            xVal(iGrid) = xyzG(jGrid,1,1,1)
            yVal(iGrid) = xyzG(jGrid,1,1,2)
C DEB       write(*,*) iMesh, iGrid, xVal(iGrid), yVal(iGrid)
         end do
         call plotCurve(1,xVal,yVal,4)
* ------ Mark triangles whith touch the interior of the circle
* ------ Realizing that the numerical cylinder has R=0.4986
C XXX    if (min(xVal(1)**2+yVal(1)**2,
C XXX&           xVal(2)**2+yVal(2)**2,
C XXX&           xVal(3)**2+yVal(3)**2).lt.0.0**2) then
C XXX       WidthCurve=10                !  OPTION EFF. REMOVED!
C XXX       call plotCurve(1,xVal,yVal,4)
C XXX       WidthCurve=3
C XXX    end if
* -------
      end do
* --- Terminate graphics
      call TermFig(1)
* --- User info
      write(*,*) '>>> View << z0_Grid.ps >>'
* --- end of subroutine
      return
      end
