************************************************************************
*****                                                              *****
***** @segment name: xs3X_compCorrMatrSub.f                        *****
***** @task: I/O routines for correlation matrix ...               *****
***** @see:  res.20100408.xAMC_ToDo.tex                            *****
*****                                                              *****
************************************************************************
* === subroutine compMeanFlowSub(fFlowsSubG,fFlowSubG)
* === subroutine subtractMeanFlowSub(fFlowsSubG,fFlowSubG)
* === subroutine compCorrMatrSub(fFlowsSubG,fMatrix)
*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012
************************************************************************ 
* === compute mean flow ============================================== *
      subroutine compMeanFlowSub(fFlowsSubG,fFlowSubG)
      implicit none
* --- input parameters for filename and domain
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
      include 'xv3X.include' ! NXSub, ...
      include 'xv3X_FlowSub.include'   ! fFlowSubG
      include 'xv3X_FlowsSub.include'  ! fFlowsSubG
* --- auxiliary dummy variables
      integer iXSub, iYSub, iZSub      ! counter for grid nodes
      integer iDummy                   ! dummy variable
      integer iFlow                    ! counter
      real*8  fDummy                   ! norm
* --- begin of segment ----------------------------------------------- *
      write(*,*) '***** Computing the mean flow ...'
      do iDummy  = 1, NVarFlowSub
      do iZSub   = 1, NZSub
      do iYSub   = 1, NYSub
      do iXSub   = 1, NXSub
        fDummy   = 0.0
        do iFlow = 1, NFlows
          fDummy = fDummy + fFlowsSubG(iXSub,iYSub,iZSub,iDummy,iFlow)
        end do
        fFlowSubG(iXSub,iYSub,iZSub,iDummy) = fDummy/NFlows
      end do
      end do
      end do
      end do
* --- end of segment ------------------------------------------------- *
      return
      end

************************************************************************ 
* === subtract mean flow ============================================= *
      subroutine subtractMeanFlowSub(fFlowsSubG,fFlowSubG)
      implicit none
* --- input parameters for filename and domain
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
      include 'xv3X.include' ! NXSub, ...
      include 'xv3X_FlowSub.include'   ! fFlowSubG
      include 'xv3X_FlowsSub.include'  ! fFlowsSubG
* --- auxiliary dummy variables
      integer iXSub, iYSub, iZSub       ! counter for grid nodes
      integer iDummy                    ! dummy variable
      integer iFlow                     ! counter
      real*8  fDummy                    ! norm
* --- begin of segment ----------------------------------------------- *
      write(*,*) '***** Subtracting the mean flow ...'
      do iFlow  = 1, NFlows
      do iDummy = 1, NVarFlowSub
      do iZSub  = 1, NZSub
      do iYSub  = 1, NYSub
      do iXSub  = 1, NXSub
          fFlowsSubG(iXSub,iYSub,iZSub,iDummy,iFlow) =
     &    fFlowsSubG(iXSub,iYSub,iZSub,iDummy,iFlow)
     &  - fFlowSubG (iXSub,iYSub,iZSub,iDummy)
      end do
      end do
      end do
      end do
      end do
* --- end of segment ------------------------------------------------- *
      return
      end

************************************************************************ 
* === compute the correlation matrix ================================= *
      subroutine compCorrMatrSub(fFlowsSubG,fMatrix)
      implicit none
* --- input parameters for filename and domain
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
      include 'xv3X.include' ! NXSub, ...
      include 'xv3X_FlowSub.include'   ! fFlowSubG
      include 'xv3X_Flow1Sub.include'  ! fFlow1SubG
      include 'xv3X_FlowsSub.include'  ! fFlowsSubG
* --- correlation matrix
      real*8 fMatrix(NFlowsP,NFlowsP)
* --- auxiliary dummy variables
      integer iFlow, jFlow
      real*8  fFctInnerProductSub
      real*8  fDummy          ! norm
* --- begin of segment ----------------------------------------------- *
* --- initialize correlation matrix
      do iFlow = 1, NFlowsP
      do jFlow = 1, NFlowsP
        fMatrix(iFlow,jFlow) = 0.0
      end do
      end do
* --- compute correlation matrix
      write(*,*) '***** Compute correlation matrix ...'
      do iFlow = 1, NFlows
      do jFlow = 1, iFlow
         call copyFromFlowsSub(fFlowSubG,fFlowsSubG,iFlow)
         call copyFromFlowsSub(fFlow1SubG,fFlowsSubG,jFlow)
         fDummy = fFctInnerProductSub(fFlowSubG,fFlow1SubG)/NFlows
         fMatrix(iFlow,jFlow) = fDummy
         fMatrix(jFlow,iFlow) = fDummy
         write(*,*) iFlow,jFlow,fDummy
      end do
      end do
* --- end of segment ------------------------------------------------- *
      return
      end
