************************************************************************
*****                                                              *****
***** @segment name: xs3c_gxCORESub.f                              *****
*****                                                              *****
***** @task: provide routines for integration and differentiation  *****
*****        for the computation of the Galerkin system,           *****
*****        i.e. m_ij, l_ij, q_ijk, g_ij.                         *****
*****        Operates on 3D Cartesian grid.                        *****
*****                                                              *****
***** @see: header of x2GalerkinProjection3c.f                     *****
*****                                                              *****
************************************************************************
*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012
* --- These subroutines are needed in x1_GalerkinApproximation ---------
* === subroutine initIntegration
* === real*8 function fFctInnerProduct(fFlowG,fFlow2G)
* ----------------------------------------------------------------------
* === subroutine initIntegrationSub
* === real*8 function fFctInnerProductSub(fFlowSubG,fFlow2SubG)
* === real*8 function fFctViscousTermSub(fFlowSubG,fFlow2SubG)
* === real*8 function fFctConvectiveTermSub(fFlowSubG,fFlow2SubG,fFlow3SubG)


************************************************************************
* === initialize integration ========================================= *
      subroutine initIntegration
      implicit none
* --- variables for xAMC data
      include 'xv3X.include'           ! for NX, ...
      include 'xv3X_Grid.include'      ! for xG, ...
      include 'xv3X_Weight.include'    ! for fWeight
* --- auxiliary variables
      integer ix, iy, iz, iVar    ! counter for grid points
      real*8  fDummy

* --- begin of segment ----------------------------------------------- *
* --- fully 3D
      if (min(NX,NY,NZ).gt.1) then
        do iz = 1, NZ
        do iy = 1, NY
        do ix = 1, NX
          fDummy =           xG(min(NX,ix+1))-xG(max(1,ix-1))
          fDummy = fDummy * (yG(min(NY,iy+1))-yG(max(1,iy-1)))
          fDummy = fDummy * (zG(min(NZ,iz+1))-zG(max(1,iz-1)))
          fWeight(ix,iy,iz) = fDummy / (2**3)
        end do
        end do
        end do
* --- 2D (x,y)
      else if (NZ.eq.1) then
        do iz = 1, NZ
        do iy = 1, NY
        do ix = 1, NX
          fDummy =           xG(min(NX,ix+1))-xG(max(1,ix-1))
          fDummy = fDummy * (yG(min(NY,iy+1))-yG(max(1,iy-1)))
          fWeight(ix,iy,iz) = fDummy / (2**3)
        end do
        end do
        end do
      else
        stop "fFctInnerProduct: Option NX=1 or NY=1 ready!"
      end if
* --- end of segment ------------------------------------------------- *
      return
      end

************************************************************************
* === inner product ================================================== *
      real*8 function fFctInnerProduct(fFlowG,fFlow2G)
      implicit none
* --- variables for xAMC data
      include 'xv3X.include'           ! for NX, ...
      include 'xv3X_Weight.include'    ! for fWeight
      include 'xv3X_Flow.include'      ! for fFlowG
      include 'xv3X_Flow2.include'     ! for fFlow2G
* --- auxiliary variables
      integer ix, iy, iz, iVar         ! counter for grid points
      real*8  fDummy
      real*8  iInnerProductCheck
      common / cInnerProduct / iInnerProductCheck
* --- begin of segment ----------------------------------------------- *
* ... initialize during first run
      if (iInnerProductCheck.ne.123456) then
         call initIntegration 
         iInnerProductCheck = 123456
      end if
      fDummy = 0.0
      do iVar = 1, NVarFlow
      do iz   = 1, NZ
      do iy   = 1, NY
      do ix   = 1, NX
        fDummy = fDummy + fWeight(ix,iy,iz)*fFlowG (ix,iy,iz,iVar)
     &                                     *fFlow2G(ix,iy,iz,iVar)
      end do
      end do
      end do
      end do
      fFctInnerProduct = fDummy
* --- end of segment ------------------------------------------------- *
      return
      end

************************************************************************
* === initialize integration ========================================= *
      subroutine initIntegrationSub
      implicit none
* --- variables for xAMC data
      include 'xv3X.include'           ! for NXSub, ...
      include 'xv3X_GridSub.include'   ! for xSubG, ...
      include 'xv3X_WeightSub.include' ! for fWeightSub
* --- auxiliary variables
      integer ix, iy, iz, iVar    ! counter for grid points
      real*8  fDummy

* --- begin of segment ----------------------------------------------- *
        do iz = 1, NZSub
        do iy = 1, NYSub
        do ix = 1, NXSub
          fDummy =           xSubG(min(NXSub,ix+1))-xSubG(max(1,ix-1))
          fDummy = fDummy * (ySubG(min(NYSub,iy+1))-ySubG(max(1,iy-1)))
          fDummy = fDummy * (zSubG(min(NZSub,iz+1))-zSubG(max(1,iz-1)))
          fWeightSub(ix,iy,iz) = fDummy / (2**3)
        end do
        end do
        end do
* --- end of segment ------------------------------------------------- *
      return
      end

************************************************************************
* === inner product ================================================== *
      real*8 function fFctInnerProductSub(fFlowSubG,fFlow2SubG)
      implicit none
* --- variables for xAMC data
      include 'xv3X.include'           ! for NXSub, ...
      include 'xv3X_WeightSub.include' ! for fWeight
      include 'xv3X_FlowSub.include'   ! for fFlowSubG
      include 'xv3X_Flow2Sub.include'  ! for fFlow2SubG
* --- auxiliary variables
      integer ix, iy, iz, iVar         ! counter for grid points
      real*8  fDummy
      real*8  iInnerProductSubCheck
      common / cInnerProductSub / iInnerProductSubCheck
* --- begin of segment ----------------------------------------------- *
* ... initialize during first run
      if (iInnerProductSubCheck.ne.123456) then
         call initIntegrationSub 
         iInnerProductSubCheck = 123456
      end if
      fDummy = 0.0
      do iVar = 1, NVarFlowSub
      do iz   = 1, NZSub
      do iy   = 1, NYSub
      do ix   = 1, NXSub
        fDummy = fDummy + fWeightSub(ix,iy,iz)
     &                   *fFlowSubG (ix,iy,iz,iVar)
     &                   *fFlow2SubG(ix,iy,iz,iVar)
      end do
      end do
      end do
      end do
      fFctInnerProductSub = fDummy
* --- end of segment ------------------------------------------------- *
      return
      end


************************************************************************
* === viscous term (l_ij^nu)  ======================================== *
      real*8 function fFctViscousTermSub(fFlowSubG,fFlow2SubG)
      implicit none
* --- variables for xAMC data
      include 'xv3X.include'           ! for NXSub, ...
      include 'xv3X_GridSub.include'   ! for xSubG,...
      include 'xv3X_WeightSub.include' ! for fWeight
      include 'xv3X_FlowSub.include'   ! for fFlowSubG
      include 'xv3X_Flow2Sub.include'  ! for fFlow2SubG
* --- auxiliary variables for integration
      integer ix, iy, iz                       ! counter for grid points
      integer iVar, iDir                       ! counter for variables
      real*8  dVCell                           ! volume of cell
      real*8  uCell         (NVarFlowSub)      ! cell-averaged flow of fFlowSubG
      real*8  uJacobianCell (NVarFlowSub,3)    ! Jacobian of cell
      real*8  u2Cell        (NVarFlowSub)      ! cell-averaged flow fFlow2SubG
      real*8  u2JacobianCell(NVarFlowSub,3)    ! Jacobian of cell
      real*8  fDummy, ffDummy                  ! dummies
* --- begin of segment ----------------------------------------------- *
      fDummy = 0.0
* --- loop over cells (hence: ixMax = NXSub-1, ...)
      do iz   = 1, NZSub-1
      do iy   = 1, NYSub-1
      do ix   = 1, NXSub-1
* ------ compute cell-averaged 0-th and 1st derivative of u, v
         call compJacobianCell(
     &            ix,iy,iz,fFlowSubG, dVCell,uCell, uJacobianCell
     &                        )
         call compJacobianCell(
     &            ix,iy,iz,fFlow2SubG,dVCell,u2Cell,u2JacobianCell
     &                        )
C DEB    write(*,*) 'ix=',ix,'; iy=',iy,'; iz=',iz,'; dV=',dVCell
C DEB    write(*,*) 'x=',xSubG(ix),'; y=',ySubG(iy),'; z=',zSubG(iz)
C DEB    write(*,*) 'x+=',xSubG(ix+1),'; y+=',ySubG(iy+1),'; z+=',zSubG(iz+1)
* ------ compute integrand of 
* ------ << L(u,v) = Int dV (Nabla u)^t : Nabla v >>
         ffDummy = 0.0
         do iVar = 1, NVarFlowSub
         do iDir = 1, min(NVarFlowSub,3)! <<< REMOVE , only for Rist data
            ffDummy = ffDummy
     &              + uJacobianCell (iVar,iDir)
     &              * u2JacobianCell(iVar,iDir)
C DEB       write(*,'(2i5,4f12.5)') iVar,iDir, 
C DEB&                 uJacobianCell (iVar,iDir),
C DEB&                 u2JacobianCell(iVar,iDir), ffDummy, fDummy
         end do
         end do
C DEB    read(*,*)
* ------ update integral
         fDummy = fDummy + ffDummy * dVCell
      end do
      end do
      end do
* --- return value of integral
      fFctViscousTermSub =  - fDummy
* --- end of segment ------------------------------------------------- *
      return
      end


************************************************************************
* === convective term q_ijk^c  ======================================= *
      real*8 function fFctConvectiveTermSub(
     &                fFlowSubG,fFlow2SubG,fFlow3SubG)
      implicit none
* --- variables for xAMC data
      include 'xv3X.include'           ! for NXSub, ...
      include 'xv3X_WeightSub.include' ! for fWeight
      include 'xv3X_FlowSub.include'   ! for fFlowSubG
      include 'xv3X_Flow2Sub.include'  ! for fFlow2SubG
      include 'xv3X_Flow3Sub.include'  ! for fFlow3SubG
* --- auxiliary variables for integration
      integer ix, iy, iz                       ! counter for grid points
      integer iVar, iDir                       ! counter for variables
      real*8  dVCell                           ! volume of cell
      real*8  uCell         (NVarFlowSub)      ! cell-averaged flow of fFlowSubG
      real*8  uJacobianCell (NVarFlowSub,3)    ! Jacobian of cell
      real*8  u2Cell        (NVarFlowSub)      ! cell-averaged flow fFlow2SubG
      real*8  u2JacobianCell(NVarFlowSub,3)    ! Jacobian of cell
      real*8  u3Cell        (NVarFlowSub)      ! cell-averaged flow fFlow3SubG
      real*8  u3JacobianCell(NVarFlowSub,3)    ! Jacobian of cell
      real*8  fDummy, ffDummy                  ! dummies
* --- begin of segment ----------------------------------------------- *
      fDummy = 0.0
* --- loop over cells (hence: ixMax = NXSub-1, ...)
      do iz   = 1, NZSub-1
      do iy   = 1, NYSub-1
      do ix   = 1, NXSub-1
* ------ compute cell-averaged 0-th and 1st derivative of u, v, w
         call compFlowCell(
     &            ix,iy,iz,fFlowSubG, dVCell,uCell
     &                        )
         call compFlowCell(
     &            ix,iy,iz,fFlow2SubG,dVCell,u2Cell
     &                        )
         call compJacobianCell(
     &            ix,iy,iz,fFlow3SubG,dVCell,u3Cell,u3JacobianCell
     &                        )
* ------ compute integrand of 
* ------ << C(u,v,w) = Int dV u . v . nabla w >>
         ffDummy = 0.0
C DEB write(*,'(a12,3i12)')  'ix,iy,iz :',ix,iy,iz
C DEB write(*,'(a12,4f12.5)') 'x,y,z,dV:',xSubG(ix),ySubG(iy),zSubG(iz),dVCell
C DEB write(*,'(2a6,5a12)') 'iVar','iDir',
C DEB&        'u(iV)','u2(iD)','u3(iV,iD)','ffDummy','fDummy'
         do iVar = 1, NVarFlowSub
         do iDir = 1, min(NVarFlowSub,3) ! <<< REMOVE, only for Rist data
            ffDummy = ffDummy
     &              + uCell         (iVar)
     &              * u2Cell        (iDir)
     &              * u3JacobianCell(iVar,iDir)
C DEB write(*,'(2i6,5f12.5)') iVar,iDir,
C DEB&        uCell (iVar),u2Cell (iDir),
C DEB&        u3JacobianCell(iVar,iDir), ffDummy, fDummy
         end do
         end do
C DEB if (ix.eq.NXSub-1.and.iy.eq.NYSub-1.and.iz.eq.NZSub-1) read(*,*)
* ------ update integral
         fDummy = fDummy + ffDummy * dVCell
      end do
      end do
      end do
* --- return value of integral (note that du/dt = - u .nabla u ...)
      fFctConvectiveTermSub =  - fDummy
* --- end of segment ------------------------------------------------- *
      return
      end


************************************************************************
* === averaged flow in cell ========================================== *
      subroutine compFlowCell(ix,iy,iz,fFlowSubG,dVCell,uCell)
      implicit none
* --- variables for xAMC data 
      include 'xv3X.include'           ! for NXSub, ...
      include 'xv3X_GridSub.include'   ! for xSubG, ...
      include 'xv3X_FlowSub.include'   ! for fFlowSubG
* --- variables for Jacobian
      integer ix, iy, iz               ! cube (lower, left, bottom corner)
      real*8  dVCell                   ! volume of cell
      real*8  uCell(NVarFlowSub)       ! cell-averaged flow
* --- auxiliary variables
      integer iVar                     ! counter for variables
      integer ixPlus, iyPlus, izPlus   ! 
      real*8  dx, dy, dz
      real*8  fDummy
* --- begin of segment ----------------------------------------------- *
      ixPlus = ix + 1 
      iyPlus = iy + 1 
      izPlus = iz + 1 
      dx = xSubG(ixPlus)-xSubG(ix)
      dy = ySubG(iyPlus)-ySubG(iy)
      dz = zSubG(izPlus)-zSubG(iz)
      dVCell = dx*dy*dz
* --- 0-th derivative
      do iVar = 1, NVarFlowSub
        uCell(iVar) = (fFlowSubG(ix,    iy,    iz,    iVar)
     &                +  fFlowSubG(ixPlus,iy,    iz,    iVar)
     &                +  fFlowSubG(ix    ,iyPlus,iz,    iVar)
     &                +  fFlowSubG(ixPlus,iyPlus,iz,    iVar)
     &                +  fFlowSubG(ix,    iy,    izPlus,iVar)
     &                +  fFlowSubG(ixPlus,iy,    izPlus,iVar)
     &                +  fFlowSubG(ix    ,iyPlus,izPlus,iVar)
     &                +  fFlowSubG(ixPlus,iyPlus,izPlus,iVar)
     &                ) / 8.
      end do
* --- end of segment ------------------------------------------------- *
      return
      end


************************************************************************
* === compute flow and first derivatives in cell  ==================== *
      subroutine compJacobianCell(
     &               ix,iy,iz,fFlowSubG,
     &               dVCell,uCell,uJacobianCell
     &                           )
      implicit none
* --- variables for xAMC data 
      include 'xv3X.include'           ! for NXSub, ...
      include 'xv3X_GridSub.include'   ! for xSubG, ...
      include 'xv3X_FlowSub.include'   ! for fFlowSubG
* --- variables for Jacobian
      integer ix, iy, iz               ! cube (lower, left, bottom corner)
      real*8  dVCell                       ! volume of cell
      real*8  uCell        (NVarFlowSub)   ! cell-averaged flow
      real*8  uJacobianCell(NVarFlowSub,3) ! Jacobian in cell
* --- auxiliary variables
      integer iVar                         ! counter for variables
      integer ixPlus, iyPlus, izPlus       ! index of upper right top corner
      real*8  dx, dy, dz
      real*8  fDummy
* --- begin of segment ----------------------------------------------- *
      ixPlus = ix + 1 
      iyPlus = iy + 1 
      izPlus = iz + 1 
      dx = xSubG(ixPlus)-xSubG(ix)
      dy = ySubG(iyPlus)-ySubG(iy)
      dz = zSubG(izPlus)-zSubG(iz)
      dVCell = dx*dy*dz
* --- 0-th derivative
      do iVar = 1, NVarFlowSub
        uCell(iVar) = (fFlowSubG(ix,    iy,    iz,    iVar)
     &                +  fFlowSubG(ixPlus,iy,    iz,    iVar)
     &                +  fFlowSubG(ix    ,iyPlus,iz,    iVar)
     &                +  fFlowSubG(ixPlus,iyPlus,iz,    iVar)
     &                +  fFlowSubG(ix,    iy,    izPlus,iVar)
     &                +  fFlowSubG(ixPlus,iy,    izPlus,iVar)
     &                +  fFlowSubG(ix    ,iyPlus,izPlus,iVar)
     &                +  fFlowSubG(ixPlus,iyPlus,izPlus,iVar)
     &                ) / 8.
      end do
* --- 1st derivative Jacobian
      do iVar = 1, NVarFlowSub
        uJacobianCell(iVar,1) =
     &                ( fFlowSubG(ixPlus,iyPlus,izPlus,iVar)
     &                - fFlowSubG(ix    ,iyPlus,izPlus,iVar)
     &                + fFlowSubG(ixPlus,iy,    izPlus,iVar)
     &                - fFlowSubG(ix,    iy,    izPlus,iVar)
     &                + fFlowSubG(ixPlus,iyPlus,iz,    iVar)
     &                - fFlowSubG(ix    ,iyPlus,iz,    iVar)
     &                + fFlowSubG(ixPlus,iy,    iz,    iVar)
     &                - fFlowSubG(ix,    iy,    iz,    iVar)
     &                ) / (4*dx)
        uJacobianCell(iVar,2) =
     &                ( fFlowSubG(ixPlus,iyPlus,izPlus,iVar)
     &                + fFlowSubG(ix    ,iyPlus,izPlus,iVar)
     &                - fFlowSubG(ixPlus,iy,    izPlus,iVar)
     &                - fFlowSubG(ix,    iy,    izPlus,iVar)
     &                + fFlowSubG(ixPlus,iyPlus,iz,    iVar)
     &                + fFlowSubG(ix    ,iyPlus,iz,    iVar)
     &                - fFlowSubG(ixPlus,iy,    iz,    iVar)
     &                - fFlowSubG(ix,    iy,    iz,    iVar)
     &                ) / (4*dy)
        uJacobianCell(iVar,3) =
     &                ( fFlowSubG(ixPlus,iyPlus,izPlus,iVar)
     &                + fFlowSubG(ix    ,iyPlus,izPlus,iVar)
     &                + fFlowSubG(ixPlus,iy,    izPlus,iVar)
     &                + fFlowSubG(ix,    iy,    izPlus,iVar)
     &                - fFlowSubG(ixPlus,iyPlus,iz,    iVar)
     &                - fFlowSubG(ix    ,iyPlus,iz,    iVar)
     &                - fFlowSubG(ixPlus,iy,    iz,    iVar)
     &                - fFlowSubG(ix,    iy,    iz,    iVar)
     &                ) / (4*dz)
      end do
* --- end of segment ------------------------------------------------- *
      return
      end
