************************************************************************
*****                                                              *****
***** @segment name: xs3c_ioFlow.f                                 *****
*****                                                              *****
***** @task: read origin flow data                                 *****
*****        This segment contains data specific routines.         *****
*****                                                              *****
************************************************************************
*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012
***** subroutine readFlow( fn,fnLength,fFlowG)
***** subroutine writeFlow(fn,fnLength,fFlowG)

************************************************************************
* === read from the original snapshot ================================ *
      subroutine readFlow(fn,fnLength,fFlowG)
      implicit none
* --- index and flow field (raw data=entire field)
      include 'xv3X.include'           ! for NX, ...
      include 'xv3X_Flow.include'      ! for fFlowG

* --- file name
      character*120 fn                 ! file name of snapshot
      integer       fnLength           ! file name length

* --- auxiliary variables
      integer       ix, iy, iz, iVar   ! counters
      real*8        fMin(NVarFlow), fMax(NVarFlow)

* --- begin of segment ----------------------------------------------- *
* --- user info
      write(*,'(a,i4,a,i4,a,i4,a,i2,3a)') 
     &           'Reading the original ',
     &            NX,'x',NY,'x',NZ,'x',NVarFlow,
     &           ' snapshot from << ', fn(1:fnLength), ' >> ...'


* --- reading the flow ----------------------------------------------- *
      open(10,file=fn(1:fnLength),status='OLD',form='unformatted')
      read(10) ((((fFlowG(ix,iy,iz,iVar),ix=1,  NX),
     &                                   iy=1,  NY),
     &                                   iz=1,  NZ),
     &                                   iVar=1,NVarFlow)
      close(10)

* --- output --------------------------------------------------------- *

      do iVar=1,NVarFlow
        fMin(iVar) =+1.E37
        fMax(iVar) =-1.E37
      end do

      do iVar=1,NVarFlow
      do iz  =1,NZ
      do iy  =1,NY
      do ix  =1,NX
        fMin(iVar) = min(fMin(iVar),1.0D0*fFlowG(ix,iy,iz,iVar))
        fMax(iVar) = max(fMax(iVar),1.0D0*fFlowG(ix,iy,iz,iVar))
      end do
      end do
      end do
      end do

      write(*,'(a3,a12,a12)') 'var','min','max'
      do iVar=1,NVarFlow
        write(*,'(i3,2f12.5)') iVar,fMin(iVar), fMax(iVar)
      end do

* --- end of segment ------------------------------------------------- *
      return
      end


************************************************************************
* === write in the format of the original snapshot =================== *
      subroutine writeFlow(fn,fnLength,fFlowG)
      implicit none
* --- index and flow field (raw data=entire field)
      include 'xv3X.include'           ! for NX, ...
      include 'xv3X_Flow.include'      ! for fFlowG

* --- file name
      character*120 fn                 ! file name of snapshot
      integer       fnLength           ! file name length

* --- auxiliary variables
      integer       ix, iy, iz, iVar   ! counters
      real*8        fMin(NVarFlow), fMax(NVarFlow)

* --- begin of segment ----------------------------------------------- *
* --- user info
      write(*,'(a,i4,a,i4,a,i4,a,i2,3a)') 
     &           'Writing the entire flow  ',
     &            NX,'x',NY,'x',NZ,'x',NVarFlow,
     &           ' in << ', fn(1:fnLength), ' >> ...'


* --- writing the flow ----------------------------------------------- *
      open(10,file=fn(1:fnLength),status='UNKNOWN',form='unformatted')
      write(10) ((((fFlowG(ix,iy,iz,iVar),ix=1,  NX),
     &                                    iy=1,  NY),
     &                                    iz=1,  NZ),
     &                                    iVar=1,NVarFlow)
      close(10)

* --- output --------------------------------------------------------- *
      write(*,*) 'Extremal values:'

      do iVar=1,NVarFlow
        fMin(iVar) =+1.E37
        fMax(iVar) =-1.E37
      end do

      do iVar=1,NVarFlow
      do iz  =1,NZ
      do iy  =1,NY
      do ix  =1,NX
        fMin(iVar) = min(fMin(iVar),1.0D0*fFlowG(ix,iy,iz,iVar))
        fMax(iVar) = max(fMax(iVar),1.0D0*fFlowG(ix,iy,iz,iVar))
      end do
      end do
      end do
      end do

      do iVar=1,NVarFlow
        write(*,'(i3,2f12.5)') iVar,fMin(iVar), fMax(iVar)
      end do

* --- end of segment ------------------------------------------------- *
      return
      end