************************************************************************
*****                                                              *****
***** @segment name: xs2u_ioFlowsSub.f                             *****
***** @task: read/write of snapshot ensemble                       *****
*****        for 2D unstructured grid (subdomain)                  *****
***** @see:  header of res.20100408.xAMC_ToDo.tex                  *****
*****                                                              *****
************************************************************************
* === subroutine copyToFlowsSub(fFlowSubG,fFlowsSubG,iFlow)
* === subroutine copyFromFlowsSub(fFlowSubG,fFlowsSubG,iFlow)
* === subroutine readFlowsSub(iDomain,fFlowsSubG)
*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012

************************************************************************ 
* === copy the flow into the snapshot ensemble ======================= *
      subroutine copyToFlowsSub(fFlowSubG,fFlowsSubG,iFlow)
      implicit none
* --- input parameters
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
      include 'xv3X.include' ! NXSub, ...
      include 'xv3X_FlowSub.include'   ! fFlowSubG  (single flow)
      include 'xv3X_FlowsSub.include'  ! fFlowsSubG (ensemble)
      integer  iFlow
* --- auxiliary dummy variables
      integer iXSub, iYSub, iZSub       ! counter for grid nodes
      integer iDummy                    ! dummy variable
* --- begin of segment ----------------------------------------------- *
* --- user info
C      write(*,'(a,i4,a)') 
C     &        'Copy the flow into ',iFlow,'th snapshot ...'
      write(*,*) NFlows,NFlowsP
* --- copy the flow nodes
      do iDummy = 1, NVarFlowSub
      do iZSub  = 1, NZSub
      do iYSub  = 1, NYSub
      do iXSub  = 1, NXSub
        fFlowsSubG(iXSub,iYSub,iZSub,iDummy,iFlow)
     & =fFlowSubG (iXSub,iYSub,iZSub,iDummy)
      end do
      end do
      end do
      end do
* --- end of segment ------------------------------------------------- *
      return
      end

************************************************************************ 
* === copy the flow from the snapshot ensemble ======================= *
      subroutine copyFromFlowsSub(fFlowSubG,fFlowsSubG,iFlow)
      implicit none
* --- input parameters
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
      include 'xv3X.include' ! NXSub, ...
      include 'xv3X_FlowSub.include'    ! fFlowSubG  (single flow)
      include 'xv3X_FlowsSub.include'   ! fFlowsSubG (ensemble)
      integer  iFlow
* --- auxiliary dummy variables
      integer iXSub, iYSub, iZSub       ! counter for grid nodes
      integer iDummy                    ! dummy variable
* --- begin of segment ----------------------------------------------- *
* --- user info
C      write(*,'(a,i4,a)') 
C     &        'Copy the flow from the ',iFlow,'th snapshot ...'
* --- copy the flow nodes
      do iDummy = 1, NVarFlowSub
      do iZSub  = 1, NZSub
      do iYSub  = 1, NYSub
      do iXSub  = 1, NXSub
        fFlowSubG (iXSub,iYSub,iZSub,iDummy)
     & =fFlowsSubG(iXSub,iYSub,iZSub,iDummy,iFlow)
      end do
      end do
      end do
      end do
* --- end of segment ------------------------------------------------- *
      return
      end

************************************************************************ 
* === read the f of a 2D unstructured grid (subdomain) =============== *
      subroutine readFlowsSub(iDomain,fFlowsSubG)
      implicit none
* --- input parameters for filename and domain
      integer iDomain                   ! domain number
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
      include 'xv3X.include' ! NXSub, ...
      include 'xv3X_FlowSub.include'    ! fFlowSubG
      include 'xv3X_FlowsSub.include'   ! fFlowsSubG
* --- snapshot ensemble
      character*120 fn        ! file name
      integer       fnLength  ! length of file name (# of characters)
* --- variables for grid and flow
* === NOTE THAT xv2u_*.include will be copied on xv3X_*.include before
* --- auxiliary variables for file name
* --- auxiliary dummy variables
      integer iXSub, iYSub, iZSub       ! counters for grid nodes
      integer iDummy                    ! dummy variable
      integer iFlow                     ! counter
      real*8  fNorm                     ! norm
* --- begin of segment ----------------------------------------------- *
      write(*,*) '***** Reading the snapshot ensamble ...'
      call countFlows1(iDummy)
      NFlows = iDummy ! Note that NFlows is saved in a common block (!)
      do iFlow = 1, NFlows
        call readFilenameFlow1(iFlow,fn,fnLength)
        call readFlowSub(iDomain,fn,fnLength,fFlowSubG)
        call maxnormFlowSub(fFlowSubG,fNorm)
        if (fNorm.eq.   0.0) stop 'Norm = 0'
        if (fNorm.gt.1000.0) stop 'Norm > 1000'
        call copyToFlowsSub(fFlowSubG,fFlowsSubG,iFlow)
      end do
* --- end of segment ------------------------------------------------- *
      return
      end
