************************************************************************
*****                                                              *****
***** @segment name: xs2u_ioForcesSub.f                            *****
***** @task: read/write volume forces                              *****
*****        for 2D unstructured grid (subdomain)                  *****
***** @see:  header of res.20100408.xAMC_ToDo.tex                  *****
*****                                                              *****
************************************************************************
* === subroutine copyToForcesSub  (fFlowSubG,fForcesSubG,iForce)
* === subroutine copyFromForcesSub(fFlowSubG,fForcesSubG,iForce)
* === subroutine readForcesSub    (iDomain,        fForcesSubG)
*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012

************************************************************************ 
* === copy the flow into the set of forces =========================== *
      subroutine copyToForcesSub(fFlowSubG,fForcesSubG,iForce)
      implicit none
* --- Galerkin system
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
* --- input parameters
      include 'xv3X.include'           ! NXSub, ...
      include 'xv3X_FlowSub.include'   ! fFlowSubG  (single flow)
      include 'xv3X_ForcesSub.include' ! fForcesSubG (set)
      integer  iForce
* --- auxiliary dummy variables
      integer iXSub, iYSub, iZSub       ! counter for grid nodes
      integer iDummy                    ! dummy variable
* --- begin of segment ----------------------------------------------- *
* --- user info
C      write(*,'(a,i4,a)') 
C     &        'Copy the flow into ',iForce,'th force ...'
* --- copy the flow nodes
      do iDummy = 1, NVarFlowSub
      do iZSub  = 1, NZSub
      do iYSub  = 1, NYSub
      do iXSub  = 1, NXSub
        fForcesSubG(iXSub,iYSub,iZSub,iDummy,iForce)
     & =fFlowSubG  (iXSub,iYSub,iZSub,iDummy)
      end do
      end do
      end do
      end do
* --- end of segment ------------------------------------------------- *
      return
      end

************************************************************************ 
* === copy the flow from the set of forces =========================== *
      subroutine copyFromForcesSub(fFlowSubG,fForcesSubG,iForce)
      implicit none
* --- Galerkin system
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
* --- input parameters
      include 'xv3X.include'           ! NXSub, ...
      include 'xv3X_FlowSub.include'   ! fFlowSubG  (single flow)
      include 'xv3X_ForcesSub.include' ! fForcesSubG (ensemble)
      integer  iForce
* --- auxiliary dummy variables
      integer iXSub, iYSub, iZSub      ! counter for grid nodes
      integer iDummy                   ! dummy variable
* --- begin of segment ----------------------------------------------- *
* --- user info
C      write(*,'(a,i4,a)') 
C     &        'Copy the flow from the ',iForce,'th force ...'
* --- copy the flow nodes
      do iDummy = 1, NVarFlowSub
      do iZSub  = 1, NZSub
      do iYSub  = 1, NYSub
      do iXSub  = 1, NXSub
        fFlowSubG  (iXSub,iYSub,iZSub,iDummy)
     & =fForcesSubG(iXSub,iYSub,iZSub,iDummy,iForce)
      end do
      end do
      end do
      end do
* --- end of segment ------------------------------------------------- *
      return
      end

************************************************************************ 
* === read the set of forces of a 2D unstructured grid (subdomain) === *
      subroutine readForcesSub(iDomain,fForcesSubG)
      implicit none
* --- Galerkin system
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
* --- input parameters for filename and domain
      integer iDomain                   ! domain number
      include 'xv3X.include'            ! NXSub, ...
      include 'xv3X_FlowSub.include'    ! fFlowSubG
      include 'xv3X_ForcesSub.include'  ! fForcesSubG
* --- Galerkin expansion
      character*120 fn        ! file name
      integer       fnLength  ! length of file name (# of characters)
* --- variables for grid and flow
* === NOTE THAT xv2u_*.include will be copied on xv3X_*.include before
* --- auxiliary variables for file name
* --- auxiliary dummy variables
      integer iForce                    ! counter
      integer iDummy                    ! dummy variable
      real*8  fNorm                     ! norm
* --- begin of segment ----------------------------------------------- *
      if (NVF.le.0) then
        write(*,*) 'No forces required ...'
        return
      end if
* ---
      write(*,*) '***** Reading the complete set of forces ...', NVF
      call countForces(iDummy)
      do iForce = 1, NVF
        call readFilenameForce(iForce,fn,fnLength)
        call readFlowSub(iDomain,fn,fnLength,fFlowSubG)
        call maxnormFlowSub(fFlowSubG,fNorm)
        if (fNorm.eq.   0.0) stop 'Norm = 0'
        if (fNorm.gt.1000.0) stop 'Norm > 1000'
        call copyToForcesSub(fFlowSubG,fForcesSubG,iForce)
      end do
* --- end of segment ------------------------------------------------- *
      return
      end
