************************************************************************
*****                                                              *****
***** @segment name: xs3c_ioGridSub.f                              *****
*****                                                              *****
***** @task: read grid data (subdomain)                            *****
*****                                                              *****
************************************************************************
*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012
***** The following inclusion is necessary, because xAMC3X *************
***** reads EITHER xs3c_ioGridSub OR xs3x_ioGrid BUT NOT BOTH. *********
      include 'xs3X_ioGrid.f'
************************************************************************
***** subroutine readGridSub(iDomain)
***** subroutine compNDomain(NDomain)
***** subroutine compIndexDomain(iDomain,ixMin,iyMin,izMin)

************************************************************************
* === read the original grid ========================================= *
      subroutine readGridSub(iDomain)
      implicit none
* --- variables for original grid
      include 'xv3X.include'       ! for NX, ..., NXSub, ...
      include 'xv3X_Grid.include'  ! for grid
* --- variables for xAMC grid
      include 'xv3X_GridSub.include'      ! for xSubG, ...
* ---
      integer       iDomain            ! domain number
* --- auxiliary variables
      integer       ix,    iy,    iz   ! counters
      integer       ixMin, iyMin, izMin

* --- begin of segment ----------------------------------------------- *
      call readGrid
      call compIndexDomain(iDomain,ixMin,iyMin,izMin)

* --- user info
      call compIndexDomain(iDomain,ixMin,iyMin,izMin)
      write(*,'(a)') 'Extracting the xAMC data ...'
* --- extract zone
      do ix   = 1, NXSub
        xSubG(ix) =  xG(ix+ixMin-1)
      end do
* ---
      do iy   = 1, NYSub
        ySubG(iy) =  yG(iy+iyMin-1)
      end do
* ---
      do iz   = 1, NZSub
        zSubG(iz) =  zG(iz+izMin-1)
      end do

* --- output --------------------------------------------------------- *
      write(*,*)

* --- end of segment ------------------------------------------------- *
      return
      end

************************************************************************
* === compute number of domains ====================================== *
      subroutine compNDomain(NDomain)
      implicit none
* --- variables for original and xAMC grid
      include 'xv3X.include' ! for NX, ..., for NXSub, ...
      integer NDomain

* --- begin of segment ----------------------------------------------- *
      NDomain = (1 + (NX-1)/NXSubP)   ! # of x-intervals
     &        * (1 + (NY-1)/NYSubP)   ! # of y-intervals
     &        * (1 + (NZ-1)/NZSubP)   ! # of z-intervals
* --- end of segment ------------------------------------------------- *
      return
      end

************************************************************************
* === compute index domain =========================================== *
      subroutine compIndexDomain(iDomain,ixMin,iyMin,izMin)
      implicit none
* --- variables for original and xAMC grid
      include 'xv3X.include' ! for NX, ..., for NXSub, ...
      integer iDomain, ixMin, iyMin, izMin
* --- auxiliary variables
      integer       ixyzDomain, ixDomain, iyDomain, izDomain
      integer                   NxDomain, NyDomain, NzDomain
* --- begin of segment ----------------------------------------------- *
      NxDomain = 1 + (NX-1)/NXSubP    ! # of x-intervals
      NyDomain = 1 + (NY-1)/NYSubP    ! # of y-intervals
      NzDomain = 1 + (NZ-1)/NZSubP    ! # of z-intervals
* --- exit
      if (iDomain.gt.NxDomain*NyDomain*NzDomain) then
        stop 'compIndexDomain: iDomain too large!'
      end if
* --- determine zone 
      ixyzDomain = 1
      ixDomain   = 0
      iyDomain   = 0
      izDomain   = 0
      do while (ixyzDomain.lt.iDomain)
        ixDomain = ixDomain + 1
        if (ixDomain.eq.NxDomain) then
          ixDomain = 0
          iyDomain = iyDomain + 1
        end if
        if (iyDomain.eq.NyDomain) then
          iyDomain = 0
          izDomain = izDomain + 1
        end if
        ixyzDomain = ixyzDomain + 1
      end do
* --- determine dimension of domain
*     e.g. NXSubP=3 => iDomain=1: ix=1 ix=2 ix=3 (ixMax = 1 +   (NXSubP-1))
*                      iDomain=2: ix=3 ix=4 ix=5 (ixMax = 1 + 2 (NXSubP-1))
*                      iDomain=3: ix=5 ix=6 ix=7 (ixMax = 1 + 3 (NXSubP-1))
      ixMin = 1 + ixDomain*(NXSubP-1) 
      iyMin = 1 + iyDomain*(NYSubP-1)
      izMin = 1 + izDomain*(NZSubP-1)
* ... NXSub = ixMax - ixMin + 1
      NXSub = min(NX,(ixDomain+1)*(NXSubP-1)+1)-ixMin+1
      NYSub = min(NY,(iyDomain+1)*(NYSubP-1)+1)-iyMin+1
      NZSub = min(NZ,(izDomain+1)*(NZSubP-1)+1)-izMin+1
      write(*,'(7a7)') 'iDomain','ixMin','iyMin','izMin',
     &                           'ixMax','iyMax','izMax'
      write(*,'(7i7)')
     &  iDomain, ixMin,         iyMin,         izMin, 
     &           ixMin+NXSub-1, iyMin+NYSub-1, izMin+NZSub-1
* --- end of segment ------------------------------------------------- *
      return
      end
