************************************************************************
*****                                                              *****
***** @segment name: xs2u_ioModesSub.f                             *****
***** @task: read/write Galerkin expansion                         *****
*****        for 2D unstructured grid (subdomain)                  *****
***** @see:  header of res.20100408.xAMC_ToDo.tex                  *****
*****                                                              *****
************************************************************************
* === subroutine copyToModesSub  (fFlowSubG,fModesSubG,iMode)
* === subroutine copyFromModesSub(fFlowSubG,fModesSubG,iMode)
* === subroutine readModesSub(iDomain, fModesSubG)
* === subroutine readModes2Sub(iDomain,fModes2SubG)
*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012

************************************************************************ 
* === copy the flow into the Galerkin expansion ======================= *
      subroutine copyToModesSub(fFlowSubG,fModesSubG,iMode)
      implicit none
* --- input parameters
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
      include 'xv3X.include' ! NXSub, ...
      include 'xv3X_FlowSub.include'    ! fFlowSubG  (single flow)
      include 'xv3X_ModesSub.include'   ! fModesSubG (ensemble)
      integer  iMode
* --- auxiliary dummy variables
      integer iXSub, iYSub, iZSub       ! counter for grid nodes
      integer iDummy                    ! dummy variable
* --- begin of segment ----------------------------------------------- *
* --- user info
C      write(*,'(a,i4,a)') 
C     &        'Copy the flow into ',iMode,'th mode ...'
      write(*,*) NAM, NEM
* --- copy the flow nodes
      do iDummy = 1, NVarFlowSub
      do iZSub  = 1, NZSub
      do iYSub  = 1, NYSub
      do iXSub  = 1, NXSub
        fModesSubG(iXSub,iYSub,iZSub,iDummy,iMode)
     & =fFlowSubG (iXSub,iYSub,iZSub,iDummy)
      end do
      end do
      end do
      end do
* --- end of segment ------------------------------------------------- *
      return
      end

************************************************************************ 
* === copy the flow from the Galerkin expansion ======================= *
      subroutine copyFromModesSub(fFlowSubG,fModesSubG,iMode)
      implicit none
* --- input parameters
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
      include 'xv3X.include' ! NXSub, ...
      include 'xv3X_FlowSub.include'    ! fFlowSubG  (single flow)
      include 'xv3X_ModesSub.include'   ! fModesSubG (ensemble)
      integer  iMode
* --- auxiliary dummy variables
      integer iXSub, iYSub, iZSub       ! counter for grid nodes
      integer iDummy                    ! dummy variable
* --- begin of segment ----------------------------------------------- *
* --- user info
C      write(*,'(a,i4,a)') 
C     &        'Copy the flow from the ',iMode,'th mode ...'
* --- copy the flow nodes
      do iDummy = 1, NVarFlowSub
      do iZSub  = 1, NZSub
      do iYSub  = 1, NYSub
      do iXSub  = 1, NXSub
        fFlowSubG (iXSub,iYSub,iZSub,iDummy)
     & =fModesSubG(iXSub,iYSub,iZSub,iDummy,iMode)
      end do
      end do
      end do
      end do
* --- end of segment ------------------------------------------------- *
      return
      end

************************************************************************ 
* === read the modes of a 2D unstructured grid (subdomain) =========== *
      subroutine readModesSub(iDomain,fModesSubG)
      implicit none
* --- input parameters for filename and domain
      integer iDomain                   ! domain number
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
      include 'xv3X.include' ! NXSub, ...
      include 'xv3X_FlowSub.include'    ! fFlowSubG
      include 'xv3X_ModesSub.include'   ! fModesSubG
* --- Galerkin expansion
      character*120 fn        ! file name
      integer       fnLength  ! length of file name (# of characters)
* --- variables for grid and flow
* === NOTE THAT xv2u_*.include will be copied on xv3X_*.include before
* --- auxiliary variables for file name
* --- auxiliary dummy variables
      integer iDummy                    ! dummy variable
      integer iMode                     ! counter
      real*8  fNorm                     ! norm
* --- begin of segment ----------------------------------------------- *
      write(*,*) '***** Reading the Galerkin expansion ...'
      call countModes1(iDummy)
      do iMode = -NAM, NEM
        call readFilenameMode1(iMode,fn,fnLength)
        call readFlowSub(iDomain,fn,fnLength,fFlowSubG)
        call maxnormFlowSub(fFlowSubG,fNorm)
        if (fNorm.eq.   0.0) stop 'Norm = 0'
        if (fNorm.gt.1000.0) stop 'Norm > 1000'
        call copyToModesSub(fFlowSubG,fModesSubG,iMode)
      end do
* --- end of segment ------------------------------------------------- *
      return
      end

************************************************************************ 
* === read the modes of a 2D unstructured grid (subdomain) =========== *
      subroutine readModes2Sub(iDomain,fModes2SubG)
      implicit none
* --- input parameters for filename and domain
      integer iDomain                   ! domain number
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
      include 'xv3X.include' ! NXSub, ...
      include 'xv3X_FlowSub.include'    ! fFlowSubG
      include 'xv3X_Modes2Sub.include'  ! fModesSubG
* --- Galerkin expansion
      character*120 fn        ! file name
      integer       fnLength  ! length of file name (# of characters)
* --- variables for grid and flow
* === NOTE THAT xv2u_*.include will be copied on xv3X_*.include before
* --- auxiliary variables for file name
* --- auxiliary dummy variables
      integer iDummy                    ! dummy variable
      integer iMode                     ! counter
      real*8  fNorm                     ! norm
* --- begin of segment ----------------------------------------------- *
      write(*,*) '***** Reading the test functions ...'
      call countModes2(iDummy)
      do iMode = 1, NEM
        call readFilenameMode2(iMode,fn,fnLength)
        call readFlowSub(iDomain,fn,fnLength,fFlowSubG)
        call maxnormFlowSub(fFlowSubG,fNorm)
        if (fNorm.eq.   0.0) stop 'Norm = 0'
        if (fNorm.gt.1000.0) stop 'Norm > 1000'
        call copyToModesSub(fFlowSubG,fModes2SubG,iMode)
      end do
* --- end of segment ------------------------------------------------- *
      return
      end
