************************************************************************
*****                                                              *****
***** @segment name: xs3X_opFlow.f                                 *****
***** @task: elementary operations of flows on unstructured grids  *****
***** @see:  header of res.20100408.xAMC_ToDo.tex                  *****
*****                                                              *****
************************************************************************
* === subroutine zeroFlow      (                 fFlowG)
* === subroutine scaleFlow     (fFactor,         fFlowG)
* === subroutine addScaledFlow (fFactor1,fFlow1G,fFlowG)
* === subroutine addProductFlow(fFlow1G, fFlow2G,fFlowG)
* === subroutine maxnormFlow   (fFlowG,fNorm)
*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012
************************************************************************ 
* === zero the flow on a 2D unstructured grid ======================== *
      subroutine zeroFlow(fFlowG)
      implicit none
* --- variables for grid and flow
      include 'xv3X.include'            ! NX, ...
      include 'xv3X_Flow.include'       ! fFlowG
* --- auxiliary dummy variables
      integer iX, iY, iZ      ! counter for grid nodes
      integer iDummy          ! dummy variables
* --- begin of segment ----------------------------------------------- *
* --- read the flow
      write(*,*) 'Zeroing a flow ...'
      do iDummy = 1, NVarFlow
      do iZ     = 1, NZP
      do iY     = 1, NYP
      do iX     = 1, NXP
        fFlowG(iX,iY,iZ,iDummy) = 0.0
      end do
      end do
      end do
      end do
* --- end of segment ------------------------------------------------- *
      return
      end

************************************************************************ 
* === scale a flow on a 2D unstructured grid ========================= *
      subroutine scaleFlow(fFactor,fFlowG)
      implicit none
* --- variables for grid and flow
      include 'xv3X.include'            ! NX, ...
      include 'xv3X_Flow.include'       ! fFlowG
      real*8   fFactor
* --- auxiliary dummy variables
      integer iX, iY, iZ      ! counter for grid nodes
      integer iDummy          ! dummy variables
* --- begin of segment ----------------------------------------------- *
* --- read the flow
      write(*,*) 'Performing ',fFactor,' x flow -> new flow ...'
      do iDummy = 1, NVarFlow
      do iZ     = 1, NZ
      do iY     = 1, NY
      do iX     = 1, NX
        fFlowG(iX,iY,iZ,iDummy) = fFlowG (iX,iY,iZ,iDummy)*fFactor
      end do
      end do
      end do
      end do
* --- end of segment ------------------------------------------------- *
      return
      end

************************************************************************ 
* === add scaled flow on a 2D unstructured grid ====================== *
      subroutine addScaledFlow(fFactor1,fFlow1G,fFlowG)
      implicit none
* --- variables for grid and flow
      include 'xv3X.include'            ! NX, ...
      include 'xv3X_Flow.include'       ! fFlowG
      include 'xv3X_Flow1.include'      ! fFlow1G
      real*8   fFactor1
* --- auxiliary dummy variables
      integer iX, iY, iZ      ! counter for grid nodes
      integer iDummy          ! dummy variables
* --- begin of segment ----------------------------------------------- *
* --- read the flow
      write(*,*) 'Performing ',fFactor1,' x flow1 + flow = new flow ...'
      do iDummy = 1, NVarFlow
      do iZ     = 1, NZ
      do iY     = 1, NY
      do iX     = 1, NX
        fFlowG(iX,iY,iZ,iDummy)= fFlowG (iX,iY,iZ,iDummy)
     &                         + fFlow1G(iX,iY,iZ,iDummy)*fFactor1
      end do
      end do
      end do
      end do
* --- end of segment ------------------------------------------------- *
      return
      end

************************************************************************ 
* === add product of two flows ======================================= *
      subroutine addProductFlow(fFlow1G,fFlow2G,fFlowG)
      implicit none
* --- variables for grid and flow
      include 'xv3X.include'            ! NX, ...
      include 'xv3X_Flow.include'       ! fFlowG
      include 'xv3X_Flow1.include'      ! fFlow1G
      include 'xv3X_Flow2.include'      ! fFlow2G
* --- auxiliary dummy variables
      integer iX, iY, iZ      ! counter for grid nodes
      integer iDummy          ! dummy variables
* --- begin of segment ----------------------------------------------- *
* --- read the flow
      write(*,*) 'Performing flow1 <- flow1 + flow2*flow3 ...'
      do iDummy = 1, NVarFlow
      do iZ     = 1, NZ
      do iY     = 1, NY
      do iX     = 1, NX
        fFlowG(iX,iY,iZ,iDummy)= fFlowG (iX,iY,iZ,iDummy)
     &                         + fFlow1G(iX,iY,iZ,iDummy)
     &                         * fFlow2G(iX,iY,iZ,iDummy)
      end do
      end do
      end do
      end do
* --- end of segment ------------------------------------------------- *
      return
      end

************************************************************************ 
* === returns the maximum norm of the flow on an unstr. grid ========= *
      subroutine maxnormFlow(fFlowG,fNorm)
      implicit none
* --- variables for grid and flow
      include 'xv3X.include'            ! NX, ...
      include 'xv3X_Flow.include'       ! fFlowG
      real*8  fNorm
* --- auxiliary dummy variables
      integer iX, iY, iZ      ! counter for grid nodes
      integer iDummy          ! dummy variable
      real*8  fDummy          ! dummy variable
      real*8  fMinVal(NVarFlow), fMaxVal(NVarFlow)
* --- begin of segment ----------------------------------------------- *
* --- read the flow
      write(*,*) 'Determining the extrema of the flow ...'
      fNorm = 0.0
      do iDummy = 1, NVarFlow
        fMinVal(iDummy) =  1.0E37
        fMaxVal(iDummy) = -1.0E37
        do iZ   = 1, NZ
        do iY   = 1, NY
        do iX   = 1, NX
          fDummy          =  fFlowG(iX,iY,iZ,iDummy)
          fNorm           = max(fNorm,      abs(fDummy))
          fMinVal(iDummy) = min(fMinVal(iDummy),fDummy)
          fMaxVal(iDummy) = max(fMaxVal(iDummy),fDummy)
        end do
        end do
        end do
      end do
      write(*,'(a5,2a12)') 'var.','min','max'
      do iDummy = 1, NVarFlow
        write(*,'(i5,2f12.5)') iDummy,fMinVal(iDummy),fMaxVal(iDummy)
      end do
      write(*,'(a, f12.5)') 'max. norm=',fNorm
* --- end of segment ------------------------------------------------- *
      return
      end
