************************************************************************
*****                                                              *****
***** @segment name: xs3X_opFlowSub.f                              *****
***** @task: elementary operations of flows on unstructured grids  *****
***** @see:  header of res.20100408.xAMC_ToDo.tex                  *****
*****                                                              *****
************************************************************************
* === subroutine zeroFlowSub      (                       fFlowSubG)
* === subroutine scaleFlowSub     (fFactor,               fFlowSubG)
* === subroutine addScaledFlowSub (fFactor1,   fFlow1SubG,fFlowSubG)
* === subroutine addProductFlowSub(fFlow1SubG, fFlow2SubG,fFlowSubG)
* === subroutine maxnormFlowSub   (fFlowSubG,fNorm)
*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012
************************************************************************ 
* === zero the flow on a 2D unstructured grid (subdomain) ============ *
      subroutine zeroFlowSub(fFlowSubG)
      implicit none
* --- variables for grid and flow
      include 'xv3X.include'            ! NXSub, ...
      include 'xv3X_FlowSub.include'    ! fFlowSubG
* --- auxiliary dummy variables
      integer iXSub, iYSub, iZSub       ! counter for grid nodes
      integer iDummy                    ! dummy variables
* --- begin of segment ----------------------------------------------- *
* --- read the flow
      write(*,*) 'Zeroing a flow (subdomain)...'
      do iDummy = 1, NVarFlowSub
      do iZSub  = 1, NZSubP
      do iYSub  = 1, NYSubP
      do iXSub  = 1, NXSubP
        fFlowSubG(iXSub,iYSub,iZSub,iDummy) = 0.0
      end do
      end do
      end do
      end do
* --- end of segment ------------------------------------------------- *
      return
      end

************************************************************************ 
* === scale a flow on a 2D unstructured grid (subdomain) ============= *
      subroutine scaleFlowSub(fFactor,fFlowSubG)
      implicit none
* --- variables for grid and flow
      include 'xv3X.include'            ! NXSub, ...
      include 'xv3X_FlowSub.include'    ! fFlowSubG
      real*8   fFactor
* --- auxiliary dummy variables
      integer iXSub, iYSub, iZSub       ! counter for grid nodes
      integer iDummy                    ! dummy variables
* --- begin of segment ----------------------------------------------- *
* --- read the flow
      write(*,*) 'Performing ',fFactor,
     &           ' x flow -> new flow (subdomain)...'
      do iDummy = 1, NVarFlowSub
      do iZSub  = 1, NZSub
      do iYSub  = 1, NYSub
      do iXSub  = 1, NXSub
        fFlowSubG(iXSub,iYSub,iZSub,iDummy)
     &= fFlowSubG(iXSub,iYSub,iZSub,iDummy)*fFactor
      end do
      end do
      end do
      end do
* --- end of segment ------------------------------------------------- *
      return
      end

************************************************************************ 
* === add scaled flow on a 2D unstructured grid (subdomain) ========== *
      subroutine addScaledFlowSub(fFactor1,fFlow1SubG,fFlowSubG)
      implicit none
* --- variables for grid and flow
      include 'xv3X.include'            ! NXSub, ...
      include 'xv3X_FlowSub.include'    ! fFlowSubG
      include 'xv3X_Flow1Sub.include'   ! fFlow1SubG
      real*8   fFactor1
* --- auxiliary dummy variables
      integer iXSub, iYSub, iZSub       ! counter for grid nodes
      integer iDummy                    ! dummy variables
* --- begin of segment ----------------------------------------------- *
* --- read the flow
      write(*,*) 'Performing ',fFactor1,
     &           ' x flow1 + flow = new flow (subdomain) ...'
      do iDummy = 1, NVarFlowSub
      do iZSub  = 1, NZSub
      do iYSub  = 1, NYSub
      do iXSub  = 1, NXSub
        fFlowSubG (iXSub,iYSub,iZSub,iDummy)
     &= fFlowSubG (iXSub,iYSub,iZSub,iDummy)
     &+ fFlow1SubG(iXSub,iYSub,iZSub,iDummy)*fFactor1
      end do
      end do
      end do
      end do
* --- end of segment ------------------------------------------------- *
      return
      end

************************************************************************ 
* === add product of two flows ======================================= *
      subroutine addProductFlowSub(fFlow1SubG,fFlow2SubG,fFlowSubG)
      implicit none
* --- variables for grid and flow
      include 'xv3X.include'            ! NXSub, ...
      include 'xv3X_FlowSub.include'    ! fFlowSubG
      include 'xv3X_Flow1Sub.include'   ! fFlow1SubG
      include 'xv3X_Flow2Sub.include'   ! fFlow2SubG
* --- auxiliary dummy variables
      integer iXSub, iYSub, iZSub       ! counter for grid nodes
      integer iDummy                    ! dummy variables
* --- begin of segment ----------------------------------------------- *
* --- read the flow
      write(*,*) 
     &     'Performing flow1 <- flow1 + flow2*flow3 (subdomain) ...'
      do iDummy = 1, NVarFlowSub
      do iZSub  = 1, NZSub
      do iYSub  = 1, NYSub
      do iXSub  = 1, NXSub
        fFlowSubG (iXSub,iYSub,iZSub,iDummy)
     &= fFlowSubG (iXSub,iYSub,iZSub,iDummy)
     &+ fFlow1SubG(iXSub,iYSub,iZSub,iDummy)
     &* fFlow2SubG(iXSub,iYSub,iZSub,iDummy)
      end do
      end do
      end do
      end do
* --- end of segment ------------------------------------------------- *
      return
      end

************************************************************************ 
* === returns the maximum norm of the flow on an unstr. grid ========= *
      subroutine maxnormFlowSub(fFlowSubG,fNorm)
      implicit none
* --- variables for grid and flow
      include 'xv3X.include'            ! NXSub, ...
      include 'xv3X_FlowSub.include'    ! fFlowSubG
      real*8  fNorm
* --- auxiliary dummy variables
      integer iXSub, iYSub, iZSub       ! counter for grid nodes
      integer iDummy                    ! dummy variable
      real*8  fDummy                    ! dummy variable
      real*8  fMinVal(NVarFlowSub)      ! minima of the flow component
      real*8  fMaxVal(NVarFlowSub)      ! maxima of the flow component
* --- begin of segment ----------------------------------------------- *
* --- read the flow
      write(*,*) 'Determining the extrema of the flow (subdomain) ...'
      fNorm = 0.0
      do iDummy = 1, NVarFlowSub
        fMinVal(iDummy) =  1.0E37
        fMaxVal(iDummy) = -1.0E37
        do iZSub = 1, NZSub
        do iYSub = 1, NYSub
        do iXSub = 1, NXSub
          fDummy          =  fFlowSubG(iXSub,iYSub,iZSub,iDummy)
          fNorm           = max(fNorm,      abs(fDummy))
          fMinVal(iDummy) = min(fMinVal(iDummy),fDummy)
          fMaxVal(iDummy) = max(fMaxVal(iDummy),fDummy)
        end do
        end do
        end do
      end do
      write(*,'(a5,2a12)') 'var.','min','max'
      do iDummy = 1, NVarFlowSub
        write(*,'(i5,2f12.5)') iDummy,fMinVal(iDummy),fMaxVal(iDummy)
      end do
      write(*,'(a, f12.5)') 'max. norm=',fNorm
* --- end of segment ------------------------------------------------- *
      return
      end
