************************************************************************
*****                                                              *****
***** @segment name: xs_subtractBaseFlowSub.f                      *****
***** @task: subtract actuation mode expansion and mean flow       *****
***** @see:  res.20100408.xAMC_ToDo.tex                            *****
*****                                                              *****
************************************************************************
* === subroutine readBaseFlowSub(iDomain,fAim,fModesSubG)
* === subroutine subtractBaseFlowSub    (fAim,fModesSubG,fFlowsSubG)
*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012

************************************************************************ 
* === read base flow (modes i=-NA,...,0 with amplitudes) ============= *
      subroutine readBaseFlowSub(iDomain,fAim,fModesSubG)
      implicit none
* --- input parameters for filename and domain
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
      include 'xv3X.include' ! NXSub, ...
      include 'xv3X_ModesSub.include'   ! fModesSubG
      integer iDomain
* --- mode amplitudes
      real*8  fTm (           NFlowsP)
      real*8  fAim(-NAMP:NEMP,NFlowsP)
* --- auxiliary dummy variables
      integer iDummy                    ! dummy variable
      real*8  fDummy                    ! norm
* --- begin of segment ----------------------------------------------- *
      write(*,*) '***** Loading the actuation modes (if necessary) ...'
      if (NAM.gt.0) then 
* ..... read amplitudes of actuation mode
        call readAmplitudes1(fAim)
* ..... read actuation modes
        iDummy = NEM
        NEM    = -1 ! This limits the loop in readModes 
        call readModesSub(iDomain,fModesSubG)
        NEM    = iDummy
      end if
* --- end of segment ------------------------------------------------- *
      return
      end

************************************************************************ 
* === subtract base flow ============================================= *
      subroutine subtractBaseFlowSub(fAim,fModesSubG,fFlowsSubG)
      implicit none
* --- input parameters for filename and domain
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
      include 'xv3X.include' ! NXSub, ...
      include 'xv3X_ModesSub.include'   ! fModesSubG
      include 'xv3X_FlowsSub.include'   ! fFlowsSubG
* --- mode amplitudes
      real*8  fAim(-NAMP:NEMP,NFlowsP)
* --- auxiliary dummy variables
      integer iXSub, iYSub, iZSub       ! counter for grid nodes
      integer iDummy                    ! dummy variable
      integer iMode                     ! counter
      integer iFlow                     ! counter
      real*8  fDummy                    ! norm
* --- begin of segment ----------------------------------------------- *
      write(*,*) '***** Subtracting AM expansion ... N_A=',NAM
      if (NAM.gt.0) then
      do iMode  = -NAM,-1
      do iFlow  = 1, NFlows
      do iDummy = 1, NVarFlowSub
      do iZSub  = 1, NZSub
      do iYSub  = 1, NYSub
      do iXSub  = 1, NXSub
          fFlowsSubG(iXSub,iYSub,iZSub,iDummy,iFlow)
     &  = fFlowsSubG(iXSub,iYSub,iZSub,iDummy,iFlow)
     &  - fModesSubG(iXSub,iYSub,iZSub,iDummy,iMode)*fAim(iMode,iFlow)
      end do
      end do
      end do
      end do
      end do
      end do
      end if
      write(*,*) '***** Computing the mean flow ... M=',NFlows
* ---
      do iDummy = 1, NVarFlowSub
      do iZSub  = 1, NZSub
      do iYSub  = 1, NYSub
      do iXSub  = 1, NXSub
        fDummy  = 0.0
        do iFlow = 1, NFlows
          fDummy = fDummy + fFlowsSubG(iXSub,iYSub,iZSub,iDummy,iFlow)
        end do
        fModesSubG(iXSub,iYSub,iZSub,iDummy,0) = fDummy/NFlows
      end do
      end do
      end do
      end do
* ---
      write(*,*) '***** Subtracting the mean flow ...'
      do iFlow  = 1, NFlows
      do iDummy = 1, NVarFlowSub
      do iZSub  = 1, NZSub
      do iYSub  = 1, NYSub
      do iXSub  = 1, NXSub
          fFlowsSubG(iXSub,iYSub,iZSub,iDummy,iFlow)
     &  = fFlowsSubG(iXSub,iYSub,iZSub,iDummy,iFlow)
     &  - fModesSubG(iXSub,iYSub,iZSub,iDummy,0)
      end do
      end do
      end do
      end do
      end do
* --- end of segment ------------------------------------------------- *
      return
      end

