************************************************************************
*****                                                              *****
***** @segment name: xs_DS_Propagator.f                            *****
***** @task: initializing and computing the propagator             *****
***** @see: res090905                                              *****
*****                                                              *****
************************************************************************
*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012
* === subroutine initFlow
* === subroutine compFlow(timeNow,fAi,fFi)
* === subroutine compJacobian(fAi,fDFij)

***********************************************************************
* === initializing the propagator =================================== *
      subroutine initFlow
      implicit none
      include 'xv_DS.include'        ! NAP,... | dimensions of DS
      include 'xv_DS_Propagator.include' ! parameter of DS
      integer*8 iDummy
      real*8    fDummy
* --- begin of subroutine ------------------------------------------- *
      write(*,*) 'Initializing the propagator...'
      NA = 2
* --- end of subroutine --------------------------------------------- *
      return
      end

***********************************************************************
* === computing the propagator ====================================== *
      subroutine compFlow(timeNow,fAi,fFi)
      implicit none
      include 'xv_DS.include'        ! NAP,... | dimensions of DS
      include 'xv_DS_State.include'      ! fAi     | state of DS
      include 'xv_DS_Flow.include'       ! fFi     | propagator of DS
C XXX include 'xv_DS_Propagator.include' ! parameters of DS, e.g. fQijk
      real*8    timeNow
      integer*8 iDummy
      real*8    fDummy
* --- begin of subroutine ------------------------------------------- *
      fDummy = (fAi(1)**2 + fAi(2)**2)
      fFi(1) = (0.1-fDummy)*fAi(1)-fAi(2)
      fFi(2) = (0.1-fDummy)*fAi(2)+fAi(1)
* --- end of subroutine --------------------------------------------- *
      return
      end

***********************************************************************
* === computing the Jacobian of the propagator ====================== *
      subroutine compJacobian(fAi,fDFij)
      implicit none
      include 'xv_DS.include'        ! NAP,... | dimensions of DS
      include 'xv_DS_State.include'      ! fAi     | state of DS
      include 'xv_DS_Jacobian.include'   ! fDFij   | Jacobian of DS
C XXX include 'xv_DS_Propagator.include' ! fQijk   | parameters of DS
      real*8    timeNow
      integer*8 iDummy, jDummy, kDummy
      real*8    fDummy
* --- begin of subroutine ------------------------------------------- *
* --- compute Jacobian
      fDfij(1,1) =  0.1 - 2*fAi(1)**2
      fDfij(2,2) =  0.1 - 2*fAi(2)**2
      fDfij(1,2) = -1.0 - 2*fAi(1)*fAi(2)
      fDfij(2,1) = +1.0 - 2*fAi(1)*fAi(2)
* --- end of subroutine --------------------------------------------- *
      return
      end
