************************************************************************
*****                                                              *****
***** @segment name: xs_DS_Propagator_clq.f                        *****
***** @task: initializing and computing the propagator             *****
*****        for a constant-linear-quadratic flow                  *****
***** @see:  res090905                                             *****
*****                                                              *****
************************************************************************
* === subroutine initFlow
* === subroutine compFlow(timeNow,fAi,fFi)
* === subroutine compJacobian(    fAi,fFi)
*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012

***********************************************************************
* === initializing the propagator =================================== *
      subroutine initFlow
      implicit none
      include 'xv_DS.include'        ! NAP,... | dimensions of DS
      include 'xv_DS_Propagator.include' ! parameter of DS
      integer*8 iDummy, jDummy, kDummy, NDummy
      real*8    fDummy
* --- begin of subroutine ------------------------------------------- *
      write(*,*) 'Initializing the C-L-Q propagator...'
      call readQijk
      call pressRETURN
* --- end of subroutine --------------------------------------------- *
      return
      end
      
* --- auxiliary routine for various subroutines --------------------- *
      include 'xs_DS_readQijk.f'

***********************************************************************
* === computing the propagator ====================================== *
      subroutine compFlow(timeNow,fAi,fFi)
      implicit none
      include 'xv_DS.include'        ! NAP,... | dimensions of DS
      include 'xv_DS_State.include'      ! fAi     | state of DS
      include 'xv_DS_Flow.include'       ! fFi     | propagator of DS
      include 'xv_DS_Propagator.include' ! parameters of DS, e.g. fQijk
      real*8    timeNow
      integer*8 iDummy, jDummy, kDummy
      real*8    fDummy
* --- begin of subroutine ------------------------------------------- *
      do iDummy = 1, NA
* ----- add constant term
        fDummy  = fQijk(iDummy,0,0)
* ----- add linear term
        do jDummy = 1, NA
          fDummy = fDummy + fQijk(iDummy,jDummy,0)*fAi(jDummy)
          fDummy = fDummy + fQijk(iDummy,0,jDummy)*fAi(jDummy)
* ------- add quadratic term
          do kDummy =1, NA
            fDummy = fDummy + fQijk(iDummy,jDummy,kDummy)
     &                      * fAi(jDummy)*fAi(kDummy)
          end do
        end do
* ----- exception for nonlinear eddy viscosity model
        if (iMode(iDummy).eq.-1002) fDummy = sqrt(max(0.0,fDummy)) 
* ----- 
        fFi(iDummy) = fDummy
      end do
* --- end of subroutine --------------------------------------------- *
      return
      end

***********************************************************************
* === computing the Jacobian of the propagator ====================== *
      subroutine compJacobian(fAi,fDFij)
      implicit none
      include 'xv_DS.include'        ! NAP,... | dimensions of DS
      include 'xv_DS_State.include'      ! fAi     | state of DS
      include 'xv_DS_Jacobian.include'   ! fDFij   | Jacobian of DS
      include 'xv_DS_Propagator.include' ! fQijk   | parameters of DS
      real*8    timeNow
      integer*8 iDummy, jDummy, kDummy
      real*8    fDummy
* --- begin of subroutine ------------------------------------------- *
* --- compute Jacobian
* --- Df_ij = q_ij0 + q_i0j + Sum_kl q_ikl D_j (a_k a_l)
* ---       = q_ij0 + q_i0j + Sum_kl q_ikl (delta_jk a_l + a_k delta_jl)
* ---       = q_ij0 + q_i0j + Sum_l q_ijl a_l + Sum_k q_ikj a_k )
* ---       = q_ij0 + q_i0j + Sum_k (q_ikj+q_ijk) a_k 
      do iDummy = 1, NA
      do jDummy = 1, NA
* ----- linear term Df = q_ij0 + q_i0j
        fDummy = fQijk(iDummy,jDummy,0)
     &         + fQijk(iDummy,0,jDummy)
        do kDummy = 1, NA
          fDummy = fDummy + fQijk(iDummy,jDummy,kDummy)*fAi(kDummy)
          fDummy = fDummy + fQijk(iDummy,kDummy,jDummy)*fAi(kDummy)
        end do
        fDFij(iDummy,jDummy) = fDummy
      end do
      end do
* --- end of subroutine --------------------------------------------- *
      return
      end
