************************************************************************
*****                                                              *****
***** @segment name: xs_DS_io.f                                    *****
***** @task: io routines for monitoring the state                  *****
***** @see:  res090905                                             *****
*****                                                              *****
************************************************************************
* === subroutine openFiles
* === subroutine closeFiles
* === subroutine saveState(timeNow,fAi)
*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012

***********************************************************************
* === opening (pre)gnuplot files for state evolution ================ *
      subroutine openFiles
      implicit none
      integer*8         iFile
      common / SaveDS / iFile
* --- begin of subroutine ------------------------------------------- *
      iFile = 0
      write(*,*) 'Opening z3_Times.dat for m-> t^m (unit=11) ...'
      open(11,file='z3_Times.dat',form='formatted',status='UNKNOWN')
      write(*,*) 'Opening z3_Amplitudes.dat ',
     &           'for m,i -> a_i^m (unit=12) ...'
      open(12,file='z3_Amplitudes.dat',
     &        form='formatted',status='UNKNOWN')
* --- end of subroutine --------------------------------------------- *
      return
      end

***********************************************************************
* === closing (pre)gnuplot files for state evolution ================ *
      subroutine closeFiles
      implicit none
      integer*8         iFile
      common / SaveDS / iFile
* --- begin of subroutine ------------------------------------------- *
      write(*,*) 'Closing z3_Times.dat ...'
      close(11)
      write(*,*) 'Closing z3_Amplitudes.dat ...'
      close(12)
      write(*,'(i5,a)') iFile, ' states saved'
* --- end of subroutine --------------------------------------------- *
      return
      end

************************************************************************
* === saving state at suitable times ================================ *
      subroutine saveState(timeNow,fAi)
      implicit none
      include 'xv_DS.include'     ! NAP,... | dimensions of DS
      include 'xv_DS_Time.include'    ! timeSave, ...
      include 'xv_DS_State.include'   ! fAi     | state of DS
      real*8    timeNow
      integer*8 iDummy, iFile
      common / SaveDS / iFile
* --- begin of subroutine -------------------------------------------- *
      if ((int((timeNow-0.5*timeStep)/timeSave).ne.
     &     int((timeNow+0.5*timeStep)/timeSave)).or. ! at t approx i*timeSave
     &    timeNow.lt.timeBegin+0.5*timeStep.or. ! at t approx tBegin
     &    timeNow.gt.timeEnd-0.5*timeStep) then ! at t approx tEnd
* ....... update z3_time.dat and z3_amplitude.dat
        iFile = iFile + 1
        write(11,'(i7,1x,f20.10)') iFile, timeNow
        do iDummy = 1, NA
          write(12,'(i8,1x,i4,1x,f20.10)') iFile, iDummy, fAi(iDummy)
        end do
      end if
* --- end of subroutine ---------------------------------------------- *
      return
      end
