************************************************************************
*****                                                              *****
***** @segment name: xs_FTT_Propagator.f                           *****
***** @task: initializing and computing the FTT propagator         *****
*****        --- always for a constant-linear-quadratic flow       *****
***** @see:  res090905                                             *****
*****                                                              *****
************************************************************************
* === subroutine initFlow
* === subroutine compFlow(timeNow,fAi,fFi)
*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012

***********************************************************************
* === initializing the propagator =================================== *
      subroutine initFlow
      implicit none
      include 'xv_DS.include'        ! NAP,... | dimensions of DS
      include 'xv_DS_Propagator.include' ! parameter of DS
      integer*8 iDummy, jDummy, kDummy, NDummy
      real*8    fDummy
* --- begin of subroutine ------------------------------------------- *
      write(*,*) 'Initializing the FTT propagator ...'
* ---
      call readQijk          ! load NA and q_ijk
      call readModeScheme    ! discriminate between A and C modes
      call readFTT           ! load X_ijk
* --- end of subroutine --------------------------------------------- *
      return
      end

***********************************************************************
* === computing the propagator ====================================== *
      subroutine compFlow(timeNow,fAi,fFi)
      implicit none
      include 'xv_DS.include'        ! NAP,... | dimensions of DS
      include 'xv_DS_State.include'      ! fAi     | state of DS
      include 'xv_DS_Flow.include'       ! fFi     | propagator of DS
      include 'xv_DS_Propagator.include' ! parameters of DS, e.g. fQijk
      include 'xv_FTT_Mode.include'      ! iModeFTT ... | mode scheme
      real*8    timeNow
* --- for FTT energy flow equation
      real*8    fMi(NAP), fEi(NAP)       ! m_i, E_i | state of FTT
      real*8    fFctQi, fFctTi           ! Q_i, T_i functions
* --- dummy variables
      integer*8 iDummy, jDummy, kDummy
      integer*8 iGroup, jGroup, kGroup
      real*8    fDummy
* --- begin of subroutine ------------------------------------------- *
* --- enforce partial thermal equilibrium in E_i
      call enforcePTE(fAi) ! acts only on C modes were a_i = E_i
* --- initialize for FTT energy equations
      do iDummy = 1, NA
        fMi(iDummy) = 0
        fEi(iDummy) = 0
        if (iModeFTT(iDummy).lt.0) fMi(iDummy)=fAi(iDummy)
        if (iModeFTT(iDummy).gt.0) fEi(iDummy)=fAi(iDummy)
      end do
* --- compute flow
      do iDummy = 1, NA
* ----- for B mode (dynamically resolved) ------------------------------
        if (iModeFTT(iDummy).eq.0) then
* ------- add constant, linear & quadratic term of A and B modes
          do jDummy = 0, NA
          do kDummy = 0, NA
            jGroup = iModeFTT(jDummy)
            kGroup = iModeFTT(kDummy)
            if (max(jGroup,kGroup).le.0) then
              fDummy = fDummy + fQijk(iDummy,jDummy,kDummy)
     &                                 * fAi(jDummy)
     &                                        * fAi(kDummy)
            end if
          end do
          end do
          fFi(iDummy) = fDummy
* ------- add eddy viscosity term 
* >>>>>>> NOT IMPLEMENTED YET
* ------- estimator for m_i and E_i (HARDWIRED !!!)
* >>>>>>> NOT IMPLEMENTED YET
* ------- for A mode (E_i=0, slaved) -----------------------------------
        else if (iModeFTT(iDummy).lt.0) then
* ------- add constant term
          fDummy  = fQijk(iDummy,0,0)
* ------- add linear term + Reynolds stress term
          do jDummy = 1, NA
            fDummy = fDummy +   fQijk(iDummy,jDummy,0)*fMi(jDummy)
            fDummy = fDummy +   fQijk(iDummy,0,jDummy)*fMi(jDummy)
            fDummy = fDummy + 2*fQijk(iDummy,jDummy,jDummy)*fEi(jDummy)
* --------- add quadratic term
            do kDummy =1, NA
              fDummy = fDummy + fQijk(iDummy,jDummy,kDummy)
     &                        * fMi(jDummy)*fMi(kDummy)
            end do
          end do
          fFi(iDummy) = fDummy
* ----- for C mode (m_i=0, thermodynamic) ------------------------------
        else if (iModeFTT(iDummy).gt.0) then
          fFi(iDummy) = fFctQi(iDummy,fMi,fEi) + fFctTi(iDummy,fEi)
* >>>>>>> no rescaling of alpha!       
        end if
* ----------------------------------------------------------------------
      end do
* --- Enforce PTE also in the propagator for the next step.
      call enforcePTE(fFi)
* --- end of subroutine --------------------------------------------- *
      return
      end

* --- auxiliary routines -------------------------------------------- *
      include 'xs_DS_readQijk.f'       ! readQijk
      include 'xs_FTT_read.f'          ! readModeScheme, readFTT
      include 'xs_FTT_eflow.f'         ! fFct[Q,t]i, enforcePTE
