************************************************************************
*****                                                              *****
***** @segment name: xs_FTT_init.f                                 *****
***** @task: routines for initializing the FTT system              *****
*****        --- always for a constant-linear-quadratic flow       *****
***** @see:  res090905                                             *****
*****                                                              *****
************************************************************************
* === subroutine initModeScheme
* === subroutine compFTT
*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012

***********************************************************************
* === initialize mode scheme ======================================== *
      subroutine initModeScheme
      implicit none
      include 'xv_DS.include'     ! NAP,... | dimensions of DS
      include 'xv_FTT_Mode.include'   ! iModeFTT, fModeFTT | mode class
      integer*8 iDummy
* --- begin of subroutine -------------------------------------------- *
      write(*,*) 'Initialize all modes as independent C modes'
      write(*,*) 'and saving this state in y4.mode.dat ...'
* --- writing
      do iDummy = 1, NA
        iModeFTT(iDummy) = iDummy
        fModeFTT(iDummy) = 1
        write(*,*) iDummy,iModeFTT(iDummy),fModeFTT(iDummy)
      end do
* --- end of subroutine ---------------------------------------------- *
      return
      end

************************************************************************
* === computing FTT system parameters for dE_i/dt = Q_i + T_i ======== *
      subroutine compFTT
      implicit none
      include 'xv_DS.include'         ! NAP,...   | dimensions of DS
      include 'xv_DS_Propagator.include'  ! fQijk ... | coeff. of DS
      include 'xv_FTT_Propagator.include' ! fXijk ... | coeff. of FTT
      integer*8 iDummy, jDummy, kDummy
      real*8    fDummy
* --- begin of subroutine -------------------------------------------- *
      write(*,*) 'Computing FTT system ...'
* --- initialize
      fAlpha = 1
      do iDummy = 1, NA
      do jDummy = 0, NA
      do kDummy = 0, NA
        fXijk(iDummy,jDummy,kDummy) = 0
      end do
      end do
      end do
* --- computing X_i
      do iDummy = 1, NA
        fXijk(iDummy,0,0) = 2 * ( fQijk(iDummy,iDummy,0)
     &                          + fQijk(iDummy,0,iDummy) )
* ----- computing X_ij
        do jDummy = 1, NA
          fXijk(iDummy,jDummy,0) = 2 * ( fQijk(iDummy,iDummy,jDummy)
     &                                 + fQijk(iDummy,jDummy,iDummy) )
* ------- computing X_ijk
          do kDummy = 1, NA
            fDummy =          abs(fQijk(iDummy,jDummy,kDummy))
            fDummy = fDummy + abs(fQijk(iDummy,kDummy,jDummy))
            fDummy = fDummy + abs(fQijk(jDummy,iDummy,kDummy))
            fDummy = fDummy + abs(fQijk(jDummy,kDummy,iDummy))
            fDummy = fDummy + abs(fQijk(kDummy,iDummy,jDummy))
            fDummy = fDummy + abs(fQijk(kDummy,jDummy,iDummy))
            fXijk(iDummy,jDummy,kDummy) = fDummy/6.
          end do
        end do
      end do
* --- end of subroutine ---------------------------------------------- *
      return
      end

***********************************************************************
* === computing FTT system parameters =============================== *
      subroutine compFTT2
      implicit none
      include 'xv_DS.include'         ! NAP,...   | dimensions of DS
      include 'xv_DS_Propagator.include'  ! fQijk ... | coeff. of DS
      include 'xv_FTT_Mode.include'       ! iModeFTT ... | mode scheme
      include 'xv_FTT_Propagator.include' ! fXijk ... | coeff. of FTT
      integer*8 iDummy, jDummy, kDummy, iGroup
      real*8    fDummy
* --- begin of subroutine -------------------------------------------- *
      write(*,*) 'Cleaning DS / FTT system for A,B,C-modes...'
* --- initialize
      do iDummy = 1, NA
        iGroup = iModeFTT(iDummy)
* ----- no energy equation for slaved / A-modes (X_ijk=0)
        if (iGroup.lt.0) then
          write(*,*) 'Zeroing X_ijk for i=',iDummy
          do jDummy = 0, NA
          do kDummy = 0, NA
            fXijk(iDummy,jDummy,kDummy) = 0.
          end do
          end do
        end if
* ----- no DS equation for thermodynamic / C-modes (q_ijk=0)
        if (iGroup.gt.0) then
          write(*,*) 'Zeroing q_ijk for i=',iDummy
          do jDummy = 0, NA
          do kDummy = 0, NA
            fQijk(iDummy,jDummy,kDummy) = 0.
          end do
          end do
        end if
      end do
* --- end of subroutine ---------------------------------------------- *
      return
      end
