************************************************************************
*****                                                              *****
***** @segment name: xs_FTT_read.f                                 *****
***** @task: reading FTT system                                    *****
*****        --- always for a constant-linear-quadratic flow       *****
***** @see:  res090905                                             *****
*****                                                              *****
************************************************************************
* === subroutine readModeScheme
* === subroutine readStateFTT(fMi,fEi)
* === subroutine readFTT
* === subroutine readFTT2
*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012

************************************************************************
* === reading mode scheme ============================================ *
      subroutine readModeScheme
      implicit none
      include 'xv_DS.include'     ! NAP,... | dimensions of DS
      include 'xv_FTT_Mode.include'   ! iModeFTT, fModeFTT | mode class
      integer*8 iDummy, jDummy
      real*8    fDummy
* --- begin of subroutine -------------------------------------------- *
      write(*,*) 'Reading the mode scheme from << y4_mode.dat >> ...'
      open(10,file='y4_Modes.dat',form='formatted',status='old')
      do iDummy = 1, NA ! assuming a COMPLETE ordered list!!!
         write(*,*) iDummy
        read(10,*) jDummy, iModeFTT(iDummy),fModeFTT(iDummy)
        write(*,'(2i5,f12.5)') 
     &             iDummy, iModeFTT(iDummy), fModeFTT(iDummy)
        if (iDummy.ne.jDummy)
     &     stop "Error in readMode: input not ordered."
      end do
      close(10)
      call pressRETURN
* --- end of subroutine ---------------------------------------------- *
      return
      end

************************************************************************
* === read state ===================================================== *
      subroutine readStateFTT(fMi,fEi)
      implicit none
      include 'xv_DS.include'     ! NAP,... | dimensions of DS
      include 'xv_FTT_Mode.include'   ! iModeFTT, fModeFTT | mode class
      real*8    fMi(NAP), fEi(NAP)    ! m_i, E_i | state of FTT
      integer*8 iDummy, jDummy
      real*8    fDummy
* --- begin of subroutine -------------------------------------------- *
      write(*,*) 'Read m_i and E_i from << y4_State.dat >> ...'
      write(*,*) 'No dynamic FTT, i.e. all modes are assumed to be'
      write(*,*) 'A modes (E_i=0) or C modes (m_i=0).'
* --- initialize
      do iDummy = 1, NAP
        fMi(iDummy) = 0.0
        fEi(iDummy) = 0.0
      end do
* --- writing
      open(10,file='y4_State.dat',form='formatted',status='old')
      do iDummy = 1, NA
        if (iModeFTT(iDummy).lt.0) then       ! A mode
          read(10,*) jDummy,fMi(iDummy)
        else if (iModeFTT(iDummy).gt.0) then  ! C mode
          read(10,*) jDummy, fEi(iDummy)
        else if (iModeFTT(iDummy).eq.0) then
          stop "Error in readStateFTT: B modes not implemented yet."
        end if
        write(*,'(2i5,2f12.5)')
     &          iDummy,iModeFTT(iDummy),fMi(iDummy),fEi(iDummy)
      end do
      close(10)
      call pressRETURN
* --- end of subroutine ---------------------------------------------- *
      return
      end

***********************************************************************
* === read FTT system parameters ==================================== *
      subroutine readFTT
      implicit none
      include 'xv_DS.include'         ! NAP,...   | dimensions of DS
      include 'xv_FTT_Mode.include'       ! iModeFTT ... | mode scheme
      include 'xv_FTT_Propagator.include' ! fXijk ... | coeff. of FTT
      integer*8 iDummy, jDummy, kDummy
      real*8    fDummy
* --- begin of subroutine -------------------------------------------- *
      write(*,*) 'Reading FTT system from << y4_FTT_Xijk.dat >> ... '
      write(*,*) '--- energy equations for B- and C-modes ---'
* --- initialize
      NA = 0
      do iDummy = 1, NAP
      do jDummy = 0, NAP
      do kDummy = 0, NAP
        fXijk(iDummy,jDummy,kDummy) = 0.0
      end do
      end do
      end do
* --- Reading FTT system
      open(10,file='y4_FTT_Xijk.dat',form='formatted',status='old')
* ---
      read(10,*) fAlpha
* ---
  111 read(10,*,end=199,err=199) iDummy, jDummy, kDummy, fDummy
      fXijk(iDummy,jDummy,kDummy) = fDummy
      NA = max(NA,iDummy)
      goto 111
 199  close(10)
      call pressRETURN
* --- end of subroutine ---------------------------------------------- *
      return
      end

***********************************************************************
* === read FTT system parameters ==================================== *
      subroutine readFTT2
      implicit none
      include 'xv_DS.include'         ! NAP,...   | dimensions of DS
      include 'xv_DS_Propagator.include'  ! fQijk ... | coeff. of FTT
      integer*8 iDummy, jDummy, kDummy
      real*8    fDummy
* --- begin of subroutine -------------------------------------------- *
      write(*,*) 'Reading FTT system from << y4_FTT_Qijk.dat >> ... '
      write(*,*) 'ODE equations for A- and B-modes ...'
* --- initialize
C XXX NA = 0
      do iDummy = 1, NAP
      do jDummy = 0, NAP
      do kDummy = 0, NAP
        fQijk(iDummy,jDummy,kDummy) = 0.0
      end do
      end do
      end do
* --- Reading FTT system
      open(10,file='y4_FTT_Qijk.dat',form='formatted',status='old')
* ---
  111 read(10,*,end=199,err=199) iDummy, jDummy, kDummy, fDummy
      fQijk(iDummy,jDummy,kDummy) = fDummy
      NA = max(NA,iDummy)
      goto 111
 199  close(10)
      call pressRETURN
* --- end of subroutine ---------------------------------------------- *
      return
      end
