************************************************************************
*****                                                              *****
***** @segment name: xs_FTT_save.f                                 *****
***** @task: save FTT system                                       *****
*****        --- always for a constant-linear-quadratic flow       *****
***** @see:  res090905                                             *****
*****                                                              *****
************************************************************************
* === subroutine saveModeScheme
* === subroutine saveStateFTT(fMi,fEi)
* === subroutine saveFTT   ! energy equation
* === subroutine saveFTT2  ! DS equation
*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012

************************************************************************
* === save mode scheme =============================================== *
      subroutine saveModeScheme
      implicit none
      include 'xv_DS.include'     ! NAP,... | dimensions of DS
      include 'xv_FTT_Mode.include'   ! iModeFTT, fModeFTT | mode class
      real*8    fEi(NAP), fQi(NAP), fTi(NAP)
      integer*8 iDummy
      real*8    fDummy
* --- begin of subroutine -------------------------------------------- *
      write(*,*) 'Save mode scheme in << y4_mode.dat >> ...'
* --- writing
      open(10,file='y4_Modes.dat',form='formatted',status='UNKNOWN')
      do iDummy = 1, NA
        write(10,'(i5,1x,i5,f20.10)') 
     &             iDummy,iModeFTT(iDummy),fModeFTT(iDummy)
      end do
      close(10)
* --- end of subroutine ---------------------------------------------- *
      return
      end

************************************************************************
* === save initial conditions ======================================== *
      subroutine saveStateFTT(fMi,fEi)
      implicit none
      include 'xv_DS.include'     ! NAP,... | dimensions of DS
      include 'xv_FTT_Mode.include'   ! iModeFTT, fModeFTT | mode class
      real*8    fMi(NAP), fEi(NAP)    ! m_i, E_i | state of FTT
      integer*8 iDummy
* --- begin of subroutine -------------------------------------------- *
      write(*,*) 'Save IC for FTT in << y4_State.dat >> ...'
      write(*,*) 'No dynamic FTT, i.e. all modes are assumed to be'
      write(*,*) 'A modes (E_i=0) or C modes (m_i=0).'
* --- writing
      open(10,file='y4_State.dat',form='formatted',status='UNKNOWN')
      do iDummy = 1, NA
        if (iModeFTT(iDummy).lt.0) then       ! A mode
          write(10,'(i5,1x,f20.10,1x,a)') 
     &             iDummy,fMi(iDummy),  ' ! m_i for A mode'
        else if (iModeFTT(iDummy).gt.0) then  ! C mode
          write(10,'(i5,1x,f20.10,1x,a)')
     &             iDummy, fEi(iDummy), ' ! E_i for A mode'
        else if (iModeFTT(iDummy).eq.0) then
          stop "Error in saveStateFTT: B modes not implemented yet."
        end if
      end do
      close(10)
* --- end of subroutine ---------------------------------------------- *
      return
      end

************************************************************************
* === save FTT system parameters ===================================== *
      subroutine saveFTT
      implicit none
      include 'xv_DS.include'         ! NAP,...   | dimensions of DS
C XXX include 'xv_gedit DS_Propagator.include'  ! fQijk ... | coeff. of DS
      include 'xv_FTT_Propagator.include' ! fXijk ... | coeff. of FTT
      integer*8 iDummy, jDummy, kDummy
      real*8    fDummy
* --- begin of subroutine -------------------------------------------- *
      write(*,*) 'Saving FTT system in << y4_FTT_Xijk.dat >> ... '
* --- Saving FTT system
      open(10,file='y4_FTT_Xijk.dat',form='formatted',status='UNKNOWN')
      write(10,'(f20.10,1x,a)') fAlpha, ' ! Alpha parameter for X_ijk'
* --- write "X_i" terms
      do iDummy = 1, NA
        fDummy = fXijk(iDummy,0,0)
        if (fDummy.ne.0) then
          write(10,'(3(i5,1x),f20.10,1x,a)') 
     &             iDummy,0,0,fDummy, ' ! X_i for Q_i'
        end if
      end do
* --- write "X_ij" terms summarizing equivalent terms
      do iDummy = 1, NA
      do jDummy = 1, NA
        fDummy = fXijk(iDummy,jDummy,0) + fXijk(iDummy,0,jDummy)
        if (fDummy.ne.0) then
          write(10,'(3(i5,1x),f20.10,1x,a)') 
     &             iDummy,jDummy,0,fDummy, ' ! X_ij for Q_i'
        end if
      end do
      end do
* --- write "X_ijk" terms (symmetrised)
      do iDummy = 1, NA
      do jDummy = 1, NA
      do kDummy = 1, NA
        fDummy = fXijk(iDummy,jDummy,kDummy)
     &         + fXijk(iDummy,kDummy,jDummy)
        fDummy = fDummy / 2
        if (fDummy.ne.0) then
          write(10,'(3(i5,1x),f20.10,1x,a)') 
     &              iDummy,jDummy,kDummy,fDummy, ' ! X_ijk for T_i'
        end if
      end do
      end do
      end do
* ---
      close(10)    
* --- end of subroutine ---------------------------------------------- *
      return
      end

************************************************************************
* === save FTT system parameters for dynamic resolution ============== *
      subroutine saveFTT2
      implicit none
      include 'xv_DS.include'         ! NAP,...   | dimensions of DS
      include 'xv_DS_Propagator.include'  ! fQijk ... | coeff. of DS
      integer*8 iDummy, jDummy, kDummy
      real*8    fDummy
* --- begin of subroutine -------------------------------------------- *
      write(*,*) 'Saving FTT system in << y4_FTT_Qijk.dat >> ... '
* --- Saving FTT system
      open(10,file='y4_FTT_Qijk.dat',form='formatted',status='UNKNOWN')
* --- write "Q_ijk" terms (symmetrised)
      do iDummy = 1, NA
      do jDummy = 0, NA
      do kDummy = 0, NA
        fDummy = fQijk(iDummy,jDummy,kDummy)
        if (fDummy.ne.0) then
          write(10,'(3(i5,1x),f20.10,1x,a)') 
     &              iDummy,jDummy,kDummy,fDummy, ' ! q_ijk for FTT'
        end if
      end do
      end do
      end do
* ---
      close(10)    
* --- end of subroutine ---------------------------------------------- *
      return
      end
