************************************************************************
*****                                                              *****
***** @segment name: xs_ioAmplitudes.f                             *****
***** @task: io routines for amplitudes                            *****
*****                                                              *****
************************************************************************
* === subroutine readTimes1      (fTm)
* === subroutine readAmplitudes1 (fAim)
* === subroutine writeAmplitudes1(fAim)
*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012
      
************************************************************************
* === reading the times of flow data ================================= *
      subroutine readTimes1(fTm)
      implicit none
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
* --- variables
      real*8    fTm(NFlowsP)
* --- auxiliary variables
      integer*8 iFlow, NFlowsX
      real*8    fDummy, fMax, fMin
* --- begin of subroutine -------------------------------------------- *
* --- user info
      write(*,*) 'Reading << z1_Times.dat >> ...'
* --- zero times
      do iFlow = 1, NFlowsP
        fTm(iFlow) = 0.
      end do
* --- initialize statistics
      NFlowsX = 0
* --- reading the times
 111  open(10,file='z1_Times.dat',form='formatted',status='old')
      read(10,*,end=119,err=119) iFlow, fDummy
      fTm(iFlow) = fDummy
      NFlowsX = max(iFlow,NFlowsX)
      goto 111
 119  close(10)
* --- user info
      write(*,'(2(a,f12.5),a,i8)') 
     &        't_min=',fTm(1),'; t_max=',fTm(NFlowsX),'; M=',NFlowsX
* --- update global variables
      NFlows = NFlowsX
* --- end of subroutine ---------------------------------------------- *
      return
      end
      
************************************************************************
* === reading the amplitudes of flow data ============================ *
      subroutine readAmplitudes1(fAim)
      implicit none
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
* --- variables
      real*8    fAim(-NAMP:NEMP,NFlowsP)
* --- auxiliary variables
      integer*8 iMode,  NModesX
      integer*8 iFlow,  NFlowsX
      real*8    fDummy, fMax, fMin
* --- begin of subroutine -------------------------------------------- *
* --- user info
      write(*,*) 'Reading << z1_Amplitudes.dat >> ...'
* --- zero amplitudes
      do iMode = -NAMP,NEMP
      do iFlow = 1, NFlowsP
        fAim(iMode,iFlow) = 0.
      end do
      end do
* --- initialize statistics
      NModesX = 0
      NFlowsX = 0
      fMin    =  1.E37
      fMax    = -1.E37
* --- read amplitudes
 121  open(10,file='z1_Amplitudes.dat',form='formatted',status='old')
      read(10,*,end=129,err=129) iFlow, iMode, fDummy
      fAim(iMode,iFlow) = fDummy !
      fMax    = max(fMax,  fDummy) 
      fMin    = min(fMin,  fDummy) 
      NModesX = max(iMode,NModesX)
      NFlowsX = max(iFlow, NFlowsX)
      goto 121
 129  close(10)
* --- user  info
      write(*,'(2(a,f12.5),2(a,i8))') 
     &        'min(a_i)=',fMin,'; max(a_i)=',fMax,
     &        '; N=',NModesX, '; M=',NFlowsX
* --- update global variables
      NEM    = NModesX
      NFlows = NFlowsX
* --- end of subroutine ---------------------------------------------- *
      return
      end

************************************************************************
* === write the amplitudes of flow data ============================== *
      subroutine writeAmplitudes1(fAim)
      implicit none
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
* --- variables
      real*8    fAim(-NAMP:NEMP,NFlowsP)
* --- auxiliary variables
      integer   iMode, iFlow

      real*8    fDummy, fMax, fMin
* --- begin of subroutine -------------------------------------------- *
* --- user info
      write(*,*) 'Writing << z1_Amplitudes.dat >> ...'
* --- initialize statistics
      fMin    =  1.E37
      fMax    = -1.E37
* --- writing the amplitudes
      open(10,file='z1_Amplitudes.dat',
     &        form='formatted',status='UNKNOWN')
      do iFlow =    1, NFlows
      do iMode = -NAM, NEM
        if (abs(fAim(iMode,iFlow)).gt.0) then
          write(10,'(i5,1x,i5,1x,f20.10)')
     &             iFlow, iMode, fAim(iMode,iFlow)
          fAim(iMode,iFlow) = fDummy !
          fMax    = max(fMax,  fDummy) 
          fMin    = min(fMin,  fDummy) 
        end if
      end do
      end do
      close(10)
      write(*,'(2(a,f12.5),2(a,i8))') 
     &        'min(a_i)=',fMin,'; max(a_i)=',fMax,
     &        '; N=',NEM, '; M=',NFlows
* --- end of subroutine ---------------------------------------------- *
      return
      end

