************************************************************************
*****                                                              *****
***** @segment name: xs_ioCORE.f                                   *****
***** @task: I/O routines for lines, filenames, ...                *****
***** @see:  res.20100408.xAMC_ToDo.tex                            *****
*****                                                              *****
************************************************************************
* === subroutine readLine    (iUnit,Line,bOK)
* === subroutine readFilename(Line,fn,fnLength)
* ===    |______ logical function bFctBeginFilename(Ch)
* === subroutine countLinesUnit(iUnit,fn,fnLength,NLine)
* === subroutine readFilenameUnit(iUnit,fnUnit,fnUnitLength,
* ===&                            iFile,fn,    fnLength)
*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012

************************************************************************
* === read line (skip lines without contents)  ======================= *
      subroutine readLine(iUnit,Line,bOK)
      implicit none
      integer       iUnit    ! unit of file  (has to integer*4)
      character*120 Line     ! line read from file
      logical       bOK      ! true if a line could be read
* --- auxiliary variables
      integer       iCh, iChMin, iChMax
* --- begin segment -------------------------------------------------- *
* --- initialize
      bOK  = .true.          ! default
  101 do iCh=1,120
         Line(iCh:iCh) = ' ' ! erase line
      end do
      read(iUnit,'(a)',end=999) Line
C XXX write(*,'(a)') Line    ! do not echo (too many calls)
* --- skip comments -------------------------------------------------- *
      iChMax = index(Line,'[')
      if (iChMax.eq.1) goto 101   ! do if << [ >> found in first column
      iChMax = iChMax -1          ! do if << [ >> found AFTER first coumn
      if (iChMax.le.0) iChMax=120 ! do if << [ >> NOT found 

* --- skip empty lines before '[' ------------------------------------ *
      iChMin = 1
      do while (iChar(Line(iChMin:iChMin)).eq.iChar(' ').and.
     &          iChMin.lt.iChMax)
         iChMin = iChMin + 1
      end do
      if (iChMin.ge.iChMax) goto 101
      return
* --- error path ----------------------------------------------------- *
  999 bOK = .false.
* --- end segment ---------------------------------------------------- *
      return
      end

************************************************************************
* === read file name ================================================= *
      subroutine readFilename(Line,fn,fnLength)
      implicit none
      character*120 Line
      character*120 fn
      integer       fnLength, iCh, iChMin, iChMax, iASCII
      logical bFctBeginFilename
* --- begin segment -------------------------------------------------- *
C XXX write(*,'(a,a,a)') 'Reading the file name from ',Line(1:20),' ...'
C XXX Do not inform use. Too many calls.
* --- truncate string from comments starting with [
      iChMax = index(Line,'[')-1
      if (iChMax.lt.0) iChMax=120
      do while (Line(iChMax:iChMax).eq.' '.and.iChMax.gt.1)
         iChMax = iChMax - 1
      end do
* --- search for first Character (which in the alphabet)
* --- Check for '/', then upper case character,
* --- then for lower case character.
      iChMin = 1
      do while (.not.bFctBeginFilename(Line(iChMin:iChMin)).and.
     &          iChMin.lt.iChMax)
         iChMin = iChMin + 1
      end do
      if (iChMin.eq.iChMax) stop 'No file name found'
* ---
      fn       = Line(iChMin:iChMax)
      fnLength = iChMax-iChMin+1
C XXX write(*,'(a,a,a,i3)') ' Name=',fn(1:fnLength),'; length=',fnLength
C XXX Do not inform user. Too many calls.
* --- end segment ---------------------------------------------------- *
      return
      end

************************************************************************
* === determine if character is permissible as first character of fn = *
      logical function bFctBeginFilename(Ch)
      implicit none
      character*1 Ch
      logical bCh
* --- begin of segment ----------------------------------------------- *
C XXX write(*,*) 'bFctBeginFilename << ',Ch,' >> ...'
      bCh = .false.
* ... '/' is a permissible letter as file name
      if (Ch.eq.'/') bCh=.true.
* ... 'A'--'Z' dto
      if (iChar(Ch).ge.iChar('A').and.
     &    iChar(Ch).le.iChar('Z')) bCh=.true.
* ... 'a'--'z' dto
      if (iChar(Ch).ge.iChar('a').and.
     &    iChar(Ch).le.iChar('z')) bCh=.true.
      bFctBeginFilename = bCh
* --- end of segment ------------------------------------------------- *
      return
      end

************************************************************************
* === count non-empty lines of a file ================================ *
      subroutine countLinesUnit(iUnit,fn,fnLength,NLine)
      implicit none
      integer       iUnit    ! unit of file  (has to integer*4)
      character*120 fn       ! file name
      integer       fnLength ! length of file name
      integer       NLine
* --- auxiliary variables
      character*120 Line     ! line read from file
      logical       bOK      ! true if a line could be read
* --- begin segment -------------------------------------------------- *
      write(*,'(3a)') 'Counting lines in <<',fn(1:fnLength),'>> ...'
* --- return negative number of files if the file does not exist
      inquire(file=fn(1:fnLength),exist=bOK)
      if (.not.bOK) then
        NLine = -1
        return
      end if
* --- count number of non-empty lines
      open(iUnit,file=fn(1:fnLength),status='old',form='formatted')
      NLine = 0
  101 call readLine(iUnit,Line,bOK)
      if (bOK) then
        NLine = NLine +1
        goto 101
      end if
      close(iUnit)
* --- end of segment ------------------------------------------------- *
      return
      end

************************************************************************
* === read file name listed in a file ================================ *
      subroutine readFilenameUnit(iUnit,fnUnit,fnUnitLength,
     &                            iFile,fn,    fnLength)
      implicit none
      integer       iUnit         ! unit of file  (has to integer*4)
      character*120 fnUnit        ! file name of Input
      integer       fnUnitLength  ! length of this file name
      integer       iFile         ! file index
      character*120 fn            ! file name
      integer       fnLength      ! length of file name
* --- auxiliary variables
      character*120 Line          ! line read from file
      logical       bOK           ! true if a line could be read
      integer       iDummy        ! dummy variables
* --- begin segment -------------------------------------------------- *
C XXX No user info: too many calls.
C XXX write(*,'(a,i6,3a)') 'Read filename #',iFile,
C XXX&                      ' from <<',fnUnit(1:fnUnitLength),'>> ...'
* --- return negative length of file name if file does not exist
      inquire(file=fnUnit(1:fnUnitLength),exist=bOK)
      if (.not.bOK) then
        fnLength=-1
        return
      end if
* --- set fnLength=0 in case the number is not found
      fn(1:1)  = '0'
      fnLength =  0
* --- count number of non-empty lines
      open(iUnit,file=fnUnit(1:fnUnitLength),
     &           status='old',form='formatted')
  101 call readLine(iUnit,Line,bOK)
* --- evaluating an existing line
      if (bOK) then
* ----- reading the file number and extracting the file
* ----- if the file number is OK.
        read(Line,*) iDummy
        if (iDummy.ne.iFile) goto 101
* ----- now the right file name has been found
        call readFilename(Line,fn,fnLength)
      end if
      close(iUnit)
* --- end of segment ------------------------------------------------- *
      return
      end