************************************************************************
*****                                                              *****
***** @segment name: xs_ioCorrMatrSub.f                            *****
***** @task: I/O routines for correlation matrix ...               *****
***** @see:  res.20100408.xAMC_ToDo.tex                            *****
*****                                                              *****
************************************************************************
* === subroutine writeCorrMatr   (        fMatrix)
* === subroutine writeCorrMatrSub(iDomain,fMatrixSub)
* === subroutine readCorrMatrSub (iDomain,fMatrixSub,bOK)
*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012

************************************************************************ 
* === write the correlation matrix =================================== *
      subroutine writeCorrMatr(fMatrix)
      implicit none
* --- input parameters for filename and domain
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
      integer iDomain                     ! domain number 
      real*8  fMatrix(NFlowsP,NFlowsP)    ! correlation matrix
      character*120 fn                    ! file name
      integer       fnLength              ! length of file name
* --- auxiliary dummy variables
      integer iFlow, jFlow
* --- begin of segment ----------------------------------------------- *
* --- user info
      write(*,*) '***** Writing the correlation matrix in ',
     &           '<<z1_CorrelationMatrix.dat>>...'
* --- write correlation matrix
      open(10,file='z1_CorrelationMatrix.dat',
     &        status='UNKNOWN',form='formatted')
      do iFlow = 1, NFlows
      do jFlow = 1, NFlows
        write(10,'(2(i5,1x),f20.10)') iFlow,jFlow,fMatrix(iFlow,jFlow)
      end do
      end do
      close(10)
* --- end of segment ------------------------------------------------- *
      return
      end

************************************************************************ 
* === write the correlation matrix (of subdomain) ==================== *
      subroutine writeCorrMatrSub(iDomain,fMatrixSub)
      implicit none
* --- input parameters for filename and domain
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
      integer iDomain                     ! domain number 
      real*8  fMatrixSub(NFlowsP,NFlowsP) ! correlation matrix
      character*120 fn                    ! file name
      integer       fnLength              ! length of file name
* --- auxiliary dummy variables
      integer iFlow, jFlow
* --- begin of segment ----------------------------------------------- *
* --- name of correlation matrix
* ................1234567890123456789012345
      fn       = 'z1_CorrelationMatrix.0000'
      fnLength =  25
      write(fn,'(a,i4.4)') 'z1_CorrelationMatrix.',iDomain
* --- user info
      write(*,*) '***** Writing the correlation matrix in <<',
     &           fn(1:fnLength),'>>...'
* --- write correlation matrix
      open(10,file=fn(1:fnLength),status='UNKNOWN',form='formatted')
      do iFlow = 1, NFlows
      do jFlow = 1, NFlows
        write(10,'(i5,1x,i5,1x,f20.10)') 
     &           iFlow,jFlow,fMatrixSub(iFlow,jFlow)
      end do
      end do
      close(10)
* --- end of segment ------------------------------------------------- *
      return
      end

************************************************************************ 
* === read the correlation matrix (of subdomain) ===================== *
      subroutine readCorrMatrSub(iDomain,fMatrixSub,bOK)
      implicit none
* --- input parameters for filename and domain
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
      integer iDomain                     ! domain number
      real*8  fMatrixSub(NFlowsP,NFlowsP) ! correlation matrix
      logical bOK                         ! boolean variable=.true.for ex.
* --- auxiliary dummy variables
      character*120 fn                    ! file name
      integer       fnLength              ! length of file name
      integer       iFlow, jFlow, iDummy
      real*8        fDummy
* --- begin of segment ----------------------------------------------- *
* --- name of correlation matrix
* ................1234567890123456789012345
      fn       = 'z1_CorrelationMatrix.0000'
      fnLength =  25
      write(fn,'(a,i4.4)') 'z1_CorrelationMatrix.',iDomain
      write(*,*) '***** Reading the mass matrix from <<',
     &           fn(1:fnLength),'>>...'
      call countLinesUnit(10,fn,fnLength,iDummy)
      if (iDummy.lt.0) then
        write(*,*) 'File does not exist!'
        bOK = .false.
        return
      else if (iDummy.eq.0) then
        write(*,*) 'File is empty!'
        bOK = .false.
        return
      else
        bOK = .true.
      end if 
* --- initialize
      do jFlow=1,NFlowsP
      do iFlow=1,NFlowsP
        fMatrixSub(iFlow,jFlow) = 0.0
      end do
      end do
* --- write correlation matrix
      open(10,file=fn(1:fnLength),status='old',form='formatted')
  101 read(10,*,end=199) iFlow,jFlow,fDummy
      fMatrixSub(iFlow,jFlow) = fDummy
      goto 101
 199  close(10)
* --- end of segment ------------------------------------------------- *
      return
      end
