************************************************************************
*****                                                              *****
***** @segment name: xs_ioDS.f                                     *****
***** @task: I/O routines for dynamical system                     *****
***** @see:  res.20110712                                          *****
*****                                                              *****
************************************************************************
* === subroutine writeYDS
* ----------------------------------------------------------------------
* === subroutine writeTime(timeNow)
* === subroutine readTime (timeNow)
* === subroutine showTime (timeNow)
* ----------------------------------------------------------------------
* === subroutine writeState(fAi)
* === subroutine readState (fAi)
* === subroutine readStateData(fAi)
* === subroutine showState (timeNow,fAi)
* ----------------------------------------------------------------------
* === subroutine writeMode
* === subroutine readMode
* ----------------------------------------------------------------------
* === subroutine writeQijk
* === subroutine readQijk
*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012

************************************************************************
* === Write job file ***************************************************
      subroutine writeYDS
      implicit none
      include 'xv_DS.include'      ! NA, NAP, iMode
      integer NModes
* --- Begin segment -------------------------------------------------- *
      write(*,*) 'Writing << y3_DS.dat >>...', NA
* --- Open file ------------------------------------------------------ *
      open(10,file='y3_DS.dat',form='formatted',status='UNKNOWN')
* --- Write in file -------------------------------------------------- *
      write(10,'(a)') '[*** standard file from xGS ***]'
      write(10,'(i5,  5x,a)')  NA,'[N]'
      write(10,'(a14, 8x,a)') 'y3_DS_Qijk.dat', 
     &                        '[DS for natural dynamics]'
      write(10,'(a12,10x,a)') 'y3_State.dat',
     &                        '[initial condition]'
      write(10,'(a11,11x,a)') 'y3_Time.dat',
     &                        '[time]'
* --- Close file ----------------------------------------------------- *
      close(10)
* --- End of segment ------------------------------------------------- *
      return
      end

************************************************************************
* === Write time integration parameters for dynamical system ***********
      subroutine writeTime
      implicit none
      include 'xv_DS_Time.include'      ! timeBegin, timeEnd, ...
* --- Begin segment -------------------------------------------------- *
      write(*,*) 'Writing << y3_Times.dat >> ...'
* --- Open file ------------------------------------------------------ *
      open(10,file='y3_Time.dat',form='formatted',status='UNKNOWN')
* --- Write in file -------------------------------------------------- *
      write(10,'(f12.5,5x,a)') timeBegin,'[t_0 ]'
      write(10,'(f12.5,5x,a)') timeEnd,  '[t_1 ]'
      write(10,'(f12.5,5x,a)') timeStep, '[dt, 1000 steps per unit]'
      write(10,'(f12.5,5x,a)') timeShow, '[dt_Show, 10 steps per unit]'
      write(10,'(f12.5,5x,a)') timeSave, '[dt_Save, 10 steps per unit]'
* --- Close file ----------------------------------------------------- *
      close(10)
* --- End of segment ------------------------------------------------- *
      return
      end

***********************************************************************
* === Reading the time integration parameters ======================= *
      subroutine readTime(timeNow)
      implicit none
      include 'xv_DS_Time.include'     ! timeBegin, timeEnd, ...
      real*8  timeNow
* --- begin of subroutine ------------------------------------------- *
      write(*,*) 'Reading << y3_Time.dat >> ...'
      open(10,file='y3_Time.dat',form='formatted',status='old')
      read(10,*) timeBegin
      read(10,*) timeEnd
      read(10,*) timeStep
      read(10,*) timeShow
      read(10,*) timeSave
      close(10)
      timeNow = timeBegin
* --- end of subroutine --------------------------------------------- *
      return
      end

***********************************************************************
* === Showing the time integration parameters ======================= *
      subroutine showTime(timeNow)
      implicit none
      include 'xv_DS_Time.include'
      real*8  timeNow
* --- begin of subroutine ------------------------------------------- *
      write(*,*) 'Showing the time integration parameters ...'
      write(*,'(3(a8,f12.5,2x))') 't=',      timeNow,
     &                            't_0=',    timeBegin,
     &                            't_1=',    timeEnd
      write(*,'(3(a8,f12.5,2x))') 'dt=',     timeStep,
     &                            'dt_Show=',timeShow,
     &                            'dt_Save=',timeSave
* --- End of subroutine ---------------------------------------------- *
      return
      end

************************************************************************
* === Write IC for dynamical system ************************************
      subroutine writeState(fAi)
      implicit none
      include 'xv_DS.include'      ! NA, NAP, iMode
      include 'xv_DS_State.include'    ! fAi(NAP)
      integer iDummy, jDummy, NDummy   ! dummy variables
      real*8  fDummy                   ! dummy variable
* --- Begin segment -------------------------------------------------- *
      write(*,*) 'Writing << y3_State.dat >> ... / N=',NA
* --- Open file
      open(11,file='y3_State.dat',form='formatted',status='UNKNOWN')
      NDummy = 0
* --- Write in file
      do iDummy = 1, NA
        NDummy = NDummy + 1
        fDummy = fAi(iDummy)
        if (fDummy.ne.0) then
          write(11,'(i5,e18.10,5x,a)') iDummy, fDummy
        end if
      end do
* --- Close file
      close(11)
      write(*,'(a,i10)') '# (non-zero elements) = ',NDummy
* --- End of segment ------------------------------------------------- *
      return
      end

***********************************************************************
* === Reading the state ============================================= *
      subroutine readState(fAi)
      implicit none
      include 'xv_DS.include'      ! NA, NAP,... | dim.  of DS
      include 'xv_DS_State.include'    ! fAi         | state of DS
      integer*8 iDummy, NDummy         ! dummy variables
      real*8    fDummy                 ! dummy variable
* --- begin of subroutine -------------------------------------------- *
      write(*,*) 'Reading << y3_State >> ...'
* --- Initialize the default
      do iDummy = 1, NAP
        fAi(iDummy)=0
      end do
      NDummy = 0             ! minimum number of non-zero elements
* --- Open file
      open(10,file='y3_State.dat',form='formatted',status='old')
* --- Read file
 111  read(10,*,end=199,err=199) iDummy, fDummy
      if (iDummy.ge.0.and.iDummy.le.NAP) then
        fAi(iDummy) = fDummy
        write(*,'(a,i4,a,f12.5)') 'a_',iDummy,'=',fAi(iDummy) 
        NDummy = NDummy + 1
      else
        stop "Error in readState: iDummy out of range!"
      end if
      goto 111
* --- Close file
 199  close(10)
      write(*,'(a,i5)') '# elements=' ,NDummy
* --- end of subroutine ---------------------------------------------- *
      return
      end

************************************************************************
* === Read state from data *********************************************
      subroutine readStateData(fAi)
      implicit none
      include 'xv_DS.include'      ! NA, NAP,... | dim.  of DS
      include 'xv_DS_State.include'    ! fAi         | state of DS
      integer*8 iDummy, jDummy, NDummy ! dummy variables
      real*8    fDummy                 ! dummy variable
      logical   bDummy                 ! dummy variable
* --- Begin segment -------------------------------------------------- *
      write(*,*) 'Reading state from << z1_Amplitudes.dat >> ...'
* --- Initialize the default
      do iDummy = 1, NAP
        fAi(iDummy)=0
      end do
      NDummy = 0             ! number of non-zero elements
* --- Open file
      inquire(file='z1_Amplitudes.dat',exist=bDummy)
      if (bDummy) then
        write(*,*) 'Reading << z1_Amplitudes.dat >> ...'
        open(10,file='z1_Amplitudes.dat',form='formatted',status='old')
 101    read(10,*,end=102,err=102) iDummy, jDummy, fDummy
        if (iDummy.eq.1.and.jDummy.gt.0) then
          fAi(jDummy) = fDummy
          NDummy = NDummy + 1
        end if
        goto 101
 102    close(10)
      else
        write(*,*) 'Initializing with a_1 = 0.01 ...'
        fAi(1) = 0.01
      end if
      write(*,'(a,i5)') '# elements=' ,NDummy
* --- End of segment ------------------------------------------------- *
      return
      end

************************************************************************
* === Showing state at suitable times (for integration) ============== *
      subroutine showState(timeNow,fAi)
      implicit none
      include 'xv_DS.include'     ! NA, NAP,... | dim. of DS
      include 'xv_DS_Time.include'    ! timeShow, ...
      include 'xv_DS_State.include'   ! fAi     | state of DS
      real*8    timeNow
      integer*8 iDummy                ! dummy variable
* --- begin of subroutine -------------------------------------------- *
      if ((int((timeNow-timeShow)/timeShow).ne.
     &     int((timeNow+timeShow)/timeShow)).or.  ! at t approx i*timeShow
     &    timeNow.lt.timeBegin+0.5*timeStep.or.   ! at t approx tBegin
     &    timeNow.gt.timeEnd  -0.5*timeStep) then ! at t approx tEnd
        write(*,'(a,f12.2,a,5f10.5)') 't=',timeNow,
     &    '; a_i=',(fAi(iDummy),iDummy=1,min(5,NA))
      end if
* --- end of subroutine ---------------------------------------------- *
      return
      end

************************************************************************
* === write descriptor system (ODE vs. algebraic eq.) ******************
      subroutine writeMode
      implicit none
      include 'xv_DS.include'      ! NA, NAP, iMode
      integer iDummy                   ! dummy parameter
* --- Begin segment -------------------------------------------------- *
      write(*,*) 'Writing << y3_Modes.dat >> ... / N=', NA
* --- Open file
      open(10,file='y3_Modes.dat',form='formatted',status='UNKNOWN')
* --- Write file
      do iDummy = 1, NA
        if (iMode(iDummy).gt.0) then
          write(10,'(2i10,2x,a)') iDummy, iMode(iDummy),
     &             ' ! dynamic mode / da_i/dt=f_i(a)'
        else if (iMode(iDummy).lt.0) then
          write(10,'(2i10,2x,a)') iDummy, iMode(iDummy),
     &             ' ! slaved mode  / a_i=f_i(a)'
        else 
          write(10,'(2i10,2x,a)') iDummy, iMode(iDummy),
     &             ' ! slaved mode  / 0=f_i(a)'
        end if
      end do
* --- Close file
      close(10)
* --- End of segment ------------------------------------------------- *
      return
      end

***********************************************************************
* === read descriptor system (ODE vs. algebraic equation) =========== *
      subroutine readMode
      implicit none
      include 'xv_DS.include'       ! NA, NAP,... | dim. of DS
      integer*8 iDummy, jDummy, NDummy  ! dummy variables
      logical bExist
* --- Begin of subroutine ------------------------------------------- *
      write(*,'(a,i5)') 'Reading << y3_Modes.dat >> ... / N=',NA
* --- Initialize a default
      do iDummy = 1, NA
        iMode(iDummy) = iDummy
      end do
* --- Reading z_i from file
      inquire(file='y3_Modes.dat',EXIST=bExist)
      if (bExist) then
 111    open(10,file='y3_Modes.dat',form='formatted',status='old')
        read(10,*,end=199,err=199) iDummy, jDummy
        iMode(iDummy) = jDummy
        goto 111
 199    close(10)
      else
        write(*,*) '<< y3_Modes.dat >> does not exist.'
      end if
* --- Show result
      NDummy = 0
      do iDummy = 1, NA
        write(*,'(3(a,i5))') 'z_(',iDummy,')=',iMode(iDummy)
        if (iMode(iDummy).gt.0) NDummy = NDummy + 1
      end do
      write(*,'(a,i10)') '# (ODE) = ',NDummy      
* --- End of subroutine --------------------------------------------- *
      return
      end

***********************************************************************
* === Write Qijk ==================================================== *
      subroutine writeQijk
      implicit none
      include 'xv_DS.include'        ! NA, NAP,... | dim.of DS
      include 'xv_DS_Propagator.include' ! parameter of DS
      integer*8 iDummy, jDummy, kDummy, NDummy
      real*8    fDummy
* --- Begin of subroutine ------------------------------------------- *
      write(*,*) 'Writing << y3_DS_Qijk.dat >> ... / N=',NA
* --- Open file
      open(10,file='y3_DS_Qijk.dat',form='formatted',status='UNKNOWN')
      NDummy = 0
* --- Write file
      do iDummy=1,NA
      do jDummy=0,NA
      do kDummy=0,NA
        fDummy = fQijk(iDummy,jDummy,kDummy)
        if (fDummy.ne.0) then 
          write(10,'(3i5x,2xe18.10)') iDummy, jDummy, kDummy, fDummy
          NDummy = NDummy + 1
        end if
      end do
      end do
      end do
* --- Close file
      close(10)
      write(*,'(a,i10)') '# (non-zero elements) = ',NDummy
* --- End of subroutine --------------------------------------------- *
      return
      end

***********************************************************************
* === read Qijk ===================================================== *
      subroutine readQijk
      implicit none
      include 'xv_DS.include'        ! NAP,... | dimensions of DS
      include 'xv_DS_Propagator.include' ! parameter of DS
      integer*8 iDummy, jDummy, kDummy, NDummy
      real*8    fDummy
* --- Begin of subroutine ------------------------------------------- *
      write(*,*) 'Reading << y3_DS_Qijk >> and determining N ...'
* --- Initialize trivial default
      NA     = 0 ! minimum dimension
      NDummy = 0 ! minimum number of elements
      do iDummy = 1, NAP
      do jDummy = 0, NAP
      do kDummy = 0, NAP
        fQijk(iDummy,jDummy,kDummy) = 0.0
      end do
      end do
      end do
* --- reading the DS coefficients
 111  open(10,file='y3_DS_Qijk.dat',form='formatted',status='old')
      read(10,*,end=199,err=199) iDummy, jDummy, kDummy,fDummy
      if (min(iDummy-1,jDummy,kDummy).ge.0.and.
     &    max(iDummy,jDummy,kDummy).le.NAP) then
        fQijk(iDummy,jDummy,kDummy) = fDummy
        write(*,'(a,i4,a,i4,a,i4,a,f12.5)')
     &          'q(',iDummy,';',jDummy,';',kDummy,')=',fDummy 
        NDummy = NDummy + 1
        NA = max(NA,iDummy)
      else
        stop "Error in readState: iDummy out of range!"
      end if
      goto 111
* --- Close file
 199  close(10)
      write(*,'(3(a,i5))') 'N=',NA, '; # elements=' ,NDummy,
     & ' <=',NA*(NA+1)*(NA+1)
* --- End of subroutine --------------------------------------------- *
      return
      end
