************************************************************************
*****                                                              *****
***** @segment name: xs_ioMEFA.f                                   *****
*****                                                              *****
***** @task: io routines for modal energy flow analysis            *****
*****                                                              *****
***** @see: header of x1_EnergyFlowAnalysis.f                      *****
*****                                                              *****
************************************************************************
*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012
************************************************************************
* === initialize modal energy flow analysis ========================== *
      subroutine initMEFA(fAVal,fKVal,fPVal,fDVal,fCVal,fTVal,fFVal)
      implicit none
* --- variables (dimensions) for file system
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
* --- variables for energy flow analysis
      real*8 fAVal(0:NEMP,0:NFlowsP)        ! a
      real*8 fKVal(0:NEMP,0:NFlowsP)        ! K 
      real*8 fPVal(0:NEMP,0:NFlowsP)        ! P
      real*8 fDVal(0:NEMP,0:NFlowsP)        ! D
      real*8 fCVal(0:NEMP,0:NFlowsP)        ! C
      real*8 fTVal(0:NEMP,0:NFlowsP)        ! T
      real*8 fFVal(0:NEMP,0:NFlowsP)        ! F
* --- auxiliary variables
      integer iMode, iFlow
* --- begin of segment ----------------------------------------------- *
      do iMode = 0, NEMP
      do iFlow = 0, NFlowsP
        fAVal(iMode,iFlow) = 0.0
        fKVal(iMode,iFlow) = 0.0
        fPVal(iMode,iFlow) = 0.0
        fDVal(iMode,iFlow) = 0.0
        fCVal(iMode,iFlow) = 0.0
        fTVal(iMode,iFlow) = 0.0
        fFVal(iMode,iFlow) = 0.0
      end do
      end do
* --- end of segment ------------------------------------------------- *
      return
      end

************************************************************************
* === output of energy flow analysis ================================= *
      subroutine writeMEFA(iDomain,fAVal,fKVal,fPVal,fDVal,
     &                                   fCVal,fTVal,fFVal)
      implicit none
* --- variables (dimensions) for file system
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
* --- 
      integer iDomain
* --- variables for energy flow analysis
      real*8 fDummyVal(0:NEMP,0:NFlowsP)        ! free variable
      real*8 fAVal(0:NEMP,0:NFlowsP)        ! a
      real*8 fKVal(0:NEMP,0:NFlowsP)        ! K 
      real*8 fPVal(0:NEMP,0:NFlowsP)        ! P
      real*8 fDVal(0:NEMP,0:NFlowsP)        ! D
      real*8 fCVal(0:NEMP,0:NFlowsP)        ! C
      real*8 fTVal(0:NEMP,0:NFlowsP)        ! T
      real*8 fFVal(0:NEMP,0:NFlowsP)        ! F
* === fKVal(iMode,iFlow) .... TKE of snapshot iFlow in mode iMode
* === fKVal(0,    iFlow) .... total TKE of snapshot iFlow
* === fKVal(iMode,    0) .... averaged TKE  in mode iMode

* --- file name (generic + energy flow specific)
      character*120 fn
      integer       fnLength
* --- auxiliary variables
      integer iMode, iFlow, iTerm
* --- begin of segment ----------------------------------------------- *
      do iTerm = -1, 5
* ----- initialize fDummyVal
        do iFlow = 0, NFlowsP
        do iMode = 0, NEMP
          fDummyVal(iMode,iFlow) = 0.0
        end do
        end do
* ----- file name specification
        if (iTerm.eq.-1) then
           do iFlow = 1, NFlows
           do iMode = 1, NEM
             fDummyVal(iMode,iFlow) = fAVal(iMode,iFlow)
           end do
           end do
        end if
* ----- file name specification
        if (iTerm.eq.0) then
           do iFlow = 1, NFlows
           do iMode = 1, NEM
             fDummyVal(iMode,iFlow) = fKVal(iMode,iFlow)
           end do
           end do
        end if
* -----
        if (iTerm.eq.1) then
           do iFlow = 1, NFlows
           do iMode = 1, NEM
             fDummyVal(iMode,iFlow) = fPVal(iMode,iFlow)
           end do
           end do
        end if
* -----
        if (iTerm.eq.2) then
           do iFlow = 1, NFlows
           do iMode = 1, NEM
             fDummyVal(iMode,iFlow) = fDVal(iMode,iFlow)
           end do
           end do
        end if
* -----
        if (iTerm.eq.3) then
           do iFlow = 1, NFlows
           do iMode = 1, NEM
             fDummyVal(iMode,iFlow) = fCVal(iMode,iFlow)
           end do
           end do
        end if
* -----
        if (iTerm.eq.4) then
           do iFlow = 1, NFlows
           do iMode = 1, NEM
             fDummyVal(iMode,iFlow) = fTVal(iMode,iFlow)
           end do
           end do
        end if
* -----
        if (iTerm.eq.5) then
           do iFlow = 1, NFlows
           do iMode = 1, NEM
             fDummyVal(iMode,iFlow) = fFVal(iMode,iFlow)
           end do
           end do
        end if
* ----- computing the averaged values
* ----- computing and saving the averaged values --------------------- *
        do iMode = 1, NEM
        do iFlow = 1, NFlows
          fDummyVal(iMode,0) = fDummyVal(iMode,0)
     &                       + fDummyVal(iMode,iFlow) / NFlows
        end do
        end do 
* ----- opening the file with instantaneous values
        call getFilenameMEFA(iDomain,iTerm,0,fn,fnLength)
        write(*,*) 'Writing in << ',fn(1:fnLength),' >> ...',
     &              NEM,' x ',NFlows
        open(10,file=fn(1:fnLength),status='UNKNOWN',form='formatted')
        do iFlow = 1, NFlows
        do iMode = 1, NEM
* >>>>>>> Note the index order if flipped during I/O: m, i -> K_i^m
          write(10,'(2i6,1e20.10)') iFlow, iMode, fDummyVal(iMode,iFlow) 
        end do
        end do
        close(10)
* ----- opening the file fo the averaged values
        call getFilenameMEFA(iDomain,iTerm,1,fn,fnLength)
        write(*,*) 'Writing in << ',fn(1:fnLength),' >> ...',
     &              NEM
        open(10,file=fn(1:fnLength),status='UNKNOWN',form='formatted')
        do iMode = 1, NEM
          write(10,'(i6,1e20.10)') iMode, fDummyVal(iMode, 0)
        end do
        close(10)
* --- end of loop over terms
      end do
* --- end of segment ------------------------------------------------- *
      return
      end

************************************************************************
* === reading the modal energy flow analysis ========================= *
      subroutine readMEFA(iDomain,fAVal,fKVal,fPVal,fDVal,
     &                                  fCVal,fTVal,fFVal)
      implicit none
* --- variables (dimensions) for file system
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
* --- 
      integer iDomain
* --- variables for energy flow analysis
      real*8 fDummyVal(0:NEMP,0:NFlowsP)   ! free variable
      real*8 fAVal(0:NEMP,0:NFlowsP)        ! a
      real*8 fKVal(0:NEMP,0:NFlowsP)        ! K 
      real*8 fPVal(0:NEMP,0:NFlowsP)        ! P
      real*8 fDVal(0:NEMP,0:NFlowsP)        ! D
      real*8 fCVal(0:NEMP,0:NFlowsP)        ! C
      real*8 fTVal(0:NEMP,0:NFlowsP)        ! T
      real*8 fFVal(0:NEMP,0:NFlowsP)        ! F
* === fKVal(iMode,iFlow) .... TKE of snapshot iFlow in mode iMode
* === fKVal(0,    iFlow) .... total TKE of snapshot iFlow
* === fKVal(iMode,    0) .... averaged TKE  in mode iMode

* --- file name (generic + energy flow specific)
      character*120 fn
      integer       fnLength
* --- auxiliary variables
      integer iMode, iFlow, iTerm
      integer iDummy1, iDummy2
* --- begin of segment ----------------------------------------------- *
      do iTerm = -1, 5
* ----- initialize fDummyVal
        do iFlow = 0, NFlowsP
        do iMode = 0, NEMP
          fDummyVal (iMode,iFlow) = 0.0
        end do
        end do
* ----- opening the file with instantaneous values ------------------- *
        call getFilenameMEFA(iDomain,iTerm,0,fn,fnLength)
        write(*,*) 'Reading in << ',fn(1:fnLength),' >> ...',
     &              NEM,' x ',NFlows
        open(10,file=fn(1:fnLength),status='UNKNOWN',form='formatted')
        do iFlow = 1, NFlows
        do iMode = 1, NEM
* >>>>>>> note the index order is flipped during I/O: m, i -> X_i^m
          read(10,*) iDummy1, iDummy2,
     &                       fDummyVal(iDummy2,iDummy1) 
        end do
        end do
        close(10)
* ----- file name specification
        if (iTerm.eq.-1) then
           do iFlow = 1, NFlows
           do iMode = 1, NEM
             fAVal(iMode,iFlow) = fDummyVal(iMode,iFlow)
           end do
           end do
        end if
* ----- file name specification
        if (iTerm.eq.0) then
           do iFlow = 1, NFlows
           do iMode = 1, NEM
             fKVal(iMode,iFlow) = fDummyVal(iMode,iFlow)
           end do
           end do
        end if
* -----
        if (iTerm.eq.1) then
           do iFlow = 1, NFlows
           do iMode = 1, NEM
             fPVal(iMode,iFlow) = fDummyVal(iMode,iFlow)
           end do
           end do
        end if
* -----
        if (iTerm.eq.2) then
           do iFlow = 1, NFlows
           do iMode = 1, NEM
             fDVal(iMode,iFlow) = fDummyVal(iMode,iFlow)
           end do
           end do
        end if
* -----
        if (iTerm.eq.3) then
           do iFlow = 1, NFlows
           do iMode = 1, NEM
             fCVal(iMode,iFlow) = fDummyVal(iMode,iFlow)
           end do
           end do
        end if
* -----
        if (iTerm.eq.4) then
           do iFlow = 1, NFlows
           do iMode = 1, NEM
             fTVal(iMode,iFlow) = fDummyVal(iMode,iFlow)
           end do
           end do
        end if
* -----
        if (iTerm.eq.5) then
           do iFlow = 1, NFlows
           do iMode = 1, NEM
             fFVal(iMode,iFlow) = fDummyVal(iMode,iFlow)
           end do
           end do
        end if
* --- end of loop over MEFA terms
      end do
* --- end of segment ------------------------------------------------- *
      return
      end

************************************************************************
* === output of energy flow analysis ================================= *
      subroutine getFilenameMEFA(iDomain,iTerm,iMoment,fn,fnLength)
* --- Example: iDomain=7,iTerm=1,iMoment=0 -> 'z1_TKE.0007'
* --- Example: iDomain-1,iTerm=1,iMoment=0 -> 'z1_TKE.dat'
* --- Example: iDomain=7,iTerm=1,iMoment=1 -> 'z1_TKE_m1.0007'
* --- iTerm =   -1    0    1    2    3    4    5
* --- means ... a_i   K_i  P_i  D_i  C_i  T_i  F_i
      implicit none
      integer iDomain, iTerm, iMoment
* --- file name (generic + energy flow specific)
      character*120 fn
      integer       fnLength
* --- begin of segment ----------------------------------------------- *
      if (iTerm.eq.-1) then
        fn  = 'z1_A' 
        fnLength = 4
      else if (iTerm.eq.0) then
        fn  = 'z1_TKE' 
        fnLength = 6
      else if (iTerm.eq.1) then
* --- ..............1234567890123456
        fn       = 'z1_ProductionTKE'
        fnLength = 16
      else if (iTerm.eq.2) then
* --- ..............12345678901234567
        fn       = 'z1_DissipationTKE'
        fnLength = 17
      else if (iTerm.eq.3) then
* --- ..............1234567890123456
        fn       = 'z1_ConvectionTKE'
        fnLength = 16
      else if (iTerm.eq.4) then
* --- ..............12345678901234
        fn       = 'z1_TransferTKE'
        fnLength = 14
      else if (iTerm.eq.5) then
* --- ..............1234567890123456789
        fn       = 'z1_PressurePowerTKE'
        fnLength = 19
      end if
* -----
      if (iDomain.gt.0.and.iMoment.eq.0) then
        write(fn,'(a,i4.4)') fn(1:fnLength)//'.',iDomain
        fnLength         = fnLength + 5
      else if (iDomain.gt.0.and.iMoment.eq.1) then
        write(fn,'(a,i4.4)') fn(1:fnLength)//'_m1.',iDomain
        fnLength         = fnLength + 3 + 5
      else if (iDomain.le.0.and.iMoment.eq.0) then
        write(fn,'(a,i4.4)') fn(1:fnLength)//'.dat'
        fnLength         = fnLength + 4
      else if (iDomain.le.0.and.iMoment.eq.1) then
        write(fn,'(a,i4.4)') fn(1:fnLength)//'_m1.dat'
        fnLength         = fnLength + 7
      else
        fnLength = 0
      end if
* --- end of segment ------------------------------------------------- *
      return
      end
