************************************************************************
*****                                                              *****
***** @segment name: xs_ioFilenames.f                              *****
***** @task: I/O routines for xAMC specific filenames, ...         *****
***** @see:  res.20100408.xAMC_ToDo.tex                            *****
*****                                                              *****
************************************************************************
* === routine countDomains(NDomain)
* === routine countFlows0(NFlows0)
* === routine countFlows1(NFlows1)
* === routine countModes1(NModes1)
* === routine countModes2(NModes2)
* === routine countForces(NForce)
* === routine readFilenameDomain(iDomain,fn,fnLength)
* === routine readFilenameFlow0(iFlow,fn,fnLength)
* === routine readFilenameFlow1(iFlow,fn,fnLength)
* === routine readFilenameFlowX(iFlow,fn,fnLength)
* === routine readFilenameMode1(iMode,fn,fnLength)
* === routine readFilenameMode2(iMode,fn,fnLength)
* === routine readFilenameModeX(iMode,fn,fnLength)
* === routine readFilenameForce(iForce,fn,fnLength)
*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012

************************************************************************
* === determine # of domains from file (1-level, for xAMC) =========== *
      subroutine countDomains(NDomain)
      implicit none
      integer       NDomain
* --- auxiliary variables
      integer       iDomain,  iUnit
      character*120 fn,       fn0
      integer       fnLength, fn0Length
* --- begin segment -------------------------------------------------- *
      write(*,*) 'Counting domains ...'
      iUnit     =  10 ! => no integer constants in argument lists
* .................12345678901234
      fn0       = 'y1_Domains.dat'
      fn0Length =  14
      call countLinesUnit(iUnit,fn0,fn0Length,NDomain)
      write(*,*) 'Domains = ',NDomain
      do iDomain=1,NDomain
        call readFilenameDomain(iDomain,fn,fnLength)
        write(*,'(i6,2x,a,2x,a)') iDomain,'-->',fn(1:fnLength)
      end do
      call pressRETURN
* --- end segment ---------------------------------------------------- *
      return
      end

************************************************************************
* === determine # of snapshots from file (0-level, data) ============= *
      subroutine countFlows0(NFlows0)
      implicit none
      integer       NFlows0
* --- auxiliary variables
      integer       iFlow,    iUnit
      character*120 fn,       fn0
      integer       fnLength, fn0Length
* --- begin segment -------------------------------------------------- *
      write(*,*) 'Counting snapshots (raw data) ...'
      iUnit     =  10 ! => no integer constants in argument lists
* .................123456789012
      fn0       = 'y0_Flows.dat'
      fn0Length =  12
      call countLinesUnit(iUnit,fn0,fn0Length,NFlows0)
      write(*,*) '# Flows = ',NFlows0
      do iFlow=1,NFlows0
        call readFilenameFlow0(iFlow,fn,fnLength)
        write(*,'(i6,2x,a,2x,a)') iFlow,'-->',fn(1:fnLength)
      end do
      call pressRETURN
* --- end segment ---------------------------------------------------- *
      return
      end

************************************************************************
* === determine # of snapshots from file (1-level, G. expansion) ===== *
      subroutine countFlows1(NFlows1)
      implicit none
      integer       NFlows1
* --- auxiliary variables
      integer       iFlow,   iUnit
      character*120 fn,       fn0
      integer       fnLength, fn0Length
* --- begin segment -------------------------------------------------- *
      write(*,*) 'Counting snapshots (for Galerkin approximation) ...'
      iUnit     =  10 ! => no integer constants in argument lists
* .................123456789012
      fn0       = 'y1_Flows.dat'
      fn0Length =  12
      call countLinesUnit(iUnit,fn0,fn0Length,NFlows1)
      write(*,*) '# Flows = ',NFlows1
      do iFlow=1,NFlows1
        call readFilenameFlow1(iFlow,fn,fnLength)
        write(*,'(i6,2x,a,2x,a)') iFlow,'-->',fn(1:fnLength)
      end do
      call pressRETURN
* --- end segment ---------------------------------------------------- *
      return
      end

************************************************************************
* === determine number of expansion modes from file (1-level) ======== *
      subroutine countModes1(NModes1)
      implicit none
      integer       NModes1
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
* --- auxiliary variables
      integer       iMode,   iUnit
      character*120 fn,       fn0
      integer       fnLength, fn0Length
* --- begin segment -------------------------------------------------- *
      write(*,*) 'Counting modes (for Galerkin expansion) ...'
      iUnit     =  10 ! => no integer constants in argument lists
* .................123456789012
      fn0       = 'y1_Modes.dat'
      fn0Length =  12
      call countLinesUnit(iUnit,fn0,fn0Length,NModes1)
      write(*,*) 'Modes = ',NModes1
      do iMode=-NAM,NEM
        call readFilenameMode1(iMode,fn,fnLength)
        write(*,'(i6,2x,a,2x,a)') iMode,'-->',fn(1:fnLength)
      end do
      call pressRETURN
* --- end segment ---------------------------------------------------- *
      return
      end

************************************************************************
* === determine number of test functions from file (2-level) ======== *
      subroutine countModes2(NModes2)
      implicit none
      integer       NModes2
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
* --- auxiliary variables
      integer       iMode,   iUnit
      character*120 fn,       fn0
      integer       fnLength, fn0Length
* --- begin segment -------------------------------------------------- *
      write(*,*) 'Counting test functions (for Galerkin projection) ...'
      iUnit     =  10 ! => no integer constants in argument lists
* .................123456789012
      fn0       = 'y2_Modes.dat'
      fn0Length =  12
      call countLinesUnit(iUnit,fn0,fn0Length,NModes2)
      write(*,*) 'Modes = ',NModes2
      do iMode=1,NEM
        call readFilenameMode2(iMode,fn,fnLength)
        write(*,'(i6,2x,a,2x,a)') iMode,'-->',fn(1:fnLength)
      end do
      call pressRETURN
* --- end segment ---------------------------------------------------- *
      return
      end

************************************************************************
* === determine number of forces from file =========================== *
      subroutine countForces(NForce)
      implicit none
      integer       NForce
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
* --- auxiliary variables
      integer       iForce,   iUnit
      character*120 fn,       fn0
      integer       fnLength, fn0Length
* --- begin segment -------------------------------------------------- *
      write(*,*) 'Counting volume forces (for Galerkin projection) ...'
      iUnit     =  10 ! => no integer constants in argument lists
* .................1234567890123
      fn0       = 'y2_Forces.dat'
      fn0Length =  13
      call countLinesUnit(iUnit,fn0,fn0Length,NForce)
      write(*,*) 'Forces = ',NForce
      do iForce=1,NVF
        call readFilenameForce(iForce,fn,fnLength)
        write(*,'(i6,2x,a,2x,a)') iForce,'-->',fn(1:fnLength)
      end do
      call pressRETURN
* --- end segment ---------------------------------------------------- *
      return
      end

************************************************************************
* === read filename of i-th domain (1 & 2-level) ===================== *
      subroutine readFilenameDomain(iDomain,fn,fnLength)
      implicit none
      integer       iDomain,  iUnit
      character*120 fn,       fn0
      integer       fnLength, fn0Length
* --- begin segment -------------------------------------------------- *
      iUnit     =  10 ! => no integer constants in argument lists
* .................1234567890123
      fn0       = 'y1_Domains.dat'
      fn0Length =  14
      call readFilenameUnit(iUnit,fn0,fn0Length,iDomain,fn,fnLength)
* --- end segment ---------------------------------------------------- *
      return
      end

************************************************************************
* === read filename of i-th snapshot (0-level, data)  ================ *
      subroutine readFilenameFlow0(iFlow,fn,fnLength)
      implicit none
      integer       iFlow,   iUnit
      character*120 fn,       fn0
      integer       fnLength, fn0Length
* --- begin segment -------------------------------------------------- *
      iUnit     =  10 ! => no integer constants in argument lists
* .................123456789012
      fn0       = 'y0_Flows.dat'
      fn0Length =  12
      call readFilenameUnit(iUnit,fn0,fn0Length,iFlow,fn,fnLength)
* --- end segment ---------------------------------------------------- *
      return
      end

************************************************************************
* === read filename of i-th snapshot (1-level)  ====================== *
      subroutine readFilenameFlow1(iFlow,fn,fnLength)
      implicit none
      integer       iFlow,    iUnit
      character*120 fn,       fn0
      integer       fnLength, fn0Length
* --- begin segment -------------------------------------------------- *
      iUnit     =  10 ! => no integer constants in argument lists
* .................123456789012
      fn0       = 'y1_Flows.dat'
      fn0Length =  12
      call readFilenameUnit(iUnit,fn0,fn0Length,iFlow,fn,fnLength)
* --- end segment ---------------------------------------------------- *
      return
      end

************************************************************************
* === create generic file name for the m-th snapshot ================= *
      subroutine readFilenameFlowX(iFlow,fn,fnLength)
      implicit none
      integer       iFlow
      character*120 fn
      integer       fnLength
* --- begin segment -------------------------------------------------- *
      write(fn,'(a,i4.4)') 'z0_Data/Flow.',iFlow
*...........................1234567890123  4567
      fnLength = 17
* --- end segment ---------------------------------------------------- *
      return
      end

************************************************************************
* === read filename of i-th mode for Galerkin expansion  ============= *
      subroutine readFilenameMode1(iMode,fn,fnLength)
      implicit none
      integer       iMode,    iUnit
      character*120 fn,       fn0
      integer       fnLength, fn0Length
* --- begin segment -------------------------------------------------- *
      iUnit     =  10 ! => no integer constants in argument lists
* .................123456789012
      fn0       = 'y1_Modes.dat'
      fn0Length =  12
      call readFilenameUnit(iUnit,fn0,fn0Length,iMode,fn,fnLength)
* --- end segment ---------------------------------------------------- *
      return
      end

************************************************************************
* === read filename of i-th mode for Galerkin projection ============= *
      subroutine readFilenameMode2(iMode,fn,fnLength)
      implicit none
      integer       iMode,   iUnit
      character*120 fn,       fn0
      integer       fnLength, fn0Length
* --- begin segment -------------------------------------------------- *
      iUnit     =  10 ! => no integer constants in argument lists
* .................123456789012
      fn0       = 'y2_Modes.dat'
      fn0Length =  12
      call readFilenameUnit(iUnit,fn0,fn0Length,iMode,fn,fnLength)
* --- end segment ---------------------------------------------------- *
      return
      end

************************************************************************
* === create generic file name for i-th mode ========================= *
      subroutine readFilenameModeX(iMode,fn,fnLength)
      implicit none
      integer       iMode
      character*120 fn
      integer       fnLength
* --- begin segment -------------------------------------------------- *
      write(fn,'(a,i4.4)') 'z1_Data/Mode.',iMode
*...........................1234567890123  4567
      fnLength = 17
* --- end segment ---------------------------------------------------- *
      return
      end

************************************************************************
* === read filename of i-th force for Galerkin projection ============= *
      subroutine readFilenameForce(iForce,fn,fnLength)
      implicit none
      integer       iForce,   iUnit
      character*120 fn,       fn0
      integer       fnLength, fn0Length
* --- begin segment -------------------------------------------------- *
      iUnit     =  10 ! => no integer constants in argument lists
* .................1234567890123
      fn0       = 'y2_Forces.dat'
      fn0Length =  13
      call readFilenameUnit(iUnit,fn0,fn0Length,iForce,fn,fnLength)
* --- end segment ---------------------------------------------------- *
      return
      end
