************************************************************************
*****                                                              *****
***** @segment name: xs_ioGS.f                                     *****
***** @task: I/O routines for Galerkin system                      *****
***** @see:  res.20100408.xAMC_ToDo.tex                            *****
*****                                                              *****
************************************************************************
* === subroutine writeMassMatrix   (fMatrix)
* === subroutine readMassMatrix    (fMatrix,bOK)
* === subroutine writeMassMatrixSub(iDomain,fMatrixSub)
* === subroutine readMassMatrixSub (iDomain,fMatrixSub,bOK)
* ----------------------------------------------------------------------
* === subroutine writeViscousTerm   (fMatrix)
* === subroutine readViscousTerm    (fMatrix,bOK)
* === subroutine writeViscousTermSub(iDomain,fMatrixSub)
* === subroutine readViscousTermSub (iDomain,fMatrixSub,bOK)
* ----------------------------------------------------------------------
* === subroutine writeConvectiveTerm   (fTensor)
* === subroutine readConvectiveTerm    (fTensor,bOK)
* === subroutine writeConvectiveTermSub(iDomain,fTensorSub)
* === subroutine readConvectiveTermSub (iDomain,fTensorSub,bOK)
* ----------------------------------------------------------------------
* === subroutine readPressureTerm (fMatrix,bOK)
* === only one routine, since the pressure term 
* === is computed "outside" the SOP.
* ----------------------------------------------------------------------
* === subroutine writeForceTerm   (fMatrix2)
* === subroutine readForceTerm    (fMatrix2,bOK)
* === subroutine writeForceTermSub(iDomain,fMatrix2Sub)
* === subroutine readForceTermSub (iDomain,fMatrix2Sub,bOK)
* ======================================================================
* === subroutine writeMatrix(fn,fnLength,fMatrix)
* === subroutine readMatrix (fn,fnLength,fMatrix,bOK)
* ----------------------------------------------------------------------
* === subroutine zeroMatrix (fMatrix)
* === subroutine zeroMatrix2(fMatrix2)
* === subroutine cleanMatrix(fMatrix)
* === subroutine add2Matrix(fMatrix,fMatrix2)
* === subroutine add2Tensor(fTensor,fTensor2)
* === subroutine zeroTensor(fTensor)
*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012

************************************************************************ 
* === write the mass matrix ========================================== *
      subroutine writeMassMatrix(fMatrix)
      implicit none
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
      real*8  fMatrix(NEMP,-NAMP:NEMP)    ! mass matrix
      character*120 fn                    ! file name
      integer       fnLength              ! length of file name
* --- begin of segment ----------------------------------------------- *
* --- user info
* ................12345678901234567
      fn       = 'z2_MassMatrix.dat'
      fnLength = 17
      call writeMatrix(fn,fnLength,fMatrix)
* --- end of segment ------------------------------------------------- *
      return
      end

************************************************************************ 
* === read the mass matrix =========================================== *
      subroutine readMassMatrix(fMatrix,bOK)
      implicit none
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
      real*8  fMatrix(NEMP,-NAMP:NEMP)    ! mass matrix
      logical       bOK                   ! boolean variable=.true.for ex.
      character*120 fn                    ! file name
      integer       fnLength              ! length of file name
* --- begin of segment ----------------------------------------------- *
* --- name of mass matrix
* ................12345678901234567
      fn       = 'z2_MassMatrix.dat'
      fnLength =  17
      call readMatrix(fn,fnLength,fMatrix,bOK)
* --- end of segment ------------------------------------------------- *
      return
      end

************************************************************************ 
* === write the mass matrix (of subdomain) =========================== *
      subroutine writeMassMatrixSub(iDomain,fMatrixSub)
      implicit none
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
      integer iDomain                     ! domain number 
      real*8  fMatrixSub(NEMP,-NAMP:NEMP) ! mass matrix
      character*120 fn                    ! file name
      integer       fnLength              ! length of file name
* --- begin of segment ----------------------------------------------- *
* --- name of mass matrix
* ................123456789012345678
      fn       = 'z2_MassMatrix.0000'
      fnLength =  18
      write(fn,'(a,i4.4)') 'z2_MassMatrix.',iDomain
      call writeMatrix(fn,fnLength,fMatrixSub)
* --- end of segment ------------------------------------------------- *
      return
      end

************************************************************************ 
* === read the mass matrix (of subdomain) ============================ *
      subroutine readMassMatrixSub(iDomain,fMatrixSub,bOK)
      implicit none
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
      integer iDomain                     ! domain number
      real*8  fMatrixSub(NEMP,-NAMP:NEMP) ! mass matrix
      logical bOK                         ! boolean variable=.true.for ex.
      character*120 fn                    ! file name
      integer       fnLength              ! length of file name
* --- begin of segment ----------------------------------------------- *
* --- name of mass matrix
* ................123456789012345678
      fn       = 'z2_MassMatrix.0000'
      fnLength =  18
      write(fn,'(a,i4.4)') 'z2_MassMatrix.',iDomain
      call readMatrix(fn,fnLength,fMatrixSub,bOK)
* --- end of segment ------------------------------------------------- *
      return
      end

************************************************************************ 
* === write the viscous term ========================================= *
      subroutine writeViscousTerm(fMatrix)
      implicit none
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
      real*8  fMatrix(NEMP,-NAMP:NEMP)    ! viscous term
      character*120 fn                    ! file name
      integer       fnLength              ! length of file name
* --- begin of segment ----------------------------------------------- *
* --- user info
* ................123456789012345678
      fn       = 'z2_ViscousTerm.dat'
      fnLength = 18
      call writeMatrix(fn,fnLength,fMatrix)
* --- end of segment ------------------------------------------------- *
      return
      end

************************************************************************ 
* === read the viscous term ========================================== *
      subroutine readViscousTerm(fMatrix,bOK)
      implicit none
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
      real*8  fMatrix(NEMP,-NAMP:NEMP)    ! viscous term
      logical       bOK                  ! boolean variable=.true.for ex.
      character*120 fn                    ! file name
      integer       fnLength              ! length of file name
* --- begin of segment ----------------------------------------------- *
* --- name of viscous term
* ................123456789012345678
      fn       = 'z2_ViscousTerm.dat'
      fnLength =  18
      call readMatrix(fn,fnLength,fMatrix,bOK)
* --- end of segment ------------------------------------------------- *
      return
      end

************************************************************************ 
* === write the viscous term (of subdomain) ========================== *
      subroutine writeViscousTermSub(iDomain,fMatrixSub)
      implicit none
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
      integer iDomain                     ! domain number 
      real*8  fMatrixSub(NEMP,-NAMP:NEMP) ! viscous term
      character*120 fn                    ! file name
      integer       fnLength              ! length of file name
* --- begin of segment ----------------------------------------------- *
* --- name of viscous term
* ................1234567890123456789
      fn       = 'z2_ViscousTerm.0000'
      fnLength =  19
      write(fn,'(a,i4.4)') 'z2_ViscousTerm.',iDomain
      call writeMatrix(fn,fnLength,fMatrixSub)
* --- end of segment ------------------------------------------------- *
      return
      end

************************************************************************ 
* === read the viscous term (of subdomain) =========================== *
      subroutine readViscousTermSub(iDomain,fMatrixSub,bOK)
      implicit none
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
      real*8  fMatrixSub(NEMP,-NAMP:NEMP) ! viscous term
      integer iDomain                     ! domain number
      logical       bOK                   ! boolean variable=.true.for ex.
      character*120 fn                    ! file name
      integer       fnLength              ! length of file name
* --- begin of segment ----------------------------------------------- *
* --- name of viscous term
* ................1234567890123456789
      fn       = 'z2_ViscousTerm.0000'
      fnLength =  19
      write(fn,'(a,i4.4)') 'z2_ViscousTerm.',iDomain
      call readMatrix(fn,fnLength,fMatrixSub,bOK)
* --- end of segment ------------------------------------------------- *
      return
      end

************************************************************************ 
* === write the convective term ====================================== *
      subroutine writeConvectiveTerm(fTensor)
      implicit none
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
      real*8  fTensor(NEMP,-NAMP:NEMP,-NAMP:NEMP)    ! conv. term
      character*120 fn                    ! file name
      integer       fnLength              ! length of file name
* --- begin of segment ----------------------------------------------- *
* --- user info
* ................123456789012345678901
      fn       = 'z2_ConvectiveTerm.dat'
      fnLength = 21
      call writeTensor(fn,fnLength,fTensor)
* --- end of segment ------------------------------------------------- *
      return
      end

************************************************************************ 
* === read the convective term ======================================= *
      subroutine readConvectiveTerm(fTensor,bOK)
      implicit none
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
      integer iDomain                     ! domain number
      real*8  fTensor(NEMP,-NAMP:NEMP,-NAMP:NEMP)    ! convective term
      logical       bOK                   ! boolean variable=.true.for ex.
      character*120 fn                    ! file name
      integer       fnLength              ! length of file name
* --- begin of segment ----------------------------------------------- *
* --- name of convective term
* ................123456789012345678901
      fn       = 'z2_ConvectiveTerm.dat'
      fnLength =  21
      call readTensor(fn,fnLength,fTensor,bOK)
* --- end of segment ------------------------------------------------- *
      return
      end

************************************************************************ 
* === write the convective term (of subdomain) ======================= *
      subroutine writeConvectiveTermSub(iDomain,fTensorSub)
      implicit none
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
      integer iDomain                     ! domain number 
      real*8  fTensorSub(NEMP,-NAMP:NEMP,-NAMP:NEMP) ! convective term
      character*120 fn                    ! file name
      integer       fnLength              ! length of file name
* --- begin of segment ----------------------------------------------- *
* --- name of convective term
* ................1234567890123456789012
      fn       = 'z2_ConvectiveTerm.0000'
      fnLength =  22
      write(fn,'(a,i4.4)') 'z2_ConvectiveTerm.',iDomain
      call writeTensor(fn,fnLength,fTensorSub)
* --- end of segment ------------------------------------------------- *
      return
      end

************************************************************************ 
* === read the viscous term (of subdomain) =========================== *
      subroutine readConvectiveTermSub(iDomain,fTensorSub,bOK)
      implicit none
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
      integer iDomain                     ! domain number
      real*8  fTensorSub(NEMP,-NAMP:NEMP,-NAMP:NEMP) ! convective term
      logical bOK                         ! boolean variable=.true.for ex.
      character*120 fn                    ! file name
      integer       fnLength              ! length of file name
* --- begin of segment ----------------------------------------------- *
* --- name of convective term
* ................1234567890123456789012
      fn       = 'z2_ConvectiveTerm.0000'
      fnLength =  22
      write(fn,'(a,i4.4)') 'z2_ConvectiveTerm.',iDomain
      call readTensor(fn,fnLength,fTensorSub,bOK)
* --- end of segment ------------------------------------------------- *
      return
      end

************************************************************************ 
* === read the pressure term ========================================== *
      subroutine readPressureTerm(fMatrix,bOK)
      implicit none
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
      real*8  fMatrix(NEMP,-NAMP:NEMP)    ! viscous term
      logical       bOK                  ! boolean variable=.true.for ex.
      character*120 fn                    ! file name
      integer       fnLength              ! length of file name
* --- begin of segment ----------------------------------------------- *
* --- name of pressure term
* ................1234567890123456789
      fn       = 'z2_PressureTerm.dat'
      fnLength =  19
      call readMatrix(fn,fnLength,fMatrix,bOK)
* --- end of segment ------------------------------------------------- *
      return
      end

************************************************************************ 
* === write the force term =========================================== *
      subroutine writeForceTerm(fMatrix2)
      implicit none
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
      real*8  fMatrix2(NEMP,NVFP)         ! force term
      character*120 fn                    ! file name
      integer       fnLength              ! length of file name
* --- begin of segment ----------------------------------------------- *
      if (NVF.le.0) return ! no action if no volume forces are used
* --- user info
* ................1234567890123456
      fn       = 'z2_ForceTerm.dat'
      fnLength = 16
      call writeMatrix2(fn,fnLength,fMatrix2)
* --- end of segment ------------------------------------------------- *
      return
      end

************************************************************************ 
* === read the force term ========================================== *
      subroutine readForceTerm(fMatrix2,bOK)
      implicit none
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
      real*8  fMatrix2(NEMP,NVFP)         ! force term
      logical       bOK                   ! boolean variable=.true.for ex.
      character*120 fn                    ! file name
      integer       fnLength              ! length of file name
* --- begin of segment ----------------------------------------------- *
      if (NVF.le.0) then
        bOK=.true.
        return ! no action if no volume forces are used
      end if
* --- name of force term
* ................1234567890123456
      fn       = 'z2_ForceTerm.dat'
      fnLength =  16
      call readMatrix2(fn,fnLength,fMatrix2,bOK)
* --- end of segment ------------------------------------------------- *
      return
      end

************************************************************************ 
* === write the force term (of subdomain) ============================ *
      subroutine writeForceTermSub(iDomain,fMatrix2Sub)
      implicit none
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
      real*8  fMatrix2Sub(NEMP,NVFP)      ! force term
      integer iDomain                     ! domain number 
      character*120 fn                    ! file name
      integer       fnLength              ! length of file name
* --- begin of segment ----------------------------------------------- *
      if (NVF.le.0) return ! no action if no volume forces are used
* --- name of force term
* ................12345678901234567
      fn       = 'z2_ForceTerm.0000'
      fnLength =  17
      write(fn,'(a,i4.4)') 'z2_ForceTerm.',iDomain
      call writeMatrix2(fn,fnLength,fMatrix2Sub)
* --- end of segment ------------------------------------------------- *
      return
      end

************************************************************************ 
* === read the force term (of subdomain) ============================= *
      subroutine readForceTermSub(iDomain,fMatrix2Sub,bOK)
      implicit none
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
      real*8  fMatrix2Sub(NEMP,NVFP)      ! force term
      integer iDomain                     ! domain number
      logical       bOK                   ! boolean variable=.true.for ex.
      character*120 fn                    ! file name
      integer       fnLength              ! length of file name
* --- begin of segment ----------------------------------------------- *
      if (NVF.le.0) then
        bOK=.true.
        return ! no action if no volume forces are used
      end if
* --- name of force term
* ................12345678901234567
      fn       = 'z2_ForceTerm.0000'
      fnLength =  17
      write(fn,'(a,i4.4)') 'z2_ForceTerm.',iDomain
      call readMatrix2(fn,fnLength,fMatrix2Sub,bOK)
* --- end of segment ------------------------------------------------- *
      return
      end

************************************************************************
* XXX auxiliary routines to the above routines XXXXXXXXXXXXXXXXXXXXXXX *
************************************************************************ 
* === write the matrix =============================================== *
      subroutine writeMatrix(fn,fnLength,fMatrix)
      implicit none
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
      real*8  fMatrix(NEMP,-NAMP:NEMP)    ! matrix
      character*120 fn                    ! file name
      integer       fnLength              ! length of file name
      integer       iMode, jMode          ! auxiliary dummy variables
* --- begin of segment ----------------------------------------------- *
* --- user info
      write(*,'(a,a,a)') '***** Writing the matrix in <<',
     & fn(1:fnLength),'>> ...'
* --- write matrix
      open(10,file=fn(1:fnLength),
     &        status='UNKNOWN',form='formatted')
      do iMode =    1, NEM
      do jMode = -NAM, NEM
        if (abs(fMatrix(iMode,jMode)).ne.0) 
     &      write(10,'(2(i5,1x),f20.10)') 
     &      iMode,jMode,fMatrix(iMode,jMode)
      end do
      end do
      close(10)
* --- end of segment ------------------------------------------------- *
      return
      end

************************************************************************ 
* === read the matrix ================================================ *
      subroutine readMatrix(fn,fnLength,fMatrix,bOK)
      implicit none
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
      real*8  fMatrix(NEMP,-NAMP:NEMP)    ! matrix
      character*120 fn                    ! file name
      integer       fnLength              ! length of file name
      logical       bOK                   ! true if exist & nonempty
      integer       iMode, jMode          ! auxiliary dummy variables
      integer       iDummy
      real*8        fDummy
* --- begin of segment ----------------------------------------------- *
* --- user info
      write(*,'(a,a,a)') '***** Reading the matrix from <<',
     & fn(1:fnLength),'>> ...'
      call countLinesUnit(10,fn,fnLength,iDummy)
      if (iDummy.lt.0) then
        write(*,*) 'File does not exist!'
        bOK = .false.
        return
      else if (iDummy.eq.0) then
        write(*,*) 'File is empty!'
        bOK = .false.
        return
      else
        bOK = .true.
      end if 
* --- initialize
      do iMode =     1,NEMP
      do jMode = -NAMP,NEMP
        fMatrix(iMode,jMode) = 0.0
      end do
      end do
* --- read viscous / other matrix
      open(10,file=fn(1:fnLength),status='old',form='formatted')
  101 read(10,*,end=199) iMode,jMode,fDummy
      fMatrix(iMode,jMode) = fDummy
      goto 101
 199  close(10)
* --- end of segment ------------------------------------------------- *
      return
      end

************************************************************************ 
* === write the matrix (volume forces) =============================== *
      subroutine writeMatrix2(fn,fnLength,fMatrix2)
      implicit none
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
      real*8  fMatrix2(NEMP,NVFP)         ! matrix
      character*120 fn                    ! file name
      integer       fnLength              ! length of file name
      integer       iMode, jMode          ! auxiliary dummy variables
* --- begin of segment ----------------------------------------------- *
      if (NVF.le.0) return
* --- user info
      write(*,'(a,a,a)') '***** Writing the matrix2 in <<',
     & fn(1:fnLength),'>> ...'
* --- write matrix
      open(10,file=fn(1:fnLength),
     &        status='UNKNOWN',form='formatted')
      do iMode =    1, NEM
      do jMode =    1, NVF
        if (abs(fMatrix2(iMode,jMode)).ne.0) 
     &      write(10,'(2(i5,1x),f20.10)') 
     &                iMode,jMode,fMatrix2(iMode,jMode)
      end do
      end do
      close(10)
* --- end of segment ------------------------------------------------- *
      return
      end

************************************************************************ 
* === read the matrix ================================================ *
      subroutine readMatrix2(fn,fnLength,fMatrix2,bOK)
      implicit none
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
      real*8  fMatrix2(NEMP,NVFP)         ! matrix
      character*120 fn                    ! file name
      integer       fnLength              ! length of file name
      logical       bOK                   ! true if exist & nonempty
      integer       iMode, jMode          ! auxiliary dummy variables
      integer       iDummy
      real*8        fDummy
* --- begin of segment ----------------------------------------------- *
      if (NVF.le.0) then
        bOK = .true.
        return
      end if
* --- user info
      write(*,'(a,a,a)') '***** Reading the matrix from <<',
     & fn(1:fnLength),'>> ...'
      call countLinesUnit(10,fn,fnLength,iDummy)
      if (iDummy.lt.0) then
        write(*,*) 'File does not exist!'
        bOK = .false.
        return
      else if (iDummy.eq.0) then
        write(*,*) 'File is empty!'
        bOK = .false.
        return
      else
        bOK = .true.
      end if 
* --- initialize
      do iMode =     1,NEMP
      do jMode =     1,NVFP
        fMatrix2(iMode,jMode) = 0.0
      end do
      end do
* --- write viscous term
      open(10,file=fn(1:fnLength),status='old',form='formatted')
  101 read(10,*,end=199) iMode,jMode,fDummy
      fMatrix2(iMode,jMode) = fDummy
      goto 101
 199  close(10)
* --- end of segment ------------------------------------------------- *
      return
      end

************************************************************************ 
* === write the tensor =============================================== *
      subroutine writeTensor(fn,fnLength,fTensor)
      implicit none
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
      real*8  fTensor(NEMP,-NAMP:NEMP,-NAMP:NEMP)    ! tensor
      character*120 fn                    ! file name
      integer       fnLength              ! length of file name
      integer       iMode, jMode, kMode   ! auxiliary dummy variables
* --- begin of segment ----------------------------------------------- *
* --- user info
      write(*,'(a,a,a)') '***** Writing the tensor in <<',
     & fn(1:fnLength),'>> ...'
* --- write tensor
      open(10,file=fn(1:fnLength),
     &        status='UNKNOWN',form='formatted')
      do iMode =    1, NEM
      do jMode = -NAM, NEM
      do kMode = -NAM, NEM
        if (abs(fTensor(iMode,jMode,kMode)).ne.0) 
     &      write(10,'(3(i5,1x),f20.10)') 
     &                iMode,jMode,kMode,fTensor(iMode,jMode,kMode)
      end do
      end do
      end do
      close(10)
* --- end of segment ------------------------------------------------- *
      return
      end

************************************************************************ 
* === read the tensor ================================================ *
      subroutine readTensor(fn,fnLength,fTensor,bOK)
      implicit none
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
      real*8  fTensor(NEMP,-NAMP:NEMP,-NAMP:NEMP)    ! tensor
      character*120 fn                    ! file name
      integer       fnLength              ! length of file name
      logical       bOK                   ! true if exist & nonempty
      integer       iMode, jMode, kMode   ! auxiliary dummy variables
      integer       iDummy
      real*8        fDummy
* --- begin of segment ----------------------------------------------- *
* --- user info
      write(*,'(a,a,a)') '***** Reading the tensor from <<',
     & fn(1:fnLength),'>> ...'
      call countLinesUnit(10,fn,fnLength,iDummy)
      if (iDummy.lt.0) then
        write(*,*) 'File does not exist!'
        bOK = .false.
        return
      else if (iDummy.eq.0) then
        write(*,*) 'File is empty!'
        bOK = .false.
        return
      else
        bOK = .true.
      end if 
* --- initialize
      do iMode =     1,NEMP
      do jMode = -NAMP,NEMP
      do kMode = -NAMP,NEMP
        fTensor(iMode,jMode,kMode) = 0.0
      end do
      end do
      end do
* --- write viscous term
      open(10,file=fn(1:fnLength),status='old',form='formatted')
  101 read(10,*,end=199) iMode,jMode,kMode,fDummy
      fTensor(iMode,jMode,kMode) = fDummy
      goto 101
 199  close(10)
* --- end of segment ------------------------------------------------- *
      return
      end

************************************************************************
* === set matrix to zero ============================================= *
      subroutine zeroMatrix(fMatrix)
      implicit none
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
      real*8  fMatrix(NEMP,-NAMP:NEMP)
      integer iMode, jMode
* --- begin of segment ----------------------------------------------- *
      do iMode=     1, NEMP ! physical dimensions
      do jMode= -NAMP, NEMP
        fMatrix(iMode,jMode) = 0.0
      end do
      end do
* --- end of segment ------------------------------------------------- *
      return
      end

************************************************************************
* === set volume force matrix to zero ================================ *
      subroutine zeroMatrix2(fMatrix2)
      implicit none
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
      real*8  fMatrix (NEMP,-NAMP:NEMP)
      real*8  fMatrix2(NEMP, NVFP)
      integer iMode, jMode
* --- begin of segment ----------------------------------------------- *
      do iMode=     1, NEMP ! physical dimensions
      do jMode=     1, NVFP
        fMatrix(iMode,jMode) = 0.0
      end do
      end do
* --- end of segment ------------------------------------------------- *
      return
      end

************************************************************************
* === clean matrix from "numerical zeros" ============================ *
      subroutine cleanMatrix(fMatrix)
      implicit none
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
      real*8  fMatrix(NEMP,-NAMP:NEMP)
      integer iMode, jMode
* --- begin of segment ----------------------------------------------- *
      do iMode=     1, NEMP ! physical dimensions
      do jMode= -NAMP, NEMP
        if (abs(fMatrix(iMode,jMode)).lt.1.E-12)
     &          fMatrix(iMode,jMode) = 0.0
      end do
      end do
* --- end of segment ------------------------------------------------- *
      return
      end

************************************************************************
* === add fMatrix to fMatrix2  ======================================= *
      subroutine add2Matrix(fMatrix,fMatrix2)
      implicit none
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
      real*8  fMatrix (NEMP,-NAMP:NEMP)
      real*8  fMatrix2(NEMP,-NAMP:NEMP)
      integer iMode, jMode
* --- begin of segment ----------------------------------------------- *
      do iMode=     1, NEM             ! actual dimensions
      do jMode=  -NAM, NEM
        fMatrix2(iMode,jMode) = fMatrix2(iMode,jMode)
     &                        + fMatrix (iMode,jMode)
      end do
      end do
* --- end of segment ------------------------------------------------- *
      return
      end

************************************************************************
* === add fMatrix to fMatrix2  ======================================= *
      subroutine add2Matrix2(fMatrix,fMatrix2)
      implicit none
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
      real*8  fMatrix (NEMP,NVFP)
      real*8  fMatrix2(NEMP,NVFP)
      integer iMode, jMode
* --- begin of segment ----------------------------------------------- *
      do iMode=     1, NEM             ! actual dimensions
      do jMode=     1, NVF
        fMatrix2(iMode,jMode) = fMatrix2(iMode,jMode)
     &                        + fMatrix (iMode,jMode)
      end do
      end do
* --- end of segment ------------------------------------------------- *
      return
      end

************************************************************************
* === set "tensor" to zero =========================================== *
      subroutine zeroTensor(fTensor)
      implicit none
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
      real*8  fTensor(NEMP,-NAMP:NEMP,-NAMP:NEMP)
      integer iMode, jMode, kMode
* --- begin of segment ----------------------------------------------- *
      do iMode=     1, NEMP            ! physical dimensions
      do jMode= -NAMP, NEMP
      do kMode= -NAMP, NEMP
        fTensor(iMode,jMode,kMode) = 0.0
      end do
      end do
      end do
* --- end of segment ------------------------------------------------- *
      return
      end

************************************************************************
* === add fTensor to fTensor2  ======================================= *
      subroutine add2Tensor(fTensor,fTensor2)
      implicit none
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
      real*8  fTensor (NEMP,-NAMP:NEMP,-NAMP:NEMP)
      real*8  fTensor2(NEMP,-NAMP:NEMP,-NAMP:NEMP)
      integer iMode, jMode, kMode
* --- begin of segment ----------------------------------------------- *
      do iMode=     1, NEM             ! actual dimensions
      do jMode=  -NAM, NEM
      do kMode=  -NAM, NEM
        fTensor2(iMode,jMode,kMode) = fTensor2(iMode,jMode,kMode)
     &                              + fTensor (iMode,jMode,kMode)
      end do
      end do
      end do
* --- end of segment ------------------------------------------------- *
      return
      end
