************************************************************************
*****                                                              *****
***** @segment name: xs_ioYGA.f                                    *****
***** @task: I/O routines for Galerkin approximation               *****
***** @see:  res.20100408.xAMC_ToDo.tex                            *****
*****                                                              *****
************************************************************************
* === readYGA
* === showYGA
*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012

************************************************************************
* === read parameters for Galerkin approximation ===================== *
      subroutine readYGA
      implicit none
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
      include 'xv_GA2.include'    ! tasks for construction of modes
* --- auxiliary variables
      character*120 Line
      logical  bOK
* --- begin of subroutine -------------------------------------------- *
      write(*,*) 'Reading << y1_GM.dat >> ...'
      open(10,file  = 'y1_GM.dat',status= 'OLD',form='formatted')

* --- Read mandatory part of << y1_GM.dat >> ------------------------- *
* ... read number of actuation modes as lower index bound: i = -NA,...,N
      call readLine(10,Line,bOK)
      if (.not.bOK) stop 'Error in reading NAM'
      read(Line,*) NAM
* ... read number of expansion modes as upper index bound
      call readLine(10,Line,bOK)
      if (.not.bOK) stop 'Error in reading NEM'
      read(Line,*) NEM
* ... read number of empirical modes (subset i=1...N_KL<=N)
      call readLine(10,Line,bOK)
      if (.not.bOK) stop 'Error in reading NKL'
      read(Line,*) NKL

* --- Read optional part of << y1_GM.dat >> [non POD modes, ...] ----- *
      NTaskGA = 0
  101 call readline(10,Line,bOK)
* --- Skip interpretation if a reading error has occured.
      if (.not.bOK) goto 999
* --- identity operation
      if (index(Line,'identity').gt.0) then
        NTaskGA = NTaskGA + 1
        aTaskGA(NTaskGA) = 0
        read(Line,*) iTaskGA(NTaskGA),jTaskGA(NTaskGA)
        goto 101
* --- initialize mode u_i=0
      else if (index(Line,'initialize').gt.0) then
        NTaskGA = NTaskGA + 1
        aTaskGA(NTaskGA) = 1
        read(Line,*) iTaskGA(NTaskGA)
        goto 101
* --- linear combination
      else if (index(Line,'linear').gt.0) then
        NTaskGA = NTaskGA + 1
        aTaskGA(NTaskGA) = 2
        read(Line,*) iTaskGA(NTaskGA),jTaskGA(NTaskGA),fTaskGA(NTaskGA)
        goto 101
* --- orthonormalize
      else if (index(Line,'orthon').gt.1) then
        NTaskGA = NTaskGA + 1
        aTaskGA(NTaskGA) = 3
        read(Line,*) iTaskGA(NTaskGA),jTaskGA(NTaskGA),kTaskGA(NTaskGA)
        goto 101
      end if

* --- End of file ---------------------------------------------------- *
  999 close(10)
* --- end of subroutine ---------------------------------------------- *
      return
      end

************************************************************************
* === show parameters for Galerkin approximation ===================== *
      subroutine showYGA
      implicit none
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
      include 'xv_GA2.include'    ! tasks for construction of modes
* --- auxiliary variables
      integer iTask
* --- begin of subroutine -------------------------------------------- *
      write(*,*) '========================================'
      write(*,*) 'Contruction of Galerkin approximation ...'
      write(*,'(2a6,2x,a)') 'act.','phys.','dimension of'
      write(*,'(2i6,2x,a)') NAM, NAMP, 'actuation modes i=-N_A...-1'
      write(*,'(2i6,2x,a)') NEM, NEMP, 'expansion modes i= 0...N'
      write(*,'(2i6,2x,a)') NKL, NKLP, 'POD/DMD/... modes N_POD<=N'
      write(*,*)
      do iTask = 1, NTaskGA
        if (aTaskGA(iTask).eq.0) then
* ....... no action
          write(*,'(a,i5,a,i5,a,i5)') 
     &            'task #',iTask,'): identity for i=',
     &             iTaskGA(iTask),' ...',jTaskGA(iTask)
        else if (aTaskGA(iTask).eq.1) then
* ....... u_i =!= 0
          write(*,'(a,i5,a,i5)') 
     &            'task #',iTask,'): initialize i=',
     &             iTaskGA(iTask)
        else if (aTaskGA(iTask).eq.2) then
* ....... u_i <- u_i + lambda u_j
          write(*,'(a,i5,a,i5,a,i5,a,f12.5,a,i5)') 
     &            'task #',iTask,'): mode',iTaskGA(iTask),
     &                          ' <- mode',iTaskGA(iTask),
     &                         ' + ',fTaskGA(iTask),
     &                         ' x mode',jTaskGA(iTask)
        else if (aTaskGA(iTask).eq.3) then
          write(*,'(a,i5,a,i5,a,i5,a,i5)') 
     &            'task #',iTask,'): orthonormalize mode ',
     &                     iTaskGA(iTask), ' w.r.t. modes',
     &                     jTaskGA(iTask), ' ...',
     &                     kTaskGA(iTask)
        end if
      end do
      call pressRETURN
* --- end of subroutine ---------------------------------------------- *
      return
      end

