************************************************************************
*****                                                              *****
***** @segment name: xs_ioYGP.f                                    *****
***** @task: I/O routines for Galerkin projection                  *****
***** @see:  res.20100408.xAMC_ToDo.tex                            *****
*****                                                              *****
************************************************************************
* === readYGP
* === showYGP
*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012

************************************************************************
* === read parameters for Galerkin projection ======================== *
      subroutine readYGP
      implicit none
* --- Galerkin system
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
      include 'xv_GP2.include'    ! tasks for Galerkin projection
* --- auxiliary variables
      character*120 Line
      logical       bOK
* --- begin of subroutine -------------------------------------------- *
      write(*,*) 'Reading << y2_GM.dat >> ...'
      open(10,file  = 'y2_GM.dat',status= 'OLD',form='formatted')
* ... read Reynolds number
      call readLine(10,Line,bOK)
      read(Line,*) fRe
* ... read number of volume forces
      call readLine(10,Line,bOK)
      read(Line,*) NVF
* --- 
      NTaskGP = 0
  101 call readline(10,Line,bOK)
* --- mass matrix
      if (index(Line,'mass').gt.0) then
        NTaskGP = NTaskGP + 1
        aTaskGP(NTaskGP) = 0
C XXX   read(Line,*) fTaskGP(NTaskGP)
        goto 101
* --- viscous term
      else if (index(Line,'viscous').gt.0) then
        NTaskGP = NTaskGP + 1
        aTaskGP(NTaskGP) = 1
        fTaskGP(NTaskGP) = 1.0
C XXX   read(Line,*) fTaskGP(NTaskGP)
        goto 101
* --- compute convective term
      else if (index(Line,'convective').gt.0) then
        NTaskGP = NTaskGP + 1
        aTaskGP(NTaskGP) = 2
        fTaskGP(NTaskGP) = 1.0
C XXX   read(Line,*) fTaskGP(NTaskGP)
        goto 101
* --- compute pressure term
      else if (index(Line,'pressure').gt.0) then
        NTaskGP = NTaskGP + 1
        aTaskGP(NTaskGP) = 3
        fTaskGP(NTaskGP) = 1.0
C XXX   read(Line,*) fTaskGP(NTaskGP)
        goto 101
      end if
C XXX write(*,*) NTaskGP, ' tasks'
* --- end of file
      close(10)
* --- end of subroutine ---------------------------------------------- *
      return
      end

************************************************************************
* === show parameters for Galerkin projection ======================== *
      subroutine showYGP
      implicit none
* --- Galerkin system
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
      include 'xv_GP2.include'    ! tasks for Galerkin projection
* --- auxiliary variables
      integer iTask
* --- begin of subroutine -------------------------------------------- *
      write(*,*) '========================================'
      write(*,*) 'Contruction of Galerkin projection ...'
      write(*,'(a,f12.2)') 'Re = ', fRe
      write(*,'(2a6,2x,a)') 'act.','phys.','dimension of'
      write(*,'(2i6,2x,a)') NVF, NVFP, 'forces l=1..N_V'
      write(*,*)
      do iTask = 1, NTaskGP
        if (aTaskGP(iTask).eq.0) then
          write(*,'(a,i5,a)') 
     &            'task #',iTask,'): compute mass matrix.'
        else if (aTaskGP(iTask).eq.1) then
          write(*,'(a,i5,a)') 
     &            'task #',iTask,'): compute viscous term.'
        else if (aTaskGP(iTask).eq.2) then
          write(*,'(a,i5,a)') 
     &            'task #',iTask,'): compute convective term.'
        else if (aTaskGP(iTask).eq.3) then
          write(*,'(a,i5,a)') 
     &            'task #',iTask,'): compute pressure term.'
        end if
      end do
      call pressRETURN  
* --- end of subroutine ---------------------------------------------- *
      return
      end
