************************************************************************
*****                                                              *****
***** @segment name: xs_plotAmplitudes.f                           *****
***** @task: reading and plotting routines for a_i(t), ...         *****
***** @see:  res.20100408.xAMC_ToDo.tex                            *****
*****                                                              *****
************************************************************************
* === subroutine readAmplitudes(fTm,fAim,NSnap)
* === subroutine plotAmplitudes(fTm,fAim,NSnap)
* === subroutine plotPhasePortrait(fTm,fAim,NSnap)
*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012

************************************************************************
* === Reading the solution =========================================== *
      subroutine readAmplitudes(fTm,fAim,NSnap)
      implicit none
      include 'xv_DS.include'      ! NAP,...   | dimensions of DS
      include 'xv_DS_Solution.include' ! NSnap,... | dimensions of array
      integer*8 iDummy, iSnap
      real*8    fDummy, fMax, fMin
      integer   iOpt                   ! option of modes
* --- file name
      character*120 fn
      integer*8     fnLength
* --- Begin of subroutine -------------------------------------------- *
* --- User input
      write(*,*) '< 1 > ... a_i of Galerkin expansion'
      write(*,*) '< 2 > ... a_i of Galerkin system solution'
      read (*,*) iOpt
* --- Reading the time ...123456789012
      if (iOpt.eq.1) fn= 'z1_Times.dat'
      if (iOpt.eq.2) fn= 'z3_Times.dat'
      fnLength = 12
      write(*,*) 'Reading << ',fn(1:fnLength),' >> ...'
      NSnap = 1
 111  open(10,file=fn(1:fnLength),form='formatted',status='old')
      read(10,*,end=199,err=199) iSnap, fDummy
      fTm(iSnap) = fDummy
      NSnap = max(iSnap,NSnap)
      goto 111
 199  close(10)
      write(*,'(2(a,f12.5),a,i8)') 
     &        't_min=',fTm(1),'; t_max=',fTm(NSnap),'; M=',NSnap
* --- Reading the amplitudes
* --- ....................12345678901234567
      if (iOpt.eq.1) fn= 'z1_Amplitudes.dat'
      if (iOpt.eq.2) fn= 'z3_Amplitudes.dat'
      fnLength = 17
      write(*,*) 'Reading << ',fn(1:fnLength),' >> ...'
      NA    = 1
      NSnap = 1
      fMin  =  1.E37
      fMax  = -1.E37
 211  open(10,file=fn(1:fnLength),form='formatted',status='old')
      read(10,*,end=299,err=299) iSnap, iDummy, fDummy
      if (iDummy.gt.0) then
        fAim(iDummy,iSnap) = fDummy !
        fMax  = max(fMax,  fDummy) 
        fMin  = min(fMin,  fDummy) 
        NA    = max(iDummy,NA)
        NSnap = max(iSnap, NSnap)
      end if
      goto 211
 299  close(10)
      write(*,'(2(a,f12.5),2(a,i8))') 
     &        'min(a_i)=',fMin,'; max(a_i)=',fMax,
     &        '; N=',NA, '; M=',NSnap
* --- End of subroutine ---------------------------------------------- *
      return
      end

************************************************************************
* === Plotting the solution ========================================== *
      subroutine plotAmplitudes(fTm,fAim,NSnap)
      implicit none
      include 'xv_DS.include'     ! NAP,...   | dimensions of DS
      include 'xv_DS_Solution.include'! NSnap,... | dimensions of array
      integer*8 iDummy, iSnap
      character*14 fName
* --- Begin of subroutine -------------------------------------------- *
      write(*,*) 'Plotting the solution ...'
      do iDummy = 1, NA
* ----- Creating gnuplot data file
* ----- 12345678901234
* ===== gnu_ta0000.dat
        write(fName,'(a,i4.4,a)') 'gnu_ta',iDummy,'.dat'
        write(*,'(a,a,a)') 'Creating <<',fName,'>> ...'
        open(10,file=fName,status='UNKNOWN',form='FORMATTED')
        do iSnap=1,NSnap
          write(10,'(f20.10,1x,f20.10)') 
     &              fTm(iSnap),fAim(iDummy,iSnap)
        end do
        close(10)
* ----- Creating gnuplot script
        open(10,file='tmp.gnuplot',status='UNKNOWN',form='FORMATTED')
        write(10,'(3a)') 'plot "',fName,'" using 1:2 with lines'
        write(10,'(a)') 'set title "solution of dynamical system"'
        write(10,'(a)') 'set xlabel "time"'
        if (NA.le.9) then
          write(10,'(a,i1.1,a)') 'set ylabel "a_',iDummy,'"'
        else if (NA.le.99) then
          write(10,'(a,i2.2,a)') 'set ylabel "a_',iDummy,'"'
        else if (NA.le.999) then
          write(10,'(a,i3.3,a)') 'set ylabel "a_',iDummy,'"'
        else
          write(10,'(a,i4.4,a)') 'set ylabel "a_',iDummy,'"'
        end if
        write(10,'(a)') 'set terminal png'
        write(fName,'(a,i4.4,a)') 'gnu_ta',iDummy,'.png'
        write(10,'(3a)') 'set output "',fName(1:14),'"'
        write(10,'(a)') 'replot'
        close(10)
* ----- Executing gnuplot script
        open(10,file='tmp.sh',status='UNKNOWN',form='FORMATTED')
        write(10,'(a)') 'gnuplot tmp.gnuplot'
        close(10)
        call system("chmod u+x tmp.sh")
        call system("./tmp.sh")
      end do
* --- Creating a LaTeX file comprising all EPS files
      write(*,*) 'Creating gnu_ta.[tex,pdf] ...'
      open(unit=10,file  ='gnu_ta.tex',    status='unknown',
     &             access='sequential', form  ='formatted')
* === NOTE: char(92)='\' should run on all fortran compilers.
* === Different fortran compilers  assume 
* === '\' exclusive or '\\' for the backslash.
      write(10,'(a)') char(92)//'documentclass[twocolumn]{article}'
      write(10,'(a)') char(92)//'usepackage{graphicx}'
      write(10,'(a)') char(92)//'DeclareGraphicsExtensions{png}'
      write(10,'(a)') char(92)//'begin{document}'
      do iDummy=1,NA
      write(10,'(a)') char(92)//'begin{figure}'
      write(10,'(2a,i4.4,a)') 
     &         char(92)//'includegraphics[width=75mm,angle=-00]',
     &         '{gnu_ta',iDummy,'.png}'
      write(10,'(a,i4,a)') 
     &         char(92)//'caption{Evolution of mode amplitude ',
     &                iDummy,'.}'
      write(10,'(a)') char(92)//'end{figure}'
* --- to prevent << too many unprocessed floats >>
      if (mod(iDummy,4).eq.0) 
     &   write(10,'(a)') char(92)//'clearpage' 
      end do
      write(10,'(a)') char(92)//'end{document}'
      close(10)
* --- Compile LaTeX file
      call system("pdflatex  gnu_ta.tex")
* ---
  901 format(1h a)
* --- End of subroutine ---------------------------------------------- *
      return
      end

************************************************************************
* === Plotting the phase portraits =================================== *
      subroutine plotPhasePortrait(fTm,fAim,NSnap)
      implicit none
      include 'xv_DS.include'     ! NAP,...   | dimensions of DS
      include 'xv_DS_Solution.include'! NSnap,... | dimensions of array
      integer*8 iDummy, jDummy, iSnap
      character*21 fName
* --- Begin of subroutine -------------------------------------------- *
      write(*,*) 'Plotting the phase portraits ...'
      do iDummy = 1, NA
      do jDummy = 1, NA
* ----- creating gnuplot data file
* ----- 123456789012345678
* ===== gnu_a0000a0000.dat
        write(fName,'(a,i4.4,a,i4.4,a)') 
     &              'gnu_a',iDummy,'a',jDummy,'.dat'
        write(*,'(a,a,a)') 'Creating <<',fName,'>> ...'
        open(10,file=fName,status='UNKNOWN',form='FORMATTED')
        do iSnap=1,NSnap
          write(10,'(f20.10,1x,f20.10)') 
     &              fAim(iDummy,iSnap), fAim(jDummy,iSnap)
        end do
        close(10)
* ----- Creating gnuplot shell script
        open(10,file='tmp.gnuplot',status='UNKNOWN',form='FORMATTED')
        write(10,'(3a)') 'plot "',fName(1:18),'" using 1:2 with lines'
        write(10,'(a)') 'set title "xAMC 2010-11-10: phase portraits"'
        if (NA.le.9) then
          write(10,'(a,i1.1,a)') 'set xlabel "a_',iDummy,'"'
        else if (NA.le.99) then
          write(10,'(a,i2.2,a)') 'set xlabel "a_',iDummy,'"'
        else if (NA.le.999) then
          write(10,'(a,i3.3,a)') 'set xlabel "a_',iDummy,'"'
        else
          write(10,'(a,i4.4,a)') 'set xlabel "a_',iDummy,'"'
        end if
        if (NA.le.9) then
          write(10,'(a,i1.1,a)') 'set ylabel "a_',jDummy,'"'
        else if (NA.le.99) then
          write(10,'(a,i2.2,a)') 'set ylabel "a_',jDummy,'"'
        else if (NA.le.999) then
          write(10,'(a,i3.3,a)') 'set ylabel "a_',jDummy,'"'
        else
          write(10,'(a,i4.4,a)') 'set ylabel "a_',jDummy,'"'
        end if
        write(10,'(a)') 'set terminal png'
        write(fName,'(a,i4.4,a,i4.4,a)') 
     &              'gnu_a',iDummy,'a',jDummy,'.png'
        write(10,'(3a)') 'set output "',fName(1:18),'"'
        write(10,'(a)') 'replot'
        close(10)
* ----- Executing gnuplot shell script
        open(10,file='tmp.sh',status='UNKNOWN',form='FORMATTED')
        write(10,'(a)') 'gnuplot tmp.gnuplot'
        close(10)
        call system("chmod u+x tmp.sh")
        call system("./tmp.sh")
      end do
      end do
* --- Creating latex file comprising
      write(*,*) 'Creating gnu_aa.[tex,pdf] ...'
      open(unit=10,file  ='gnu_aa.tex',    status='unknown',
     &             access='sequential', form  ='formatted')
* === NOTE: char(92)='\' should run on all fortran compilers.
* === Different fortran compilers  assume 
* === '\' exclusive or '\\' for the backslash.
      write(10,'(a)')
     &      char(92)//'documentclass[twocolumn,a4paper]{article}'
      write(10,'(a)') char(92)//'usepackage{graphicx}'
C === write(10,'(a)') char(92)//'DeclareGraphicsExtensions{png}'
      write(10,'(a)') char(92)//'begin{document}'
      do iDummy=1,NA
        if (iDummy.gt.1) write(10,'(a,i4,a)') char(92)//'clearpage'
        write(10,'(a,i4,a)') char(92)//'section*{Mode ',iDummy,'}'
        write(10,'(a)') char(92)//'newpage'
        do jDummy=1,NA
          write(10,'(a)') char(92)//'begin{figure}'
          write(10,'(2a,i4.4,a,i4.4,a)') 
     &         char(92)//'includegraphics[width=70mm,angle=-00]',
     &         '{gnu_a',iDummy,'a',jDummy,'.png}'
          write(10,'(a,i4,a,i4,a)') 
     &         char(92)//'caption{Phase portrait ',
     &                iDummy,' -',jDummy,' plane.}'
          write(10,'(a)') char(92)//'end{figure}'
          if (mod(jDummy,6).eq.3) 
     &      write(10,'(a)') char(92)//'clearpage' 
        end do
        write(10,'(a)') char(92)//'clearpage' 
      end do
      write(10,'(a)') char(92)//'end{document}'
      close(10)
* --- Compile the LaTeX file
      call system("pdflatex  gnu_aa.tex")
* ---
  901 format(1h a)
* --- End of subroutine ---------------------------------------------- *
      return
      end

